// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package hello;

import java.awt.*;
import java.awt.event.*;

public class Client extends java.applet.Applet implements ActionListener
{
    static int
    run(org.omg.CORBA.ORB orb, String[] args)
        throws org.omg.CORBA.UserException
    {
        //
        // Resolve Root POA
        //
        org.omg.PortableServer.POA root =
            org.omg.PortableServer.POAHelper.narrow(
                orb.resolve_initial_references("RootPOA"));
        org.omg.PortableServer.POAManager manager = root.the_POAManager();

        //
        // Create callback implementation object
        //
        HelloCallback_impl cbImpl = new HelloCallback_impl(root);
        HelloCallback cb = cbImpl._this(orb);

        //
        // Get "hello" object
        //
        org.omg.CORBA.Object obj = orb.string_to_object("relfile:/Hello.ref");
        if(obj == null)
        {
            System.err.println("hello.Client: cannot read IOR from Hello.ref");
            return 1;
        }
        Hello hello = HelloHelper.narrow(obj);

        //
        // Activate the POAManager so that callbacks will be dispatched
        //
        manager.activate();

        //
        // Main loop
        //
        System.out.println("Enter 'h' for hello, 's' to shutdown server or " +
                           "'x' for exit:");

        int c;

        try
        {
            String input;
            do
            {
                System.out.print("> ");
                java.io.BufferedReader in = new java.io.BufferedReader(
                    new java.io.InputStreamReader(System.in));
                input = in.readLine();

                if(input.equals("h"))
                    hello.say_hello(cb);
                else if(input.equals("s"))
                    hello.deactivate();
            }
            while(!input.equals("x"));
        }
        catch(java.io.IOException ex)
        {
            System.err.println("Can't read from `" + ex.getMessage() + "'");
            return 1;
        }

        return 0;
    }

    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        int status = 0;
        org.omg.CORBA.ORB orb = null;

        try
        {
            //
            // Initialize the ORB
            //
            orb = org.omg.CORBA.ORB.init(args, props);

            status = run(orb, args);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }

    //
    // Members only needed for applet
    //

    private static class AppletCallback_impl extends HelloCallbackPOA
    {
        private org.omg.PortableServer.POA poa_;
        private Label label_;
        private int counter_;

        AppletCallback_impl(org.omg.PortableServer.POA poa, Label label)
        {
            poa_ = poa;
            label_ = label;
            counter_ = 0;
        }

        public void
        hello_callback()
        {
            counter_++;
            label_.setText("Callback #" + counter_);
        }
    }

    private Hello hello_;
    private Button button_;
    private HelloCallback callback_;

    //
    // Applet initialization
    //
    public void
    init()
    {
        try
        {
            String ior = getParameter("ior");

            //
            // Set configuration properties for applet
            //
	    java.util.Properties props = new java.util.Properties();
            props.put("ooc.oci.client", "iiop,bidir --protocol iiop");
            props.put("ooc.oci.server", "iiop,bidir --protocol iiop");
            props.put("ooc.orb.oa.endpoint", "bidir_iiop --callback");
            props.put("ooc.orb.oa.conc_model", "threaded");
            props.put("ooc.bidir.peer", "HelloClientApplet");

            //
            // Create ORB
            //
            org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(this, props);

            //
            // Resolve Root POA
            //
            org.omg.PortableServer.POA root =
                org.omg.PortableServer.POAHelper.narrow(
                    orb.resolve_initial_references("RootPOA"));
            org.omg.PortableServer.POAManager manager = root.the_POAManager();

            //
            // Create callback implementation object
            //
            Label label = new Label("No callbacks yet", Label.CENTER);
            AppletCallback_impl cbImpl = new AppletCallback_impl(root, label);
            callback_ = cbImpl._this(orb);

            //
            // Get "hello" object
            //
            org.omg.CORBA.Object obj = orb.string_to_object(ior);
            if(obj == null)
                throw new RuntimeException();
            hello_ = HelloHelper.narrow(obj);

            //
            // Activate the POAManager so that callbacks will be dispatched
            //
            manager.activate();

            //
            // Add GUI components
            //
            setLayout(new BorderLayout());
            this.add(label, BorderLayout.CENTER);
            button_ = new Button("Hello");
            button_.addActionListener(this);
            Panel buttonPanel = new Panel();
            buttonPanel.add(button_);
            this.add(buttonPanel, BorderLayout.SOUTH);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
        }
    }

    //
    // Handle events
    //
    public void
    actionPerformed(ActionEvent event)
    {
        hello_.say_hello(callback_);
    }
}
