// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.BiDir;

//
// Message represents an outgoing message and encapsulates the
// header buffer (i.e., the buffer holding the bi-directional protocol
// header) and the body buffer.
//
class Message
{
    //
    // The header and body buffers
    //
    private com.ooc.OCI.Buffer header_;
    private com.ooc.OCI.Buffer body_;

    //
    // My state
    //
    private static final int Unsent    = 0;
    private static final int Sent      = 1;
    private static final int Exception = 2;
    private int state_;
    private java.lang.Object stateMonitor_ = new java.lang.Object();

    //
    // The exception
    //
    private org.omg.CORBA.SystemException ex_;

    // ------------------------------------------------------------------
    // Private member implementations
    // ------------------------------------------------------------------

    // ------------------------------------------------------------------
    // Constructors
    // ------------------------------------------------------------------

    Message(com.ooc.OCI.Buffer header, com.ooc.OCI.Buffer body)
    {
        header_ = header;
        body_ = body;
        state_ = Unsent;
    }

    // ------------------------------------------------------------------
    // Package member implementations
    // ------------------------------------------------------------------

    com.ooc.OCI.Buffer
    header()
    {
        return header_;
    }

    com.ooc.OCI.Buffer
    body()
    {
        return body_;
    }

    void
    waitUntilSent(boolean block)
    {
        if(!block)
            Thread.yield();

        synchronized(stateMonitor_)
        {
            while(state_ == Unsent)
            {
                if(!block)
                    return;

                try
                {
                    stateMonitor_.wait();
                }
                catch(InterruptedException ex)
                {
                }
            }
        }

        if(ex_ != null)
            throw ex_;
    }

    void
    waitUntilSent(int t)
    {
        if(t == 0)
            Thread.yield();

        synchronized(stateMonitor_)
        {
            while(state_ == Unsent)
            {
                try
                {
                    int oldState = state_;

                    stateMonitor_.wait(t);

                    if(state_ == oldState)
                        throw new org.omg.CORBA.NO_RESPONSE();
                }
                catch(InterruptedException ex)
                {
                }
            }
        }

        if(ex_ != null)
            throw ex_;
    }

    void
    sent()
    {
        synchronized(stateMonitor_)
        {
            state_ = Sent;
            stateMonitor_.notify();
        }
    }

    void
    exception(org.omg.CORBA.SystemException ex)
    {
        synchronized(stateMonitor_)
        {
            state_ = Exception;
            ex_ = ex;
            stateMonitor_.notify();
        }
    }
}
