// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

public class Consumer
{
    static private org.omg.CosEventChannelAdmin.EventChannel
    GetEventChannel(org.omg.CORBA.ORB orb, String channelRef)
    {
	//
	// Get a reference to the Event Service EventChannel
	//
	org.omg.CosEventChannelAdmin.EventChannel eventChannel = null;
	try
	{
	    org.omg.CORBA.Object obj = null;

	    if(channelRef != null)
		obj = orb.string_to_object(channelRef);
	    else
		obj = orb.resolve_initial_references("EventService");

	    //
	    // Narrow the org.omg.CORBA.Object reference to an
	    // EventChannel reference so we can invoke its methods
	    //
	    eventChannel =
		org.omg.CosEventChannelAdmin.EventChannelHelper.narrow(obj);
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	    System.err.println("Can't resolve `EventService'");
	    System.exit(1);
	}
	catch(org.omg.CORBA.BAD_PARAM ex)
	{
	    // narrow() failed
	}
	    
	//
	// Make sure the CORBA object was really an EventChannelFactory
	//
	if(eventChannel == null)
	{
	    System.err.println("`EventService' initial reference is not a " +
			       "org.omg.CosEventChannelAdmin.EventChannel\n");
	    System.exit(1);
	}

	return eventChannel;
    }

    static int
    run(org.omg.CORBA.ORB orb, String args[])
        throws org.omg.CORBA.UserException
    {
	int totalEvents = 0;
	String outputFile = null;
	String channelRef = null;
	boolean verbose = false;

        //
        // Get command line options
        //
        for(int i = 0 ; i < args.length ; i++)
        {
	    if(args[i].equals("--total-events"))
		totalEvents = Integer.parseInt(args[++i]);

	    else if(args[i].equals("--output-file"))
		outputFile = args[++i];

	    else if(args[i].equals("--channel-ref"))
		channelRef = args[++i];

	    else if(args[i].equals("--verbose"))
		verbose = true;
        }

        //
        // Resolve Root POA.
        //
        org.omg.PortableServer.POA rootPOA =
	    org.omg.PortableServer.POAHelper.narrow(
		orb.resolve_initial_references("RootPOA"));

        //
        // Get a reference to the POA manager.
        //
        org.omg.PortableServer.POAManager manager = rootPOA.the_POAManager();

        //
        // Get Event Channel
        //
        org.omg.CosEventChannelAdmin.EventChannel eventChannel =
	    GetEventChannel(orb, channelRef);

	//
	// Obtain the Consumer Admin
	//
	org.omg.CosEventChannelAdmin.ConsumerAdmin consumerAdmin =
	    eventChannel.for_consumers();

	//
	// Obtain Proxy Supplier -- we are using the "Push" model
	//
	org.omg.CosEventChannelAdmin.ProxyPushSupplier proxyPushSupplier =
	    consumerAdmin.obtain_push_supplier();

        //
	// Create the Push Consumer and connect to the Proxy Supplier
        //
	StringBuffer sout = new StringBuffer();
        PushConsumer_impl impl = new PushConsumer_impl(orb,
						       rootPOA,
						       totalEvents,
						       verbose,
						       sout);

	//
	// Connect the consumer to the POA
	//
        org.omg.CosEventComm.PushConsumer pushConsumer = impl._this(orb);

	try 
	{
	    proxyPushSupplier.connect_push_consumer(pushConsumer);
	}
	catch(org.omg.CosEventChannelAdmin.AlreadyConnected ex)
	{
	}

        //
        // Activate the POAManager.
        //
        manager.activate();

	//
	// Run implementation.
	//
	orb.run();

	//
	// Remove the Proxy Push Supplier
	//
        try
        {
	    proxyPushSupplier.disconnect_push_supplier();
        }
        catch(org.omg.CORBA.TRANSIENT ex)
        {
        }
	catch(org.omg.CORBA.COMM_FAILURE ex)
	{
	}
	catch(org.omg.CORBA.NO_RESPONSE ex)
	{
	}
	catch(org.omg.CORBA.OBJECT_NOT_EXIST ex)
	{
	}
    
	//
	// Display our results
	//
	if(outputFile != null)
	{
	    try
	    {
		java.io.FileOutputStream file =
		    new java.io.FileOutputStream(outputFile);
		java.io.PrintWriter out = new java.io.PrintWriter(file);
		out.print(sout.toString());
		out.flush();
		file.close();
	    }
	    catch(java.io.IOException ex)
	    {
		System.err.println("can't write to `" + ex.getMessage() + "'");
		return 1;
	    }
	}
	else
	    System.out.print(sout.toString());

	return 0;
    }

    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        int status = 0;
        org.omg.CORBA.ORB orb = null;

        try
        {
            args = com.ooc.CORBA.ORB.ParseArgs(args, props, null);

	    props.put("ooc.orb.conc_model", "threaded");
	    props.put("ooc.orb.oa.conc_model", "threaded");

	    //
	    // Create ORB.
	    //
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);
            status = run(orb, argsH.value);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
