// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

public class Supplier
{
    static private org.omg.CORBA.Any
    CreateNewEvent(org.omg.CORBA.ORB orb, String eventDataFile)
    {
	String s = "";

	try
	{
	    if(eventDataFile != null)
	    {
		java.io.FileInputStream file =
		    new java.io.FileInputStream(eventDataFile);
		java.io.BufferedReader in = new java.io.BufferedReader(
		    new java.io.InputStreamReader(file));

		int MaxArraySize = 5000;
		char[] buf = new char[MaxArraySize];
		int len = -1;
		try
		{
		    len = in.read(buf, 0, MaxArraySize - 1);
		}
		catch(java.io.IOException ex)
		{
		}

		in.close();

		if(len > 0)
		    s = new String(buf, 0, len);
	    }
	}
	catch(java.io.IOException ex)
	{
	}

	org.omg.CORBA.Any any = orb.create_any();
	any.insert_string(s);

	return any;
    }

    static private org.omg.CosEventChannelAdmin.EventChannel
    GetEventChannel(org.omg.CORBA.ORB orb, String channelRef)
    {
	//
	// Get a reference to the Event Service EventChannel
	//
	org.omg.CosEventChannelAdmin.EventChannel eventChannel = null;
	try
	{
	    org.omg.CORBA.Object obj = null;

	    if(channelRef != null)
		obj = orb.string_to_object(channelRef);
	    else
		obj = orb.resolve_initial_references("EventService");

	    //
	    // Narrow the org.omg.CORBA.Object reference to an
	    // EventChannel reference so we can invoke its methods
	    //
	    eventChannel =
		org.omg.CosEventChannelAdmin.EventChannelHelper.narrow(obj);
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	    System.err.println("Can't resolve `EventService'");
	    System.exit(1);
	}
	catch(org.omg.CORBA.BAD_PARAM ex)
	{
	    // narrow() failed
	}
	    
	//
	// Make sure the CORBA object was really an EventChannelFactory
	//
	if(eventChannel == null)
	{
	    System.err.println("`EventService' initial reference is not a " +
			       "org.omg.CosEventChannelAdmin.EventChannel\n");
	    System.exit(1);
	}

	return eventChannel;
    }

    static int
    run(org.omg.CORBA.ORB orb, String args[])
        throws org.omg.CORBA.UserException
    {
	String eventDataFile = null;
	int totalEvents = 0;
	String outputFile = null;
	String channelRef = null;
	boolean verbose = false;

        //
        // Get command line options
        //
        for(int i = 0 ; i < args.length ; i++)
        {
	    if(args[i].equals("--total-events"))
		totalEvents = Integer.parseInt(args[++i]);

	    else if(args[i].equals("--event-data-file"))
		eventDataFile = args[++i];

	    else if(args[i].equals("--output-file"))
		outputFile = args[++i];

	    else if(args[i].equals("--channel-ref"))
		channelRef = args[++i];

	    else if(args[i].equals("--verbose"))
		verbose = true;
        }

        //
        // Get event channel
        //
        org.omg.CosEventChannelAdmin.EventChannel eventChannel =
	    GetEventChannel(orb, channelRef);
	    
        //
        // Obtain a Supplier Admin
        //
        org.omg.CosEventChannelAdmin.SupplierAdmin supplierAdmin =
	    eventChannel.for_suppliers();

	//
	// Obtain a Proxy Consumer -- we are using the "Push" model
	//
        org.omg.CosEventChannelAdmin.ProxyPushConsumer
	    proxyPushConsumer = supplierAdmin.obtain_push_consumer();

        //
	// Connect to the Proxy Consumer
        //
        proxyPushConsumer.connect_push_supplier(null);

	//
	// Create an event
	//
	org.omg.CORBA.Any event = CreateNewEvent(orb, eventDataFile);

	//
	// Let things settle down before we start pushing events
	//
	try
	{
	    Thread.sleep(1000);
	}
	catch(java.lang.InterruptedException ex)
	{
	}

	//
	// Push events
	//
	int i = 0;
	StringBuffer sout = new StringBuffer();
	while(i < totalEvents)
	{
	    if(i == 0)
	    {
		long now = System.currentTimeMillis();

		sout.append("First event sent by supplier: ");
		sout.append(now / 1000);
		sout.append(".");
		if(now % 1000 < 100)
		    sout.append("0");
		if(now % 1000 < 10)
		    sout.append("0");
		sout.append(now % 1000);
		sout.append("\n");
	    }
	    
	    try
	    {
		//
		// Push the event
		//
	        proxyPushConsumer.push(event);
		if(verbose && (i % 10) == 0)
		{
		    System.err.print("-s-" + i + " ");
		    System.err.flush();
		}
	    }
	    catch(org.omg.CosEventComm.Disconnected ex)
	    {
		System.err.println("Caught Disconnected Exception in the " +
				   "Supplier while pushing events.  At " +
				   "event " + i + " of " + totalEvents);
	        break;
	    }
	    catch(org.omg.CORBA.SystemException ex)
	    {
		System.err.println("Caught System Exception in the Supplier" +
				   "while pushing events.  At event " + i +
				   " of " + totalEvents);
	        ex.printStackTrace();
	        break;
	    }
	    ++i;
	}
	if(verbose)
	{
	    System.err.print("-s-Disconnect ");
	    System.err.flush();
	}

	//
	// Remove the Proxy Push Consumer
	//
        try
        {
	    proxyPushConsumer.disconnect_push_consumer();
        }
        catch(org.omg.CORBA.TRANSIENT ex)
        {
        }
	catch(org.omg.CORBA.COMM_FAILURE ex)
	{
	}

	//
	// Display our results
	//
	if(outputFile != null)
	{
	    try
	    {
		java.io.FileOutputStream file =
		    new java.io.FileOutputStream(outputFile);
		java.io.PrintWriter out = new java.io.PrintWriter(file);
		out.print(sout.toString());
		out.flush();
		file.close();
	    }
	    catch(java.io.IOException ex)
	    {
		System.err.println("can't write to `" + ex.getMessage() + "'");
		return 1;
	    }
	}
	else
	    System.out.print(sout.toString());

	return 0;
    }

    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        int status = 0;
        org.omg.CORBA.ORB orb = null;

        try
        {
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);
            status = run(orb, argsH.value);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
