// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosEvent;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.CosEventChannelAdmin.*;
import org.omg.CosTypedEventChannelAdmin.*;

final class EventAdmin
{
    final static String progName = "com.ooc.CosEvent.EventAdmin";

    private static void
    usage()
    {
        System.err.println(
            "Usage:\n" +
            progName +  " [options] command\n" +
    	    "\n" +
	    "Options:\n" +
	    "-h, --help             Show this message.\n" +
	    "-v, --version          Show version number.\n" +
            "-t, --typed-service    Admin for typed event service.\n" +
            "-u, --untyped-service  " +
	    "Admin for untyped event service [default].\n" +
	    "\n" +
	    "Commands:\n" +
	    "create channel-id      Create a new event channel.\n" +
	    "destroy channel-id     Destroy an event channel.\n" +
	    "list                   Show the active event channels.\n" +
	    "shutdown               Terminate the event service.\n"
	    );
    }

    static int
    run(ORB orb, String[] args)
        throws UserException
    {
	int i;
	boolean typedService = false;

 	//
	// Resolve root POA
	//
	POA rootPOA =
            POAHelper.narrow(orb.resolve_initial_references("RootPOA"));

        //
	// Get options
	//
	for(i = 0 ; i < args.length && args[i].charAt(0) == '-' ; ++i)
	{
	    if(args[i].equals("--help") || args[i].equals("-h"))
	    {
		usage();
		return 0;
	    }
	    else if(args[i].equals("--version") || args[i].equals("-v"))
	    {
		System.err.println("ORBacus Event Service " +
				   com.ooc.OB.Version.version);
		return 0;
	    }
	    else if(args[i].equals("--untyped-service") ||
		    args[i].equals("-u"))
	    {
	    }
	    else if(args[i].equals("--typed-service") ||
		    args[i].equals("-t"))
            {
                typedService = true;
            }
	    else
	    {
		System.err.println(progName +  ": unknown option `" +
		    		   args[i] + "'\n");
		usage();
		return 1;
	    }
	}

        if(i == args.length)
        {
            usage();
	    return 1;
        }

        String  serviceName = typedService 
	    ? "TypedEventChannelFactory" : "EventChannelFactory";
        
        org.omg.CORBA.Object obj = null;
        try
	{
            obj = orb.resolve_initial_references(serviceName);
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName ex)
        {
        }

        if(obj == null)
        {
	    System.err.println("No `" + serviceName + "' initial reference.");
  	    return 1;
        }

        if(typedService)
        {
            com.ooc.OBTypedEventChannelFactory.TypedEventChannelFactory f =
                null;
            try
            {
                f = com.ooc.OBTypedEventChannelFactory.
		    TypedEventChannelFactoryHelper.narrow(obj);
            }
            catch(BAD_PARAM ex)
            {
                // narrow() failed
            }

            if(f == null)
            {
		System.err.println("`" + serviceName +
		    		   "' initial reference is invalid.");
		return 1;
            }

            if(i + 1 < args.length && args[i].equals("create"))
            {
                try
                {
                    TypedEventChannel channel = f.create_channel(args[i + 1]);
                }
                catch(com.ooc.OBEventChannelFactory.ChannelAlreadyExists ex)
                {
		    System.err.println("Channel `" + args[i + 1] +
				       "' already exists");
                    return 1;
                }
            }
            else if(i + 1 < args.length && args[i].equals("destroy"))
            {
                try
                {
                    TypedEventChannel channel =
                        f.get_channel_by_id(args[i + 1]);
                    channel.destroy();
                }
                catch(com.ooc.OBEventChannelFactory.ChannelNotAvailable ex)
                {
		    System.err.println("Channel `" + args[i + 1] + 
                                       "' not found");
                    return 1;
                }
            }
            else if(i < args.length && args[i].equals("list"))
            {
                String[] ids = f.get_channels();
                if(ids.length == 0)
                    System.err.println("No active channels");
                else
                {
                    for(int j = 0 ; j < ids.length ; ++j)
                        System.out.println(ids[j]);
                }
            }
            else if(i < args.length && args[i].equals("shutdown"))
            {
                try
                {
                    f.shutdown();
                }
                catch(org.omg.CORBA.TRANSIENT ex)
                {
                    //
                    // This call will deactivate the server and thus
                    // throw an exception.
                    //
                }
            }
            else
                usage();
        }
        else
        {
            com.ooc.OBEventChannelFactory.EventChannelFactory f =
                null;
            try
            {
                f = com.ooc.OBEventChannelFactory.
                    EventChannelFactoryHelper.narrow(obj);
            }
            catch(BAD_PARAM ex)
            {
                // narrow() failed
            }

            if(f == null)
            {
		System.err.println("`" + serviceName +
		    		   "' initial reference is invalid.");
                return 1;
            }

            if(i + 1 < args.length && args[i].equals("create"))
            {
                try
                {
                    EventChannel channel = f.create_channel(args[i + 1]);
                }
                catch(com.ooc.OBEventChannelFactory.ChannelAlreadyExists ex)
                {
		    System.err.println("Channel `" + args[i + 1] +
				       "' already exists");
                    return 1;
                }
            }
            else if(i + 1 < args.length && args[i].equals("destroy"))
            {
                try
                {
                    EventChannel channel = f.get_channel_by_id(args[i + 1]);
                    channel.destroy();
                }
                catch(com.ooc.OBEventChannelFactory.ChannelNotAvailable ex)
                {
		    System.err.println("Channel `" + args[i + 1] +
				       "' not found");
                    return 1;
                }
            }
            else if(i < args.length && args[i].equals("list"))
            {
                String[] ids = f.get_channels();
                if(ids.length == 0)
                    System.out.println("No active channels");
                else
                {
                    for(int j = 0 ; j < ids.length ; ++j)
                        System.out.println(ids[j]);
                }
            }
            else if(i < args.length && args[i].equals("shutdown"))
            {
                try
                {
                    f.shutdown();
                }
                catch(org.omg.CORBA.TRANSIENT ex)
                {
                    //
                    // This call will deactivate the server and thus
                    // throw an exception.
                    //
                }
            }
            else
                usage();
        }
	
	return 0;
    }

    public static void
    main(String args[])
    {
        int status = 0;
        ORB orb = null;

        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        try
        {
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);
            status = run(orb, argsH.value);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                System.err.println(ex.getMessage());
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
