// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosTypedEventChannelAdmin;

import com.ooc.CosEvent.*;
import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POAPackage.*;

final class TypedProxyPushConsumerDSI_impl
    extends org.omg.PortableServer.DynamicImplementation
    implements ProxyBase
{
    //
    // The ProxyBaseImpl Object
    //
    private ProxyBaseImpl proxyImpl_;

    //
    // The channel type information.
    //
    private ChannelType type_;

    //
    // The associated proxy
    //
    private ProxyBase proxy_;

    // -----------------------------------------------------------------
    // TypedProxyPushConsumerDSI_impl constructor
    // -----------------------------------------------------------------

    TypedProxyPushConsumerDSI_impl(ProxyConstructorInfo info,
    				   ChannelType type, ProxyBase proxy)
    {
	info.base = "TypedProxyPushConsumerDSI";
	proxyImpl_ = new ProxyBaseImpl(info, this);
	proxyImpl_.setState(ProxyState.Connected);
	type_ = type;
	proxy_ = proxy;
    }

    protected void
    finalize()
        throws Throwable
    {
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public synchronized boolean
    _is_a(String name)
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	proxy_.stampRequest();

	return type_.isCompatible(name);
    }

    public synchronized String[]
    _all_interfaces(POA poa, byte[] oid)
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	proxy_.stampRequest();

	return type_.getTypeIds();
    }

    public POA
    _default_POA()
    {
        return proxyImpl_.getPoa();
    }

    public void
    invoke(ServerRequest req)
    {
	synchronized(this)
	{
	    //
	    // If the proxy is dead, then don't accept any more method
	    // invocations.
	    //
	    if(proxyImpl_.getState() == ProxyState.Destroyed)
	    {
		Any any = proxyImpl_.getOrb().create_any();
		OBJECT_NOT_EXISTHelper.insert(any, new OBJECT_NOT_EXIST());
		req.set_exception(any);
		return;
	    }

	    if(proxyImpl_.getState() == ProxyState.Disconnected)
	    {
		Any any = proxyImpl_.getOrb().create_any();
		TRANSIENTHelper.insert(any, new TRANSIENT());
		req.set_exception(any);
		return;
	    }
	}

	proxy_.stampRequest();
	
	//
	// Rip out the operation name.
	//
	String opName = req.operation();

	//
	// Get a list of all of the operations supported by this
	// InterfaceDef.  Includes inherited interfaces.
	//
	OperationDescription operations[] = type_.getOperations();

	//
	// Look up the operation in the IR.
	//
	for(int i = 0; i < operations.length; i++)
	{
	    if(opName.equals(operations[i].name))
	    {
		//
		// Validate the parameters. All in parameters, and no
		// return value, no exceptions.
		//
		if(operations[i].result.kind() != TCKind.tk_void)
		{
		    Any any = proxyImpl_.getOrb().create_any();
		    BAD_OPERATIONHelper.insert(any, new BAD_OPERATION());
		    req.set_exception(any);
		    return;
		}
		
		int j;
		for(j = 0 ; j < operations[i].parameters.length; j++)
		    if(operations[i].parameters[j].mode !=
		       ParameterMode.PARAM_IN)
		    {
			Any any = proxyImpl_.getOrb().create_any();
			BAD_OPERATIONHelper.insert(any, new BAD_OPERATION());
			req.set_exception(any);
			return;
		    }
    
		if(operations[i].exceptions.length > 0)
		{
		    Any any = proxyImpl_.getOrb().create_any();
		    BAD_OPERATIONHelper.insert(any, new BAD_OPERATION());
		    req.set_exception(any);
		    return;
		}

		//
		// Create an NVList to contain the parameters for this
		// method. Firstly create the NVList.
		//
		NVList params = proxyImpl_.getOrb().create_list(0);
		
		//
		// Now process each parameter.
		//
		for(j = 0 ; j < operations[i].parameters.length; j++)
		{
		    ParameterDescription par = operations[i].parameters[j];
		    
		    //
		    // Get the name.
		    //
		    String name = par.name;
		    
		    //
		    // Create an any with the appropriate typecode.
		    //
		    Any any = proxyImpl_.getOrb().create_any();
		    any.type(par.type);
		    
		    //
		    // Add this parameter.
		    //
		    params.add_value(name, any, ARG_IN.value);
		}
		
		if(proxyImpl_.getLogger().traceEvents() >= 1)
		{
		    String msg = "invoking `" + opName + 
		                 "'\nnumber of parameters: " + params.count();
		    try
		    {
			for(int param = 0; param < params.count(); ++param)
			{
			    NamedValue nv = params.item(param);
			    msg += "\nname: " +  nv.name();
			}
		    }
		    catch(Bounds ex)
		    {
			throw new RuntimeException();
		    }
		    proxyImpl_.getLogger().trace("event", msg);
		}

		req.arguments(params);
		
		//
		// Create a new Event_impl, and forward this to the event
		// channel.
		//
		Event_impl event = new Event_impl(proxyImpl_.getLogger(),
						  params, opName,
						  proxyImpl_.getId());
		proxyImpl_.getParentPipe().receive(event);
		return;
	    }
	}
	
	//
	// No such operation.
	//
	Any any = proxyImpl_.getOrb().create_any();
	BAD_OPERATIONHelper.insert(any, new BAD_OPERATION());
	req.set_exception(any);
    }

    // ------------------------------------------------------------------
    // ProxyBase implementation
    // ------------------------------------------------------------------

    //
    // Disconnect the servant from the ORB
    //
    public void deactivateObject()
    {
	POA poa = _default_POA();
	try
	{
	    byte[] oid = poa.servant_to_id(this);
	    poa.deactivate_object(oid);
	}
	catch(ServantNotActive ex)
	{
	    throw new RuntimeException();
	}
	catch(WrongPolicy ex)
	{
	    throw new RuntimeException();
	}
	catch(ObjectNotActive ex)
	{
	    throw new RuntimeException();
	}
    }
    
    //
    // Called during disconnect
    //
    public void disconnectPeer()
    {
	proxy_ = null;
    }

    //
    // Disconnect the proxy
    //
    public synchronized void disconnect()
    {
	proxyImpl_.disconnect();
    }

    //
    // Don't accept further requests
    //
    public synchronized void discardRequests()
    {
	proxyImpl_.discardRequests();
    }

    //
    // Has this proxy been destroyed?
    //
    public synchronized boolean destroyed()
    {
	return proxyImpl_.destroyed();
    }

    //
    // Can this proxy be reaperd?
    //
    public boolean canReap()
    {
	return false;
    }

    //
    // What was the time of the last request?
    //
    public long lastRequestTime()
    {
	return proxyImpl_.lastRequestTime();
    }

    //
    // Set the time of the last request
    //
    public void stampRequest()
    {
	proxyImpl_.stampRequest();
    }

    //
    // Get the name of the proxy
    //
    public String getName()
    {
	return proxyImpl_.getName();
    }
}
