// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************
 
package com.ooc.OBTypedEventChannelFactory;

import com.ooc.CosEvent.*;
import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POAPackage.*;
import org.omg.PortableServer.POAManagerPackage.*;

//
// The typed event channel factory implementation.
//
final public class TypedEventChannelFactory_impl
    extends com.ooc.OBTypedEventChannelFactory.TypedEventChannelFactoryPOA
{
    private ORB orb_; // The ORB.
    private POA poa_; // The POA.
    private EventLogger logger_; // The message logger

    private Reaper reaper_; // The proxy reaper

    //
    // Mapping from String to
    // com.ooc.OBTypedEventChannelAdmin.TypedEventChannel_impl
    //
    private java.util.Hashtable channels_;

    // ------------------------------------------------------------------
    // TypedEventChannelFactory private member implementation
    // ------------------------------------------------------------------

    synchronized private void
    reap()
    {
	java.util.Enumeration keys = channels_.keys();
	while(keys.hasMoreElements())
	{
	    java.lang.Object key = keys.nextElement();
	    com.ooc.CosTypedEventChannelAdmin.TypedEventChannel_impl c = 
		(com.ooc.CosTypedEventChannelAdmin.TypedEventChannel_impl)
		channels_.get(key);
	    if(c != null)
	    {
		if(c.destroyed())
		    channels_.remove(key);
	    }
	}
    }

    // ------------------------------------------------------------------
    // TypedEventChannelFactory constructor.
    // ------------------------------------------------------------------
    
    public TypedEventChannelFactory_impl(ORB orb, POA poa, EventLogger logger,
    					 Reaper reaper)
    {
	orb_ = orb;
	poa_ = poa;
	logger_ = logger;
	reaper_ = reaper;

	channels_ = new java.util.Hashtable(63);

	if(logger_.traceLifecycle() >= 1)
	    logger_.trace("event", "creating TypedEventChannelFactory");
    }

    protected void
    finalize()
	throws Throwable
    {
	if(logger_.traceLifecycle() >= 1)
	    logger_.trace("event", "destroying TypedEventChannelFactory");
	com.ooc.OB.Assert._OB_assert(channels_ == null);
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping.
    // ------------------------------------------------------------------
    
    //
    // Create TypedEventChannel object.
    //
    public org.omg.CosTypedEventChannelAdmin.TypedEventChannel
    create_channel(String id)
	throws com.ooc.OBEventChannelFactory.ChannelAlreadyExists
    {
	reap();

	com.ooc.CosTypedEventChannelAdmin.TypedEventChannel_impl c =
	    new com.ooc.CosTypedEventChannelAdmin.
	    TypedEventChannel_impl(orb_, poa_, logger_, reaper_, id);

	byte[] oid = id.getBytes();

	try
	{
	    poa_.activate_object_with_id(oid, c);
	}
	catch(ObjectAlreadyActive ex)
	{
	    throw new com.ooc.OBEventChannelFactory.ChannelAlreadyExists();
	}
	catch(ServantAlreadyActive ex)
	{
	    throw new com.ooc.OBEventChannelFactory.ChannelAlreadyExists();
	}
	catch(WrongPolicy ex)
	{
	    throw new RuntimeException();
	}
    
	channels_.put(id, c);
	
	return c._this(orb_);
    }

    //
    // Get TypedEventChannel object by id.
    //
    synchronized public org.omg.CosTypedEventChannelAdmin.TypedEventChannel
    get_channel_by_id(String id)
	throws com.ooc.OBEventChannelFactory.ChannelNotAvailable
    {
	reap();

	com.ooc.CosTypedEventChannelAdmin.TypedEventChannel_impl c =
	    (com.ooc.CosTypedEventChannelAdmin.TypedEventChannel_impl)
	    channels_.get(id);

	if(c == null)
	    throw new com.ooc.OBEventChannelFactory.ChannelNotAvailable();

	try
	{
	    return c._this();
	}
	catch(OBJ_ADAPTER ex)
	{
	    throw new com.ooc.OBEventChannelFactory.ChannelNotAvailable();
	}
    }

    //
    // Get TypedEventChannel object ids.
    //
    public String[]
    get_channels()
    {
	reap();

	java.util.Vector v = new java.util.Vector();

  	java.util.Enumeration keys = channels_.keys();
  	while(keys.hasMoreElements())
	    v.addElement((String)keys.nextElement());

  	String[] channelIds = new String[v.size()];
	v.copyInto(channelIds);

	return channelIds;
    }

    //
    // Shutdown the event service.
    //
    synchronized public void
    shutdown()
    {
	if(orb_ == null)
	    return;

	//
	// Inform the POA Manager to hold requests.
	//
	POAManager manager = poa_.the_POAManager();
	try
	{
	    manager.discard_requests(false);
	}
	catch(AdapterInactive ex)
	{
	    throw new RuntimeException();
	}

	//
	// Stop event channels from accepting new requests. Destroy each
	// channel
	//
	java.util.Enumeration elements = channels_.elements();
	
	while(elements.hasMoreElements())
	{
	    com.ooc.CosTypedEventChannelAdmin.TypedEventChannel_impl c = 
		(com.ooc.CosTypedEventChannelAdmin.TypedEventChannel_impl)
		elements.nextElement();
	    if(c != null)
	    {
		c.discardRequests();
		if(!c.destroyed())
		    c.destroy();
	    }
	}

	channels_.clear();
	channels_ = null;

	//
	// Shutdown the ORB
	//
	orb_.shutdown(false);
	orb_ = null;
    }

    public org.omg.PortableServer.POA
    _default_POA()
    {
	return poa_;
    }
}
