// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POAPackage.*;
import org.omg.CosEventChannelAdmin.*;
import org.omg.CosEventComm.*;
import java.io.*;
import java.util.*;

class PullSupplier_impl extends PullSupplierPOA
{
    private POA poa_;    
    private ORB orb_;
    private Vector eventList_;
    private int current_;
    private static int tracelevel_ = 0;
    
    //
    // numEventPulls_ is used as a way of determining if there should
    // be a set number of pulls before disconnecting.  If it is -2, then
    // we are not concerned with the number of pulls.  Otherwise we use
    // this value to tell how many "auto" events are left to be pulled.
    //
    private int numEventPulls_;
    
    //
    // streaming_ is used as a flag indicating that a continuous
    // stream of "auto" events are to be generated and pulled.
    //
    private int streaming_;

    public
    PullSupplier_impl(Vector eventList, POA poa, ORB orb)
    {
        poa_ = poa;
        orb_ = orb;
	eventList_ = eventList;
	current_ = 0;
	numEventPulls_ = -2;

	if (((String)eventList.elementAt(0)).equals("streaming"))
	    streaming_ = 1;
	else
	{
	    try 
	    {
		if (Integer.parseInt(((String)eventList.elementAt(0))) > 0)
		    numEventPulls_ = 
			Integer.parseInt((String)eventList.elementAt(0));
	    }
	    catch(NumberFormatException ex)
	    {
		numEventPulls_ = -2;
	    }
	}
    }

    public void
    disconnect_pull_supplier()
	throws SystemException 
    {
	byte[] oid = null;
	try
	{
	    oid = ((com.ooc.OBPortableServer.POA_impl)poa_).
		servant_to_id((org.omg.PortableServer.Servant)this);
	}
        catch(ServantNotActive ex)
        {
            throw new RuntimeException();
        }
        catch(WrongPolicy ex)
        {
            throw new RuntimeException();
        }

	try
	{
	    poa_.deactivate_object(oid);
        }
        catch(ObjectNotActive ex)
        {
            throw new RuntimeException();
        }
        catch(WrongPolicy ex)
        {
            throw new RuntimeException();
        }
	orb_.shutdown(false);
    }

    public Any
    pull()
	throws Disconnected
    {
	Any any = orb_.create_any();

	if (numEventPulls_ == -1)
	{
	    throw new InternalError();
	}

	if (numEventPulls_ > -1)
	{
	    any.insert_string("AUTO");
	    TSU.PrintStatusMsg(tracelevel_, "PullEventServer", 3, 
			       "Pulling Any Auto Event");
	    numEventPulls_--;
	    return any;
	}
	    
	if (streaming_ == 1)
	{
	    any.insert_string((String)eventList_.elementAt(current_));
	    TSU.PrintStatusMsg(tracelevel_, "PullEventServer", 3, 
				            "Pulling Any Auto Event");
	    return any;
	}

	if(current_ >= eventList_.size())
	{
	    throw new Disconnected();
	}
	else 
	{
	    any.insert_string((String)eventList_.elementAt(current_));
	    current_++;
	}
	TSU.PrintStatusMsg(tracelevel_, "PullEventServer", 3, 
				        "Pulling Any Event");
	return any;
    }
    
    public Any
    try_pull(BooleanHolder hasEvent)
    {
	Any any = orb_.create_any();

	if (numEventPulls_ == -1)
	{
	    hasEvent.value = false;
	    throw new InternalError();
	}

	if (numEventPulls_ > -1)
	{
	    any.insert_string("AUTO");
	    hasEvent.value = true;
	    TSU.PrintStatusMsg(tracelevel_, "PullEventServer", 3, 
			       "Pulling Any Auto Event");
	    numEventPulls_--;
	    return any;
	}
	    
	if (streaming_ == 1)
	{
	    any.insert_string((String)eventList_.elementAt(current_));
	    hasEvent.value = true;
	    TSU.PrintStatusMsg(tracelevel_, "PullEventServer", 3, 
			       "Pulling Any Auto Event");
	    return any;
	}

	if(current_ >= eventList_.size())
	{
	    hasEvent.value = false;
	    throw new InternalError();
	}
	else 
	{
	    any.insert_string((String)eventList_.elementAt(current_));
	    current_++;
	}
	hasEvent.value = true;
	
	TSU.PrintStatusMsg(tracelevel_, "PullEventServer", 1, 
			   "Pulling Any Event");
	return any;
    }
}

public class PullEventServer
{
    private static int tracelevel_ = 0;

    private static void
    Usage() 
    {   
	System.out.println("pulleventserver <proxy-ior> [eventList]\n" +
			   "  eventList: 'streaming', a number, or a " +
			   "list of events\n");
    }
   

    static int
    run(ORB orb, String args[])
    {
 
	if(args.length < 1)
	{
	    Usage();
	    return 1;
	}

	TSU.PrintStatusMsg(tracelevel_, "PullEventServer", 1, "Starting");

	//
	// Resolve Root POA.
	//
	POA rootPOA = null;
	try
	{
	    rootPOA = POAHelper.narrow(
		orb.resolve_initial_references("RootPOA"));
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	    throw new RuntimeException();
	}

	//
	// Get a reference to the POA manager.
	//
	POAManager manager = rootPOA.the_POAManager();

	//
	// Get the event descriptions from argv[2]+
	//
	Vector eventList = new Vector();
	for(int i = 1 ; i < args.length ; i++)
	{
	    eventList.addElement(args[i]);
	}

	//
	// Create a persistent POA.
	//
	Policy[] pl = new Policy[2];
	pl[0] = rootPOA.create_lifespan_policy(
	    org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);
	pl[1] = rootPOA.create_id_assignment_policy(
	    org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);

	POA supplierPOA = null;
	try
	{
	    supplierPOA = rootPOA.create_POA("supplier", manager, pl);
	}
	catch(org.omg.PortableServer.POAPackage.AdapterAlreadyExists ex)
	{
	    throw new RuntimeException();
	}
	catch(org.omg.PortableServer.POAPackage.InvalidPolicy ex)
	{
	    throw new RuntimeException();
	}
      
	try 
	{ 
	    //
	    // Initialize the proxy.
	    //
	    org.omg.CORBA.Object obj = orb.string_to_object(args[0]);
	    ProxyPullConsumer proxyConsumer = 
		ProxyPullConsumerHelper.narrow(obj);

	    //
	    // Create the implementation object.
	    //
	    PullSupplier_impl ps = 
		new PullSupplier_impl(eventList, supplierPOA, orb);
	    
	    //
	    // Connect the supplier to the POA with the same name each
	    // time.
	    //
	    byte[] oid = "DefaultSupplier".getBytes();
	    try
	    {
		supplierPOA.activate_object_with_id(oid, ps);
	    }
	    catch(org.omg.PortableServer.POAPackage.ServantAlreadyActive ex)
	    {
		throw new RuntimeException();
	    }
	    catch(org.omg.PortableServer.POAPackage.ObjectAlreadyActive ex)
	    {
		throw new RuntimeException();
	    }
	    catch(org.omg.PortableServer.POAPackage.WrongPolicy ex)
	    {
		throw new RuntimeException();
	    }
	    org.omg.CosEventComm.PullSupplier supplier = ps._this(orb);

	    //
	    // Connect the proxy. If we're already connected we'll get an
	    // exception. This should only happen if we're reusing the
	    // same proxy-object -- this this case the connected
	    // PullSupplier must be the same servant.
	    //
	    try
	    {
		proxyConsumer.connect_pull_supplier(supplier);
		TSU.PrintStatusMsg(tracelevel_, "PullEventServer", 1, 
				   "Connected Any Pull Supplier");
	    }
	    catch(AlreadyConnected ex)
	    {
		ex.printStackTrace();
		return 1;
	    }
	    catch(TypeError ex)
	    {
		ex.printStackTrace();
		return 1;
	    }
	}
	catch(BAD_PARAM ex)
	{
	    System.err.println("Couldn't narrow Proxy Pull Consumer");
	    return 1;
	}

	//
	// Activate the POAManager.
	//
	try
	{
	    manager.activate();
	}
	catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
	{
	    throw new RuntimeException();
	}

	//
	// Allow the orb to process any events in the pull
	// supplier.
	//
	try
	{
	    orb.run();
	    orb.shutdown(true);
	}
	//
	// Expected if ORB/Client has already been shutdown.
	//
	catch(TRANSIENT ex)
	{
	}
	catch(BAD_INV_ORDER ex)
	{
	}
	catch(INITIALIZE ex)
	{
	}
	catch(NO_RESPONSE ex)
	{
	}

	return 0;
    }

    public static void
    main(String args[])
    {
        Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

    	int status = 0;
	ORB orb = null;	

        try
        {
            //
            // Create ORB.
            //
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);

	    java.util.Properties prop = 
		((com.ooc.OBCORBA.ORB_impl)orb).properties();
	    java.util.Enumeration keys = prop.keys();

	    //
	    // Get the trace level.
	    //
	    while(keys.hasMoreElements())
	    {
		String key = (String)keys.nextElement();
		String value = prop.getProperty(key);

		if(key.startsWith("ooc.testsuite.event.trace"))
		{
		    tracelevel_ = Integer.parseInt(prop.getProperty(key));
		    break;
		}
	    }

	    status = run(orb, argsH.value);
	}	
	catch(Exception ex)
	{
	    ex.printStackTrace();
	    status = 1;
	}
	
	if(orb != null)
	{
	    try
	    {
	        orb.destroy();
	    }
	    catch(Exception ex)
	    {
		ex.printStackTrace();
		status = 1;
	    }
	}

	System.exit(status);
    }
}
