// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

import org.omg.CORBA.*;
import org.omg.CosTypedEventChannelAdmin.*;
import org.omg.CosEventChannelAdmin.*;
import java.io.*;
import java.util.*;

public class PushEventServer 
{
    private static int tracelevel_ = 0;

    private static void 
    Usage()
    {
	System.out.println("pusheventserver <proxy-ior> <EventList>\n" +
			   "    EventList:  'auto', or a list of events " +
			   "separated by spaces\n");
    }
    
    static int
    run(ORB orb, String args[])
    {
        try
	{

	    if(args.length < 1)
	    {
		Usage();
		return 1;
	    }
	    
	    //
	    // Get the event descriptions from argv[2]+
	    //
	    Vector eventDescriptions = new Vector();
	    for(int i = 1 ; i < args.length ; i++)
	    {
		eventDescriptions.addElement(args[i]);
	    }

	    TSU.PrintStatusMsg(tracelevel_, "PushEventServer", 1, "Starting");

	    //
	    // Initialize the proxy.
	    //
	    org.omg.CORBA.Object obj = orb.string_to_object(args[0]);
	    ProxyPushConsumer proxyConsumer = 
		ProxyPushConsumerHelper.narrow(obj);

	    //
	    // Since callbacks are not needed a PushSupplier_impl is not
	    // created
	    // 
            try
            {
		proxyConsumer.connect_push_supplier(null);
            }
            catch(AlreadyConnected ex)
            {
                ex.printStackTrace();
                return 1;
            }
 
	    //
	    // Put streaming event generation and pushing here.
	    //
	    if (((String)eventDescriptions.elementAt(0)).
		    equalsIgnoreCase("auto"))
	    {
		while(true)
		{
		    Any e = orb.create_any();
		    e.insert_string("AUTO");

		    try
		    {
			proxyConsumer.push(e);
			TSU.PrintStatusMsg(tracelevel_, "PushEventServer", 1, 
				           "Pushed AUTO Event");
		    }
		    catch(org.omg.CosEventComm.Disconnected ex)
		    {
			break;
		    }
		}
	    }
	    
	    //
	    // Put 'manual' event generation and pushing here.
	    //
	    for(int j = 0 ; j < eventDescriptions.size() ; j++)
	    {
		Any e = orb.create_any();
		e.insert_string((String)eventDescriptions.elementAt(j));
		
		try
		{
		    proxyConsumer.push(e);
		    TSU.PrintStatusMsg(tracelevel_, "PushEventServer", 1, 
				       "Pushed Event");
		}
		catch(org.omg.CosEventComm.Disconnected ex)
		{
		    break;
		}
	    }
	   
            try
            {
		proxyConsumer.disconnect_push_consumer();
		TSU.PrintStatusMsg(tracelevel_, "PushEventServer", 1, 
				   "Disconnected Any Push Consumer");
            }
            //
            // Expected if event service has done away
            //
            catch(org.omg.CORBA.TRANSIENT ex)
            {
            }
            catch(org.omg.CORBA.COMM_FAILURE ex)
            {
            }
	}
	catch(BAD_PARAM ex)
	{
	    System.err.println("Failure on narrow call");
	    return 1;
	}
	catch(SystemException ex)
	{
	    TSU.PrintStatusMsg(tracelevel_, "PushEventServer", 3, 
	    		       "Caught System Exception");
	    ex.printStackTrace();
            return 1;
	}

	return 0;
    }

    public static void
    main(String args[])
    {
        Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

    	int status = 0;
	ORB orb = null;	

        try
        {
            //
            // Create ORB.
            //
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);

	    java.util.Properties prop = 
		((com.ooc.OBCORBA.ORB_impl)orb).properties();
	    java.util.Enumeration keys = prop.keys();

	    //
	    // Get the trace level.
	    //
	    while(keys.hasMoreElements())
	    {
		String key = (String)keys.nextElement();
		String value = prop.getProperty(key);

		if(key.startsWith("ooc.testsuite.event.trace"))
		{
		    tracelevel_ = Integer.parseInt(prop.getProperty(key));
		    break;
		}
	    }

	    status = run(orb, argsH.value);
	}	
	catch(Exception ex)
	{
	    ex.printStackTrace();
	    status = 1;
	}
	
	if(orb != null)
	{
	    try
	    {
	        orb.destroy();
	    }
	    catch(Exception ex)
	    {
		ex.printStackTrace();
		status = 1;
	    }
	}

	System.exit(status);
    }
}
