// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

import org.omg.CORBA.*;
import org.omg.CosTypedEventChannelAdmin.*;
import org.omg.CosEventChannelAdmin.*;
import org.omg.PortableServer.*;
import org.omg.CosEventComm.*;
import java.io.*;
import java.util.*;

public class TypedPushEventClient
{
    private static int tracelevel_ = 0;

    private static void 
    Usage()
    {
	System.out.println("typedpusheventclient <proxy-ior> [--disconnect]\n" +
			   "   --disconnect:  Call connect on the proxy and " +
			   "disconnect\n");
    }
    
    static int
    run(ORB orb, String args[])
    {
        StringBuffer outputBuffer = new StringBuffer();
 
        try
	{
	    //
	    // Resolve Root POA.
	    //
	    POA rootPOA = null;
	    try
	    {
		    rootPOA = POAHelper.narrow(
			orb.resolve_initial_references("RootPOA"));
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		    throw new RuntimeException();
	    }

	    //
	    // Get a reference to the POA manager.
	    //
	    POAManager manager = rootPOA.the_POAManager();

	    if(args.length > 3)
	    {
		Usage();
		return 1;
	    }

	    //
	    // Just connect and exit.
	    //
	    boolean disconnect = false;

	    //
	    // Find whether to append timestamps or not and make sure that no 
	    // "new" arguments are given to ClientCommon.
	    //
	    for(int i = 0 ; i < args.length ; i++)
	    {
		if(args[i].equalsIgnoreCase("--disconnect"))
		{
		    disconnect=true;
		}
	    }
	    TSU.PrintStatusMsg(tracelevel_, "TypedPushEventClient", 2, 
			       "Starting");
	   
	    //
	    // Create a persistent POA.
	    //
	    Policy[] pl = new Policy[2];
	    pl[0] = rootPOA.create_lifespan_policy(
		org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);
	    pl[1] = rootPOA.create_id_assignment_policy(
		org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);

	    POA consumerPOA = null;
	    try
	    {
		consumerPOA = rootPOA.create_POA("consumer", manager, pl);
	    }
	    catch(org.omg.PortableServer.POAPackage.AdapterAlreadyExists ex)
	    {
		throw new RuntimeException();
	    }
	    catch(org.omg.PortableServer.POAPackage.InvalidPolicy ex)
	    {
		throw new RuntimeException();
	    }
    
	    //
	    // Create the servant.
	    //
	    Echo_impl pc = new Echo_impl(orb, consumerPOA, outputBuffer);

	    //
	    // Connect the consumer to the POA with the same name each
	    // time.
	    //
	    byte[] oid = ("DefaultConsumer").getBytes();
	    try
	    {
		consumerPOA.activate_object_with_id(oid, pc);
	    }
	    catch(org.omg.PortableServer.POAPackage.ServantAlreadyActive ex)
	    {
		throw new RuntimeException();
	    }
	    catch(org.omg.PortableServer.POAPackage.ObjectAlreadyActive ex)
	    {
		throw new RuntimeException();
	    }
	    catch(org.omg.PortableServer.POAPackage.WrongPolicy ex)
	    {
		throw new RuntimeException();
	    }
	    org.omg.CosEventComm.PushConsumer consumer = pc._this(orb);
	    
	    TSU.PrintStatusMsg(tracelevel_, "TypedPushEventClient", 2, 
			       "Created and connected to Echo");
	    
	    //
	    // Get the proxy object.
	    //
	    org.omg.CORBA.Object obj = orb.string_to_object(args[0]);
	    ProxyPushSupplier proxy = 
		ProxyPushSupplierHelper.narrow(obj);

	    //
	    // Activate the POAManager.
	    //
	    try
	    {
		manager.activate();
	    }
	    catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
	    {
		throw new RuntimeException();
	    }

	    //
	    // Connect the proxy. If we're already connected we'll get an
	    // exception. This should only happen if we're reusing the
	    // same proxy-object -- this this case the connected
	    // SequencePushConsumer must be the same servant.
	    //
	    try
	    {
		proxy.connect_push_consumer(consumer);
	    }
	    catch(TypeError ex)
	    {
		ex.printStackTrace();
		return 1;
	    }
	    catch(AlreadyConnected ex)
	    {
	    }

	    if(disconnect)
	    {
		TSU.PrintStatusMsg(tracelevel_, "TypedPushEventClient", 2, 
			           "Connected and exiting.");
		return 0;
	    }

	    orb.run();
	}
	catch(BAD_PARAM ex)
	{
	    System.err.println("Call to narrow failed!");
	    return 1;
	}

        //
        // Expected if ORB/Client has already been shutdown
        //
        catch(TRANSIENT ex)
        {
        }
        catch(BAD_INV_ORDER ex)
        {
        }
        catch(INITIALIZE ex)
        {
        }
        catch(NO_RESPONSE ex)
        {
        }
	catch(SystemException ex)
	{	
	    TSU.PrintStatusMsg(tracelevel_, "TypedPushEventClient", 2, 
			       "Caught Unexpected System Exception");
	    ex.printStackTrace();
	    return 1;
	}
        System.out.println(outputBuffer);
	return 0;
    }
    
    public static void
    main(String args[])
    {
        Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

    	int status = 0;
	ORB orb = null;	

        try
        {
            //
            // Create ORB.
            //
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);

	    java.util.Properties prop = 
		((com.ooc.OBCORBA.ORB_impl)orb).properties();
	    java.util.Enumeration keys = prop.propertyNames();

	    //
	    // Get the trace level.
	    //
	    while(keys.hasMoreElements())
	    {
		String key = (String)keys.nextElement();
		String value = prop.getProperty(key);

		if(key.startsWith("ooc.testsuite.event.trace"))
		{
		    tracelevel_ = Integer.parseInt(prop.getProperty(key));
		    break;
		}
	    }

	    status = run(orb, argsH.value);
	}	
	catch(Exception ex)
	{
	    ex.printStackTrace();
	    status = 1;
	}
	
	if(orb != null)
	{
	    try
	    {
	        orb.destroy();
	    }
	    catch(Exception ex)
	    {
		ex.printStackTrace();
		status = 1;
	    }
	}

	System.exit(status);
    }
}
