// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

import org.omg.CORBA.*;
import org.omg.CosTypedEventChannelAdmin.*;
import org.omg.CosEventChannelAdmin.*;
import java.io.*;
import java.util.*;

public class TypedPushEventServer
{
    private static int tracelevel_ = 0;

    private static void 
    Usage()
    {
	System.out.println("pusheventserver <proxy-ior) <EventList>\n" +
		           "  EventList:  'auto', or a list of events " +
			   "seperated by spaces\n");
    }


    static int
    run(ORB orb, String args[])
    {
        try
	{
	    if(args.length < 3)
	    {
		Usage();
		return 1;
	    }
	    
	    //
	    // Get the event descriptions from argv[4]+
	    //
	    Vector eventDescriptions = new Vector();
	    for(int i = 1 ; i < args.length ; i++)
	    {
		eventDescriptions.addElement(args[i]);
	    }

	    TSU.PrintStatusMsg(tracelevel_, "TypedPushEventServer", 1, 
		   	       "Starting");

	    //
	    // Initialize the server and get the proxy object.
	    //
	    org.omg.CORBA.Object obj = orb.string_to_object(args[0]);
	    TypedProxyPushConsumer proxyConsumer = 
		TypedProxyPushConsumerHelper.narrow(obj);

	    try 
	    {
		proxyConsumer.connect_push_supplier(null);
	    }
	    catch(AlreadyConnected ex)
	    {
	    }

	    org.omg.CORBA.Object typedConsumer = 
		proxyConsumer.get_typed_consumer();

	    Echo echo = EchoHelper.narrow(typedConsumer);
	    if(echo == null)
	    {
		System.out.println("narrow failed!");
		return 1;
	    }

	    //
	    // Put streaming event generation and pushing here.
	    //
	    if (((String)eventDescriptions.elementAt(0)).equals("auto"))
	    {
		TSU.PrintStatusMsg(tracelevel_, "TypedPushEventServer", 1, 
		                   "Pushing Typed AUTO Events");
		while(true)
		{
		    echo.echo_string("AUTO");
		}
	    }
	    
	    //
	    // Put 'manual' event generation and pushing here.
	    //
	    for(int j = 0 ; j < eventDescriptions.size() ; j++)
	    {
		echo.echo_string((String)eventDescriptions.elementAt(j));
		TSU.PrintStatusMsg(tracelevel_, "TypedPushEventServer", 1, 
				   "Pushed Any Event");
	    }
	    
	    try
	    {
		proxyConsumer.disconnect_push_consumer();
		TSU.PrintStatusMsg(tracelevel_, "TypedPushEventServer", 1, 
				   "Disconnected Any Push Consumer");
	    }
	    catch(org.omg.CORBA.TRANSIENT ex)
	    {
	    }
	    catch(org.omg.CORBA.COMM_FAILURE ex)
	    {
	    }
	}
	catch(BAD_PARAM ex)
	{
	    System.err.println("Couldn't narrow Proxy Push Consumer");
	    return 1;
	}
	catch(SystemException ex)
	{
	    TSU.PrintStatusMsg(tracelevel_, "TypedPushEventServer", 3, 
			       "Caught System Exception");
	    ex.printStackTrace();
            return 1;
	}
        return 0;
    }


    public static void
    main(String args[])
    {
        Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

    	int status = 0;
	ORB orb = null;	

        try
        {
            //
            // Create ORB.
            //
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);

	    java.util.Properties prop = 
		((com.ooc.OBCORBA.ORB_impl)orb).properties();
	    java.util.Enumeration keys = prop.propertyNames();

	    //
	    // Get the trace level.
	    //
	    while(keys.hasMoreElements())
	    {
		String key = (String)keys.nextElement();
		String value = prop.getProperty(key);

		if(key.startsWith("ooc.testsuite.event.trace"))
		{
		    tracelevel_ = Integer.parseInt(prop.getProperty(key));
		    break;
		}
	    }

	    status = run(orb, argsH.value);
	}	
	catch(Exception ex)
	{
	    ex.printStackTrace();
	    status = 1;
	}
	
	if(orb != null)
	{
	    try
	    {
	        orb.destroy();
	    }
	    catch(Exception ex)
	    {
		ex.printStackTrace();
		status = 1;
	    }
	}

	System.exit(status);
    }
}
