// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA  
//                     
// All Rights Reserved   
//   
// **********************************************************************

package com.ooc.IMRConsole;

import com.ooc.Util.*;
import com.ooc.Util.GUI.*;
import java.net.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import javax.swing.event.*;
import org.omg.CORBA.*;
import com.ooc.IMR.*;

final class Console extends JFrame 
    implements TreeSelectionListener, ClipboardListener
{
    private JFrame frame_;
    private JPopupMenu popup_;

    private static ObjectTree objectTree_;
    private MainViewPanel mainViewPnl_;
    private StatusBar status_;

    private ServiceManager serviceMgr_;
    private ServiceObject daObj_;
    
    private JPanel dynamicPnl_;

    private CardLayout cards_;

    private AbstractAction exitAction_;
    private AbstractAction createAction_;
    private AbstractAction deleteAction_;
    private AbstractAction cutAction_;
    private AbstractAction pasteAction_;
    private AbstractAction modifyAction_;
    private AbstractAction startAction_;
    private AbstractAction stopAction_;
    private AbstractAction resetAction_;
    private AbstractAction refreshAction_;
    private AbstractAction aboutAction_;

    private ImageIcon createIcon_;
    private ImageIcon modifyIcon_;
    private ImageIcon deleteIcon_;
    private ImageIcon cutIcon_;
    private ImageIcon pasteIcon_;
    private ImageIcon startIcon_;
    private ImageIcon stopIcon_;
    private ImageIcon resetIcon_;
    private ImageIcon refreshIcon_;
    private ImageIcon aboutIcon_;

    private boolean wasCut_;

    public void
    valueChanged(TreeSelectionEvent e)
    {
	try
	{
	    ServiceObject selectedObj = objectTree_.getSelectedObject();

	    clearMenuItems();
	    status_.clear();

	    if(selectedObj == null)
		return;
	
	    if(selectedObj.getType() == ServiceObject.DOMAIN)
	    {
		cards_.show(dynamicPnl_, "blank");

		ServerDomain da = (ServerDomain)selectedObj.toObject();
		da._non_existent();
	    }
	    else if(selectedObj.getType() == ServiceObject.OAD &&
		    selectedObj.getName().equals(
			ServiceObject.ORPHANED_SERVER_OAD_NAME))
	    {
		cards_.show(dynamicPnl_, "blank");
	    }
	    else
	    {
		mainViewPnl_.refresh(selectedObj);
		cards_.show(dynamicPnl_, "main_view");
	    }
		
	    refreshMenuItems();
	    status_.setText("IMR Console Ready."); 
	}
	catch(UnfoundObjectReference ex)
	{
	    ErrorDialog.showUnfoundObjectReference(ex);
	}
	catch(org.omg.CORBA.SystemException ex)
	{
	    ErrorDialog.showSystemException(ex); 
	}
    }

    public void
    clipboardChanged(ClipboardData data)
    {
	refreshMenuItems();
    }

    Console(ORB orb, ServerDomain da) 
    {
        super("IMR Console");
        frame_ = this;

        initializeActions();
        initializeIcons();
	setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

	addWindowListener(
	    new WindowAdapter()
	    {
		public void
	        windowClosing(WindowEvent e)
		{
		    if(ErrorDialog.showConfirm("Confirm Exit",
					       "Do you really want to quit?"))
			System.exit(0);
		}
	    });
	
	status_ = new StatusBar();
	Clipboard.instance().addClipboardListener(this);
        serviceMgr_ = new ServiceManager(this, orb, da);

        daObj_ = serviceMgr_.createDomainObject();

	//
	// initialize and layout the tree
	//
        objectTree_ = new ObjectTree(this, daObj_, serviceMgr_);
	objectTree_.addTreeSelectionListener(this);
	objectTree_.addMouseListener(new PopupListener());

	JScrollPane objectTreeSP = new JScrollPane(objectTree_);

	JLabel objectTreeLbl = new JLabel("IMR Object Structure");

	JPanel objectTreePnl = new JPanel(new BorderLayout());
	objectTreePnl.setPreferredSize(new Dimension(250, 440));
	objectTreePnl.setBorder(BorderFactory.createEtchedBorder());
	objectTreePnl.add(objectTreeLbl, BorderLayout.NORTH);
	objectTreePnl.add(objectTreeSP, BorderLayout.CENTER);

        mainViewPnl_ = new MainViewPanel(this);
        mainViewPnl_.setPreferredSize(new Dimension(450, 220));

	ErrorDialog.initialize(this);

        JMenuBar menuBar = new JMenuBar();
        setupMenuBar(menuBar);
        setJMenuBar(menuBar);

        JToolBar toolBar = new JToolBar();
        setupToolBar(toolBar);

        setupPopupMenu();

	JPanel blankWhitePnl = new JPanel();
	blankWhitePnl.setBorder(BorderFactory.createEtchedBorder());
	blankWhitePnl.setBackground(Color.white);

	JPanel blankPnl = new JPanel(new BorderLayout());
	blankPnl.setBorder(BorderFactory.createEtchedBorder());
	blankPnl.add(new JLabel(" "), BorderLayout.NORTH);
	blankPnl.add(blankWhitePnl, BorderLayout.CENTER);
        
	cards_ = new CardLayout();
	dynamicPnl_ = new JPanel(cards_);
	dynamicPnl_.add(mainViewPnl_, "main_view");
	dynamicPnl_.add(blankPnl, "blank");

        JSplitPane sp2 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
	sp2.setBorder(null);
        sp2.setLeftComponent(objectTreePnl);
        sp2.setRightComponent(dynamicPnl_);

        java.awt.Container contentPane = getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.add(toolBar, BorderLayout.NORTH);
        contentPane.add(sp2, BorderLayout.CENTER);
        contentPane.add(status_, BorderLayout.SOUTH);

        pack();

	clearMenuItems();
	cards_.show(dynamicPnl_, "blank");
	status_.setText("IMR Console Ready."); 

        setVisible(true);
    }

    private void
    initializeActions()
    {
        exitAction_ = new ExitAction();
        createAction_ = new CreateAction();
        deleteAction_ = new DeleteAction();
	cutAction_ = new CutAction();
	pasteAction_ = new PasteAction();
        modifyAction_ = new ModifyAction();
        startAction_ = new StartAction();
        stopAction_ = new StopAction();
        resetAction_ = new ResetAction();
        refreshAction_ = new RefreshAction();
        aboutAction_ = new AboutAction();
    }

    private void
    initializeIcons()
    {
        createIcon_ = Util.getImageResource("create.gif", this);
        modifyIcon_ = Util.getImageResource("edit.gif", this);
        deleteIcon_ = Util.getImageResource("destroy.gif", this);
	cutIcon_ = Util.getImageResource("cut.gif", this);
	pasteIcon_ = Util.getImageResource("paste.gif", this);
        startIcon_ = Util.getImageResource("start.gif", this);
        stopIcon_ = Util.getImageResource("stop.gif", this);
        resetIcon_ = Util.getImageResource("reset.gif", this);
        refreshIcon_ = Util.getImageResource("refresh.gif", this);
        aboutIcon_ = Util.getImageResource("obimr.gif", this);
    }

    private void
    setupMenuBar(JMenuBar menuBar)
    {
	JMenu menu = new JMenu(AppStandards.getString("FileMenuKey"), true);
	menu.setMnemonic(AppStandards.getChar("FileMenuMnemKey"));
        menuBar.add(menu);

	createMenuItem(menu, "Exit", exitAction_);

        menu = new JMenu("Edit");
	menu.setMnemonic(AppStandards.getChar("EditMenuMnemKey"));
        menuBar.add(menu);

	createMenuItem(menu, "Create", createAction_);
	createMenuItem(menu, "Modify", modifyAction_);
	createMenuItem(menu, "Delete", deleteAction_);

        menu.addSeparator();
	createMenuItem(menu, "Cut", cutAction_);
	createMenuItem(menu, "Paste", pasteAction_);

	menu.addSeparator();
	createMenuItem(menu, "Start", startAction_);
	createMenuItem(menu, "Stop", stopAction_);
	createMenuItem(menu, "Reset", resetAction_);

        menu = new JMenu("View");
	menu.setMnemonic(AppStandards.getChar("ViewMenuMnemKey"));
        menuBar.add(menu);

	createMenuItem(menu, "Refresh", refreshAction_);

        menu = new JMenu("Help");
	menu.setMnemonic(AppStandards.getChar("HelpMenuMnemKey"));
        menuBar.add(menu);

	createMenuItem(menu, "About", aboutAction_);
    }

    //
    // Create menu item with accelerator and action
    //
    private JMenuItem
    createMenuItem(JMenu menu, String key, AbstractAction action)
    {
	JMenuItem item = menu.add(action);

	item.setText(AppStandards.getString(key + "Key"));
	item.setMnemonic(AppStandards.getChar(key + "MnemKey"));

	char accel = AppStandards.getChar(key + "AccelKey");
	if(accel != 0)
	{
	    //
	    // Simple hack for keys without Ctrl
	    //
	    int keyMask = 0;
	    if(!key.equals("Delete") &&
	       !key.equals("About") &&
	       !key.equals("Refresh"))
		keyMask = getToolkit().getMenuShortcutKeyMask();
	    item.setAccelerator(KeyStroke.getKeyStroke(accel, keyMask));
	}

	return item;
    }

    //
    // Create popup menu item with and action
    //
    private JMenuItem
    createMenuItem(JPopupMenu menu, String key, AbstractAction action)
    {
	JMenuItem item = menu.add(action);

	item.setText(AppStandards.getString(key + "Key"));
	item.setMnemonic(AppStandards.getChar(key + "MnemKey"));

	return item;
    }

    private void
    setupToolBar(JToolBar toolBar)
    {
        toolBar.setFloatable(false);

	JButton b = toolBar.add(createAction_);
        b.setIcon(createIcon_);
        b.setToolTipText("Create");
        b = toolBar.add(modifyAction_);
        b.setIcon(modifyIcon_);
        b.setToolTipText("Modify");
        b = toolBar.add(deleteAction_);
        b.setIcon(deleteIcon_);
        b.setToolTipText("Delete");

        toolBar.addSeparator();
	b = toolBar.add(cutAction_);
	b.setIcon(cutIcon_);
	b.setToolTipText("Cut");
	b = toolBar.add(pasteAction_);
	b.setIcon(pasteIcon_);
	b.setToolTipText("Paste");

        toolBar.addSeparator();
        b = toolBar.add(startAction_);
        b.setIcon(startIcon_);
        b.setToolTipText("Start");
        b = toolBar.add(stopAction_);
        b.setIcon(stopIcon_);
        b.setToolTipText("Stop");
        b = toolBar.add(resetAction_);
        b.setIcon(resetIcon_);
        b.setToolTipText("Reset");

        toolBar.addSeparator();
        b = toolBar.add(refreshAction_);
        b.setIcon(refreshIcon_);
        b.setToolTipText("Refresh");
    }

    private void
    setupPopupMenu()
    {
        popup_ = new JPopupMenu();

    	createMenuItem(popup_, "Create", createAction_);
    	createMenuItem(popup_, "Modify", modifyAction_);
    	createMenuItem(popup_, "Delete", deleteAction_);

	popup_.addSeparator();
	createMenuItem(popup_, "Cut", cutAction_);
	createMenuItem(popup_, "Paste", pasteAction_);

        popup_.addSeparator();
	createMenuItem(popup_, "Start", startAction_);
	createMenuItem(popup_, "Stop", stopAction_);
	createMenuItem(popup_, "Reset", resetAction_);

        popup_.addSeparator();
	createMenuItem(popup_, "Refresh", refreshAction_);
    }

    private void
    createOAD()
    {
	OADInfo oadInfo = OADDialog.show(frame_, "Create OAD", null);

	if(oadInfo != null && serviceMgr_.createOAD(oadInfo.host))
	{
	    objectTree_.add(oadInfo.host, ServiceObject.OAD, daObj_);
	    status_.setText("Success: Created `OAD " + oadInfo.host + "'"); 
	}
    }

    private void
    createServer(ServiceObject oadObj)
    {
	ServerProps props =
	    ServerDialog.show(frame_, "Create Server", null);

	if(props != null)
	    if(serviceMgr_.createServer(props.getName(),
					props.getExec(),
					props.getArgs(),
					props.getDirectory(),
					props.getMode(),
					props.getActivatePOAs(),
					props.getUpdateTimeout(),
					props.getFailureTimeout(),
					props.getMaxForks(),
					oadObj))
	    {
		objectTree_.add(
		    props.getName(), ServiceObject.SERVER, oadObj);
		status_.setText(
		    "Success: Created `Server " + props.getName() + "'"); 
	    }
    }
	
    private void
    createPOA(ServiceObject serverObj)
    {
	POAInfo poaInfo = POADialog.show(frame_,
					 "Create POA",
					 null);

	if(poaInfo != null)
	    if(serviceMgr_.createPOA(poaInfo.poa, serverObj))
	    {
		String sPOAName = "";
		for(int i = 0; i < poaInfo.poa.length; i++)
		{
		    if(i != 0)
			sPOAName += "/";
		    sPOAName += poaInfo.poa[i];
		}   

		objectTree_.add(sPOAName, ServiceObject.POA, serverObj);
		status_.setText("Success: Created `POA " + sPOAName + "'"); 
	    }
    }

    private void
    deleteOAD(ServiceObject oadObj)
    {
	if(ErrorDialog.showConfirm("Confirm Delete?",
		                   "Delete `" + oadObj.toString() + "'?"))
	{
	    if(serviceMgr_.destroyOAD(oadObj))
	    {
		status_.setText("Success: Deleted `" + 
				oadObj.toString() + "'");

		objectTree_.remove(oadObj);

		objectTree_.refresh();
		cards_.show(dynamicPnl_, "blank");
	    }
	}
    }

    private void
    deleteServer(ServiceObject serverObj)
    {
	if(ErrorDialog.showConfirm("Confirm Delete?", 
		                   "Delete `" + 
				   serverObj.toString() + "'?"))
	{
	    if(serviceMgr_.destroyServer(serverObj))
	    {
		status_.setText("Success: Deleted `" + 
				serverObj.toString() + "'");
		objectTree_.remove(serverObj);
		cards_.show(dynamicPnl_, "blank");
	    }
	}
    }

    private void
    deletePOA(ServiceObject poaObj)
    {
	if(ErrorDialog.showConfirm("Confirm Delete?", 
		                   "Delete `" + 
				   poaObj.toString() + "'?"))
	{
	    if(serviceMgr_.destroyPOA(poaObj))
	    {
		status_.setText("Success: Deleted `" + 
				poaObj.toString() + "'");
		objectTree_.remove(poaObj);
		cards_.show(dynamicPnl_, "blank");
	    }
	}
    }

    private void
    startServer(ServiceObject serverObj)
    {
	if(ErrorDialog.showConfirm("Confirm Start?", 
		                   "Start `" + 
				   serverObj.toString() + "'?"))
	{
	    if(serviceMgr_.startServer(serverObj))
		status_.setText("Success: Started `" + 
				serverObj.toString() + "'");
	    else
		status_.setText("Failure: `" + 
				serverObj.toString() + "' was not started");
	}
    }

    private void
    stopServer(ServiceObject serverObj)
    {
	if(ErrorDialog.showConfirm("Confirm Stop?", 
		                   "Stop `" + 
				   serverObj.toString() + "'?"))
	{
	    if(serviceMgr_.stopServer(serverObj))
		status_.setText("Success: Stopped `" + 
				serverObj.toString() + "'");
	    else
		status_.setText("Failure: `" + 
				serverObj.toString() + "' was not stopped");
	}
    }

    private void
    resetServer(ServiceObject serverObj)
    {
	if(ErrorDialog.showConfirm("Confirm Reset?", 
		                   "Reset `" + 
				   serverObj.toString() + "'?"))
	{
	    if(serviceMgr_.resetServer(serverObj))
		status_.setText("Success: Reset `" + 
				serverObj.toString() + "'");
	    else
		status_.setText("Failure: `" + 
				serverObj.toString() + "' was not reset");
	}
    }

    private void
    clearMenuItems()
    {
	createAction_.setEnabled(false);
	modifyAction_.setEnabled(false);
	deleteAction_.setEnabled(false);

	cutAction_.setEnabled(false);
	pasteAction_.setEnabled(false);

	startAction_.setEnabled(false);
	stopAction_.setEnabled(false);
	resetAction_.setEnabled(false);
	refreshAction_.setEnabled(true);
    }
    
    private void
    refreshMenuItems()
    {
	ServiceObject selectedObj = objectTree_.getSelectedObject();
	int type = ServiceObject.DOMAIN;
	if(selectedObj != null)
	    type = selectedObj.getType();

	boolean isOrphanedServerOAD =
	    type == ServiceObject.OAD && selectedObj.isOrphanedServerOAD();
	boolean isOrphanedServer =
	    type == ServiceObject.SERVER &&
	    ((ServiceObject)selectedObj.getParent()).isOrphanedServerOAD();

	createAction_.setEnabled(type != ServiceObject.POA &&
				 !isOrphanedServerOAD);
	modifyAction_.setEnabled(type != ServiceObject.DOMAIN &&
				 !isOrphanedServerOAD);
	deleteAction_.setEnabled(type != ServiceObject.DOMAIN &&
				 !isOrphanedServerOAD);

	cutAction_.setEnabled(type == ServiceObject.SERVER);
	pasteAction_.setEnabled(Clipboard.instance().getData() != null &&
				type == ServiceObject.OAD);

	boolean serverStopped = true;
	if(type == ServiceObject.SERVER)
	{
	    try
	    {
		Server s = (Server)selectedObj.toObject();
		serverStopped = s.status() == ServerStatus.STOPPED;
	    }
	    catch(UnfoundObjectReference ex)
	    {
		ErrorDialog.showUnfoundObjectReference(ex);
	    }
	}
	
	startAction_.setEnabled(type == ServiceObject.SERVER &&
				!isOrphanedServer &&
				serverStopped);
	stopAction_.setEnabled(type == ServiceObject.SERVER &&
			       !isOrphanedServer &&
			       !serverStopped);
	resetAction_.setEnabled(type == ServiceObject.SERVER &&
				!isOrphanedServer);

	refreshAction_.setEnabled(true);
    }

    private class ExitAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
	    if(ErrorDialog.showConfirm("Confirm Exit",
				       "Do you really want to quit?"))
                System.exit(1);

	    status_.setText("Exit cancelled.");
        }
    }

    private class CreateAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
	    ServiceObject selectedObj = objectTree_.getSelectedObject();

	    try
	    {
		if(selectedObj != null)
		{
		    switch(selectedObj.getType())
		    {
		    case ServiceObject.DOMAIN:
			createOAD();
			break;
		    case ServiceObject.OAD:
			createServer(selectedObj);
			break;
		    case ServiceObject.SERVER:
			createPOA(selectedObj);
			break;
		    }
		}
	    }
            catch(org.omg.CORBA.NO_PERMISSION ex)
            {
	        ErrorDialog.showNoPermissionException(); 
            }
            catch(org.omg.CORBA.SystemException ex)
            {
	        ErrorDialog.showSystemException(ex); 
            }
        }
    }

    private class ModifyAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
	    ServiceObject selectedObj = objectTree_.getSelectedObject();

	    if(selectedObj == null)
		return;

	    try
	    {
		int type = selectedObj.getType();

		if(type == ServiceObject.OAD)
		{
		    OADInfo oadInfo = (OADInfo)selectedObj.toObject();
		    OADInfo newOADInfo = OADDialog.show(frame_,
							"OAD Setup",
							oadInfo);
		}
		else if(type == ServiceObject.SERVER)
		{
		    Server sa = (Server)selectedObj.toObject();

		    ServerProps props =
			new ServerProps(sa.name(),
					sa.status(),
					sa.manual(),
					sa.updateTime(),
					sa.timesForked());
		    props.setHost(sa.host());
		    props.setExec(sa.exec());
		    props.setArgs(sa.args());
		    props.setDirectory(sa.directory());
		    props.setMode(sa.mode());
		    props.setActivatePOAs(sa.activatePOAs());
		    props.setUpdateTimeout(sa.updateTimeout());
		    props.setFailureTimeout(sa.failureTimeout());
		    props.setMaxForks(sa.maxForks());
		    
		    ServerProps newProps =
			ServerDialog.show(frame_, "Server Setup", props);

		    if(newProps != null)
		    {
			sa.exec(newProps.getExec());
			sa.args(newProps.getArgs());
			sa.directory(newProps.getDirectory());
			sa.mode(newProps.getMode());
			sa.activatePOAs(newProps.getActivatePOAs());
			sa.updateTimeout(newProps.getUpdateTimeout());
			sa.failureTimeout(newProps.getFailureTimeout());
			sa.maxForks(newProps.getMaxForks());

			try
			{
			    Thread.currentThread().sleep(500);
			}
			catch(InterruptedException ex)
			{
			}

			try
			{
			    objectTree_.refreshServer(selectedObj);
			}
			catch(UnfoundObjectReference ex)
			{
			    ErrorDialog.showUnfoundObjectReference(ex);
			}

			mainViewPnl_.refresh(selectedObj);
			refreshMenuItems();
		    }
		}
		else if(type == ServiceObject.POA)
		{
		    POAInfo poaInfo = (POAInfo)selectedObj.toObject();
		    POAInfo newPOAInfo = POADialog.show(frame_,
							"POA Setup",
							poaInfo);
		}
	    }
	    catch(UnfoundObjectReference ex)
	    {
		ErrorDialog.showUnfoundObjectReference(ex);
	    }
            catch(org.omg.CORBA.NO_PERMISSION ex)
            {
	        ErrorDialog.showNoPermissionException(); 
            }
	    catch(org.omg.CORBA.SystemException ex)
	    {
		ErrorDialog.showSystemException(ex); 
	    }
	}
    }

    private class DeleteAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
	    ServiceObject selectedObj = objectTree_.getSelectedObject();

	    try
	    {
                if(selectedObj != null)
	        {
		    switch(selectedObj.getType())
		    {
		    case ServiceObject.OAD:
			deleteOAD(selectedObj);
			break;
		    case ServiceObject.SERVER:
			deleteServer(selectedObj);
			break;
		    case ServiceObject.POA:
			deletePOA(selectedObj);
			break;
		    }
		}
	    }
            catch(org.omg.CORBA.NO_PERMISSION ex)
            {
	        ErrorDialog.showNoPermissionException(); 
            }
            catch(org.omg.CORBA.SystemException ex)
            {
  	        ErrorDialog.showSystemException(ex); 
            }
        }
    }

    private class StartAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
	    ServiceObject selectedObj = objectTree_.getSelectedObject();

	    try
	    {
                if(selectedObj != null &&
		   selectedObj.getType() == ServiceObject.SERVER)
		    startServer(selectedObj);
	    }
            catch(org.omg.CORBA.SystemException ex)
            {
	        ErrorDialog.showSystemException(ex); 
            }

	    try
	    {
		Thread.currentThread().sleep(500);
	    }
	    catch(InterruptedException ex)
	    {
	    }

	    try
	    {
		objectTree_.refreshServer(selectedObj);
	    }
	    catch(UnfoundObjectReference ex)
	    {
		ErrorDialog.showUnfoundObjectReference(ex);
	    }

	    mainViewPnl_.refresh(selectedObj);
	    refreshMenuItems();
        }
    }

    private class StopAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
	    ServiceObject selectedObj = objectTree_.getSelectedObject();

	    try
	    {
                if(selectedObj != null &&
		   selectedObj.getType() == ServiceObject.SERVER)
		    stopServer(selectedObj);
	    }
            catch(org.omg.CORBA.SystemException ex)
            {
	        ErrorDialog.showSystemException(ex); 
            }

	    try
	    {
		Thread.currentThread().sleep(500);
	    }
	    catch(InterruptedException ex)
	    {
	    }

	    try
	    {
		objectTree_.refreshServer(selectedObj);
	    }
	    catch(UnfoundObjectReference ex)
	    {
		ErrorDialog.showUnfoundObjectReference(ex);
	    }

	    mainViewPnl_.refresh(selectedObj);
	    refreshMenuItems();
        }
    }

    private class ResetAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
	    ServiceObject selectedObj = objectTree_.getSelectedObject();

	    try
	    {
                if(selectedObj != null &&
		   selectedObj.getType() == ServiceObject.SERVER)
		    resetServer(selectedObj);
	    }
            catch(org.omg.CORBA.SystemException ex)
            {
	        ErrorDialog.showSystemException(ex); 
            }

	    try
	    {
		Thread.currentThread().sleep(500);
	    }
	    catch(InterruptedException ex)
	    {
	    }

	    try
	    {
		objectTree_.refreshServer(selectedObj);
	    }
	    catch(UnfoundObjectReference ex)
	    {
		ErrorDialog.showUnfoundObjectReference(ex);
	    }

	    mainViewPnl_.refresh(selectedObj);
	    refreshMenuItems();
        }
    }

    private class RefreshAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
	    try
	    {
	        status_.setText("Refreshing...");

		objectTree_.refresh();

		ServiceObject selectedObj = objectTree_.getSelectedObject();
		if(selectedObj != null)
		    mainViewPnl_.refresh(selectedObj);
		refreshMenuItems();

	        status_.setText("Refreshing... done.");
	    }
            catch(org.omg.CORBA.SystemException ex)
            {
	        ErrorDialog.showSystemException(ex); 
            }
        }
    }

    private class CutAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    ServiceObject obj = objectTree_.getSelectedObject();
	    Clipboard.instance().setData(new ClipboardData(null, obj));

	    wasCut_ = true;
	}
    }

    private class PasteAction extends AbstractAction
    {
	public void
	actionPerformed(ActionEvent e)
	{
	    ServiceObject target = objectTree_.getSelectedObject();
	    ServiceObject obj =
		(ServiceObject)Clipboard.instance().getData().data;

	    if(target.getType() == ServiceObject.OAD &&
	       obj.getType() == ServiceObject.SERVER &&
	       wasCut_)
	    {
		try
		{
		    OADInfo oadInfo = (OADInfo)target.toObject();
		    Server server = (Server)obj.toObject();

		    server.host(oadInfo.host);
  		    objectTree_.move(obj, target);
		
		    status_.setText("Success: Moved server `" +
				    obj.toString() + "'");
		    Clipboard.instance().setData(null);
		}
		catch(UnfoundObjectReference ex)
		{
		    ErrorDialog.showUnfoundObjectReference(ex);
		}
                catch(org.omg.CORBA.NO_PERMISSION ex)
                {
	            ErrorDialog.showNoPermissionException(); 
                }
		catch(org.omg.CORBA.SystemException ex)
		{
		    ErrorDialog.showSystemException(ex);
		}
	    }
	}
    }

    private class AboutAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
	    ErrorDialog.showAbout(aboutIcon_);
        }
    }

    private class PopupListener extends MouseAdapter 
    {
        public void 
        mousePressed(MouseEvent e) 
        {
            TreePath selPath = objectTree_.getPathForLocation(e.getX(),
							      e.getY());
	    objectTree_.setSelectionPath(selPath);
            showPopup(e);
        }

        public void 
        mouseReleased(MouseEvent e) 
        {
            showPopup(e);
        }

        private void 
        showPopup(MouseEvent e) 
        {
            if(e.isPopupTrigger()) 
                popup_.show(e.getComponent(), e.getX(), e.getY());
        }
    }
}
