// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.IMRConsole;

import java.lang.*;
import java.util.Date;
import java.text.DateFormat;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import com.ooc.IMR.*;

final class MainTableModel extends AbstractTableModel
{
    String[] columnNames = {"Property", "Value"};
    Object[][] data = null;

    public String
    getColumnName(int col) 
    { 
	return columnNames[col]; 
    }

    public int
    getRowCount() 
    { 
	return data.length; 
    }

    public int
    getColumnCount() 
    { 
	return columnNames.length; 
    }

    public Object
    getValueAt(int row, int col) 
    { 
	return data[row][col]; 
    }

    public boolean
    isCellEditable(int row, int col)
    { 
	return false; 
    }

    public void
    setValueAt(Object value, int row, int col) 
    {
	data[row][col] = value;
	fireTableCellUpdated(row, col);
    }

    MainTableModel()
    {
	data = new Object[0][2];
    }

    void
    updateData(OADInfo oadInfo)
    {
	data = new Object[2][2];
	    
	data[0][0] = "Host";
	data[0][1] = oadInfo.host;
	    
	data[1][0] = "Up";
	data[1][1] = oadInfo.up ? "Yes" : "No";

	fireTableDataChanged();
    }

    void
    updateData(ServerProps props)
    {
	data = new Object[12][2];
	    
	data[0][0] = "Name";
	data[0][1] = props.getName();
	    
	data[1][0] = "Execution Path";
	data[1][1] = props.getExec();

	data[2][0] = "Arguments";
	String[] args = props.getArgs();
	String sArgs = "";
	for(int i = 0; i < args.length; i++)
	{
	    if(i != 0)
		sArgs += " ";
	    sArgs += args[i];
	}
	data[2][1] = sArgs;

	data[3][0] = "Directory";
	data[3][1] = props.getDirectory();
	    
	data[4][0] = "Activation Mode";
	String mode = "";
	if(props.getMode() == ServerActivationMode.SHARED)
	    mode = "Shared";
	else if(props.getMode() == ServerActivationMode.PERSISTENT)
	    mode = "Persistent";
	data[4][1] = mode;
	    
	data[5][0] = "Activate POAs";
	data[5][1] = props.getActivatePOAs() ? "Yes" : "No";
	    
	data[6][0] = "Maximum Spawn Count";
	data[6][1] = new Short(props.getMaxForks());
	    
	data[7][0] = "Failure Timeout (secs)";
	data[7][1] = new Integer(props.getFailureTimeout());

	data[8][0] = "Update Timeout (ms)";
	data[8][1] = new Integer(props.getUpdateTimeout());
	    
	data[9][0] = "Status";
	String status;
	if(props.getStatus() == ServerStatus.FORKED)
	    status = "Forked";
	else if(props.getStatus() == ServerStatus.STARTING)
	    status = "Starting";
	else if(props.getStatus() == ServerStatus.RUNNING)
	    status = "Running";
	else if(props.getStatus() == ServerStatus.STOPPING)
	    status = "Stopping";
	else if(props.getStatus() == ServerStatus.STOPPED)
	    status = "Stopped";
	else
	    status = "";
	data[9][1] = status;
	    
	data[10][0] = "Started Manually";
	data[10][1] = props.getManual() ? "Yes" : "No";
	    
	data[11][0] = "Number of Times Spawned";
	data[11][1] = new Short(props.getTimesForked());
	    
	fireTableDataChanged();
    }

    void
    updateData(POAInfo poaInfo)
    {
	data = new Object[2][2];

	String sPoaName = "";
	for(int i = 0; i < poaInfo.poa.length; i++)
	{
	    if(i != 0)
		sPoaName += "/";
	    sPoaName += poaInfo.poa[i];
	}
	    
	data[0][0] = "Name";
	data[0][1] = sPoaName;
	    
	data[1][0] = "State";
	if(poaInfo.state == POAStatus.INACTIVE)
	    data[1][1] = "Inactive";
	else if(poaInfo.state == POAStatus.ACTIVE)
	    data[1][1] = "Active";
	else if(poaInfo.state == POAStatus.HOLDING)
	    data[1][1] = "Holding";
	else if(poaInfo.state == POAStatus.DISCARDING)
	    data[1][1] = "Discarding";
	else if(poaInfo.state == POAStatus.NON_EXISTENT)
	    data[1][1] = "";

	fireTableDataChanged();
    }

    void
    clear()
    {
	data = new Object[0][2];
	fireTableDataChanged();
    }
}
