// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.IMRConsole;

import java.lang.*;
import java.util.Date;
import java.text.DateFormat;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import com.ooc.IMR.*;

final class MainViewPanel extends JPanel 
{
    private JFrame parent_;
    private int type_ = ServiceObject.UNDEFINED;
    private MainTableModel model_;
    
    MainViewPanel(JFrame parent)
    {
	parent_ = parent;

        JTable table =
	    new JTable(0, 2)
	    {
		private DefaultTableCellRenderer myRenderer_ =
	            new DefaultTableCellRenderer()
		    {
			public Component
			getTableCellRendererComponent(
			    JTable tbl, Object value, boolean isSelected,
			    boolean hasFocus, int r, int c)
			{
			    Component comp =
				super.getTableCellRendererComponent(
				    tbl, value, isSelected, hasFocus, r, c);
			    if(r % 4 == 0 || r % 4 == 1)
				comp.setBackground(new Color(239, 255, 239));
			    else
				comp.setBackground(new Color(255, 255, 239));
			    return comp;
			}
		    };
	    
		public TableCellRenderer
		getCellRenderer(int row, int col)
	        {
		    return myRenderer_;
		}
	    };

        model_ = new MainTableModel();
        table.setModel(model_);
        table.setRowSelectionAllowed(false);
        table.setColumnSelectionAllowed(false);
        table.setCellSelectionEnabled(false);

	JLabel lbl = new JLabel("Properties");
	lbl.setOpaque(true);

        JPanel p1 = new JPanel(new BorderLayout());
        p1.setBackground(Color.white);
        p1.add(lbl, BorderLayout.NORTH);
        p1.add(new JScrollPane(table), BorderLayout.CENTER);

        setLayout(new BorderLayout());
	setBorder(BorderFactory.createEtchedBorder());
        add(p1, BorderLayout.CENTER);
    }

    void
    refresh(ServiceObject object)
    {
	type_ = object.getType();

        try 
	{
	    if(type_ == ServiceObject.OAD)
		model_.updateData((OADInfo)object.toObject());
	    else if(type_ == ServiceObject.SERVER)
	    {
		Server sa = (Server)object.toObject();

		ServerProps props =
		    new ServerProps(sa.name(),
				    sa.status(),
				    sa.manual(),
				    sa.updateTime(),
				    sa.timesForked());
		props.setHost(sa.host());
		props.setExec(sa.exec());
		props.setArgs(sa.args());
		props.setDirectory(sa.directory());
		props.setMode(sa.mode());
		props.setActivatePOAs(sa.activatePOAs());
		props.setUpdateTimeout(sa.updateTimeout());
		props.setFailureTimeout(sa.failureTimeout());
		props.setMaxForks(sa.maxForks());
		    
		model_.updateData(props);
	    }
	    else if(type_ == ServiceObject.POA)
		model_.updateData((POAInfo)object.toObject());
	    else
		clear();
	}
	catch(UnfoundObjectReference ex)
	{
	    ErrorDialog.showUnfoundObjectReference(ex);
	}
	catch(org.omg.CORBA.SystemException ex)
	{
	    ErrorDialog.showSystemException(ex); 
	}
    }

    void
    clear()
    {
	model_.clear();
    }
}
