// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.IMRConsole;

import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import com.ooc.IMR.*;

class POADialog extends JDialog
{
    private static boolean new_;

    private static JTextField poaTxt_;
    private static JTextField stateTxt_;
    
    private static POADialog dialog_;
    private static POAInfo poaInfo_;
    private static POAInfo rtnPOAInfo_;

    private static AbstractAction okAction_;
    private static AbstractAction cancelAction_;

    static POAInfo
    show(JFrame parent, String title, POAInfo poaInfo)
    {
	new_ = poaInfo == null;
	if(new_)
	    poaInfo_ = new POAInfo(new String[0], POAStatus.NON_EXISTENT);
	else
	    poaInfo_ = poaInfo;
	    
        dialog_ = new POADialog(parent, title);

	dialog_.setPOAInfo();

        dialog_.setLocationRelativeTo(parent);
        dialog_.setVisible(true);

        return rtnPOAInfo_;
    }

    private
    POADialog(JFrame parent, String title)
    {
        super(parent, title, true);
        initializeActions();
	rtnPOAInfo_ = null;
        setResizable(false);
        
	JPanel buttonPnl = initializeButtonPanel();
	JPanel setPOAInfoPnl = initializePOAInfoPanel();
	
        JPanel mainPnl = new JPanel();
        mainPnl.setLayout(new BorderLayout());
        mainPnl.setBorder(BorderFactory.createEtchedBorder());
        mainPnl.add(setPOAInfoPnl, BorderLayout.CENTER);

        JPanel holderPnl = new JPanel();
        holderPnl.setLayout(new BorderLayout());
        holderPnl.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        holderPnl.add(mainPnl, BorderLayout.CENTER);
        holderPnl.add(buttonPnl, BorderLayout.SOUTH);

        Container contentPane = getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.add(holderPnl, BorderLayout.SOUTH);

        pack();
    }

    private JPanel
    initializeButtonPanel()
    {
	JButton okBtn = new JButton("Ok");
        okBtn.addActionListener(okAction_);

        JButton cancelBtn = new JButton("Cancel");
        cancelBtn.addActionListener(cancelAction_);

	JPanel pnl = new JPanel();
        pnl.setLayout(new BoxLayout(pnl, BoxLayout.X_AXIS));
        pnl.setBorder(BorderFactory.createEmptyBorder(5, 0, 0, 0));
        pnl.add(Box.createHorizontalGlue());
        pnl.add(okBtn);
        pnl.add(Box.createRigidArea(new Dimension(5, 0)));
        pnl.add(cancelBtn);

	return pnl;
    }

    private JPanel
    initializePOAInfoPanel()
    {
	poaTxt_ = new JTextField(30);
	poaTxt_.setEnabled(new_);

	stateTxt_ = new JTextField(30);
	stateTxt_.setEnabled(false);

	JPanel pnl = new JPanel(new GridBagLayout());

	Util.addDialogHSpace(pnl);
	Util.addDialogParameterPair(pnl, "Name:", poaTxt_);
	Util.addDialogParameterPair(pnl,  "State:", stateTxt_);

	Util.addDialogHSpace(pnl);

	return pnl;
    }

    private void
    initializeActions()
    {
        okAction_ = new OkAction();
        cancelAction_ = new CancelAction();
    }

    private void
    setPOAInfo()
    {
	String[] poa = poaInfo_.poa;
	String sPOA = "";
	for(int i = 0; i < poaInfo_.poa.length; i++)
	{
	    if(i != 0)
		sPOA += "/";
	    sPOA += poaInfo_.poa[i];
	}
	poaTxt_.setText(sPOA);

	if(poaInfo_.state == POAStatus.INACTIVE)
	    stateTxt_.setText("Inactive");
	else if(poaInfo_.state == POAStatus.ACTIVE)
	    stateTxt_.setText("Active");
	else if(poaInfo_.state == POAStatus.HOLDING)
	    stateTxt_.setText("Holding");
	else if(poaInfo_.state == POAStatus.DISCARDING)
	    stateTxt_.setText("Discarding");
	else if(poaInfo_.state == POAStatus.NON_EXISTENT)
	    stateTxt_.setText("");
    }

    private void
    loadPOAInfo()
    {
	rtnPOAInfo_ = poaInfo_;

	StringTokenizer st = new StringTokenizer(poaTxt_.getText(), "/");
	String[] poa = new String[st.countTokens()];
	for(int i = 0; st.hasMoreTokens(); i++)
	    poa[i] = st.nextToken();
	rtnPOAInfo_.poa = poa;
    }
    
    private class OkAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
	    String sPOA = poaTxt_.getText();

	    int i = 0;
	    for(; i < sPOA.length(); i++)
	    {
		char c = sPOA.charAt(i);

		if(i == 0)
		{
		    if(!Character.isUnicodeIdentifierStart(c))
			break;
		}
		else
		    if(!Character.isUnicodeIdentifierPart(c))
			if((c == '/') &&
			   (i < sPOA.length() - 1) &&
			   (Character.isUnicodeIdentifierStart(
			       sPOA.charAt(i+1))))
			    i = i + 1;
			else
			    break;
	    }

	    if(i == sPOA.length())
	    {
		loadPOAInfo();
                dialog_.setVisible(false);
	    }
	    else
		ErrorDialog.showBadValue(poaTxt_, sPOA);
        }
    }

    private class CancelAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
            dialog_.setVisible(false);
        }
    }  
}
