// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.IMRConsole;

import java.lang.*;
import java.util.*;
import java.text.DateFormat;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import com.ooc.IMR.*;

class ServerDialog extends JDialog
{
    private static boolean new_;

    private static JTextField statusTxt_;
    private static JTextField manualTxt_;

    private static NumberField timesForkedNum_;
    private static NumberField updateTimeoutNum_;
    private static NumberField failureTimeoutNum_;
    private static NumberField maxForksNum_;

    private static JTextField nameTxt_;
    private static JTextField execTxt_;
    private static JTextField argsTxt_;
    private static JTextField directoryTxt_;

    private static final String[] modes_ = {"Shared", "Persistent"};

    private static JComboBox modeField_;
    private static JCheckBox activatePOAsField_;
    
    private static ServerDialog dialog_;
    private static ServerProps props_;
    private static ServerProps rtnProps_;

    private static AbstractAction okAction_;
    private static AbstractAction cancelAction_;

    static ServerProps
    show(JFrame parent, String title, ServerProps props)
    {
	new_ = (props == null);
	if(new_)
	    props_ = new ServerProps();
	else
	    props_ = props;
	    
        dialog_ = new ServerDialog(parent, title);

	dialog_.setProps();

        dialog_.setLocationRelativeTo(parent);
        dialog_.setVisible(true);

        return rtnProps_;
    }

    private
    ServerDialog(JFrame parent, String title)
    {
        super(parent, title, true);
        initializeActions();
	rtnProps_ = null;
        setResizable(false);
        
	JPanel buttonPnl = initializeButtonPanel();
	JPanel setPropsPnl = initializePropsPanel();
	
        JPanel mainPnl = new JPanel();
        mainPnl.setLayout(new BorderLayout());
        mainPnl.setBorder(BorderFactory.createEtchedBorder());
        mainPnl.add(setPropsPnl, BorderLayout.CENTER);

        JPanel holderPnl = new JPanel();
        holderPnl.setLayout(new BorderLayout());
        holderPnl.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        holderPnl.add(mainPnl, BorderLayout.CENTER);
        holderPnl.add(buttonPnl, BorderLayout.SOUTH);

        Container contentPane = getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.add(holderPnl, BorderLayout.SOUTH);

        pack();
    }

    private JPanel
    initializeButtonPanel()
    {
	JButton okBtn = new JButton("Ok");
        okBtn.addActionListener(okAction_);

        JButton cancelBtn = new JButton("Cancel");
        cancelBtn.addActionListener(cancelAction_);

	JPanel pnl = new JPanel();
        pnl.setLayout(new BoxLayout(pnl, BoxLayout.X_AXIS));
        pnl.setBorder(BorderFactory.createEmptyBorder(5, 0, 0, 0));
        pnl.add(Box.createHorizontalGlue());
        pnl.add(okBtn);
        pnl.add(Box.createRigidArea(new Dimension(5, 0)));
        pnl.add(cancelBtn);

	return pnl;
    }

    private JPanel
    initializePropsPanel()
    {
	statusTxt_ = new JTextField(30);
	statusTxt_.setEnabled(false);
	manualTxt_ = new JTextField(30);
	manualTxt_.setEnabled(false);

	timesForkedNum_ = new NumberField("0", NumberField.SHORT);
	timesForkedNum_.setEnabled(false);

	updateTimeoutNum_ = new NumberField("0", NumberField.LONG);
	failureTimeoutNum_ = new NumberField("0", NumberField.LONG);
	maxForksNum_ = new NumberField("0", NumberField.SHORT);

	nameTxt_ = new JTextField(30);
	nameTxt_.setEnabled(new_);

	execTxt_ = new JTextField(30);
	argsTxt_ = new JTextField(30);
	directoryTxt_ = new JTextField(30);

	modeField_ = new JComboBox(modes_);
	activatePOAsField_ = new JCheckBox();

	JPanel pnl = new JPanel(new GridBagLayout());

	Util.addDialogHSpace(pnl);
	Util.addDialogParameterPair(pnl, "Name:", nameTxt_);

	Util.addDialogHSpace(pnl);
	Util.addDialogParameterPair(pnl, "Execution Path:", execTxt_);
	Util.addDialogParameterPair(pnl, "Arguments:", argsTxt_);
	Util.addDialogParameterPair(pnl, "Directory:", directoryTxt_);

	Util.addDialogHSpace(pnl);
	Util.addDialogParameterPair(pnl, "Activate Mode:", modeField_);
	Util.addDialogParameterPair(pnl, "Activate POAs:", activatePOAsField_);

	Util.addDialogHSpace(pnl);
	Util.addDialogParameterPair(pnl, "Maximum Spawn Count:", maxForksNum_);
	Util.addDialogParameterPair(pnl,
				    "Failure Timeout (secs):",
				    failureTimeoutNum_);
	Util.addDialogParameterPair(pnl,
				    "Update Timeout (ms):",
				    updateTimeoutNum_);

	Util.addDialogHSpace(pnl);
	Util.addDialogParameterPair(pnl, "Status:", statusTxt_);
	Util.addDialogParameterPair(pnl, "Started Manually:", manualTxt_);
	Util.addDialogParameterPair(pnl,
				    "Number of Times Spawned:",
				    timesForkedNum_);

	Util.addDialogHSpace(pnl);

	return pnl;
    }

    private void
    initializeActions()
    {
        okAction_ = new OkAction();
        cancelAction_ = new CancelAction();
    }

    private void
    setProps()
    {
	String status;
	if(props_.getStatus() == ServerStatus.FORKED)
	    status = "Forked";
	else if(props_.getStatus() == ServerStatus.STARTING)
	    status = "Starting";
	else if(props_.getStatus() == ServerStatus.RUNNING)
	    status = "Running";
	else if(props_.getStatus() == ServerStatus.STOPPING)
	    status = "Stopping";
	else
	    status = "Stopped";
	statusTxt_.setText(status);

	manualTxt_.setText(props_.getManual() ? "Yes" : "No");

	timesForkedNum_.setText(String.valueOf(props_.getTimesForked()));
	updateTimeoutNum_.setText(String.valueOf(props_.getUpdateTimeout()));
	failureTimeoutNum_.setText(String.valueOf(props_.getFailureTimeout()));
	maxForksNum_.setText(String.valueOf(props_.getMaxForks()));

	nameTxt_.setText(props_.getName());
	execTxt_.setText(props_.getExec());

	String[] args = props_.getArgs();
	String sArgs = "";
	for(int i = 0; i < args.length; i++)
	{
	    if(i != 0)
		sArgs += " ";
	    sArgs += args[i];
	}
	argsTxt_.setText(sArgs);

	directoryTxt_.setText(props_.getDirectory());

	String mode = "";
	if(props_.getMode() == ServerActivationMode.SHARED)
	    mode = "Shared";
	else if(props_.getMode() == ServerActivationMode.PERSISTENT)
	    mode = "Persistent";
	modeField_.setSelectedItem(mode);

	activatePOAsField_.setSelected(props_.getActivatePOAs());

    }

    private void
    loadProps()
    {
	rtnProps_ = (ServerProps)props_.clone();

	rtnProps_.setName(nameTxt_.getText());
	rtnProps_.setExec(execTxt_.getText());

	StringTokenizer st = new StringTokenizer(argsTxt_.getText());
	String[] args = new String[st.countTokens()];
	for(int i = 0; st.hasMoreTokens(); i++)
	    args[i] = st.nextToken();
	rtnProps_.setArgs(args);

	rtnProps_.setDirectory(directoryTxt_.getText());

	String mode = (String)modeField_.getSelectedItem();
	if(mode.equals("Shared"))
	    rtnProps_.setMode(ServerActivationMode.SHARED);
	if(mode.equals("Persistent"))
	    rtnProps_.setMode(ServerActivationMode.PERSISTENT);

	rtnProps_.setActivatePOAs(activatePOAsField_.isSelected());

	rtnProps_.setUpdateTimeout(
	    Integer.parseInt(updateTimeoutNum_.getText()));
	rtnProps_.setFailureTimeout(
	    Integer.parseInt(failureTimeoutNum_.getText()));
	rtnProps_.setMaxForks(
	    Short.parseShort(maxForksNum_.getText()));
    }

    private boolean
    fieldIsValid(NumberField field, String name)
    {
	if(!field.isValid())
	{
	    ErrorDialog.showBadValue(field, name);
	    return false;
        }

	return true;
    }
    
    private class OkAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
	    if(fieldIsValid(updateTimeoutNum_, "Update Timeout") &&
	       fieldIsValid(failureTimeoutNum_, "Failure Timeout") &&
	       fieldIsValid(maxForksNum_, "Maximum Spawn Count"))
	    {
                loadProps();
                dialog_.setVisible(false);
	    }
        }
    }

    private class CancelAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
            dialog_.setVisible(false);
        }
    }  
}
