// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA  
//                  
// All Rights Reserved   
//   
// **********************************************************************

package com.ooc.IMRConsole;

import java.net.*;
import java.util.*;
import javax.swing.*;
import org.omg.CORBA.*;
import com.ooc.IMR.*;

final class ServiceManager 
{
    private JFrame parent_;
    private ORB orb_;
    private ServerDomain da_;

    ServiceManager(JFrame parent, ORB orb, ServerDomain da)
    {
	parent_ = parent;
	orb_ = orb;
        da_ = da;
    }

    ServiceObject
    createDomainObject()
    {
	return new ServiceObject(null,
				 null,
				 ServiceObject.DOMAIN,
				 ServiceObject.UNDEFINED,
				 da_);
    }

    ServiceObject
    createObject(String objectName, int objectType, ServiceObject parent)
    {
	String serverName;
	if(objectType == ServiceObject.SERVER)
	    serverName = objectName;
	else
	    serverName = parent.getServerName();
	
	return new ServiceObject(objectName,
				 serverName,
		                 objectType,
				 parent.getType(),
				 da_);
    }

    boolean
    createOAD(String name)
    {
	try
	{
	    da_.create_oad_record(name);
	}
	catch(OADAlreadyExists ex)
	{
            ErrorDialog.showOADAlreadyExists(ex);
	    return false;
	}

	return true;
    } 

    boolean
    createServer(String name,
		 String exec,
		 String[] args,
		 String directory,
		 ServerActivationMode mode,
		 boolean activatePOAs,
		 int updateTimeout,
		 int failureTimeout,
		 short maxForks,
		 ServiceObject oadObj)
    {
        try
        {
	    ServerFactory sf = da_.get_server_factory();
            Server s = sf.create_server_record(name);

	    OADInfo oadInfo = (OADInfo)oadObj.toObject();

	    s.exec(exec);
	    s.host(oadInfo.host);
	    s.args(args);
	    s.directory(directory);
	    s.mode(mode);
	    s.activatePOAs(activatePOAs);
	    s.updateTimeout(updateTimeout);
	    s.failureTimeout(failureTimeout);
	    s.maxForks(maxForks);
        }
        catch(ServerAlreadyRegistered ex)
        {
            ErrorDialog.showServerAlreadyRegistered(ex);
	    return false;
        }
        catch(UnfoundObjectReference ex)
        {
            ErrorDialog.showUnfoundObjectReference(ex);
	    return false;
        }

	return true;
    }

    boolean
    createPOA(String[] name, ServiceObject saObj)
    {
	try
	{
            Server s = (Server)saObj.toObject();

	    String[] newName = new String[name.length + 1];
	    newName[0] = s.name();
	    for(int i = 0; i < name.length; i++)
		newName[i + 1] = name[i];
	    
	    s.create_poa_record(newName);
	}
	catch(UnfoundObjectReference ex)
	{
            ErrorDialog.showUnfoundObjectReference(ex);
	    return false;
	}
	catch(POAAlreadyRegistered ex)
	{
            ErrorDialog.showPOAAlreadyRegistered(ex);
	    return false;
	}

	return true;
    } 

    boolean
    destroyOAD(ServiceObject oadObj)
    {
	try
	{
            OADInfo oadInfo = (OADInfo)oadObj.toObject();
	    da_.remove_oad_record(oadInfo.host);
	}
	catch(NoSuchOAD ex)
	{
            ErrorDialog.showNoSuchOAD(ex);
	    return false;
	}
	catch(OADRunning ex)
	{
            ErrorDialog.showOADRunning(ex);
	    return false;
	}
	catch(UnfoundObjectReference ex)
	{
            ErrorDialog.showUnfoundObjectReference(ex);
	    return false;
	}

	return true;
    }

    boolean
    destroyServer(ServiceObject saObj)
    {
	try
	{
            Server s = (Server)saObj.toObject();
            s.destroy();
	}
	catch(ServerRunning ex)
	{
            ErrorDialog.showServerRunning(ex);
	    return false;
	}
	catch(UnfoundObjectReference ex)
	{
            ErrorDialog.showUnfoundObjectReference(ex);
	    return false;
	}

	return true;
    }

    boolean
    destroyPOA(ServiceObject poaObj)
    {
	try
	{
	    ServerFactory sf = da_.get_server_factory();
	    Server s = sf.get_server(poaObj.getServerName());

	    String sPOAName = poaObj.getName();
	    StringTokenizer st = new StringTokenizer(sPOAName, "/");
	    String[] poaName = new String[st.countTokens() + 1];
	    poaName[0] = s.name();
	    for(int i = 0; st.hasMoreTokens(); i++)
		poaName[i + 1] = st.nextToken();

	    s.remove_poa_record(poaName);
	}
	catch(NoSuchServer ex)
	{
            ErrorDialog.showNoSuchServer(ex);
	    return false;
	}
	catch(_NoSuchPOA ex)
	{
            ErrorDialog.showNoSuchPOA(ex);
	    return false;
	}

	return true;
    }

    boolean
    startServer(ServiceObject saObj)
    {
	try
	{
            Server s = (Server)saObj.toObject();
	    s.start();
	}
	catch(UnfoundObjectReference ex)
	{
            ErrorDialog.showUnfoundObjectReference(ex);
	    return false;
	}
	catch(ServerRunning ex)
	{
            ErrorDialog.showServerRunning(ex);
	    return false;
	}

	return true;
    }

    boolean
    stopServer(ServiceObject saObj)
    {
	try
	{
            Server s = (Server)saObj.toObject();
	    s.stop();
	}
	catch(UnfoundObjectReference ex)
	{
            ErrorDialog.showUnfoundObjectReference(ex);
	    return false;
	}
	catch(OADNotRunning ex)
	{
            ErrorDialog.showOADNotRunning(ex);
	    return false;
	}
	catch(ServerNotRunning ex)
	{
            ErrorDialog.showServerNotRunning(ex);
	    return false;
	}

	return true;
    }

    boolean
    resetServer(ServiceObject saObj)
    {
	try
	{
            Server s = (Server)saObj.toObject();
	    s.clear_error_state();
	}
	catch(UnfoundObjectReference ex)
	{
            ErrorDialog.showUnfoundObjectReference(ex);
	    return false;
	}

	return true;
    }

    String[] 
    getOADNames()
    {
	ServerFactory sf = da_.get_server_factory();
	Server[] servers = sf.list_orphaned_servers();
	boolean orphanedServers = servers.length > 0;

	OADInfo[] oadInfos = da_.list_oads();
	
	String oadNames[] =
	    new String[oadInfos.length + (orphanedServers ? 1 : 0)];
	for(int i = 0; i < oadInfos.length; i++)
	    oadNames[i] = oadInfos[i].host;
	if(orphanedServers)
	    oadNames[oadInfos.length] = ServiceObject.ORPHANED_SERVER_OAD_NAME;

	return oadNames;
    }

    String[] 
    getServerNames(ServiceObject oadObj)
    {
	try
	{
	    ServerFactory sf = da_.get_server_factory();

	    Server[] servers = null;
	    if(oadObj.getName().equals(ServiceObject.ORPHANED_SERVER_OAD_NAME))
		servers = sf.list_orphaned_servers();
	    else
	    {
		OADInfo oadInfo = (OADInfo)oadObj.toObject();
		servers = sf.list_servers_by_host(oadInfo.host);
	    }

	    String[] serverNames = new String[servers.length];
	    for(int i = 0; i < servers.length; i++)
		serverNames[i] = servers[i].name();

  	    return serverNames;
	}
	catch(UnfoundObjectReference ex)
	{
            ErrorDialog.showUnfoundObjectReference(ex);
	    return null;
	}
    }

    String[] 
    getPOANames(ServiceObject saObj)
    {
	try
	{
            Server s = (Server)saObj.toObject();
	    POAInfo poaInfos[] = s.list_poas();

	    String sPoaNames[] = new String[poaInfos.length];
	    
	    for(int i = 0; i < poaInfos.length; i++)
	    {
		sPoaNames[i] = "";
		for(int j = 1; j < poaInfos[i].poa.length; j++)
		{
		    if(j != 1)
			sPoaNames[i] += "/";
		    sPoaNames[i] += poaInfos[i].poa[j];
		}
	    }
		    
	    return sPoaNames;
	}
	catch(UnfoundObjectReference ex)
	{
            ErrorDialog.showUnfoundObjectReference(ex);
	    return null;
	}
    }
}
