// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA  
//              
// All Rights Reserved   
//   
// **********************************************************************

package com.ooc.IMRConsole;

import java.util.*;
import javax.swing.tree.*;
import com.ooc.IMR.*;

final class ServiceObject extends DefaultMutableTreeNode
{
    final static int UNDEFINED = -1;
    final static int DOMAIN = 0;
    final static int OAD = 1;
    final static int SERVER = 2;
    final static int POA = 3;

    final static String ORPHANED_SERVER_OAD_NAME = "Orphaned Servers";

    private String name_;
    private String serverName_;
    private boolean valid_;
    private int type_;
    private int parentType_;
    private ServerDomain da_;
    private boolean isOrphanedServerOAD_;

    ServiceObject()
    {
	name_ = null;
	serverName_ = null;
	type_ = UNDEFINED;
	parentType_ = UNDEFINED;
	da_ = null;
	isOrphanedServerOAD_ = false;
    }

    ServiceObject(String name,
		  String serverName,
		  int type,
		  int parentType,
		  ServerDomain da)
    {
        name_ = name;
        serverName_ = serverName;
        type_ = type;
        parentType_ = parentType;
        da_ = da;
	isOrphanedServerOAD_ =
	    type_ == OAD && name_.equals(ORPHANED_SERVER_OAD_NAME);
    }

    boolean
    isType(int type)
    {
        return type_ == type;
    }

    boolean
    isValid()
    {
        return valid_;
    }

    int
    getType()
    {
        return type_;
    }

    boolean
    isOrphanedServerOAD()
    {
        return isOrphanedServerOAD_;
    }

    int
    getParentType()
    {
        return parentType_;
    }

    String
    getName()
    {
        return name_;
    }
 
    String
    getServerName()
    {
        return serverName_;
    }

    Object
    toObject() throws UnfoundObjectReference
    {
	try
        {
	    if(type_ == DOMAIN)
	    {
		valid_ = !da_._non_existent();
		if(valid_)
		    return da_;

		ErrorDialog.showWarning("IMR not available.");
	    }
	    else if(type_ == OAD)
	    {
		OADInfo oadInfo = da_.get_oad_record(name_);
		valid_ = oadInfo.up;

		return oadInfo;
	    }
	    else if(type_ == SERVER)
	    {
		ServerFactory sf = da_.get_server_factory();
		Server s = sf.get_server(name_);

		valid_ = s.status() != ServerStatus.STOPPED;

		return s;
	    }
	    else if(type_ == POA)
	    {
		ServerFactory sf = da_.get_server_factory();
		Server s = sf.get_server(serverName_);
		
		StringTokenizer st = new StringTokenizer(name_, "/");
		String[] poaName = new String[st.countTokens() + 1];
		poaName[0] = s.name();
		for(int i = 1; st.hasMoreTokens(); i++)
		    poaName[i] = st.nextToken();

		POAInfo poaInfo = s.get_poa_info(poaName);
		String[] poa = new String[poaInfo.poa.length - 1];
		for(int i = 1; i < poaInfo.poa.length; i++)
		    poa[i - 1] = poaInfo.poa[i];
		poaInfo.poa = poa;

		valid_ = poaInfo.state != POAStatus.INACTIVE &&
			 poaInfo.state != POAStatus.NON_EXISTENT;
		
		return poaInfo;
	    }
	}
	catch(NoSuchOAD ex)
        {
            ErrorDialog.showWarning("OAD not found.");
        }
        catch(NoSuchServer ex)
        {
	    ErrorDialog.showWarning("Server not found.");
        }
	catch(_NoSuchPOA ex)
        {
            ErrorDialog.showWarning("POA not found.");
        }

	valid_ = false;
        throw new UnfoundObjectReference(toString());
    }

    public String
    toString()
    {
        switch(type_)
        {
	    case DOMAIN:
		return "IMR Domain";
	    case OAD:
		if(isOrphanedServerOAD_)
		    return ORPHANED_SERVER_OAD_NAME;
		return "OAD " + name_;
	    case SERVER:
		return "Server " + name_;
	    case POA:
		return "POA " + name_;
	    default:
		return "Unknown Object";
        }
    }
}
