// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************
 
package naming;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import java.io.*;

public class Server
{
    
    public static void
    main(String args[])
    {
	java.util.Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");

	int status = 0;
	ORB orb = null;
	
	try
	{
	    //
	    // Create ORB
	    //
	    orb = ORB.init(args, props);
	    status = run(orb, args);
        }
	catch(Exception ex)
	{
	    ex.printStackTrace();
	    status = 1;
	}

	if(orb != null)
	{
	    try
	    {
	        orb.destroy();
	    }
	    catch(Exception ex)
	    {
	        System.err.println(ex.getMessage());
		ex.printStackTrace();
		status = 1;
	    }
	}

        System.exit(status);
    }

    static int
    run(ORB orb, String args[])
    {
        //
        // Parse command line arguments
        //
        String namingFile = null;
        int argc = 0;
        while(argc < args.length)
        {
	    if(args[argc].equals("-f"))
	    {
	        if(argc + 1 < args.length)
	        {
		    argc++;
		    namingFile = args[argc];
	        }
	        else
		    usage();
	    }
	    else if(args[argc].equals("-h") || args[argc].equals("--help"))
	        usage();
		
	    argc++;
        }
	    
        //
        // Resolve Root POA
        //
        org.omg.CORBA.Object poaObj = null;
        try
        {
            poaObj = orb.resolve_initial_references("RootPOA");
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName ex)
        {
            throw new RuntimeException();
        }

        POA rootPOA = POAHelper.narrow(poaObj);

        //
	// Get a reference to the POA manager
        //
        POAManager manager = rootPOA.the_POAManager();
        com.ooc.OBPortableServer.POAManager obManager =
            com.ooc.OBPortableServer.POAManagerHelper.narrow(manager);

        //
        // Create some implementations
        //
        Named_impl implA = new Named_impl();
        Named_impl implA1 = new Named_impl();
        Named_impl implA2 = new Named_impl();
        Named_impl implA3 = new Named_impl();
        Named_impl implB = new Named_impl();
        Named_impl implC = new Named_impl();
        Named a = implA._this(orb);
        Named a1 = implA1._this(orb);
        Named a2 = implA2._this(orb);
        Named a3 = implA3._this(orb);
        Named b = implB._this(orb);
        Named c = implC._this(orb);
	    
        //
        // Get naming service
        //
        org.omg.CORBA.Object obj = null;
    
        if(namingFile != null)
        {
	    try
	    {
	        FileReader r = new FileReader(namingFile);
	        BufferedReader in = new BufferedReader(r);
	        String ref = in.readLine();
	        r.close();
    
	        obj = orb.string_to_object(ref);
	    }
	    catch(IOException e)
	    {
	        e.printStackTrace();
	        return 1;
	    }
        }
        else
        {
	    try
	    {
	        obj = orb.resolve_initial_references("NameService");
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
	        System.out.println("Can't resolve `NameService'");
	        return 1;
	    }
        }

        if(obj == null)
        {
	    System.out.println("`NameService' is a nil object reference");
	    return 1;
        }

        NamingContext nc = null;
        try
        {
            nc = NamingContextHelper.narrow(obj);
        }
        catch(org.omg.CORBA.BAD_PARAM ex)
        {
	    System.out.println("`NameService' is not " +
			       "a NamingContext object reference");
	    return 1;
        }

        try
        {
	    //
	    // Create and bind some Naming Contexts
	    //
	    NameComponent[] nc1Name = new NameComponent[1];
	    nc1Name[0] = new NameComponent();
	    nc1Name[0].id = "nc1";
	    nc1Name[0].kind = "";
	    NamingContext nc1 = nc.bind_new_context(nc1Name);
    
	    NameComponent[] nc2Name = new NameComponent[2];
	    nc2Name[0] = new NameComponent();
	    nc2Name[0].id = "nc1";
	    nc2Name[0].kind = "";
	    nc2Name[1] = new NameComponent();
	    nc2Name[1].id = "nc2";
	    nc2Name[1].kind = "";
	    NamingContext nc2 = nc.bind_new_context(nc2Name);

	    //
	    // Bind names with the Naming Service
	    //
	    NameComponent[] aName = new NameComponent[1];
	    aName[0] = new NameComponent();
	    aName[0].id = "a";
	    aName[0].kind = "";
	    nc.bind(aName, a);

	    NameComponent[] a1Name = new NameComponent[1];
	    a1Name[0] = new NameComponent();
	    a1Name[0].id = "a1";
	    a1Name[0].kind = "";
	    nc.bind(a1Name, a1);

	    NameComponent[] a2Name = new NameComponent[1];
	    a2Name[0] = new NameComponent();
	    a2Name[0].id = "a2";
	    a2Name[0].kind = "";
	    nc.bind(a2Name, a2);

	    NameComponent[] a3Name = new NameComponent[1];
	    a3Name[0] = new NameComponent();
	    a3Name[0].id = "a3";
	    a3Name[0].kind = "";
	    nc.bind(a3Name, a3);

	    NameComponent[] bName = new NameComponent[2];
	    bName[0] = new NameComponent();
	    bName[0].id = "nc1";
	    bName[0].kind = "";
	    bName[1] = new NameComponent();
	    bName[1].id = "b";
	    bName[1].kind = "";
	    nc.bind(bName, b);

	    NameComponent[] cName = new NameComponent[3];
	    cName[0] = new NameComponent();
	    cName[0].id = "nc1";
	    cName[0].kind = "";
	    cName[1] = new NameComponent();
	    cName[1].id = "nc2";
	    cName[1].kind = "";
	    cName[2] = new NameComponent();
	    cName[2].id = "c";
	    cName[2].kind = "";
	    nc.bind(cName, c);

	    //
	    // Run implementation
	    //
	    try
	    {
	        manager.activate();
	    }
	    catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
	    {
	        throw new RuntimeException();
	    }
	    orb.run();

	    //
	    // Unregister names with the Naming Service
	    //
	    nc.unbind(cName);
	    nc.unbind(bName);
	    nc.unbind(aName);
	    nc.unbind(a1Name);
	    nc.unbind(a2Name);
	    nc.unbind(a3Name);
	    nc.unbind(nc2Name);
	    nc.unbind(nc1Name);
	    try
	    {
		nc2.destroy();
		nc1.destroy();
	    }
	    catch(NotEmpty ex)
	    {
		throw new RuntimeException();
	    }
        }
        catch(NotFound ex)
        {
	    System.err.print("Got a `NotFound' exception (");
	    switch(ex.why.value())
	    {
	    case NotFoundReason._missing_node:
	        System.err.print("missing node");
	        break;

	    case NotFoundReason._not_context:
	        System.err.print("not context");
	        break;
    
	    case NotFoundReason._not_object:
	        System.err.print("not object");
	        break;
	    }
	    System.err.println(")");
	    ex.printStackTrace();
	    return 1;
        }
        catch(CannotProceed ex)
        {
	    System.err.println("Got a `CannotProceed' exception");
	    ex.printStackTrace();
	    return 1;
        }
        catch(InvalidName ex)
        {
	    System.err.println("Got an `InvalidName' exception");
	    ex.printStackTrace();
	    return 1;
        }
        catch(AlreadyBound ex)
        {
	    System.err.println("Got an `AlreadyBound' exception");
	    ex.printStackTrace();
	    return 1;
        }

	return 0;
    }

    private static void
    usage()
    {
	System.out.println(
	    "Usage: java naming.Server [options]\n\n" +
	    "Options:\n" +
	    "-f FILE     Read the NamingService IOR from file FILE.\n" +
	    "-h, --help  Display this help message.");

        System.exit(1);
    }
}
