// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************
 
package com.ooc.CosNaming;

import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;

final class NSAdmin
{
    final static String progName = "com.ooc.CosNaming.NSAdmin";

    private static void
    usage()
    {
	System.err.println(
	    "Usage:\n"
	    + progName + " options\n" +
	    "\n" +
	    "Options:\n" +
	    "-h, --help                     Display this information.\n" +
	    "-b, --bind NAME IOR            Create a new object binding.\n" +
	    "-C, --bind_context NAME IOR    Create a new context binding.\n" +
	    "-c, --bind_new_context NAME    " +
	    "Create a new context and bind it.\n" +
	    "-d, --destroy NAME             " +
	    "Destroy an empty context.\n" +
	    "-l, --list [NAME]              " +
	    "Display the bindings in a context,\n" +
	    "                               " +
	    "or root context if no name given.\n" +
	    "-B, --rebind NAME IOR          Modify an object binding.\n" +
	    "-R, --rebind_context NAME IOR  Modify a context binding.\n" +
	    "-r, --resolve NAME             Display the IOR of a binding.\n" +
	    "-u, --unbind NAME              Remove a binding.\n"
	    );
    }

    private static void
    printException(UserException uex, NamingContextExt root, String name)
    {
	try
	{
	    NotFound notFound = (NotFound)uex;

	    NameComponent[] n = new NameComponent[1];
	    n[0] = notFound.rest_of_name[0];
	    String str = null;
	    try
	    {
		str = root.to_string(n);
	    }
	    catch(InvalidName ex)
	    {
		throw new RuntimeException();
	    }
	    if(notFound.why == NotFoundReason.missing_node)
		System.err.println("`" + str + "' was not found");
	    else if(notFound.why == NotFoundReason.not_context)
		System.err.println("`" + str + "' is not a context");
	    else if(notFound.why == NotFoundReason.not_object)
		System.err.println("`" + str + "' is not an object");
	    System.exit(1);
	}
	catch(ClassCastException ex)
	{
	}

	try
	{
	    CannotProceed cannotProceed = (CannotProceed)uex;

	    String rest = null;
	    try
	    {
		rest = root.to_string(cannotProceed.rest_of_name);
	    }
	    catch(InvalidName ex)
	    {
		throw new RuntimeException();
	    }
	    System.err.println("Cannot proceed, rest of name = `" + rest +
			       "'");
	    System.exit(1);
	}
	catch(ClassCastException ex)
	{
	}

	try
	{
	    InvalidName invalidName = (InvalidName)uex;

	    com.ooc.OB.Assert._OB_assert(name != null);
	    System.err.println("`" + name + "' is an invalid name");
	    System.exit(1);
	}
	catch(ClassCastException ex)
	{
	}

	try
	{
	    AlreadyBound alreadyBound = (AlreadyBound)uex;

	    com.ooc.OB.Assert._OB_assert(name != null);
	    System.err.println("`" + name + "' already exists");
	    System.exit(1);
	}
	catch(ClassCastException ex)
	{
	}

	try
	{
	    NotEmpty notEmpty = (NotEmpty)uex;

	    com.ooc.OB.Assert._OB_assert(name != null);
	    System.err.println("`" + name + "' is not empty");
	    System.exit(1);
	}
	catch(ClassCastException ex)
	{
	}
    }

    private static org.omg.CORBA.Object
    stringToObject(ORB orb, String str)
    {
	org.omg.CORBA.Object obj = null;

	try
	{
	    obj = orb.string_to_object(str);
	}
	catch(BAD_PARAM ex)
	{
	    System.err.println("Invalid object reference");
	    System.exit(1);
	}

	return obj;
    }

    private static NamingContext
    stringToNC(ORB orb, String str)
    {
	org.omg.CORBA.Object obj = stringToObject(orb, str);
	org.omg.CosNaming.NamingContext nc = null;

	try
	{
	    nc = NamingContextHelper.narrow(obj);
	}
        catch(BAD_PARAM ex)
        {
            System.err.println(
                "IOR does not denote a CosNaming::NamingContext");
            System.exit(1);
        }
	catch(SystemException ex)
	{
	    System.err.println("A system exception occurred while narrowing" +
			       " the given object reference\n" +
			       "to CosNaming::NamingContext:");
            System.err.println(ex.getMessage());
	    System.exit(1);
	}

	return nc;
    }

    private static void
    bind(ORB orb, NamingContextExt root, String[] args)
    {
	if(args.length != 3)
	{
	    usage();
	    System.exit(1);
	}

	String name = args[1];
	String ior = args[2];
	try
	{
	    NameComponent[] n = root.to_name(name);
	    org.omg.CORBA.Object obj = stringToObject(orb, ior);
	    root.bind(n, obj);
	}
	catch(UserException ex)
	{
	    printException(ex, root, name);
	}
    }

    private static void
    bindContext(ORB orb, NamingContextExt root, String[] args)
    {
	if(args.length != 3)
	{
	    usage();
	    return;
	}

	String name = args[1];
	String ior = args[2];
	try
	{
	    NameComponent[] n = root.to_name(name);
	    org.omg.CosNaming.NamingContext nc = stringToNC(orb, ior);
	    root.bind_context(n, nc);
	}
	catch(UserException ex)
	{
	    printException(ex, root, name);
	}
    }

    private static void
    bindNewContext(ORB orb, NamingContextExt root, String[] args)
    {
	if(args.length != 2)
	{
	    usage();
	    return;
	}

	String name = args[1];
	try
	{
	    NameComponent[] n = root.to_name(name);
	    NamingContext nc = root.bind_new_context(n);
	}
	catch(UserException ex)
	{
	    printException(ex, root, name);
	}
    }

    private static void
    destroy(ORB orb, NamingContextExt root, String[] args)
    {
	if(args.length != 2)
	{
	    usage();
	    return;
	}

	String name = args[1];
	try
	{
	    org.omg.CORBA.Object obj = root.resolve_str(name);
	    NamingContext nc = NamingContextHelper.narrow(obj);
	    nc.destroy();
	}
        catch(BAD_PARAM ex)
        {
            System.err.println("`" + name + "' is not a context");
            System.exit(1);
        }
        catch(OBJECT_NOT_EXIST ex)
        {
            System.err.println("Object reference for `" + name +
                               "' does not exist");
            System.exit(1);
        }
	catch(UserException ex)
	{
	    printException(ex, root, name);
	}
    }

    private static void
    list(ORB orb, NamingContextExt root, String[] args)
    {
	if(args.length > 2)
	{
	    usage();
	    return;
	}

	NamingContext nc = null;
	if(args.length == 2)
	{
	    String name = args[1];
	    try
	    {
		org.omg.CORBA.Object obj = root.resolve_str(name);
		try
		{
		    nc = NamingContextHelper.narrow(obj);
		}
		catch(BAD_PARAM ex)
		{
		    System.err.println("`" + name + "' is not a context");
		    System.exit(1);
		}
		catch(OBJECT_NOT_EXIST ex)
		{
		    System.err.println("Object reference for `" + name +
                                       "' does not exist");
		    System.exit(1);
		}
	    }
	    catch(UserException ex)
	    {
		printException(ex, root, name);
	    }
	}
	else
	    nc = root;

        BindingListHolder blH = new BindingListHolder();
        BindingIteratorHolder biH = new BindingIteratorHolder();
	nc.list(100, blH, biH);
	Binding[] bl = blH.value;
	BindingIterator bi = biH.value;
	if(bi != null)
	{
	    boolean b;
	    do
	    {
                BindingListHolder lH = new BindingListHolder();
		b = bi.next_n(100, lH);
		Binding[] l = lH.value;
		Binding[] blNew = new Binding[bl.length + l.length];
		System.arraycopy(bl, 0, blNew, 0, bl.length);
		System.arraycopy(l, 0, blNew, bl.length, l.length);
		bl = blNew;
	    } while(b);

	    bi.destroy();
	}

	if(bl.length == 0)
	    System.out.println("No bindings found");
	else
	{
	    System.out.println("Found " + bl.length + " binding" +
			       (bl.length != 1 ? "s" : "") + ":");
	    for(int i = 0 ; i < bl.length ; i++)
	    {
		String name = null;
		try
		{
		    name = root.to_string(bl[i].binding_name);
		}
		catch(InvalidName ex)
		{
		    throw new RuntimeException();
		}
		System.out.println(name + " " +
				   (bl[i].binding_type == BindingType.nobject
				    ? "[object]" : "[context]"));
	    }
	}
    }

    private static void
    rebind(ORB orb, NamingContextExt root, String[] args)
    {
	if(args.length != 3)
	{
	    usage();
	    return;
	}

	String name = args[1];
	String ior = args[2];
	try
	{
	    NameComponent[] n = root.to_name(name);
	    org.omg.CORBA.Object obj = stringToObject(orb, ior);
	    root.rebind(n, obj);
	}
	catch(UserException ex)
	{
	    printException(ex, root, name);
	}
    }

    private static void
    rebindContext(ORB orb, NamingContextExt root, String[] args)
    {
	if(args.length != 3)
	{
	    usage();
	    return;
	}

	String name = args[1];
	String ior = args[2];
	try
	{
	    NameComponent[] n = root.to_name(name);
	    NamingContext nc = stringToNC(orb, ior);
	    root.rebind_context(n, nc);
	}
	catch(UserException ex)
	{
	    printException(ex, root, name);
	}
    }

    private static void
    resolve(ORB orb, NamingContextExt root, String[] args)
    {
	if(args.length != 2)
	{
	    usage();
	    return;
	}

	String name = args[1];
	try
	{
	    org.omg.CORBA.Object obj = root.resolve_str(name);
	    if(obj == null)
                System.out.println("Binding has a nil object reference");
	    else
	    {
		String ref = orb.object_to_string(obj);
		System.out.println(ref);
	    }
	}
	catch(UserException ex)
	{
	    printException(ex, root, name);
	}
    }

    private static void
    unbind(ORB orb, NamingContextExt root, String[] args)
    {
	if(args.length != 2)
	{
	    usage();
	    return;
	}

	String name = args[1];
	try
	{
	    NameComponent[] n = root.to_name(name);
	    root.unbind(n);
	}
	catch(UserException ex)
	{
	    printException(ex, root, name);
	}
    }

    static int
    run(ORB orb, String[] args)
    {
	if(args.length == 0)
	{
	    usage();
	    return 1;
	}

	final int BIND = 0;
	final int BIND_CONTEXT = 1;
	final int BIND_NEW_CONTEXT = 2;
	final int DESTROY = 3;
	final int LIST = 4;
	final int REBIND = 5;
	final int REBIND_CONTEXT = 6;
	final int RESOLVE = 7;
	final int UNBIND = 8;
	int command = -1;

	//
	// Execute command
	//
	if(args[0].equals("-b") || args[0].equals("--bind"))
	{
	    command = BIND;
	}
	else if(args[0].equals("-C") || args[0].equals("--bind_context"))
	{
	    command = BIND_CONTEXT;
	}
	else if(args[0].equals("-c") ||
		args[0].equals("--bind_new_context"))
	{
	    command = BIND_NEW_CONTEXT;
            }
	else if(args[0].equals("-d") || args[0].equals("--destroy"))
	{
	    command = DESTROY;
	}
	else if(args[0].equals("-l") || args[0].equals("--list"))
	{
	    command = LIST;
	}
	else if(args[0].equals("-B") || args[0].equals("--rebind"))
	{
	    command = REBIND;
	}
	else if(args[0].equals("-R") || args[0].equals("--rebind_context"))
	{
	    command = REBIND_CONTEXT;
	}
	else if(args[0].equals("-r") || args[0].equals("--resolve"))
	{
	    command = RESOLVE;
	}
	else if(args[0].equals("-u") || args[0].equals("--unbind"))
	{
                command = UNBIND;
	}
	else
	{
	    usage();
	    return 1;
	}

	//
	// Resolve the naming service
	//
	org.omg.CORBA.Object obj = null;
	try
	{
	    obj = orb.resolve_initial_references("NameService");
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	    System.err.println(
		"Unable to resolve `NameService' initial reference");
	    return 1;
	}

	NamingContextExt root = null;
	try
	{
	    root = NamingContextExtHelper.narrow(obj);
	}
	catch(BAD_PARAM ex)
	{
	    System.err.println("`NameService' initial reference is not " +
			       "a CosNaming::NamingContextExt object.");
	    return 1;
	}

	//
	// Execute command
	//
	switch(command)
	{
	case BIND:
	    bind(orb, root, args);
	    break;
	case BIND_CONTEXT:
	    bindContext(orb, root, args);
	    break;
	case BIND_NEW_CONTEXT:
	    bindNewContext(orb, root, args);
	    break;
	case DESTROY:
	    destroy(orb, root, args);
	    break;
	case LIST:
	    list(orb, root, args);
	    break;
	case REBIND:
	    rebind(orb, root, args);
	    break;
	case REBIND_CONTEXT:
	    rebindContext(orb, root, args);
	    break;
	case RESOLVE:
	    resolve(orb, root, args);
	    break;
	case UNBIND:
	    unbind(orb, root, args);
	    break;
	}

	return 0;
    }

    public static void
    main(String[] args)
    {
	int status = 0;
	ORB orb = null;

        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

	try
	{
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);
	    status = run(orb, argsH.value);
	}
	catch(Exception ex)
	{
	    ex.printStackTrace();
	    status = 1;
	}

	if(orb != null)
	{
	    try
	    {
		orb.destroy();
	    }
	    catch(Exception ex)
	    {
		System.err.println(ex.getMessage());
		ex.printStackTrace();
		status = 1;
	    }
	}

	System.exit(status);
    }
} 
