// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNaming;

import java.io.*;
import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POAPackage.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import com.ooc.CosNaming.OBNamingContextPackage.*;
import com.ooc.CosNaming.Database.*;

//
// This class reads or writes the contents of a NamingContext to or from
// a persistent store.
//
final class NamingDatabase
{
    private POA userPOA_; // The USER_ID POA
    private POA systemPOA_; // The SYSTEM_ID POA
    private ORB orb_; // The ORB

    private String host_; // The host
    private short port_; // The port

    private String logFile_; // The name of the logfile.
    private String logFileNew_; // The name of the new logfile.
    private String logFileBack_; // The name of the backup logfile.

    private java.util.Hashtable ncs_; // The current set of naming contexts.
    private FileOutputStream os_; // The current output stream.
    private Compactor compactor_; // The database compactor

    Policy[] timeoutPolicy_; // Timeout for callbacks

    private boolean noUpdates_; // No automatic updates.
    private boolean initializing_; // Are we restoring the naming contexts?
    private boolean update_; // Do we need a journal update?
    private boolean noflush_; // Do we want to flush on each update?
    private boolean usingIMR_; // Are we using the IMR?

    private org.omg.IOP.Codec cdrCodec_; // The CDR Codec

    private com.ooc.OB.Logger logger_; // Tracing information
    private int maxBindings_;
    private int traceLevel_;

    // ----------------------------------------------------------------------
    // NamingDatabase private member implementation
    // ----------------------------------------------------------------------
 
    //
    // Dump the contents of a naming context. This is the set of bindings
    // in the context.
    //
    private void
    dumpContext(byte[] key, org.omg.CosNaming.NamingContext nc)
	throws UserException
    {
	//
	// Retrieve all bindings.
	//
	ExtendedBinding[] b =
	    ((com.ooc.CosNaming.OBNamingContext)nc).list_extended();

	for(int i = 0 ; i < b.length ; i++)
	    bind(key, b[i].binding_name, b[i].bound_obj, b[i].timestamp, false);
    }

    private void
    writeAny(Any any)
    {
	//
	// Serialize the Any
	//
	// TODO: we should cache the cdrCodec
	//
	byte[] data = null;
	try
	{
	    data = cdrCodec_.encode(any);
	}
	catch(org.omg.IOP.CodecPackage.InvalidTypeForEncoding ex)
	{
	    throw new RuntimeException();
	}

	com.ooc.OB.Assert._OB_assert(os_ != null);

	try
	{
            byte[] ascii =
                com.ooc.OB.HexConverter.octetsToAsciiBytes(data, data.length);

	    os_.write(ascii);
	    os_.write('\n');
	    if(!noflush_)
		os_.flush();
	}
	catch(IOException ex)
	{
            logger_.error("Write failed!");
            throw new PERSIST_STORE();
	}
    }

    //
    // Rename the file identified by `from' to the new name identified by
    // `to'.
    //
    private boolean
    doRename(String from, String to)
    {
	boolean result = (new File(from)).renameTo(new File(to));
	if(!result)
	{
	    logger_.error("Rename " + from + " to " + to + " failed.");

	    return false;
	}

	return true;
    }

    //
    // Check to see if the file identified by `file' exists.
    //
    private boolean
    exists(String file)
    {
	return (new File(file).exists());
    }

    //
    // Open the logfile.
    //
    private void
    openLog()
    {
	//
	// Open the logFile_.
	//
	try
	{
	    os_ = new FileOutputStream(logFile_, true);
	}
	catch(IOException ex)
	{
	    logger_.error("Cannot open `" + logFile_ + "': " + ex.toString());
	    throw new PERSIST_STORE();
	}
    }

    private Servant
    key_to_servant(byte[] key)
    {
        int idx = 0;
        byte[] rootKey = ("NameService").getBytes();
	if(key.length == rootKey.length)
	    for(; idx < key.length; ++idx)
	        if(key[idx] != rootKey[idx])
		    break;
	boolean isRoot = (idx == key.length);
        try
        {
            if(isRoot)
	        return userPOA_.id_to_servant(key);
	    else
	        return systemPOA_.id_to_servant(key);
        }
        catch(WrongPolicy ex)
        {
	    throw new RuntimeException();
        }
        catch(ObjectNotActive ex)
        {
	    throw new RuntimeException();
        }
    }

    //
    // Constructor
    //
    NamingDatabase(POA userPOA, POA systemPOA, ORB orb, String log,
    		   java.util.Hashtable ncs, Policy[] timeoutPolicy,
		   boolean noUpdates, boolean start, int timeout,
		   int maxBindings, int traceLevel)
    {
	userPOA_ = userPOA;
	systemPOA_ = systemPOA;
	orb_ = orb;
	ncs_ = ncs;
	timeoutPolicy_ = timeoutPolicy;
	noUpdates_ = noUpdates;
	initializing_ = false;
	update_ = false;
	noflush_ = false;
	maxBindings_ = maxBindings;
	traceLevel_= traceLevel;

	com.ooc.OBCORBA.ORB_impl oborb = (com.ooc.OBCORBA.ORB_impl)orb;
	logger_ = oborb.logger();

        //
	// Are we using the IMR
	//
	java.util.Properties properties = oborb.properties();
	String value = properties.getProperty("ooc.orb.server_name");
	if(value != null)
	    usingIMR_ = true;
	else
	    usingIMR_ = false;

	//
	// Grab a copy of the CDR Codec
	//
	org.omg.IOP.CodecFactory factory = null;
	try
	{
	    factory = org.omg.IOP.CodecFactoryHelper.narrow(
		orb.resolve_initial_references("CodecFactory"));
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	    throw new RuntimeException();
	}

	org.omg.IOP.Encoding how = new org.omg.IOP.Encoding(
	    (byte)org.omg.IOP.ENCODING_CDR_ENCAPS.value, (byte)1, (byte)2);

	try
	{
	    cdrCodec_ = factory.create_codec(how);
	}
	catch(org.omg.IOP.CodecFactoryPackage.UnknownEncoding ex)
	{
	    throw new RuntimeException();
	}

	//
	// Determine host and port.
	//
	POAManager manager = userPOA_.the_POAManager();
	com.ooc.OBPortableServer.POAManager obManager =
	    com.ooc.OBPortableServer.POAManagerHelper.narrow(manager);

	com.ooc.OCI.Acceptor[] acceptors = null;
	try
	{
	    acceptors = obManager.get_acceptors();
	}
	catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
	{
	    throw new RuntimeException();
	}

	for(int i = 0 ; i < acceptors.length ; i++)
	{
	    com.ooc.OCI.AcceptorInfo accInfo = acceptors[i].get_info();
	    com.ooc.OCI.IIOP.AcceptorInfo iiopInfo =
		com.ooc.OCI.IIOP.AcceptorInfoHelper.narrow(accInfo);
	    if(iiopInfo != null)
	    {
                String[] hosts = iiopInfo.hosts();
		host_ = hosts[0];
		port_ = iiopInfo.port();

		break;
	    }
	}

        //
        // Create the database compactor
        //
        compactor_ = new Compactor(orb_, this, timeout, traceLevel_);

	//
	// Setup the filenames.
	//
	logFile_ = log;
	logFileNew_ = log + ".new";
	logFileBack_ = log + ".bak";

	//
	// If we're running for the first time, then we need to verify
	// that the logfile is not already present.
	//
	if(start)
	{
	    if(exists(logFile_))
	    {
		logger_.error("Cannot specify --start option. " +
			      "There is an existing log file: `" +
			      logFile_ + "'.");
		throw new PERSIST_STORE();
	    }
	    openLog();
            compactor_.start();
	    return;
	}

	//
	// If the logfile doesn't exist, and the backup file does then
	// this is probably indicative of a crash.
	//
	if(!exists(logFile_) && exists(logFileBack_))
	{
	    logger_.warning("NamingDatabase file is not present: " +
			    logFile_ + ". Using backup file: " + logFileBack_);
	    
	    if(!doRename(logFileBack_, logFile_))
	    {
		logger_.warning("Rename of backup file failed.");
		throw new PERSIST_STORE();
	    }
	}
  
	//
	// Does the logfile exist?
	//
	if(!exists(logFile_))
	{
	    logger_.error("NamingDatabase file is not present: `" +
			  logFile_ + "'. Run with --start option if " +
			  "running for the first time.");
	    throw new PERSIST_STORE();
	}

	//
	// Ok, let's get down to business. Read the contents of the
	// logfile.  This is a set of strings, each of which is an octet
	// encoding of a marshaled any.
	//
	FileInputStream is = null;
	BufferedReader reader = null;
	try
	{
	    is = new FileInputStream(logFile_);
	    reader = new BufferedReader(new InputStreamReader(is));
	}
	catch(IOException ex)
	{
	    logger_.error("Cannot open store file: " + logFile_);
	    throw new PERSIST_STORE();
	}

	initializing_ = true;
	int rec = 0;
	String cmd;

	try
	{
	    try
	    {
		try
		{
		    while(true)
		    {
			cmd = reader.readLine();
			if(cmd == null)
			    break;

			//
			// Convert the octet character string to an
			// octet sequence and create input stream.
			//
			byte[] data =
                            com.ooc.OB.HexConverter.asciiToOctets(cmd);

			//
			// Unserialize the Any
			//
			Any any = null;
			try
			{
			    any = cdrCodec_.decode(data);
			}
			catch(org.omg.IOP.CodecPackage.FormatMismatch ex)
			{
			    throw new RuntimeException();
			}

			//
			// Next extract the instruction.
			//
			Version version;
			Info inf;
			Create create;
			Bind bind;
			Destroy destroy;
			Unbind unbind;

			TypeCode tc = any.type();
			if(tc.equal(VersionHelper.type()))
			{
			    // No action has to be taken here
			}
			else if(tc.equal(CreateHelper.type()))
			{
			    create = CreateHelper.extract(any);
			    
			    //
			    // Create a new naming context with the
			    // provided key.
			    //
			    new OBNamingContext_impl(userPOA_, systemPOA_,
			    			     orb_, this, ncs_,
						     create.key,
                                                     timeoutPolicy_,
						     noUpdates_,
						     maxBindings_,
						     traceLevel_);
			}
			else if(tc.equal(BindHelper.type()))
			{
			    bind = BindHelper.extract(any);

			    Servant servant = key_to_servant(bind.key);
			    OBNamingContext_impl nc =
				(OBNamingContext_impl)servant;

			    //
			    // Create a new binding.
			    //
			    if(bind.type.discriminator() ==
                               BindingType.ncontext)
			    {
				if(bind.rebind)
				    nc.rebind_context(bind.name,
						      bind.type.nc());
				else
				{
				    BindingUnion boundObj =
					new BindingUnion();
				    boundObj.nc(bind.type.nc());
				    nc._OB_bindWithTimestamp(bind.name,
							     boundObj,
							     bind.timestamp);
				}
			    }
			    else if(bind.rebind)
			    {
				nc.rebind(bind.name, bind.type.obj());
			    }
			    else
			    {
				BindingUnion boundObj = new BindingUnion();
				boundObj.obj(bind.type.obj());
				nc._OB_bindWithTimestamp(bind.name,
							 boundObj,
							 bind.timestamp);
			    }
			}
			else if(tc.equal(DestroyHelper.type()))
			{
			    destroy = DestroyHelper.extract(any);

			    Servant servant = key_to_servant(destroy.key);
			    OBNamingContext_impl nc =
				(OBNamingContext_impl)servant;

			    //
			    // Destroy a context.
			    //
			    nc.destroy();
			}
			else if(tc.equal(UnbindHelper.type()))
			{
			    unbind = UnbindHelper.extract(any);

			    Servant servant = key_to_servant(unbind.key);
			    OBNamingContext_impl nc =
				(OBNamingContext_impl)servant;

			    //
			    // Do an unbind.
			    //
			    nc.unbind(unbind.name);
			}
			else if(tc.equal(InfoHelper.type()))
			{
			    inf = InfoHelper.extract(any);

			    if(inf.imr && !usingIMR_)
			    {
			        logger_.error("Current database must be" +
				              " used with the Implementation" +
					      " Repository");
				throw new PERSIST_STORE();
			    }
			    else if(!inf.imr)
			    {
			        if(usingIMR_)
				{
				    logger_.error("Current database cannot" +
				                  " be used with the" +
						  " Implementation Repository");
				    throw new PERSIST_STORE();
			        }
			        else if(!host_.equals(inf.host) ||
				        port_ != inf.port)
			        {
				    logger_.error("Wrong host or port: " + 
				                  host_ + ":" + port_ +
					          ". Expected host and port " +
					          inf.host + ":" + inf.port);
				    throw new PERSIST_STORE();
			        }
			    }
			}
			else
			{
			    logger_.error("Unknown any in database.");
			    throw new PERSIST_STORE();
			}
			rec++;
		    }
		}
		catch(NotFound ex)
		{
		    String msg = "Restore failed: NotFound: ";
		    msg += "Reason: ";
		    if(ex.why == NotFoundReason.missing_node)
			msg += "missing_node";
		    else if(ex.why == NotFoundReason.not_context)
			msg += "not_context";
		    else
			msg += "not_object";
		    msg += ". Rest of name: ";
		    for(int i = 0 ; i < ex.rest_of_name.length ; i++)
		    {
			msg += "(" + ex.rest_of_name[i].id + "." +
			    ex.rest_of_name[i].kind + ") ";
		    }
		    logger_.error(msg);
		    throw new PERSIST_STORE();
		}
		catch(CannotProceed ex)
		{
		    String msg = "Restore failed: CannotProceed";
		    msg += ". Rest of name: ";
		    for(int i = 0 ; i < ex.rest_of_name.length ; i++)
			msg += "(" + ex.rest_of_name[i].id + "." +
					 ex.rest_of_name[i].kind + ") ";
		    logger_.error(msg);
		    throw new PERSIST_STORE();
		}
		catch(InvalidName ex)
		{
		    logger_.error("Restore failed: InvalidName");
		    throw new PERSIST_STORE();
		}
		catch(AlreadyBound ex)
		{
		    logger_.error("Restore failed: AlreadyBound");
		    throw new PERSIST_STORE();
		}
		catch(NotEmpty ex)
		{
		    logger_.error("Restore failed: NotEmpty");
		    throw new PERSIST_STORE();
		}
	    }
	    catch(PERSIST_STORE ex)
	    {
		logger_.error("Error processing record: " + rec);
		is.close();
		throw ex;
	    }
	}
	catch(IOException ex)
	{
	}

	try
	{
	    is.close();
	}
	catch(IOException ex)
	{
	}

	initializing_ = false;

	//
	// Re-open the logfile for new journal entries.
	//
	openLog();
	update_ = true;

	//
	// Start Compactor
	//
	compactor_.start();
    }

    protected void
    finalize()
        throws Throwable
    {
        compactor_.halt();
	
	try
	{
	    os_.close();
	}
	catch(IOException ex)
	{
	}
        super.finalize();
    }

    synchronized public void
    updateJournal()
    {
	if(!update_)
	    return;

	try
	{
	    //
	    // Close old journal.
	    //
	    os_.close();

	    os_ = new FileOutputStream(logFileNew_);
	}
	catch(IOException ex)
	{
	    logger_.error("Cannot open `" + logFileNew_ + "': " +
			  ex.toString());
	    throw new PERSIST_STORE();
	}

	//
	// Write
	//
	noflush_ = true;

	//
	// Write version, host and port
	//
	info();

	//
	// We write the entire contents of our local naming tree to the
	// journal.  We do this in two steps. First we write the create
	// instructions.  Next we write the bindings. It's important that
	// this order is preserved else we can have a binding to a local
	// naming context *before* that context exists.
	//
	try
	{
	    try
	    {
		synchronized(ncs_)
		{
		    java.util.Enumeration keys = ncs_.keys();
		    while(keys.hasMoreElements())
		    {
			com.ooc.OB.ObjectIdHasher key =
			    (com.ooc.OB.ObjectIdHasher)
			        keys.nextElement();
			create(key.getObjectId(), 0);
		    }

		    keys = ncs_.keys();
		    java.util.Enumeration values = ncs_.elements();
		    while(keys.hasMoreElements())
		    {
			com.ooc.OB.ObjectIdHasher key =
			    (com.ooc.OB.ObjectIdHasher)
			        keys.nextElement();

			org.omg.CosNaming.NamingContext nc =
			    (org.omg.CosNaming.NamingContext)
			        values.nextElement();
			try
			{
			    dumpContext(key.getObjectId(), nc);
			}
			catch(org.omg.CORBA.UserException ex)
			{
			}
			catch(OBJECT_NOT_EXIST ex)
			{
			    //
			    // Ignore this. If this occurs then the
			    // context is in the middle of being
			    // destroyed and the destroy record will
			    // be added to the store once we've
			    // finished dumping the journal.
			    //
			}
		    }
		}
	    }
	    catch(SystemException ex)
	    {
		logger_.error("Compact failed: SystemException");
		ex.printStackTrace();
		throw new PERSIST_STORE();
	    }
	}
	catch(PERSIST_STORE ex)
	{
	    noflush_ = false;
	    try
	    {
		os_.close();
	    }
	    catch(IOException e)
	    {
	    }

	    //
	    // Re-open old log file.
	    //
	    openLog();

	    throw ex;
	}

	try
	{
	    os_.close();
	}
	catch(IOException e)
	{
	}

	noflush_ = false;

	//
	// Move the current to backup. If this fails, it's not a big deal.
	//
	doRename(logFile_, logFileBack_);
 
	//
	// Move the new logfile to current. This failing IS a big deal!
	//
	if(!doRename(logFileNew_, logFile_))
	    throw new PERSIST_STORE();
 
	(new File(logFileBack_)).delete();

	//
	// Open new.
	//
	openLog();
	update_ = false;
    }

    //
    // Add a create instruction to the journal.
    //
    synchronized public void
    create(byte[] oid, long timestamp)
    {
	if(initializing_)
	    return;

	Create create = new Create(oid, (int)timestamp);

	Any any = orb_.create_any();
	CreateHelper.insert(any, create);

	writeAny(any);

	update_ = true;
    }

    //
    // Add a destroy instruction to the journal.
    //
    synchronized public void
    destroy(byte[] oid)
    {
	if(initializing_)
	    return;
 
	Destroy destroy = new Destroy(oid);
	Any any = orb_.create_any();
	DestroyHelper.insert(any, destroy);
	writeAny(any);
 
	update_ = true;
    }

    //
    // Add a bind instruction to the journal.
    //
    synchronized public void
    bind(byte[] oid, NameComponent[] name, BindingUnion boundObj,
         long timestamp, boolean rebind)
    {
	if(initializing_)
	    return;

	Bind bind = new Bind(rebind, oid, name, boundObj, (int)timestamp);
	Any any = orb_.create_any();
	BindHelper.insert(any, bind);
	writeAny(any);
 
	update_ = true;
    }

    //
    // Add an unbind instruction to the journal.
    //
    synchronized public void
    unbind(byte[] oid, NameComponent[] name)
    {
	if(initializing_)
	    return;
 
	Unbind unbind = new Unbind(oid, name);
	Any any = orb_.create_any();
	UnbindHelper.insert(any, unbind);
	writeAny(any);
 
	update_ = true;
    }

    //
    // Add host and port to the journal.
    //
    synchronized public void
    info()
    {
	if(initializing_)
	    return;
 
	Any any = orb_.create_any();

	//
	// Write version.
	//
	Version version = new Version(com.ooc.OB.Version.version);
	VersionHelper.insert(any, version);
	writeAny(any);

	//
	// Write host and port.
	//
	Info inf = new Info(host_, port_, usingIMR_);
	InfoHelper.insert(any, inf);
	writeAny(any);
 
	update_ = true;
    }
}
