// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNaming;

import org.omg.CORBA.*;
import java.util.Hashtable;

public class NamingService
{
    //
    // My ORB
    //
    private ORB orb_;

    //
    // My root context POA
    //
    private org.omg.PortableServer.POA rootContextPOA_;

    //
    // My context POA
    //
    private org.omg.PortableServer.POA contextPOA_;

    //
    // My set of naming contexts
    //
    Hashtable ncs_;

    //
    // My naming service database
    //
    private NamingDatabase store_;

    //
    // My root naming context
    //
    private org.omg.CosNaming.NamingContext root_;

    //
    // The naming service options
    //
    private NamingServiceOptions options_;

    //
    // Are we initialized?
    //
    private boolean initialized_;

    // ------------------------------------------------------------------
    // NamingService constructor
    // ------------------------------------------------------------------

    public NamingService(ORB orb, org.omg.PortableServer.POA rootContextPOA,
    			 org.omg.PortableServer.POA contextPOA,
			 NamingServiceOptions options)
    {
	orb_ = orb;
	rootContextPOA_ = rootContextPOA;
	contextPOA_ = contextPOA;
	options_ = options;
	initialized_ = false;
    }

    // ------------------------------------------------------------------
    // NamingService public member implementation
    // ------------------------------------------------------------------

    //
    // Shutdown the naming service
    //
    public void shutdown()
    {
	//
	// Release the root naming context
	//
	root_ = null;

	//
	// Clean up all contexts
	//
	ncs_.clear();
	ncs_ = null;

	//
	// Release the naming service database
	//
	store_ = null;
    }

    //
    // Initialize the naming service
    //
    public void initialize()
    {
	if(initialized_)
	    return;

	initialized_ = true;

	//
	// Create the set of naming contexts
	//
	ncs_ = new Hashtable(63);

	//
	// Create the timeout policy
	//
	Any timeoutAny = orb_.create_any();
	timeoutAny.insert_ulong(1000 * options_.callbackTimeout);
	Policy[] timeoutPolicy = new Policy[1];
	try
	{
	    timeoutPolicy[0] = ((com.ooc.CORBA.ORB)orb_).
		create_policy(com.ooc.OB.TIMEOUT_POLICY_ID.value, timeoutAny);
	}
	catch(org.omg.CORBA.PolicyError ex)
	{
	    throw new RuntimeException();
	}

	if(options_.databaseName != null)
	{
	    //
	    // Create a new naming service database
	    //
	    store_ = new NamingDatabase(rootContextPOA_, contextPOA_, orb_,
	    				options_.databaseName, ncs_,
					timeoutPolicy, options_.noUpdates,
					!options_.databaseExists,
					60 * options_.databaseTimeout,
					options_.maxBindings,
					options_.traceLevel);

	    //
	    // Add version, host and port to the journal
	    //
	    store_.info();
	}

	//
	// Get the root naming context
	//
	byte[] nsObjId = ("NameService").getBytes();
	if(!options_.databaseExists || options_.databaseName == null)
	{
	    com.ooc.CosNaming.OBNamingContext_impl root =
		new com.ooc.CosNaming.OBNamingContext_impl(
		    rootContextPOA_, contextPOA_, orb_, store_, ncs_,
		    nsObjId, timeoutPolicy, options_.noUpdates,
		    options_.maxBindings, options_.traceLevel);
	
	    root_ = root._this();
	}
	else
	{
	    com.ooc.OB.ObjectIdHasher key = 
		new com.ooc.OB.ObjectIdHasher(nsObjId);
	    root_ = (org.omg.CosNaming.NamingContext) ncs_.get(key);
	}
    }

    //
    // Get the root naming context
    //
    public org.omg.CosNaming.NamingContext
    rootNamingContext()
    {
	return root_;
    }
}
