// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNaming;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import java.io.*;

//
// NameService Server class
//
final public class Server
{
    static void
    usage(String progName)
    {
        System.err.println("Usage:");
        System.err.println(progName + " [options]\n" +
                           "\n" +
                           "Options:\n" +
                           "-h, --help                   " +
                           "Show this message.\n" +
                           "-v, --version                " +
                           "Show Naming Service version.\n" +
                           "-i, --ior                    " +
                           "Print IOR on standard output.\n" +
			   "-n, --no-updates             " +
			   "No automatic updates with ORBacus Names.\n" +
			   "-s, --start                  " +
			   "Run for the first time.\n" +
			   "-d, --database FILE          " +
			   "Use FILE as the database.\n" +
			   "-l, --limit COUNT            " +
			   "Limit bindings returned by list operation.\n" +
			   "-t, --timeout MINS           " +
			   "Purge the database every MINS minutes.\n" +
			   "                             " +
                           "(The default is 5 minutes.)\n" +
			   "-c, --callback-timeout SECS  " +
			   "Timeout for callbacks in seconds.\n" +
			   "                             " +
			   "(The default is 5 seconds.)");
    }

    static int
    run(org.omg.CORBA.ORB orb, String progName, String[] args,
	java.util.Properties props)
    {
	//
	// Get options
	//
	boolean ior = false;
	NamingServiceOptions options = new NamingServiceOptions();

	//
	// Check properties first
	//
	String propRoot = "ooc.naming.";
	java.util.Enumeration keys = props.propertyNames();
	while(keys.hasMoreElements())
	{
	    String key = (String)keys.nextElement();
	    if(!key.startsWith(propRoot))
		continue;
	    
	    String value = props.getProperty(key);

	    if(key.equals("ooc.naming.database"))
	    {
	        options.databaseName = value;
	    }
	    else if(key.equals("ooc.naming.limit"))
	    {
		try
		{
		    options.maxBindings = Integer.parseInt(value);
		}
		catch(NumberFormatException ex)
		{
		    System.err.println("Invalid value for ooc.naming.limit");
		    return 1;
		}
	    }
	    else if(key.equals("ooc.naming.timeout"))
	    {
		try
		{
		    options.databaseTimeout = Integer.parseInt(value);
		}
		catch(NumberFormatException ex)
		{
		    System.err.println("Invalid value for ooc.naming.timeout");
		    return 1;
		}
	    }
	    else if(key.equals("ooc.naming.callback_timeout"))
	    {
		try
		{
		    options.callbackTimeout = Integer.parseInt(value);
		}
		catch(NumberFormatException ex)
		{
		    System.err.println(
			"Invalid value for ooc.naming.callback_timeout");
		    return 1;
		}
	    }
	    else if(key.equals("ooc.naming.no_updates"))
	    {
	        options.noUpdates = value.equals("true") ||
	    	    value.equals("TRUE") || value.equals("1");
	    }
	    else if(key.equals("ooc.naming.trace_level"))
	    {
	        options.traceLevel = Integer.parseInt(value);
	    }
	    else if(key.equals("ooc.naming.endpoint"))
	    {
		//
		// Set the ooc.orb.oa.endpoint property from this value
		//
		String oaEndpoint = props.getProperty("ooc.orb.oa.endpoint");
		if(oaEndpoint == null)
		    props.put("ooc.orb.oa.endpoint", value);
	    }
	    else
	    {
		System.err.println(progName + ": `" + key + "': " +
				   "unknown property");
	    }
	}

	//
	// Resolve Root POA
	//
	org.omg.CORBA.Object poaObj = null;
	try
	{
	    poaObj = orb.resolve_initial_references("RootPOA");
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	    throw new RuntimeException();
	}
	POA rootPOA = POAHelper.narrow(poaObj);

        //
        // Get a reference to the POA manager
        //
        POAManager manager = rootPOA.the_POAManager();

	//
	// The persistent poa
	//
	POA rootContextPOA = null;
	POA contextPOA = null;
	try
	{
	    Policy[] policies = new org.omg.CORBA.Policy[2];
	    policies[0] = 
		rootPOA.create_lifespan_policy(
		    org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);
	    policies[1] = 
		rootPOA.create_id_assignment_policy(
		    org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);
	    rootContextPOA = 
		rootPOA.create_POA("RootContextPOA", manager, policies);

	    policies[1] =
	        rootPOA.create_id_assignment_policy(
		    org.omg.PortableServer.IdAssignmentPolicyValue.SYSTEM_ID);
	    contextPOA = rootPOA.create_POA("ContextPOA", manager, policies);
	}
	catch(org.omg.PortableServer.POAPackage.AdapterAlreadyExists ex)
	{
	}
	catch(org.omg.PortableServer.POAPackage.InvalidPolicy ex)
	{
	}
	com.ooc.OB.Assert._OB_assert(rootContextPOA != null);
	com.ooc.OB.Assert._OB_assert(contextPOA != null);

        for(int i = 0 ; i < args.length && args[i].startsWith("-") ; i++)
        {
            if(args[i].equals("--help") || args[i].equals("-h"))
            {
                usage(progName);
                return 0;
            }
            else if(args[i].equals("--version") || args[i].equals("-v"))
            {
                System.err.println("ORBacus Names" +
                                   com.ooc.OB.Version.version);
                return 0;
            }
            else if(args[i].equals("--ior") || args[i].equals("-i"))
            {
                ior = true;
            }
	    else if(args[i].equals("--no-updates") || args[i].equals("-n"))
	    {
	        options.noUpdates = true;
	    }
	    else if(args[i].equals("--limit") || args[i].equals("-l"))
            {
                ++i;
	        if(i >= args.length)
	        {
	    	    System.err.println(progName + ": " + args[i - 1] +
	    			       "expects argument");
	    	    usage(progName);
	    	    return 1;
	        }
		options.maxBindings = Integer.parseInt(args[i]);
	    }
	    else if(args[i].equals("--timeout") || args[i].equals("-t"))
            {
                ++i;
	        if(i >= args.length)
	        {
	    	    System.err.println(progName + ": " + args[i - 1] +
	    			       "expects argument");
	    	    usage(progName);
	    	    return 1;
	        }
    
		options.databaseTimeout = Integer.parseInt(args[i]);
            }
	    else if(args[i].equals("--callback-timeout") ||
                    args[i].equals("-c"))
            {
                ++i;
		if(i >= args.length)
		{
		    System.err.println(progName + ": " + args[i - 1] +
				       "expects argument");
		    usage(progName);
		    return 1;
		}

		options.callbackTimeout = Integer.parseInt(args[i]);
	    }
	    else if(args[i].equals("--database") ||
		    args[i].equals("-d"))
	    {
		++i;
		if(i >= args.length)
		{
		    System.err.println(progName + ": " + args[i - 1] +
				       "expects argument");
		    usage(progName);
		    return 1;
		}

	        options.databaseName = args[i];
	    }
	    else if(args[i].equals("--start") || args[i].equals("-s"))
            {
	        options.databaseExists = false;
	    }
	    else
	    {
	        System.err.println(progName + ": unknown option `"
	    		           + args[i] + "'\n");
	        usage(progName);
	        return 1;
	    }
        }

	if(options.databaseTimeout <= 0)
	{
	    System.err.println(progName + ": timeout value must be > 0");
	    return 1;
	}

	if(options.maxBindings < 0)
	{
	    System.err.println(progName + ": binding limit must be >= 0");
	    return 1;
	}

	if(!options.databaseExists && options.databaseName == null)
	{
	    System.err.println(progName + ": --start (-s) requires the " +
			       "--database (-d) option");
	    return 1;
	}

	//
	// Initialize the naming service.
	//
	NamingService namingService = new NamingService(orb, rootContextPOA,
							contextPOA, options);
	namingService.initialize();

	//
	// Get the root naming context.
	//
	org.omg.CosNaming.NamingContext root =
	    namingService.rootNamingContext();

	//
	// Add reference to boot manager
	//
	try
	{
	    byte[] oid = ("NameService").getBytes();
	    com.ooc.OB.BootManager bootManager = 
		com.ooc.OB.BootManagerHelper.narrow(
		    orb.resolve_initial_references("BootManager"));
	    bootManager.add_binding(oid, root);
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	    throw new RuntimeException();
	}
	catch(com.ooc.OB.BootManagerPackage.AlreadyExists ex)
	{
	    throw new RuntimeException();
	}

	//
	// Print IOR on standard output
	//
	if(ior)
	{
	    System.out.println(orb.object_to_string(root));
	}

        //
        // Run implementation
        //
        try
        {
            manager.activate();
        }
        catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
        {
            throw new RuntimeException();
        }
        orb.run();

	return 0;
    }

    public static void
    main(String args[])
    {
	java.util.Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");

	int status = 0;
	org.omg.CORBA.ORB orb = null;
	ShutdownHook shutdownHook_ = null;

	//
	// Set the program name
	//
	String progName = "com.ooc.CosNaming.Server";

        try
        {
	    args = com.ooc.CORBA.ORB.ParseArgs(args, props, null);
	    
	    //
	    // Check to see if the concurrency model is usable for nested
	    // method invocations. Threaded/thread_per_request/thread_pool
	    // is needed.
	    //
            String oaModel = props.getProperty("ooc.orb.oa.conc_model");
            String orbModel = props.getProperty("ooc.orb.conc_model");
	    boolean valid;
	    if (oaModel != null && orbModel != null)
	    {
                valid = (orbModel.equals("threaded") &&
			 (oaModel.equals("thread_per_request") ||
			  oaModel.equals("thread_pool")));
	    }
	    else
	        valid = false;

            //
            // Set the concurrency model to threaded/threaded if a valid
            // concurrency model wasn't selected
            //
            if(!valid)
            {
	        props.put("ooc.orb.conc_model", "threaded");
	        props.put("ooc.orb.oa.conc_model", "thread_per_request");
            }

            //
            // Create ORB
            //
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);

	    //
	    // Create and register ORB shutdown hook for JVM exit handling
	    //
	    shutdownHook_ = new ShutdownHook(orb);
	    shutdownHook_.register();

	    //
	    // Run the service
	    //
	    status = run(orb, progName, argsH.value, props);
 	}
	catch(Exception ex)
        {
            ex.printStackTrace();
	    status = 1;
        }

	if(orb != null)
	{
	    try
	    {
		orb.destroy();
	    }
	    catch(Exception ex)
	    {
		ex.printStackTrace();
		status = 1;
	    }
	}

	System.exit(status);
    }
}
