// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNamingConsole;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import com.ooc.Util.*;
import com.ooc.Util.GUI.*;
import com.ooc.Util.CORBA.*;
import com.ooc.Util.CORBA.GUI.*;

final public class Main
{
    //
    // NOTE: If you want to use a locale-specific resource bundle,
    //       you do NOT need to change the value for bundle_
    //
    static final String bundle_ =
        "com.ooc.CosNamingConsole.Util.AppResources";
 
    //
    // The official name for the initial reference for the Naming Service
    //
    static final String serviceName_ = "NameService";

    static int
    run(ORB orb, String[] args)
    {
        GUIOptionFilter opt;
        BooleanOption helpOption;
	BooleanOption versionOption;
	BooleanOption noUpdatesOption;
        StringOption fileOption;

        opt = new GUIOptionFilter();
        helpOption = new BooleanOption('h', "help");
	versionOption = new BooleanOption('v', "version");
	noUpdatesOption = new BooleanOption('n', "no-updates");
        fileOption = new StringOption('f', "file");
        opt.addOption(helpOption);
	opt.addOption(versionOption);
	opt.addOption(noUpdatesOption);
        opt.addOption(fileOption);

        //
        // Initialize the AppStandards class, which loads our locale-
        // specific resource bundle
        //
        AppStandards.init(bundle_);

        //
        // Process the command-line arguments
        //
        try
        {
            args = opt.parse(args);
        }
        catch(OptionException ex)
        {
            System.err.println("Error: " + ex.getMessage());
            System.err.println();
            usage();
        }

	//
	// Any remaining options? If there are, they are unknown
	//
	if(args.length != 0)
	    usage();

	if(versionOption.isDefined())
	{
	    System.out.println("ORBacus Names Console " +
			       com.ooc.OB.Version.version);
	    return 0;
	}
        else if(helpOption.isDefined())
            usage();

        //
        // Resolve Root POA
        //
        org.omg.CORBA.Object poaObj = null;

        try
        {
            poaObj = orb.resolve_initial_references("RootPOA");
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName ex)
        {
            throw new RuntimeException();
        }

        org.omg.PortableServer.POA rootPOA =
            org.omg.PortableServer.POAHelper.narrow(poaObj);

        //
        // Get a reference to the POA manager
        //
        org.omg.PortableServer.POAManager manager =
            rootPOA.the_POAManager();

        //
        // Set default retry policy to true
        //
        Any retryAny = orb.create_any();
        retryAny.insert_long(com.ooc.OB.RETRY_ALWAYS.value);
        Any timeoutAny = orb.create_any();
        timeoutAny.insert_long(5000);
        Policy[] policies = new Policy[2];

        try
        {
	    policies[0] = ((com.ooc.CORBA.ORB)orb).
	        create_policy(com.ooc.OB.RETRY_POLICY_ID.value, retryAny);
	    policies[1] = ((com.ooc.CORBA.ORB)orb).
	        create_policy(com.ooc.OB.TIMEOUT_POLICY_ID.value, timeoutAny);
    
	    PolicyManager pm = PolicyManagerHelper.narrow(
	        orb.resolve_initial_references("ORBPolicyManager"));

	    pm.add_policy_overrides(policies);
        }
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	    // Do nothing if anything fails
        }
        catch(org.omg.CORBA.PolicyManagerPackage.InvalidPolicies ex)
        {
	    // Do nothing if anything fails
        }
        catch(PolicyError ex)
        {
	    // Do nothing if anything fails
        }

        //
        // Get naming service with the following priorities:
        // 1. If a file was specified, read the IOR from this file.
        // 2. Try to resolve the NameService initial reference.
        //
        org.omg.CORBA.Object namingObj = null;

        if(fileOption.isDefined())
        {
	    try
	    {
	        FileReader r = new FileReader(fileOption.getValue());
	        BufferedReader in = new BufferedReader(r);
	        String ref = in.readLine();
	        r.close();
    
	        namingObj = orb.string_to_object(ref);
	    }
	    catch(IOException e)
	    {
	        e.printStackTrace();
	        return 1;
	    }
        }
        else
        {
	    //
	    // Check for initial service
	    //
	    try
	    {
	        namingObj = orb.resolve_initial_references("NameService");
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
	    }
        }

        boolean exists = true;

        //
        // Check if service is accessible
        //
        if(namingObj != null)
        {
	    try
	    {
	        exists = !namingObj._non_existent();
	    }
	    catch(COMM_FAILURE ex)
	    {
	        exists = false;
	    }
        }
        else
	    exists = false;

        if(!exists)
        {
	    MessageDialog.showError(null, "Couldn't access Naming Service.");
	    return 1;
        }

        ClientWindow.setNoUpdates(noUpdatesOption.isDefined());
        ClientWindow clientWindow = new ClientWindow(
    	    new Point(100, 100), orb, namingObj);

        try
        {
            manager.activate();
        }
        catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
        {
            throw new RuntimeException();
        }

         orb.run();

	 return 0;
    }

    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        int status = 0;
        ORB orb = null;

        try
        {
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = ORBManager.init(argsH, props);
            status = run(orb, argsH.value);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                System.err.println(ex.getMessage());
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }

    private static void
    showError(String errorMessage)
    {
        JFrame frame = new JFrame("");
        java.lang.Object[] options = { "OK" };
        JOptionPane.showOptionDialog(null, errorMessage, "Warning",
            JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, null,
            options, options[0]);
        System.exit(1);
    }

    private static void
    usage()
    {
	System.out.println(AppStandards.getString("UsageKey"));
        System.exit(1);
    }
}
