// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.CosNaming.*;

public class Client
{
    static int
    run(ORB orb, String args[])
        throws UserException
    {
	org.omg.CORBA.Object obj = null;

	//
	// Resolve Root POA
	//
	POA root =
            POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
	POAManager poaManager = root.the_POAManager();

        //
        // Initialize the server so that incoming requests are
        // handled from this point on
        //
        poaManager.activate();
 
        //
        // Get naming service
        //
        NamingContextExt nc = null;
        try
        {
            nc = NamingContextExtHelper.narrow(
                orb.resolve_initial_references("NameService"));
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName ex)
        {
            System.err.println("Can't resolve `NameService'");
            return 1;
        }
        catch(BAD_PARAM ex)
        {
            System.err.println("`NameService' is not " +
                               "a NamingContextExt object reference");
            return 1;
        }

	if(nc == null)
        {
            System.err.println("`NameService' is a nil object reference");
            return 1;
        }
 
        System.out.print("Testing bind()... ");
        TestBind.test(orb, nc);
        System.out.println("Done!");

        System.out.print("Testing rebind()... ");
        TestRebind.test(orb, nc);
        System.out.println("Done!");

        System.out.print("Testing bind_context()... ");
        TestBindContext.test(orb, nc);
        System.out.println("Done!");

        System.out.print("Testing rebind_context()... ");
        TestRebindContext.test(orb, nc);
        System.out.println("Done!");

        System.out.print("Testing resolve()... ");
        TestResolve.test(orb, nc);
        System.out.println("Done!");

        System.out.print("Testing unbind()... ");
        TestUnbind.test(orb, nc);
        System.out.println("Done!");

        System.out.print("Testing destroy()... ");
        TestDestroy.test(orb, nc);
        System.out.println("Done!");

        System.out.print("Testing list()... ");
        TestList.test(orb, nc);
        System.out.println("Done!");

        System.out.print("Testing list_extended()... ");
        TestListExtended.test(orb, nc);
        System.out.println("Done!");

        System.out.print("Testing rename()... ");
        TestRename.test(orb, nc);
        System.out.println("Done!");

        System.out.print("Testing callbacks... ");
        TestCallbacks.test(orb, nc);
        System.out.println("Done!");

        System.out.print("Testing to_string()... ");
        TestToString.test(orb, nc);
        System.out.println("Done!");

        System.out.print("Testing to_name()... ");
        TestToName.test(orb, nc);
        System.out.println("Done!");

        System.out.print("Testing to_url()... ");
        TestToURL.test(orb, nc);
        System.out.println("Done!");

        System.out.print("Testing resolve_str()... ");
        TestResolveStr.test(orb, nc);
        System.out.println("Done!");

        return 0;
    }

    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        int status = 0;
        ORB orb = null;

        try
        {
            orb = ORB.init(args, props);
            status = run(orb, args);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
