#!/bin/sh
# **********************************************************************
#
# Copyright (c) 2002
# IONA Technologies, Inc.
# Waltham, MA, USA
#
# All Rights Reserved
#
# **********************************************************************

#
# Function to set Java boot classpath if JDK 1.4 is being used
#
set_bootclasspath()
{
    java_version=`$JAVA -version 2>&1`
    if `$echo $java_version | grep '1\.4' > /dev/null`
    then
        JAVA="$JAVA -Xbootclasspath/p:$CLASSPATH"
    fi
}

#
# Function to check for Java distribution and set the Java CLASSPATH.
#
check_for_CPP()
{
    have_java=
    if test -n "$OB_CPP_DIST" -a \
	    -r "$OB_CPP_DIST/ob/test/multi/server${exe}" -a \
	    -r "$OB_CPP_DIST/ob/test/multi/client${exe}"
    then
	have_cpp="yes"
    fi

    CPP_TEST_DIR="$OB_CPP_DIST/ob/test"
}

#
# Messages for the non-availability of Java and set the
# clients and servers for auto testing.
#
set_auto_values_and_msgs()
{
    auto_clients="3"
    auto_servers="6 7 8 9"

    cpp_not_available=
    if test -z "$have_cpp"
    then
	cpp_not_available=" (not available)"
    else
	auto_clients="1 2 $auto_clients"
	auto_servers="1 2 3 4 5 $auto_servers"
    fi
}

#
# Function to set the client executable command
#
set_client()
{
    num=$1;
    cpp_dir=$2;
    java_pkg=$3;

    #
    # If a separate java package is not specified, create one based on
    # $cpp_dir
    #
    if test -z "$java_pkg"
    then
        java_pkg="test.$cpp_dir"
    fi

    if test "$num" = "1" -a -z "$cpp_not_available"
    then
	client="$CPP_TEST_DIR/$cpp_dir/client${exe} -ORBreactive"
    fi

    if test "$num" = "2" -a -z "$cpp_not_available"
    then
        client="$CPP_TEST_DIR/$cpp_dir/client${exe} -ORBthreaded"
    fi

    if test "$num" = "3"
    then
	client="$JAVA $java_pkg.Client -ORBthreaded"
    fi
}

#
# Function to set the server executable command
#
set_server()
{
    num=$1;
    cpp_dir=$2;
    java_pkg=$3;

    #
    # If a separate java package is not specified, create one based on
    # $cpp_dir
    #
    if test -z "$java_pkg"
    then
        java_pkg="test.$cpp_dir"
    fi

    if test "$num" = "1" -a -z "$cpp_not_available"
    then
	server="$CPP_TEST_DIR/$cpp_dir/server${exe} -OAreactive"
    fi

    if test "$num" = "2" -a -z "$cpp_not_available"
    then
	server="$CPP_TEST_DIR/$cpp_dir/server${exe} -OAthreaded"
    fi

    if test "$num" = "3" -a -z "$cpp_not_available"
    then
	server="$CPP_TEST_DIR/$cpp_dir/server${exe} -OAthread_per_client"
    fi

    if test "$num" = "4" -a -z "$cpp_not_available"
    then
	server="$CPP_TEST_DIR/$cpp_dir/server${exe} -OAthread_per_request"
    fi

    if test "$num" = "5" -a -z "$cpp_not_available"
    then
	server="$CPP_TEST_DIR/$cpp_dir/server${exe} -OAthread_pool 10"
    fi

    if test "$num" = "6"
    then
	server="$JAVA $java_pkg.Server -OAthreaded"
    fi

    if test "$num" = "7"
    then
	server="$JAVA $java_pkg.Server -OAthread_per_client"
    fi

    if test "$num" = "8"
    then
	server="$JAVA $java_pkg.Server -OAthread_per_request"
    fi

    if test "$num" = "9"
    then
	server="$JAVA $java_pkg.Server -OAthread_pool 10"
    fi
    server="$server -IIOPhost localhost"
}

#
# Function to set the collocated server executable command
#
set_colloc()
{
    num=$1;
    cpp_dir=$2;
    java_pkg=$3;

    #
    # If a separate java package is not specified, create one based on
    # $cpp_dir
    #
    if test -z "$java_pkg"
    then
        java_pkg="test.$cpp_dir"
    fi

    if test "$server_num" = "1" -a -z "$cpp_not_available"
    then
	colloc="$CPP_TEST_DIR/$cpp_dir/collocated${exe} -OAreactive"
    fi

    if test "$server_num" = "2" -a -z "$cpp_not_available"
    then
	colloc="$CPP_TEST_DIR/$cpp_dir/collocated${exe} -OAthreaded"
    fi

    if test "$server_num" = "3" -a -z "$cpp_not_available"
    then
	colloc="$CPP_TEST_DIR/$cpp_dir/collocated${exe} -OAthread_per_client"
    fi

    if test "$server_num" = "4" -a -z "$cpp_not_available"
    then
	colloc="$CPP_TEST_DIR/$cpp_dir/collocated${exe} -OAthread_per_request"
    fi

    if test "$server_num" = "5" -a -z "$cpp_not_available"
    then
	colloc="$CPP_TEST_DIR/$cpp_dir/collocated${exe} -OAthread_pool 10"
    fi

    if test "$server_num" = "6"
    then
	colloc="$JAVA $java_pkg.Collocated -OAthreaded"
    fi

    if test "$server_num" = "7"
    then
	colloc="$JAVA $java_pkg.Collocated -OAthread_per_client"
    fi

    if test "$server_num" = "8"
    then
	colloc="$JAVA $java_pkg.Collocated -OAthread_per_request"
    fi

    if test "$server_num" = "9"
    then
	colloc="$JAVA $java_pkg.Collocated -OAthread_pool 10"
    fi
}

#
# Delete old log files
#
delete_logfiles()
{
    type=$1

    rm -f "$type"_colloc.log
    rm -f "$type"_client.log
    rm -f "$type"_server.log
}

prompt_for_client_and_server()
{
    #
    # Ask for client
    #
    while test -z "$client_num"
    do
	$echo
	$echo "Select test client to use:"
	$echo
	$echo "1: C++ client, reactive${cpp_not_available}"
	$echo "2: C++ client, threaded${cpp_not_available}"
	$echo
	$echo "3: Java client, threaded"
	$echo
	$echo "---> \c"

	read num

	if test "$num" = "1" -a -z "$cpp_not_available"
	then
	    client_num=1
	elif test "$num" = "2" -a -z "$cpp_not_available"
	then
	    client_num=2
	elif test "$num" = "3"
	then
	    client_num=3
        else
	    client_num=
	fi
    done

    #
    # Ask for server
    #
    while test -z "$server_num"
    do
	$echo
	$echo "Select test server to use:"
	$echo
	$echo "1: C++ server, reactive${cpp_not_available}"
	$echo "2: C++ server, threaded${cpp_not_available}"
	$echo "3: C++ server, thread per client${cpp_not_available}"
	$echo "4: C++ server, thread per request${cpp_not_available}"
	$echo "5: C++ server, thread pool${cpp_not_available}"
	$echo
	$echo "6: Java server, threaded"
	$echo "7: Java server, thread per client"
	$echo "8: Java server, thread per request"
	$echo "9: Java server, thread pool"
	$echo
	$echo "---> \c"
	
	read num

	if test "$num" = "1" -a -z "$cpp_not_available"
	then
	    server_num=1
	elif test "$num" = "2" -a -z "$cpp_not_available"
	then
	    server_num=2
	elif test "$num" = "3" -a -z "$cpp_not_available"
	then
	    server_num=3
	elif test "$num" = "4" -a -z "$cpp_not_available"
	then
	    server_num=4
	elif test "$num" = "5" -a -z "$cpp_not_available"
	then
	    server_num=5
	elif test "$num" = "6"
	then
	    server_num=6
	elif test "$num" = "7"
	then
	    server_num=7
	elif test "$num" = "8"
	then
	    server_num=8
	elif test "$num" = "9"
	then
	    server_num=9
	else
	    server_num=
	fi
    done
}

#
# Function to run the test executables.
#
execute_test()
{
    #
    # The type of test being preformed
    #
    type=$1

    #
    # Set name of reference file
    #
    ref=$2

    #
    # Display info
    #
    $echo
    $echo Java CLASSPATH is \"CLASSPATH=$CLASSPATH\"
    $echo Server command is \"$server\"
    $echo Client command is \"$client\"

    #
    # Run remote tests
    #
    $echo
    $echo "Running remote tests:"

    #
    # Start server
    #
    $echo
    $echo "Waiting for server to start up... \c"
    rm -f $ref
    ( $server & echo $! > srvid ) 2>&1 | tee -a "$type"_server.log &
    count=0
    while test ! -s $ref -a $count -lt 16
    do
	sleep 1
	count=`expr $count + 1`
    done
    if test ! -s $ref
    then
	$echo "Failed!"
	$echo "(Server was not started)"
	exit
    else
	$echo "Done!"
    fi

    #
    # Start client
    #
    $client 2>&1 | tee -a "$type"_client.log

    #
    # Wait for server deactivation
    #
    $echo "Waiting for server to deactivate... \c"
    count=0
    while test -r $ref -a $count -lt 3
    do
	sleep 1
	count=`expr $count + 1`
    done

    if test -r $ref
    then
	$echo "Failed!"
	$echo "(Server was not deactivated by client - deactivating server now)"
	deactivate
    else
	$echo "Done!"
    fi
}


#
# Function to run the collocated test executables.
#
execute_colloc_test()
{
    type=$1

    $echo Collocated command is \"$colloc\"

    #
    # Run collocated tests
    #
    $echo
    $echo "Running collocated tests:"
    $echo
    $colloc 2>&1 | tee -a "$type"_colloc.log
}
