// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package benchmark;

import benchmark.*;
import benchmark.Benchmark.*;

public class CallBasicTester
{
    private org.omg.CORBA.ORB orb_;

    public
    CallBasicTester(org.omg.CORBA.ORB orb)
    {
        orb_ = orb;
    }
    
    public void
    ping(BasicTester obj, long iterations)
    {
        System.out.print(iterations + " Pings:     \t");
        Timer t = new Timer();

        try 
        {
            //
       	    // Warm up ping to establish connection
	    //
    	    obj.ping();
		
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.ping();

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("ping(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    short_in_t(BasicTester obj, long iterations)
    {
        short s = 7;

        System.out.print(iterations + " Shorts in: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.short_in_t(s);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("short_in_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    short_out_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.ShortHolder s = new org.omg.CORBA.ShortHolder();

        System.out.print(iterations + " Shorts out: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.short_out_t(s);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("short_out_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    short_inout_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.ShortHolder s = new org.omg.CORBA.ShortHolder();
	s.value = 18;

        System.out.print(iterations + " Shorts i/o: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.short_inout_t(s);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("short_inout_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    short_ret_t(BasicTester obj, long iterations)
    {
        short s;

        System.out.print(iterations + " Shorts ret: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        s = obj.short_ret_t();

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("short_ret_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    long_in_t(BasicTester obj, long iterations)
    {
        int s = 7;

        System.out.print(iterations + " Longs in:  \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.long_in_t(s);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("long_in_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    long_out_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.IntHolder l = new org.omg.CORBA.IntHolder();

        System.out.print(iterations + " Longs out: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.long_out_t(l);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("long_out_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    long_inout_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.IntHolder l = new org.omg.CORBA.IntHolder();
	l.value = 18;

        System.out.print(iterations + " Longs i/o: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.long_inout_t(l);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("long_inout_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    long_ret_t(BasicTester obj, long iterations)
    {
        int l;

        System.out.print(iterations + " Longs ret: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        l = obj.long_ret_t();

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("long_ret_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    longlong_in_t(BasicTester obj, long iterations)
    {
        long ll = 7L;

        System.out.print(iterations + " Longlongs in: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.longlong_in_t(ll);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("longlong_in_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    longlong_out_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.LongHolder ll = new org.omg.CORBA.LongHolder();

        System.out.print(iterations + " Longlongs out: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.longlong_out_t(ll);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("longlong_out_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    longlong_inout_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.LongHolder ll = new org.omg.CORBA.LongHolder();
	ll.value = 18L;

        System.out.print(iterations + " Longlongs i/o: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.longlong_inout_t(ll);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("longlong_inout_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    longlong_ret_t(BasicTester obj, long iterations)
    {
        long ll;

        System.out.print(iterations + " Longlongs ret: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        ll = obj.longlong_ret_t();

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("longlong_ret_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }
    
    public void
    ushort_in_t(BasicTester obj, long iterations)
    {
        short us = 7;

        System.out.print(iterations + " UShorts in: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.ushort_in_t(us);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("ushort_in_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    ushort_out_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.ShortHolder us = new org.omg.CORBA.ShortHolder();

        System.out.print(iterations + " UShorts out: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.ushort_out_t(us);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("ushort_out_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    ushort_inout_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.ShortHolder us = new org.omg.CORBA.ShortHolder();
	us.value = 18;

        System.out.print(iterations + " UShorts i/o: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.ushort_inout_t(us);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("ushort_inout_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    ushort_ret_t(BasicTester obj, long iterations)
    {
        short us;

        System.out.print(iterations + " UShorts ret: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        us = obj.ushort_ret_t();

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("ushort_ret_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    ulong_in_t(BasicTester obj, long iterations)
    {
        int ul = 7;

        System.out.print(iterations + " ULongs in: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.ulong_in_t(ul);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("ulong_in_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    ulong_out_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.IntHolder ul = new org.omg.CORBA.IntHolder();

        System.out.print(iterations + " ULongs out: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.ulong_out_t(ul);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("ulong_out_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    ulong_inout_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.IntHolder ul = new org.omg.CORBA.IntHolder();
	ul.value = 18;

        System.out.print(iterations + " ULongs i/o: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.ulong_inout_t(ul);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("ulong_inout_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    ulong_ret_t(BasicTester obj, long iterations)
    {
        int ul;

        System.out.print(iterations + " ULongs ret: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        ul = obj.ulong_ret_t();

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("ulong_ret_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    ulonglong_in_t(BasicTester obj, long iterations)
    {
        long ull = 7L;

        System.out.print(iterations + " ULonglongs in: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.ulonglong_in_t(ull);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("ulonglong_in_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    ulonglong_out_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.LongHolder ull = new org.omg.CORBA.LongHolder();

        System.out.print(iterations + " ULonglongs out: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.ulonglong_out_t(ull);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("ulonglong_out_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    ulonglong_inout_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.LongHolder ull = new org.omg.CORBA.LongHolder();
	ull.value = 18L;

        System.out.print(iterations + " ULonglongs i/o: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.ulonglong_inout_t(ull);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("ulonglong_inout_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    ulonglong_ret_t(BasicTester obj, long iterations)
    {
        long ull;

        System.out.print(iterations + " ULonglongs ret: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        ull = obj.ulonglong_ret_t();

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("ulonglong_ret_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    float_in_t(BasicTester obj, long iterations)
    {
        float f = 77.34F;

        System.out.print(iterations + " Floats in: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.float_in_t(f);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("float_in_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    float_out_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.FloatHolder f = new org.omg.CORBA.FloatHolder();

        System.out.print(iterations + " Floats out: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.float_out_t(f);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("float_out_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    float_inout_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.FloatHolder f = new org.omg.CORBA.FloatHolder();
	f.value = 18.34F;

        System.out.print(iterations + " Floats i/o: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.float_inout_t(f);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("float_inout_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    float_ret_t(BasicTester obj, long iterations)
    {
        float f;

        System.out.print(iterations + " Floats ret: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        f = obj.float_ret_t();

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("float_ret_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    double_in_t(BasicTester obj, long iterations)
    {
        double d = 77.34;

        System.out.print(iterations + " Doubles in: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.double_in_t(d);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("double_in_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    double_out_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.DoubleHolder d = new org.omg.CORBA.DoubleHolder();

        System.out.print(iterations + " Doubles out: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.double_out_t(d);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("double_out_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    double_inout_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.DoubleHolder d = new org.omg.CORBA.DoubleHolder();
	d.value = 18.34;

        System.out.print(iterations + " Doubles i/o: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.double_inout_t(d);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("double_inout_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    double_ret_t(BasicTester obj, long iterations)
    {
        double d;

        System.out.print(iterations + " Doubles ret: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        d = obj.double_ret_t();

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("double_ret_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    char_in_t(BasicTester obj, long iterations)
    {
        char c = 'f';

        System.out.print(iterations + " Chars in:  \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.char_in_t(c);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("char_in_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    char_out_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.CharHolder c = new org.omg.CORBA.CharHolder();

        System.out.print(iterations + " Chars out: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.char_out_t(c);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("char_out_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    char_inout_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.CharHolder c = new org.omg.CORBA.CharHolder();
	c.value = 'B';

        System.out.print(iterations + " Chars i/o: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.char_inout_t(c);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("char_inout_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    char_ret_t(BasicTester obj, long iterations)
    {
        char c;

        System.out.print(iterations + " Chars ret: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        c = obj.char_ret_t();

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("char_ret_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    wchar_in_t(BasicTester obj, long iterations)
    {
        char wc = 'f';

        System.out.print(iterations + " WChars in: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.wchar_in_t(wc);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("wchar_in_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    wchar_out_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.CharHolder wc = new org.omg.CORBA.CharHolder();

        System.out.print(iterations + " WChars out: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.wchar_out_t(wc);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("wchar_out_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    wchar_inout_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.CharHolder wc = new org.omg.CORBA.CharHolder();
	wc.value = 'B';

        System.out.print(iterations + " WChars i/o: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.wchar_inout_t(wc);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("wchar_inout_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    wchar_ret_t(BasicTester obj, long iterations)
    {
        char wc;

        System.out.print(iterations + " WChars ret: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        wc = obj.wchar_ret_t();

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("wchar_ret_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    string_in_t(BasicTester obj, long iterations)
    {
        String s = "Mountain bikes rule!";

        System.out.print(iterations + " Strings in: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.string_in_t(s);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("string_in_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    string_out_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.StringHolder s = new org.omg.CORBA.StringHolder();

        System.out.print(iterations + " Strings out: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.string_out_t(s);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("string_out_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    string_inout_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.StringHolder s = new org.omg.CORBA.StringHolder();
	s.value = "1400 Fashion Island Blvd, San Mateo, CA 94404";

        System.out.print(iterations + " Strings i/o: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.string_inout_t(s);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("string_inout_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    string_ret_t(BasicTester obj, long iterations)
    {
        String s;

        System.out.print(iterations + " Strings ret: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        s = obj.string_ret_t();

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("string_ret_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    wstring_in_t(BasicTester obj, long iterations)
    {
        String ws = "Mountain bikes rule!";

        System.out.print(iterations + " WStrings in: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.wstring_in_t(ws);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("wstring_in_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    wstring_out_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.StringHolder ws = new org.omg.CORBA.StringHolder();

        System.out.print(iterations + " WStrings out: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.wstring_out_t(ws);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("wstring_out_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    wstring_inout_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.StringHolder ws = new org.omg.CORBA.StringHolder();
	ws.value = "1400 Fashion Island Blvd, San Mateo, CA 94404";

        System.out.print(iterations + " WStrings i/o: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.wstring_inout_t(ws);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("wstring_inout_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    wstring_ret_t(BasicTester obj, long iterations)
    {
        String ws;

        System.out.print(iterations + " WStrings ret: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        ws = obj.wstring_ret_t();

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("wstring_ret_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }
    
    public void
    boolean_in_t(BasicTester obj, long iterations)
    {
        boolean b = true;

        System.out.print(iterations + " Booleans in: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.boolean_in_t(b);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("boolean_in_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    boolean_out_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.BooleanHolder b = new org.omg.CORBA.BooleanHolder();

        System.out.print(iterations + " Booleans out: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.boolean_out_t(b);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("boolean_out_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    boolean_inout_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.BooleanHolder b = new org.omg.CORBA.BooleanHolder();
	b.value = false;

        System.out.print(iterations + " Booleans i/o: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.boolean_inout_t(b);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("boolean_inout_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    boolean_ret_t(BasicTester obj, long iterations)
    {
        boolean b;

        System.out.print(iterations + " Booleans ret: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        b = obj.boolean_ret_t();

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("boolean_ret_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    octet_in_t(BasicTester obj, long iterations)
    {
        byte o = 7;

        System.out.print(iterations + " Octets in: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.octet_in_t(o);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("octet_in_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    octet_out_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.ByteHolder o = new org.omg.CORBA.ByteHolder();

        System.out.print(iterations + " Octets out: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.octet_out_t(o);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("octet_out_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    octet_inout_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.ByteHolder o = new org.omg.CORBA.ByteHolder();
	o.value = 0;

        System.out.print(iterations + " Octets i/o: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.octet_inout_t(o);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("octet_inout_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    octet_ret_t(BasicTester obj, long iterations)
    {
        byte o;

        System.out.print(iterations + " Octets ret: \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        o = obj.octet_ret_t();

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("octet_ret_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    any_in_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.Any a = orb_.create_any();
	a.insert_string("What's up, Chuck?");

        System.out.print(iterations + " Anys in:   \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.any_in_t(a);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("any_in_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    any_out_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.AnyHolder a = new org.omg.CORBA.AnyHolder();

        System.out.print(iterations + " Anys out:  \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.any_out_t(a);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("any_out_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    any_inout_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.AnyHolder a = new org.omg.CORBA.AnyHolder();
	a.value = orb_.create_any();
	a.value.insert_float(44.372F);

        System.out.print(iterations + " Anys i/o:  \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        obj.any_inout_t(a);

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("any_inout_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }

    public void
    any_ret_t(BasicTester obj, long iterations)
    {
        org.omg.CORBA.Any a;

        System.out.print(iterations + " Anys ret:  \t");
        Timer t = new Timer();

        try 
        {
    	    t.start();

    	    for(long i = 0; i < iterations; i++)
       	        a = obj.any_ret_t();

            t.stop();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
    	    System.err.println("any_ret_t(): call failed\n" + ex);
    	    return;
        }

	System.out.println("\t" + t.timeElapsed() + " seconds (" +
			   (iterations / t.timeElapsed()) + "/sec)");
    }
}
