// ********************************************************************* 
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
// 
// All Rights Reserved
//
// **********************************************************************

import java.util.*;
import org.omg.CORBA.*;
import org.omg.DynamicAny.*;

public class util 
{
    static ORB orb = null;                      // Initialized from main()
    static DynAnyFactory daf = null;            // Initialized from main()

    // Indent to the current level.
    static final int INDENT = 4;

    static private void
    indent(int indent_lvl)
    {
	for(int i = 0 ; i < INDENT * indent_lvl ; i++)
	    System.out.print(" ");
    }

    static private void
    show_label(Any ap)
    {
	try
        {
	    TypeCode tc = ap.type();
	    if(tc.kind() == TCKind.tk_octet)
            {
		System.out.println("default:");
	    }
            else
            {
		System.out.print("case ");
		switch(tc.kind().value())
                {
		case TCKind._tk_short:
		    short s = ap.extract_short();
		    System.out.print(s);
		    break;
		case TCKind._tk_long:
		    int l = ap.extract_long();
		    System.out.print(l);
		    break;
		case TCKind._tk_ushort:
		    short us = ap.extract_short();
		    System.out.print(us);
		    break;
		case TCKind._tk_ulong:
		    int ul = ap.extract_long();
		    System.out.print(ul);
		    break;
		case TCKind._tk_boolean:
		    boolean b = ap.extract_boolean();
		    if(b)
			System.out.print("TRUE");
		    else
			System.out.print("FALSE");
		    break;
		case TCKind._tk_char:
		    char c = ap.extract_char();
		    System.out.print(c);
		    break;
		case TCKind._tk_longlong:
		{
		    long ll = ap.extract_longlong();
		    System.out.print(ll);
                    break;
		}
		case TCKind._tk_ulonglong:
		{
		    long ull = ap.extract_longlong();
		    System.out.print(ull);
                    break;
		}
		case TCKind._tk_wchar:
		    char wc = ap.extract_wchar();
		    System.out.print("'" + wc + "'");
		    break;
		case TCKind._tk_enum:
		{
		    org.omg.DynamicAny.DynAny da = daf.create_dyn_any(ap);
		    org.omg.DynamicAny.DynEnum de =
			org.omg.DynamicAny.DynEnumHelper.narrow(da);
		    String str = de.get_as_string();
		    System.out.print(str);
		    da.destroy();
                    break;
		}
		default:
		    // Union discriminator can't be anything else
		    throw new RuntimeException();
		}
		System.out.println(":");
	    }
	}
        catch(org.omg.CORBA.UserException ex)
        {
	    ex.printStackTrace();
	    throw new RuntimeException();
	}
        catch(org.omg.CORBA.SystemException ex)
        {
	    ex.printStackTrace();
	    throw new RuntimeException();
	}
    }

    // Show the contents of a type code. 'tcp' is the type code to
    // show, 'tlist' is the list of type codes seen so far,
    // 'indent_lvl' is the current nesting level. 'tlist' is used
    // to prevent getting trapped in an infinite loop for recursive
    // structures and unions.

    static private void
    show_TC(
	TypeCode        tcp,
	Vector          tlist,
	int             indent_lvl)
    {
	final String kind_name[] =
        {
	    "tk_null", "void", "short", "long",
	    "unsigned short", "unsigned long", "float",
	    "double", "boolean", "char", "octet", "any",
	    "CORBA::TypeCode", "CORBA::Principal",
	    "interface", "struct", "union", "enum",
	    "string", "sequence", "array", "typedef",
	    "exception", "long long", "unsigned long long",
	    "long double", "wchar", "wstring", "fixed"
	};

	indent(indent_lvl);

	// Print the TCKind value.
	System.out.print(kind_name[tcp.kind().value()]);   

	// Print name and repository ID for those type codes
	// that have these parameters.
	switch(tcp.kind().value())
        {
	case TCKind._tk_objref:
	case TCKind._tk_struct:
	case TCKind._tk_union:
	case TCKind._tk_except:
	case TCKind._tk_enum:
	case TCKind._tk_alias:
	    try
            {
	        System.out.println(" " + tcp.name() + " (" + tcp.id() + "):");
	    }
            catch(org.omg.CORBA.UserException ex)
            {
	        ex.printStackTrace();
	        throw new RuntimeException();
	    }

	default:
	    ;       // Do nothing
	}

	// For type codes that have other parameters,
	// show the remaining parameters.
	switch(tcp.kind().value())
        {
	default:                    // No other params to print
	    System.out.println();
	    break;

	    // For fixed types, show digits and scale.
	case TCKind._tk_fixed:
	    try
            {
	        System.out.println("<" + tcp.fixed_digits() + "," + 
			           tcp.fixed_scale() + ">");
	    }
            catch(org.omg.CORBA.UserException ex)
            {
	        ex.printStackTrace();
	        throw new RuntimeException();
	    }
	    break;

	    // For enumerations, show the enumerators.
	case TCKind._tk_enum:
        {
            indent(indent_lvl + 1);
	    try
            {
                for(int i = 0 ; i < tcp.member_count() ; i++)
                {
                    System.out.print(tcp.member_name(i));
                    if(i < tcp.member_count() - 1)
                        System.out.print(", ");
                }
	    }
            catch(org.omg.CORBA.UserException ex)
            {
	        ex.printStackTrace();
	    	throw new RuntimeException();
	    }
            System.out.println();
            break;
        }

	// For strings, show the bound (if any).
	case TCKind._tk_string:
	case TCKind._tk_wstring:
	{
	    try
            {
	        int l = tcp.length();
	        if(l != 0)
                    System.out.print("<" + l + ">");
                System.out.println();
	    }
            catch(org.omg.CORBA.UserException ex)
            {
	        ex.printStackTrace();
	    	throw new RuntimeException();
	    }
            break;
        }

	// For sequences, show the bound (if any) and
	// the element type.
	case TCKind._tk_sequence:
        {
	    try
            {
                int l = tcp.length();
                if(l != 0)
                    System.out.print("<" + l + ">");
                System.out.println(":");
                TypeCode etype = tcp.content_type();
                show_TC(etype, tlist, indent_lvl + 1);
	    }
            catch(org.omg.CORBA.UserException ex)
            {
	    	ex.printStackTrace();
		throw new RuntimeException();
	    }
            break;
        }

	// For arrays, show the dimension and element type.
	case TCKind._tk_array:
        {
	    try
            {
                int l = tcp.length();
                System.out.println("[" + l + "]:");
                TypeCode etype = tcp.content_type();
                show_TC(etype, tlist, indent_lvl + 1);
	    }
            catch(org.omg.CORBA.UserException ex)
            {
	    	ex.printStackTrace();
		throw new RuntimeException();
	    }
            break;
        }

	// For typedefs, show the type of the aliased type.
	case TCKind._tk_alias:
        {
	    try
            {
                TypeCode atype = tcp.content_type();
                show_TC(atype, tlist, indent_lvl + 1);
	    }
            catch(org.omg.CORBA.UserException ex)
            {
	    	ex.printStackTrace();
		throw new RuntimeException();
	    }
            break;
        }

	// For structures and exceptions, show the
	// names and types of each member.
	case TCKind._tk_struct:
	case TCKind._tk_except:
        {
            // Avoid a recursive loop by checking whether we
            // have shown this type code before.
            // If we have not seen this type code before, add it
            // to the list of type codes processed so far and
            // decode the member type codes.
            try
            {
	        if(!tlist.contains(tcp))
                {
	            tlist.addElement(tcp);
                    for(int i = 0 ; i < tcp.member_count() ; i++)
                    {
                        System.out.println(tcp.member_name(i) + ":");
                    	indent(indent_lvl + 1);
                        TypeCode mt = tcp.member_type(i);
                        show_TC(mt, tlist, indent_lvl + 2);
	            }
	        }
                else
                {
                    System.out.println(" " + tcp.name() + " (" + tcp.id() + 
				       ")");
	        }
	    }
            catch(org.omg.CORBA.UserException ex)
            {
	        ex.printStackTrace();
	        throw new RuntimeException();
	    }
            break;
        }

	// For unions, show the discriminator type.
	// Then, for each member, show the case label,
	// member name, and member type. To show the case
	// label, we use the show_label() helper function.
	case TCKind._tk_union:
        {
	    try
            {
		// Avoid getting trapped in a recursive loop.
		// Show the members only if we haven't shown this type
		// code before.
		if(!tlist.contains(tcp))
                {
		    tlist.addElement(tcp);
		    indent(indent_lvl + 1);

		    // Show discriminator type.
		    System.out.println("Discriminator type:");
		    TypeCode dt = tcp.discriminator_type();
		    show_TC(dt, tlist, indent_lvl + 2);

		    // Show case label, member name, and
		    // member type for each member.
		    for(int i = 0 ; i < tcp.member_count() ; i++)
                    {
                        Any label = tcp.member_label(i);
                        indent(indent_lvl + 1);
                        show_label(label);
                        indent(indent_lvl + 2);
                        System.out.println(tcp.member_name(i) + ":");
                        TypeCode mt = tcp.member_type(i);
                        show_TC(mt, tlist, indent_lvl + 3);
		    }
		}
                else
                {
		    System.out.println(" " + tcp.name() + " (" + tcp.id() + 
				       ")");
		}
	    }
            catch(org.omg.CORBA.UserException ex)
            {
	        ex.printStackTrace();
	        throw new RuntimeException();
	    }
            break;
        }
	}
    }

    // Show the contents of a type code.

    static public void 
    show_TC(TypeCode tcp)
    {
	java.util.Vector tclist = new java.util.Vector();
	show_TC(tcp, tclist, 0);
    }
}
