// ********************************************************************* 
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
// 
// All Rights Reserved
//
// **********************************************************************

class DeviceLocator_impl extends org.omg.PortableServer.ServantLocatorPOA 
{
    private Controller_impl      m_ctrl = null;
    private final int            MAX_EQ_SIZE = 100;
    private java.util.Vector     m_eq = new java.util.Vector();
    private java.util.Hashtable  m_aom = new java.util.Hashtable();

    DeviceLocator_impl(Controller_impl ctrl)
    {
	m_ctrl = ctrl;
    }

    public org.omg.PortableServer.Servant 
    preinvoke(
	byte[] oid, 
	org.omg.PortableServer.POA poa,        
	String operation, 
	org.omg.PortableServer.ServantLocatorPackage.CookieHolder cookie) 
	throws org.omg.PortableServer.ForwardRequest
    {
	String oid_string = new String(oid);

	if(oid_string.equals(Controller_impl.Controller_oid))
	    return m_ctrl;

	// Convert object id into asset number.
	Integer anum = new Integer(oid_string);

	// Check whether the device is known.
	if(!m_ctrl.exists(anum.intValue()))
	    throw new org.omg.CORBA.OBJECT_NOT_EXIST();

	// Look at the object map to find out whether
	// we have a servant in memory.
	Thermometer_impl thermometer_servant = null;
	Thermostat_impl  thermostat_servant = null;

	if((anum.intValue() % 2) != 0)
	    thermometer_servant = (Thermometer_impl)m_aom.get(anum);
	else
	    thermostat_servant = (Thermostat_impl)m_aom.get(anum);

	if((thermometer_servant == null) && (thermostat_servant == null))
        {
	    // No servant in memory. If evictor queue is full,
	    // evict servant at head of queue.
	    if(m_eq.size() == MAX_EQ_SIZE)
            {
		Integer assetNum = (Integer)m_eq.elementAt(0);
		m_eq.removeElementAt(0);
		m_aom.remove(assetNum);
	    }

	    // Instantiate correct type of servant.
	    String buf = (String)ICP.get(anum.intValue(), "model");
	    if(buf == null)
		throw new RuntimeException();

	    if(buf.equals("Sens-A-Temp") == true)
		thermometer_servant = new Thermometer_impl(anum.intValue());
	    else
		thermostat_servant = new Thermostat_impl(anum.intValue());
	}
        else
        {
	    // Servant already in memory.
	    m_eq.removeElement(anum);      // Remove from queue

	    // If operation is "remove", also remove entry from
	    // active object map -- the object is about to be deleted.
	    if(operation.equals("remove"))
		m_aom.remove(anum);
	}

	// We found a servant, or just instantiated it.
	// If the operation is not a remove, move
	// the servant to the tail of the evictor queue
	// and update its queue position in the map.
	if(!operation.equals("remove"))
        {
	    m_eq.addElement(anum);
	    if(thermometer_servant != null)
	        m_aom.put(anum, thermometer_servant);
	    else
	        m_aom.put(anum, thermostat_servant);
	}
        else
        {
	    m_ctrl.remove_impl(anum.intValue());
	}

	if(thermometer_servant != null)
	    return thermometer_servant;
	else
	    return thermostat_servant;
    }

    public void 
    postinvoke( 
	byte[] oid, 
	org.omg.PortableServer.POA poa, 
	String operation, 
	java.lang.Object cookie, 
	org.omg.PortableServer.Servant servant) 
    {
	// Do nothing if this was an operation on the controller.
        if(servant == m_ctrl)
            return;

        // If the operation that just ran was a remove, we delete the
        // servant.
        if(operation.equals("remove"))
            servant = null;
    } 
}
