// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

public class Server
{
    static int
    run(org.omg.CORBA.ORB orb, String[] args)
        throws org.omg.CORBA.UserException
    {
	// Resolve Root POA
	org.omg.PortableServer.POA poa =
	    org.omg.PortableServer.POAHelper.narrow(
		orb.resolve_initial_references("RootPOA"));

	// Get a POA manager
	org.omg.PortableServer.POAManager poa_mgr = poa.the_POAManager();

	// Create a policy list. We use persistent objects with
	// user-assigned IDs, and explicit activation.
	org.omg.CORBA.Policy[] policy_list = new org.omg.CORBA.Policy[3];
	policy_list[0] = poa.create_lifespan_policy(
	    org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);

	policy_list[1] = poa.create_id_assignment_policy(
	    org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);

	policy_list[2] = poa.create_implicit_activation_policy(
	    org.omg.PortableServer.ImplicitActivationPolicyValue.
	    NO_IMPLICIT_ACTIVATION);

	// Create a POA for the controller.
	org.omg.PortableServer.POA ctrl_poa =
	    poa.create_POA("child", null, policy_list);

	org.omg.CORBA.Policy[] new_policy_list = new org.omg.CORBA.Policy[5];
	System.arraycopy(policy_list, 0, new_policy_list, 0, 
			 policy_list.length);
	new_policy_list[3] = poa.create_request_processing_policy(
	    org.omg.PortableServer.RequestProcessingPolicyValue.
	    USE_SERVANT_MANAGER);
	new_policy_list[4] = poa.create_servant_retention_policy(
	    org.omg.PortableServer.ServantRetentionPolicyValue.NON_RETAIN);

	// Create a POA for the devices.
	org.omg.PortableServer.POA dev_poa =
	    ctrl_poa.create_POA("DevPOA", poa_mgr, new_policy_list);

	// Create a controller and set static m_ctrl member
	// for thermostats and thermometers.
	Controller_impl ctrl_servant = 
	    new Controller_impl(dev_poa, "/tmp/CCS_assets");
	Thermometer_impl.m_ctrl = ctrl_servant;
	Thermostat_impl.m_ctrl = ctrl_servant;

	org.omg.CORBA.Object obj = ctrl_servant._this(orb);

	// Write a reference for the controller to stdout.
	String ref = orb.object_to_string(obj);
	System.out.println(ref);

	// Instantiate the servant locator for devices.
	DeviceLocator_impl my_locator = new DeviceLocator_impl(ctrl_servant);
	org.omg.PortableServer.ServantManager locator = my_locator._this(orb);

	// Set servant locator.
        dev_poa.set_servant_manager(locator);

	// Activate the POA manager.
        poa_mgr.activate();

	// Accept requests.
	orb.run();

        return 0;
    }

    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

	int status = 0;
        org.omg.CORBA.ORB orb = null;

        try
        {
            orb = org.omg.CORBA.ORB.init(args, props);
            status = run(orb, args);
        }
        catch(Exception ex)
        {
	    ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
	    try
            {
		orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
