// ********************************************************************* 
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
// 
// All Rights Reserved
//
// **********************************************************************

class Thermometer_impl extends CCS.ThermometerPOA
{
    public static Controller_impl m_ctrl; // My controller
    public final int              m_anum; // My asset number
    
    // IDL model attribute.
    
    public String  
    model()
    {
	return get_model(); 
    }
    
    // IDL asset_num attribute.

    public int  
    asset_num()
    {
        return m_anum;
    }

    // IDL temperature attribute.

    public short   
    temperature()
    {
        return get_temp();
    }

    // IDL location attribute accessor.

    public String    
    location()
    {
        return get_loc();
    }

    // IDL location attribute modifier.

    public void 
    location(String loc)
    {
        set_loc(loc);
    }

    // Constructor

    Thermometer_impl(int anum, String location)
    {
	m_anum = anum;
	if(ICP.online(anum) != 0)         // Mark device as on-line
            throw new RuntimeException();
        set_loc(location);
    }

    // Helper function to read the model string from a device.

    private String  
    get_model()
    {
	String buf = (String)ICP.get(m_anum, "model");
	if(buf == null)
	    throw new RuntimeException();
	return buf;
    }

    // Helper function to read the temperature from a device.

    private short 
    get_temp()
    {
	Short temp = (Short)ICP.get(m_anum, "temperature");
	if(temp == null)
	    throw new RuntimeException();
        return temp.shortValue();
    }
    
    // Helper function to read the location from a device.

    private String 
    get_loc()
    {
	String buf = (String)ICP.get(m_anum, "location");
	if(buf == null)
	    throw new RuntimeException();
	return buf;
    }
    
    // Helper function to set the location of a device.

    private void 
    set_loc(String new_loc)
    {
        if(ICP.set(m_anum, "location", new_loc) != 0)
            throw new RuntimeException();
    }
    
    // Finalizer

    protected void
    finalize()
        throws Throwable
    {
	m_ctrl.remove_impl(m_anum);      // Remove self from map
	if(ICP.offline(m_anum) != 0) // Mark device as off-line
	    throw new RuntimeException();	

        super.finalize();
    }
}
