// ********************************************************************* 
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
// 
// All Rights Reserved
//
// **********************************************************************

class Thermostat_impl extends CCS.ThermostatPOA
{
    public static Controller_impl m_ctrl;                // My controller
    public final int              m_anum;                // My asset number

    // CORBA operations

    public short 
    get_nominal()
    {
        return get_nominal_temp();
    }

    public short 
    set_nominal(short new_temp) 
        throws CCS.ThermostatPackage.BadTemp
    {
        return set_nominal_temp(new_temp);
    }

    // Methods from Thermometer interface

    // IDL model attribute.

    public String
    model()
    {
        return get_model();
    }

    // IDL asset_num attribute.

    public int
    asset_num()
    {
        return m_anum;
    }

    // IDL temperature attribute.

    public short
    temperature()
    {
        return get_temp();
    }

    // IDL location attribute accessor.

    public String
    location()
    {
        return get_loc();
    }

    // IDL location attribute modifier.

    public void
    location(String loc)
    {
        set_loc(loc);
    }

    public void
    remove()
    {
        // Intentionally left empty
    }

    // Helper function to read the model string from a device.

    private String
    get_model()
    {
        String buf = (String)ICP.get(m_anum, "model");
        if(buf == null)
            throw new RuntimeException();
        return buf;
    }

    // Helper function to read the temperature from a device.

    private short
    get_temp()
    {
        Short temp = (Short)ICP.get(m_anum, "temperature");
        if(temp == null)
            throw new RuntimeException();
        return temp.shortValue();
    }

    // Helper function to read the location from a device.

    private String
    get_loc()
    {
        String buf = (String)ICP.get(m_anum, "location");
        if(buf == null)
            throw new RuntimeException();
        return buf;
    }
    
    // Helper function to set the location of a device.

    private void
    set_loc(String new_loc)
    {
        if(ICP.set(m_anum, "location", new_loc) != 0)
            throw new RuntimeException();
    }

    // Constructor & destructor
    Thermostat_impl(int anum, String location, short nominal_temp)
        throws CCS.ThermostatPackage.BadTemp
    {
	m_anum = anum;
	if(ICP.online(anum) != 0)         // Mark device as on-line
            throw new RuntimeException();
        set_loc(location);
	
        // Set the nominal temperature.
        set_nominal_temp(nominal_temp);
    }

    // Private Helper functions
    private short   
    get_nominal_temp()
    {
        Short temp = (Short)ICP.get(m_anum, "nominal_temp");
	if(temp == null)
            throw new RuntimeException();
        return temp.shortValue();
    }

    private short 
    set_nominal_temp(short newTemp)
        throws CCS.ThermostatPackage.BadTemp
    {
	// We need to return the previous nominal temperature,
	// so we first read the current nominal temperature before
	// changing it.
	Short old_temp = (Short)ICP.get(m_anum, "nominal_temp");
	if(old_temp == null) 
	    throw new RuntimeException();
	
	// Now set the nominal temperature to the new value.
	Short new_temp = new Short(newTemp);
	if(ICP.set(m_anum, "nominal_temp", new_temp) != 0) 
	{
	    // If ICP_set() failed, read this thermostat's minimum
	    // and maximum so we can initialize the BadTemp exception.
	    CCS.ThermostatPackage.BtData btd = 
		new CCS.ThermostatPackage.BtData();
	    
	    Short permitted_vals = (Short)ICP.get(m_anum, "MIN_TEMP");
	    if(permitted_vals == null)
		throw new RuntimeException();
	    btd.min_permitted = permitted_vals.shortValue();
	    permitted_vals = (Short)ICP.get(m_anum, "MAX_TEMP");
	    if(permitted_vals == null)
		throw new RuntimeException();
	    btd.max_permitted = permitted_vals.shortValue();
	    
	    btd.requested = new_temp.shortValue();
	    btd.error_msg = (new_temp.shortValue() > btd.max_permitted) ? 
		"Too hot" : "Too cold";
	    throw new CCS.ThermostatPackage.BadTemp(btd);
	}
	return old_temp.shortValue();
    }
}
