// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package chat;

import chat.BroadcasterAdminPackage.*;

final public class BroadcasterAdmin_impl extends BroadcasterAdminPOA
{
    private org.omg.CORBA.ORB orb_;

    private org.omg.PortableServer.POA adminPoa_;
    private org.omg.PortableServer.POA broadcasterPoa_;

    private class BroadcasterDesc
    {
	public ReceiverDesc receiverDesc;
	public int broadcasterId;
    }

    private BroadcasterDesc[] broadcasters = new BroadcasterDesc[0];

    private int nextBroadcasterId_ = 0;


    BroadcasterAdmin_impl(org.omg.CORBA.ORB orb, 
			  org.omg.PortableServer.POA adminPoa, 
			  org.omg.PortableServer.POA broadcasterPoa)
    {
        orb_ = orb;
        adminPoa_ = adminPoa;
	broadcasterPoa_ = broadcasterPoa;
    }

    //
    // Add element to ReceiverDesc array
    //
    private int
    inc()
    {
        int len = broadcasters.length + 1;
        BroadcasterDesc[] r = new BroadcasterDesc[len];
        System.arraycopy(broadcasters, 0, r, 0, len - 1);
        broadcasters = r;
        broadcasters[len - 1] = new BroadcasterDesc();

        return len;
    }

    //
    // Remove last element from ReceiverDescription array
    //
    private void
    dec()
    {
        int len = broadcasters.length - 1;
        BroadcasterDesc[] r = new BroadcasterDesc[len];
        System.arraycopy(broadcasters, 0, r, 0, len);
        broadcasters = r;
    }

    //
    // Add receiver
    //
    public Broadcaster
    add(ReceiverDesc receiverDesc)
        throws NickExists
    {
        //
        // Check if nick already exists
        //
        for(int i = 0 ; i < broadcasters.length ; i++)
	{
            if(broadcasters[i].receiverDesc.nick.equals(receiverDesc.nick))
	    {
                throw new NickExists();
	    }
	}

	try 
	{
	    //
	    // Add receiver description
	    //
	    int len = inc();
	    
	    broadcasters[len - 1].receiverDesc = receiverDesc;

	    nextBroadcasterId_++;

	    Broadcaster_impl newBroadcasterImpl = 
		new Broadcaster_impl(this, nextBroadcasterId_, broadcasterPoa_);

	    Broadcaster newBroadcaster = newBroadcasterImpl._this(orb_);

	    broadcasters[len - 1].broadcasterId = nextBroadcasterId_;

	    return newBroadcaster;
	}
	catch(Exception ex)
	{
	    System.err.println(ex);
	}
	return null;
    }

    //
    // Remove receiver
    //
    void
    remove(int broadcasterId)
    {
        //
        // Search for receiver
        //
        for(int i = 0 ; i < broadcasters.length ; i++)
        {
            if(broadcasters[i].broadcasterId == broadcasterId)
            {
                //
                // Receiver found, remove it
                //
                while(i < broadcasters.length - 1)
                {
                    broadcasters[i] = broadcasters[i + 1];
                    i++;
                }
                dec();

                return;
            }
        }
   }

    //        
    // Register receiver
    //
    public Broadcaster
    subscribe(ReceiverDesc receiverDesc)
        throws NickExists
    {
        //
        // Do nothing if this is a nil object reference
        //
        if(receiverDesc.recv == null)
	{
            return null;
	}

        //
        // Add receiver
        //
        Broadcaster newBroadcaster = add(receiverDesc);

        //
        // Send `join' message to others
        //
        String message = "*** " + 
	    receiverDesc.nick + 
	    " (" + 
	    receiverDesc.id + 
	    "@" + 
	    receiverDesc.host +
            ") has joined the chat";

        say(message);

	return newBroadcaster;
    }

    //        
    // Register receiver with check
    //
    public Broadcaster
    subscribeWithCheck(ReceiverDesc receiverDesc)
        throws ReceiverUnreachable, NickExists
    {
        //
        // Do nothing if this is a nil object reference
        //
        if(receiverDesc.recv == null)
	{
            return null;
	}

        //
        // Try to ping the receiver
        //
        boolean nonExist = true;
        try
        {
            nonExist = receiverDesc.recv._non_existent();
        }
        catch(org.omg.CORBA.COMM_FAILURE ex)
        {
        }
        catch(org.omg.CORBA.TRANSIENT ex)
        {
        }

        if(nonExist)
	{
            throw new ReceiverUnreachable();
	}

        //
        // Register receiver
        //
        return subscribe(receiverDesc);
    }


    //        
    // Unregister receiver
    //
    public void
    unregister(int broadcasterId)
    {
        //
        // Remember nick name
        //
        String nick = null;
        for(int i = 0 ; i < broadcasters.length ; i++)
	{
	    if(broadcasters[i].broadcasterId == broadcasterId) 
	    {
		nick = broadcasters[i].receiverDesc.nick;
	    }
	}

        //
        // Remove receiver
        //
        remove(broadcasterId);

        //
        // Send `leave' message to other
        //
        if(nick != null)
        {
            String message = "*** " + nick + " has left the chat";

            say(message);
        }        
    }

    //        
    // Set the nick name
    //
    public void
    setNickName(int broadcasterId, String nick)
        throws NickExists
    {
        //
        // Check if nick already exists
        //
        for(int i = 0 ; i < broadcasters.length ; i++)
        {
            if(broadcasters[i].receiverDesc.nick.equals(nick))
	    {
                throw new NickExists();
	    }
        }

        //
        // Search for receiver
        //
        for(int i = 0 ; i < broadcasters.length ; i++)
        {
            if(broadcasters[i].broadcasterId == broadcasterId)
            {
                //
                // Receiver found, set nick name
                //
                String oldNick = broadcasters[i].receiverDesc.nick;
                broadcasters[i].receiverDesc.nick = nick;

                //
                // Send `is known as' message to all
                //
                String message = "*** " + oldNick +
                    " is now known as " + nick;

                say(message);

                return;
            }
        }
   }

    //        
    // Get a receiver by its nick name
    //
    public void
    getReceiverByNick(String nick, ReceiverDescHolder desc)
        throws UnknownNick
    {
        //
        // Search for nick name
        //
        for(int i = 0 ; i < broadcasters.length ; i++)
	{
            if(broadcasters[i].receiverDesc.nick.equals(nick))
	    {
		desc.value = broadcasters[i].receiverDesc;
		return;
	    }
	}

        //
        // Nick name was not found
        //
        throw new UnknownNick();
    }

    //        
    // Get all receiver names
    //
    public String[]
    getReceiverNames()
    {
        String[] names = new String[broadcasters.length];

        for(int i = 0 ; i < broadcasters.length ; i++)
	{
            names[i] = broadcasters[i].receiverDesc.nick + " (" +
                broadcasters[i].receiverDesc.id + "@" +
                broadcasters[i].receiverDesc.host + ")";
	}

        return names;
    }

    //
    // Say something
    //
    public void
    say(String text)
    {
        //
        // Remember `bad' object references in a `bad sequence'
        //
        java.util.Vector badSeq = new java.util.Vector();

        for(int i = 0 ; i < broadcasters.length ; i++)
        {
            try
            {
                broadcasters[i].receiverDesc.recv.print(text);
            }
            catch(org.omg.CORBA.COMM_FAILURE ex)
            {
                badSeq.addElement(broadcasters[i]);
            }
            catch(org.omg.CORBA.TRANSIENT ex)
            {
                badSeq.addElement(broadcasters[i]);
            }
        }

        //
        // Unregister `bad' object references
        //
	for(int i = 0 ; i < badSeq.size() ; i++)
	{
	    remove(((BroadcasterDesc) (badSeq.elementAt(i))).broadcasterId);
	}

        //
        // Send `vanish' messages
        //
        for(int i = 0 ; i < badSeq.size() ; i++)
        {
            String message = "*** " +
                ((BroadcasterDesc)(badSeq.elementAt(i))).receiverDesc.nick +
                " vanished";

            say(message);
        }

        //
        // Print text as log message, too
        //
        System.out.println(text);
    }


    public org.omg.PortableServer.POA
    _default_POA()
    {
        return adminPoa_;
    }
}
