// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package chat;

import java.applet.*;

public class Client extends Applet
{
    static int
    run(org.omg.CORBA.ORB orb, String host, Integer port, String[] args)
        throws org.omg.CORBA.UserException
    {
	//
	// Resolve Root POA
	//
	org.omg.PortableServer.POA rootPOA =
	    org.omg.PortableServer.POAHelper.narrow(
		orb.resolve_initial_references("RootPOA"));

	org.omg.PortableServer.POAManager manager = rootPOA.the_POAManager();

	//
	// Get the broadcaster object
	//
	String str = "corbaloc::";
	str += host;
	str += ":";
	str += port;
	str += "/broadcasterAdmin";
	org.omg.CORBA.Object obj = orb.string_to_object(str);
	if(obj == null)
	    throw new RuntimeException();
	BroadcasterAdmin server = BroadcasterAdminHelper.narrow(obj);

	//
	// Activate and initialize
	//
        manager.activate();

	//
	// Create new client window
	//
	new ClientWindow(orb, rootPOA, server);

	//
	// Run implementation
	//
	orb.run();

	return 0;
    }

    //
    // Standalone program initialization
    //
    public static
    void main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");
	//
	// Random ORB ID - create distinct ORB IDs when running multiple
	// instances/applications in the same JVM. 
	//
	int random = new java.util.Random().nextInt();
        props.put("ooc.orb.id", "Chat-Client-"+random);

        //
        // Get host and port
        //
        String host = null;
        Integer port = null;
        try
        {
            java.io.BufferedReader dataIn =
                new java.io.BufferedReader(
                    new java.io.InputStreamReader(System.in));
            System.out.print("Host name? ");
            host = dataIn.readLine();
            System.out.print("Port number? ");
            String p = dataIn.readLine();
            port = new Integer(p);
        }
        catch(java.io.IOException ex)
        {
            System.err.println("Can't read from `" +
                               ex.getMessage() + "'");
	    throw new RuntimeException();
        }

        System.out.println("Using object `broadcaster' from host `" +
                           host + "' on port " + port.intValue() + "...");


	int status = 0;
	org.omg.CORBA.ORB orb = null;

        try
	{
	    orb = org.omg.CORBA.ORB.init(args, props);
	    status = run(orb, host, port, args);
	}
        catch(Exception ex)
        {
            ex.printStackTrace();
	    status = 1;
        }

	if(orb != null)
	{
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
	}

        System.exit(status);
    }

    //
    // Applet start
    //
    public void
    start()
    {
        ClientWindow window = null;

        try
        {
            //
            // Create ORB
            //
            org.omg.CORBA.ORB plainORB = org.omg.CORBA.ORB.init(this, null);
            com.ooc.CORBA.ORB orb = (com.ooc.CORBA.ORB)plainORB;

            //
            // Resolve Root POA
            //
            org.omg.PortableServer.POA rootPOA = null;
	    try
	    {
                rootPOA = org.omg.PortableServer.POAHelper.narrow(
		    orb.resolve_initial_references("RootPOA"));
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		ex.printStackTrace();
		throw new RuntimeException();
	    }

            //
            // Get a reference to the POA manager
            //
            org.omg.PortableServer.POAManager manager = 
		rootPOA.the_POAManager();

            //
            // Get "server" object
            //
            String url = getParameter("ior");
            org.omg.CORBA.Object obj = orb.string_to_object(url);
            if(obj == null)
                throw new RuntimeException();

            BroadcasterAdmin server = BroadcasterAdminHelper.narrow(obj);

            //
            // Run implementation
            //
            try
            {
                manager.activate();
            }
            catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
            {
		ex.printStackTrace();
                throw new RuntimeException();
            }

            //
            // Create new client window
            //
            window = new ClientWindow(orb, rootPOA, server);

            orb.run();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            System.err.println(ex.getMessage());
            ex.printStackTrace();
        }

        if(window != null)
            window.dispose();
    }
}
