// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package chat;

final public class Server
{
    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");
        props.put("ooc.orb.id", "Chat-Server");

        try
        {
            //
            // Create ORB
            //
            org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, props);

            //
            // Resolve Root POA
            //
            org.omg.PortableServer.POA rootPoa = null;
	    try
	    {
                rootPoa = org.omg.PortableServer.POAHelper.narrow(
		    orb.resolve_initial_references("RootPOA"));
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		ex.printStackTrace();
		throw new RuntimeException();
	    }

            org.omg.PortableServer.POAManager manager =
                rootPoa.the_POAManager();
            com.ooc.OBPortableServer.POAManager obManager =
                com.ooc.OBPortableServer.POAManagerHelper.narrow(manager);

            //
            // Create persistent POA
            //
	    org.omg.PortableServer.POA adminPoa = null;
            try
            {
                org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[2];
                policies[0] =
                    rootPoa.create_lifespan_policy(
                        org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);
                policies[1] =
                    rootPoa.create_id_assignment_policy(
                        org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);

                adminPoa = rootPoa.create_POA("chatPOA", manager, policies);
            }
            catch(org.omg.PortableServer.POAPackage.AdapterAlreadyExists ex)
            {
		ex.printStackTrace();
		throw new RuntimeException();
            }
            catch(org.omg.PortableServer.POAPackage.InvalidPolicy ex)
            {
		ex.printStackTrace();
		throw new RuntimeException();
            }

            //
            // Resolve the Boot Manager
            //
            com.ooc.OB.BootManager bootManager = null;
	    try
	    {
		bootManager = com.ooc.OB.BootManagerHelper.narrow(
		    orb.resolve_initial_references("BootManager"));
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		ex.printStackTrace();
		throw new RuntimeException();
	    }

            //
            // Create implementation object
            //
            BroadcasterAdmin_impl broadcasterAdminImpl = 
		new BroadcasterAdmin_impl(orb, adminPoa, rootPoa);
            byte[] id = ("broadcasterAdmin").getBytes();
            try
            {
                adminPoa.activate_object_with_id(id, broadcasterAdminImpl);
            }
            catch(org.omg.PortableServer.POAPackage.WrongPolicy ex)
            {
		ex.printStackTrace();
		throw new RuntimeException();
            }
            catch(org.omg.PortableServer.POAPackage.ObjectAlreadyActive ex)
            {
		ex.printStackTrace();
		throw new RuntimeException();
            }
            catch(org.omg.PortableServer.POAPackage.ServantAlreadyActive ex)
            {
		ex.printStackTrace();
		throw new RuntimeException();
            }

            BroadcasterAdmin p = broadcasterAdminImpl._this(orb);

	    try
            {
                bootManager.add_binding(id, p);
            }
            catch(com.ooc.OB.BootManagerPackage.AlreadyExists ex)
            {
		ex.printStackTrace();
                throw new RuntimeException();
            }

            //
            // Save reference as html
            //
            try
            {
                String ref = orb.object_to_string(p);
                String refFile = "Chat.html";
                java.io.FileOutputStream file =
                    new java.io.FileOutputStream(refFile);
                java.io.PrintWriter out = new java.io.PrintWriter(file);
                out.println("<applet codebase=\"classes\" " +
                            "code=\"chat/Client.class\" " +
                            "archive=\"OB.jar\" " +
                            "width=500 height=300>");
                out.println("<param name=ior value=\"" + ref + "\">");
                out.println("<param name=org.omg.CORBA.ORBClass " +
                            "value=com.ooc.CORBA.ORB>");
                //
                // No need to specify ORBSingletonClass - it's ignored anyway
                //
                //out.println("<param name=org.omg.CORBA.ORBSingletonClass " +
                //            "value=com.ooc.CORBA.ORBSingleton>");
                out.println("</applet>");
                out.flush();
                file.close();
            }
            catch(java.io.IOException ex)
            {
                System.err.println("Can't write to `" +
                                   ex.getMessage() + "'");
                System.exit(1);
            }

            //
            // Print "server started" message
            //
            java.util.Date date = new java.util.Date();
            String d = date.toString();
            System.out.println("*** Server started at " + d);;

            //
            // Display the host and port
            //
            com.ooc.OCI.Acceptor[] acceptors = null;
	    try
	    {
                acceptors = obManager.get_acceptors();
	    }
	    catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
	    {
		ex.printStackTrace();
		throw new RuntimeException();
	    }

            for(int i = 0 ; i < acceptors.length ; i++)
            {
                com.ooc.OCI.AcceptorInfo info = acceptors[i].get_info();
                com.ooc.OCI.IIOP.AcceptorInfo iiopInfo =
                    com.ooc.OCI.IIOP.AcceptorInfoHelper.narrow(info);

                if(iiopInfo != null)
                {
                    String[] hosts = iiopInfo.hosts();
                    short portShort = iiopInfo.port();
                    int port;
                    if(portShort < 0)
                        port = 0xffff + (int)portShort + 1;
                    else
                        port = (int)portShort;

                    System.out.println("*** Running on host `" + hosts[0] +
                                       "', port number " + port);
                }
            }

            //
            // Run implementation
            //
            try
            {
                manager.activate();
            }
            catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
            {
		ex.printStackTrace();
                throw new RuntimeException();
            }
            orb.run();
	}
        catch(org.omg.CORBA.SystemException ex)
        {
            System.err.println(ex.getMessage());
            ex.printStackTrace();
            System.exit(1);
        }

        System.exit(0);
   }
}
