// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package factory;

public class Locator extends org.omg.PortableServer.ServantLocatorPOA
{
    private Database database_;
    private Reaper reaper_;
    private boolean debug_;

    public
    Locator(Database database, Reaper reaper, boolean debug)
    {
        database_ = database;
        reaper_ = reaper;
	debug_ = debug;
    }

    public org.omg.PortableServer.Servant
    preinvoke(byte[] oid,
              org.omg.PortableServer.POA poa,
              String operation,
              org.omg.PortableServer.ServantLocatorPackage.CookieHolder c)
    {
        //
        // First attempt to find the servant in the reaper
        //
        org.omg.PortableServer.Servant result = reaper_.acquireServant(oid);

        //
        // If that fails, then make sure the database contains a
        // record for the key, then create a new servant
        //
        if(result == null)
        {
            String id = new String(oid);
            int key = 0;
            try
            {
                key = Integer.parseInt(id);
            }
            catch(NumberFormatException ex)
            {
                throw new RuntimeException();
            }
            Database.DBRecordData data = database_.find(key);
            if(data != null)
            {
                result = new LocRecord_impl(data, debug_);
                reaper_.addServant(oid, result);
                reaper_.acquireServant(oid);
            }
        }

        if(result == null)
            throw new org.omg.CORBA.OBJECT_NOT_EXIST();

        return result;
    }

    public void
    postinvoke(byte[] oid,
               org.omg.PortableServer.POA poa,
               String operation,
               java.lang.Object the_cookie,
               org.omg.PortableServer.Servant the_servant)
    {
        reaper_.releaseServant(oid);
    }
}
