// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package pi;

import java.io.*;

final class Server
{
    private static int
    run(org.omg.CORBA.ORB orb, String[] args)
	throws org.omg.CORBA.UserException
    {
	//
	// Get the logging level from the command-line
	//
	if(args.length > 1)
	{
	    System.err.println("Usage: server [log-level]");
	    return 1;
	}

	short logLevel = 1;
	if(args.length > 0)
	    logLevel = Short.parseShort(args[0]);

	//
	// Resolve Root POA
	//
	org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
	org.omg.PortableServer.POA rootPOA = null;
	try
	{
	    rootPOA = org.omg.PortableServer.POAHelper.narrow(obj);
	}
	catch(org.omg.CORBA.BAD_PARAM ex)
	{
	    throw new RuntimeException();
	}

	//
	// Get a reference to the POA manager
	//
	org.omg.PortableServer.POAManager manager = rootPOA.the_POAManager();

	//
	// Create a persistent POA for the servants (use the logger policy
	// to set the server logging level)
	//
	org.omg.CORBA.Any any = orb.create_any();
	org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[6];

	org.omg.PortableServer.LifespanPolicyValueHelper.insert(any,
	    org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);
	policies[0] = orb.create_policy(
	    org.omg.PortableServer.LIFESPAN_POLICY_ID.value, any);

	org.omg.PortableServer.IdAssignmentPolicyValueHelper.insert(any,
	    org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);
	policies[1] = orb.create_policy(
	    org.omg.PortableServer.ID_ASSIGNMENT_POLICY_ID.value, any);

	org.omg.PortableServer.RequestProcessingPolicyValueHelper.insert(any,
	    org.omg.PortableServer.RequestProcessingPolicyValue.
		USE_SERVANT_MANAGER);
	policies[2] = orb.create_policy(
	    org.omg.PortableServer.REQUEST_PROCESSING_POLICY_ID.value, any);

	org.omg.PortableServer.ServantRetentionPolicyValueHelper.insert(any,
	    org.omg.PortableServer.ServantRetentionPolicyValue.NON_RETAIN);
	policies[3] = orb.create_policy(
	    org.omg.PortableServer.SERVANT_RETENTION_POLICY_ID.value, any);

	org.omg.PortableServer.ImplicitActivationPolicyValueHelper.insert(any,
	    org.omg.PortableServer.ImplicitActivationPolicyValue.
		NO_IMPLICIT_ACTIVATION);
	policies[4] = orb.create_policy(
	    org.omg.PortableServer.IMPLICIT_ACTIVATION_POLICY_ID.value, any);

	any.insert_short(logLevel);
	policies[5] = orb.create_policy(LOGGER_POLICY_ID.value, any);
	
	org.omg.PortableServer.POA persistentPOA =
	    rootPOA.create_POA("persistent", manager, policies);

	//
	// Create implementation objects
	//
	PIDemo_impl piDemoImpl = new PIDemo_impl(orb, persistentPOA);
	PIDemoDSI_impl piDemoDSIImpl = new PIDemoDSI_impl(orb, persistentPOA);

	//
	// Create the servant locator and register it with the POA
	//
	PIDemoLocator_impl locatorImpl =
	    new PIDemoLocator_impl(orb, piDemoImpl, piDemoDSIImpl);
	org.omg.PortableServer.ServantLocator locator = locatorImpl._this(orb);
	persistentPOA.set_servant_manager(locator);

	//
	// The PIDemo::call_other_impl method results in calls between
	// the two implementations.  Hence, the servers are also clients.
	// Create policy to enable client logging.
	//
	policies = new org.omg.CORBA.Policy[1];
	any.insert_short(logLevel);
	policies[0] = orb.create_policy(LOGGER_POLICY_ID.value, any);

	//
	// Get the object references (with the logging policy) for each
	// implementation
	//
	PIDemo piDemo = null;
	PIDemo piDemoDSI = null;
	try
	{
	    byte[] oid = ("PIDemo").getBytes();
	    obj = persistentPOA.create_reference_with_id(oid, "IDL:PIDemo:1.0");
	    obj = obj._set_policy_override(policies,
		org.omg.CORBA.SetOverrideType.ADD_OVERRIDE);
	    piDemo = PIDemoHelper.narrow(obj);

	    oid = ("PIDemoDSI").getBytes();
	    obj = persistentPOA.create_reference_with_id(oid, "IDL:PIDemo:1.0");
	    obj = obj._set_policy_override(policies,
		org.omg.CORBA.SetOverrideType.ADD_OVERRIDE);
	    piDemoDSI = PIDemoHelper.narrow(obj);
	}
	catch(org.omg.CORBA.BAD_PARAM ex)
	{
	    throw new RuntimeException();
	}

	//
	// Pass each implementation the other's corresponding object
	// reference
	//
	piDemoImpl.setOtherObj(piDemoDSI);
	piDemoDSIImpl.setOtherObj(piDemo);

	//
	// Write IOR of first object to file
	//
	try
	{
	    FileOutputStream file = new FileOutputStream("PIDemo.ref");
	    PrintWriter out = new PrintWriter(file);

	    out.println(orb.object_to_string(piDemo));

	    out.flush();
	    file.close();
	}
	catch(IOException ex)
	{
	    System.err.println("Can't write to `" + ex.getMessage() + "'");
	    return 1;
	}

	//
	// Run implementation
	//
	manager.activate();
	orb.run();

	return 0;
    }

    public static void
    main(String[] args)
    {
	java.util.Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");

	int status = 0;
	org.omg.CORBA.ORB orb = null;

	try
	{
	    //
	    // Initialize the request interceptors
	    //
	    RILogger.init(props);

	    //
	    // Initialize the ORB
	    //
	    orb = org.omg.CORBA.ORB.init(args, props);

	    //
	    // Start the server
	    //
	    status = run(orb, args);
	}
	catch(Exception ex)
	{
	    ex.printStackTrace();
	    status = 1;
	}

	if(orb != null)
	{
	    try
	    {
		orb.destroy();
	    }
	    catch(Exception ex)
	    {
		ex.printStackTrace();
		status = 1;
	    }
	}

	System.exit(status);
    }
}
