// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OB;

//
// DowncallStub is equivalent to the C++ class OB::MarshalStubImpl
//
public final class DowncallStub
{
    //
    // The ORBInstance object
    //
    private ORBInstance orbInstance_;

    //
    // The IOR and the original IOR
    //
    private org.omg.IOP.IOR IOR_;
    private org.omg.IOP.IOR origIOR_;

    //
    // The list of policies
    //
    private RefCountPolicyList policies_;

    //
    // All client/profile pairs
    //
    private java.util.Vector clientProfilePairs_;

    //
    // We need a class to carry the DowncallStub and Downcall across
    // a portable stub invocation
    //
    private class InvocationContext
    {
        DowncallStub downcallStub;
        Downcall downcall;
    }

    // ------------------------------------------------------------------
    // Private and protected member implementations
    // ------------------------------------------------------------------

    private synchronized Client
    getClientProfilePair(com.ooc.OCI.ProfileInfoHolder profileInfo)
        throws FailureException
    {
        //
        // Lazy initialization of the client/profile pairs
        //
        if(clientProfilePairs_ == null)
        {
            //
            // Get all clients that can be used
            //
            ClientManager clientManager = orbInstance_.getClientManager();
            clientProfilePairs_ =
                clientManager.getClientProfilePairs(IOR_, policies_.value);
        }

        //
        // If we can't get any client/profile pairs, set and raise the
        // failure exception, and let the stub handle this.
        //
        if(clientProfilePairs_.isEmpty())
        {
            CoreTraceLevels coreTraceLevels =
                orbInstance_.getCoreTraceLevels();
            if(coreTraceLevels.traceRetry() >= 2)
            {
                orbInstance_.getLogger().trace("retry",
                                               "no profiles available");
            }

            throw new FailureException(
                new org.omg.CORBA.TRANSIENT(
                    MinorCodes.describeTransient(
                        MinorCodes.MinorNoUsableProfileInIOR),
                    MinorCodes.MinorNoUsableProfileInIOR,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO));
        }

        ClientProfilePair clientProfilePair =
            (ClientProfilePair)clientProfilePairs_.elementAt(0);
        profileInfo.value = clientProfilePair.profile;
        return clientProfilePair.client;
    }

    private void
    destroy()
    {
        //
        // If the ORB has been destroyed then the clientManager can be nil
        //
        ClientManager clientManager = orbInstance_.getClientManager();

        if(clientManager != null && clientProfilePairs_ != null)
        {
            for(int i = 0 ; i < clientProfilePairs_.size() ; i++)
            {
                ClientProfilePair pair =
                    (ClientProfilePair)clientProfilePairs_.elementAt(i);
                clientManager.releaseClient(pair.client);
            }
        }

        clientProfilePairs_.removeAllElements();
    }

    protected void
    finalize()
        throws Throwable
    {
        destroy();

        super.finalize();
    }

    // ------------------------------------------------------------------
    // Public member implementations
    // ------------------------------------------------------------------

    public
    DowncallStub(ORBInstance orbInstance,
                 org.omg.IOP.IOR ior,
                 org.omg.IOP.IOR origIOR,
                 RefCountPolicyList policies)
    {
        clientProfilePairs_ = null;

        //
        // Save the ORBInstance object
        //
        orbInstance_ = orbInstance;

        //
        // Save the IOR
        //
        IOR_ = ior;
        origIOR_ = origIOR;

        //
        // Save the policies
        //
        policies_ = policies;
    }

    //
    // Operations to create new Downcall objects
    //
    public Downcall
    createDowncall(String op, boolean resp)
        throws FailureException
    {
        com.ooc.OCI.ProfileInfoHolder profile =
            new com.ooc.OCI.ProfileInfoHolder();
        Client client = getClientProfilePair(profile);
        Assert._OB_assert(client != null);

        if(!policies_.interceptor)
            return new Downcall(orbInstance_, client, profile.value,
                                policies_, op, resp);

 	PIManager piManager = orbInstance_.getPIManager();
	if(piManager.haveClientInterceptors())
	{
	    return new PIDowncall(orbInstance_, client, profile.value,
				  policies_, op, resp, IOR_, origIOR_,
				  piManager);
	}
	else
	{
            return new Downcall(orbInstance_, client, profile.value,
                                policies_, op, resp);
	}
    }

    public Downcall
    createLocateRequestDowncall()
        throws FailureException
    {
        com.ooc.OCI.ProfileInfoHolder profile =
            new com.ooc.OCI.ProfileInfoHolder();
        Client client = getClientProfilePair(profile);
        Assert._OB_assert(client != null);

	//
	// A LocateRequest is not seen by the interceptors
	//
	return new Downcall(orbInstance_, client, profile.value,
	                    policies_, "_locate", true);
    }

    public Downcall
    createPIArgsDowncall(String op, boolean resp,
                         ParameterDesc[] argDesc,
                         ParameterDesc retDesc,
                         org.omg.CORBA.TypeCode[] exceptionTC)
        throws FailureException
    {
        com.ooc.OCI.ProfileInfoHolder profile =
            new com.ooc.OCI.ProfileInfoHolder();
        Client client = getClientProfilePair(profile);
        Assert._OB_assert(client != null);

        if(!policies_.interceptor)
            return new Downcall(orbInstance_, client,
                                profile.value, policies_, op, resp);

 	PIManager piManager = orbInstance_.getPIManager();
	if(piManager.haveClientInterceptors())
	{
	    return new PIArgsDowncall(orbInstance_, client,
				      profile.value, policies_,
				      op, resp, IOR_, origIOR_, piManager,
				      argDesc, retDesc, exceptionTC);
	}
	else
	{
            return new Downcall(orbInstance_, client,
                                profile.value, policies_, op, resp);
	}
    }

    public Downcall
    createPIDIIDowncall(String op, boolean resp,
                        org.omg.CORBA.NVList args,
                        org.omg.CORBA.NamedValue result,
                        org.omg.CORBA.ExceptionList exceptions)
        throws FailureException
    {
        com.ooc.OCI.ProfileInfoHolder profile =
            new com.ooc.OCI.ProfileInfoHolder();
        Client client = getClientProfilePair(profile);
        Assert._OB_assert(client != null);

        if(!policies_.interceptor)
            return new Downcall(orbInstance_, client,
                                profile.value, policies_, op, resp);

 	PIManager piManager = orbInstance_.getPIManager();
	if(piManager.haveClientInterceptors())
	{
	    return new PIDIIDowncall(orbInstance_, client,
				     profile.value, policies_,
				     op, resp, IOR_, origIOR_, piManager,
				     args, result, exceptions);
	}
	else
	{
            return new Downcall(orbInstance_, client,
                                profile.value, policies_, op, resp);
	}
    }

    //
    // Marshalling interception points
    //

    public com.ooc.CORBA.OutputStream
    preMarshal(Downcall down)
        throws LocationForward, FailureException
    {
        return down.preMarshal();
    }

    public void
    marshalEx(Downcall down,
                  org.omg.CORBA.SystemException ex)
        throws LocationForward, FailureException
    {
        down.marshalEx(ex);
    }

    public void
    postMarshal(Downcall down)
        throws LocationForward, FailureException
    {
        down.postMarshal();
    }

    //
    // Methods to invoke requests and pick up replies
    //

    public void
    locate(Downcall down)
        throws LocationForward, FailureException
    {
        down.locate();
    }

    public void
    request(Downcall down)
        throws LocationForward, FailureException
    {
        down.request();
    }

    public void
    oneway(Downcall down)
        throws LocationForward, FailureException
    {
        down.oneway();
    }

    public void
    deferred(Downcall down)
        throws LocationForward, FailureException
    {
        down.deferred();
    }

    public void
    response(Downcall down)
        throws LocationForward, FailureException
    {
        down.response();
    }

    public boolean
    poll(Downcall down)
        throws LocationForward, FailureException
    {
        return down.poll();
    }

    //
    // Unmarshalling interception points
    //

    public com.ooc.CORBA.InputStream
    preUnmarshal(Downcall down)
        throws LocationForward, FailureException
    {
        return down.preUnmarshal();
    }

    public com.ooc.CORBA.InputStream
    preUnmarshal(Downcall down,
                     org.omg.CORBA.BooleanHolder uex)
        throws LocationForward, FailureException
    {
        com.ooc.CORBA.InputStream in = down.preUnmarshal();
        uex.value = down.userException();
        return in;
    }

    public void
    unmarshalEx(Downcall down,
                    org.omg.CORBA.SystemException ex)
        throws LocationForward, FailureException
    {
        down.unmarshalEx(ex);
    }

    public void
    postUnmarshal(Downcall down)
        throws LocationForward, FailureException
    {
        down.postUnmarshal();
    }

    //
    // Operations for handling user exceptions
    //

    public String
    unmarshalExceptionId(Downcall down)
    {
        return down.unmarshalExceptionId();
    }

    public void
    setUserException(Downcall down,
                     org.omg.CORBA.UserException ex,
                     String exId)
    {
        down.setUserException(ex, exId);
    }

    public void
    setUserException(Downcall down, org.omg.CORBA.UserException ex)
    {
        down.setUserException(ex);
    }

    //
    // Handle a FailureException
    //
    public synchronized void
    handleFailureException(Downcall down, FailureException ex)
        throws FailureException
    {
        //
        // Only called if there is really a failure
        //
        Assert._OB_assert(ex.exception != null);

        //
        // If there was a failure, release the client and remove the
        // faulty client/profile pair, whether we retry or not
        //
        Client client = down.client();
        com.ooc.OCI.ProfileInfo profile = down.profileInfo();

        for(int i = 0 ; i < clientProfilePairs_.size() ; i++)
        {
            ClientProfilePair pair =
                (ClientProfilePair)clientProfilePairs_.elementAt(i);
            if(pair.client == client && pair.profile == profile)
            {
                ClientManager clientManager =
                    orbInstance_.getClientManager();

		//
		// Make sure the ORB has not been destroyed
		//
		if(clientManager == null)
		    throw new org.omg.CORBA.BAD_INV_ORDER(
		        MinorCodes.describeBadInvOrder(
			    MinorCodes.MinorShutdownCalled),
			MinorCodes.MinorShutdownCalled,
			org.omg.CORBA.CompletionStatus.COMPLETED_NO);

                clientManager.releaseClient(pair.client);
                clientProfilePairs_.removeElementAt(i);
                break;
            }
        }

	//
	// We only retry upon COMM_FAILURE, TRANSIENT, and NO_RESPONSE
	//
	try
	{
	    throw ex.exception;
	}
        catch(org.omg.CORBA.COMM_FAILURE e)
	{
	}
        catch(org.omg.CORBA.TRANSIENT e)
	{
	}
        catch(org.omg.CORBA.NO_RESPONSE e)
	{
	}
        catch(org.omg.CORBA.SystemException e)
	{
            throw ex; // Not "throw e;"!
	}

        //
        // We can't retry if RETRY_STRICT or RETRY_NEVER is set and the
	// completion status is not COMPLETED_NO
        //
        if(policies_.retry.mode != RETRY_ALWAYS.value &&
           ex.exception.completed !=
           org.omg.CORBA.CompletionStatus.COMPLETED_NO)
            throw ex;

        //
        // If no client/profile pairs are left, we cannot retry either
        //
        if(clientProfilePairs_.isEmpty())
        {
            CoreTraceLevels coreTraceLevels =
                orbInstance_.getCoreTraceLevels();
            if(coreTraceLevels.traceRetry() >= 2)
            {
                String msg = "no profiles left to try\n" +
                             ex.exception.getMessage();

                orbInstance_.getLogger().trace("retry", msg);
            }

            throw ex;
        }

        //
        // OK, let's continue with the next profile
        //
        CoreTraceLevels coreTraceLevels = orbInstance_.getCoreTraceLevels();
        if(coreTraceLevels.traceRetry() >= 2)
        {
            String msg = "trying next profile\n" +
                         ex.exception.getMessage();

            orbInstance_.getLogger().trace("retry", msg);
        }
    }

    public boolean
    locate_request()
        throws LocationForward, FailureException
    {
        while(true)
        {
            com.ooc.OB.Downcall down = createLocateRequestDowncall();

            try
            {
                try
                {
                    //
                    // Get the client and force a binding
                    //
                    Client client = down.client();
                    client.bind(policies_.connectTimeout);

                    //
                    // If the LocateRequest policy is false, then return now
                    //
                    if(!policies_.locateRequest)
                        return true;

                    //
                    // If the client doesn't support twoway invocations,
                    // then silently pretend the locate request succeeded
                    //
                    if(!client.twoway())
                        return true;
                }
                catch(org.omg.CORBA.SystemException ex)
                {
                    throw new FailureException(ex);
                }

                preMarshal(down);
                postMarshal(down);
                locate(down);
                preUnmarshal(down);
                postUnmarshal(down);
                return true;
            }
            catch(org.omg.CORBA.OBJECT_NOT_EXIST ex)
            {
                return false;
            }
            catch(FailureException ex)
            {
                handleFailureException(down, ex);
            }
        }
    }

    public com.ooc.OCI.ConnectorInfo
    get_oci_connector_info()
    {
        try
        {
            com.ooc.OCI.ProfileInfoHolder profileInfo =
                new com.ooc.OCI.ProfileInfoHolder();
            com.ooc.OB.Client client = getClientProfilePair(profileInfo);
            com.ooc.OB.Assert._OB_assert(client != null);
            return client.connectorInfo();
        }
        catch(com.ooc.OB.FailureException ex)
        {
            Assert._OB_assert(false);
            return null; // The compiler needs this
        }
    }

    public com.ooc.OCI.TransportInfo
    get_oci_transport_info()
    {
        try
        {
            com.ooc.OCI.ProfileInfoHolder profileInfo =
                new com.ooc.OCI.ProfileInfoHolder();
            com.ooc.OB.Client client = getClientProfilePair(profileInfo);
            com.ooc.OB.Assert._OB_assert(client != null);
            return client.transportInfo();
        }
        catch(com.ooc.OB.FailureException ex)
        {
            Assert._OB_assert(false);
            return null; // The compiler needs this
        }
    }

    //
    // Prepare a request from a portable stub
    //
    public com.ooc.CORBA.OutputStream
    setupRequest(org.omg.CORBA.Object self,
                 String operation,
                 boolean responseExpected)
        throws LocationForward, FailureException
    {
        while(true)
        {
            com.ooc.OB.Downcall downcall =
                createDowncall(operation, responseExpected);

            try
            {
                com.ooc.CORBA.OutputStream out = preMarshal(downcall);
                //
                // The InvocationContext is associated with the OutputStream
                // and retrieved by invoke()
                //
                InvocationContext ctx = new InvocationContext();
                ctx.downcallStub = this;
                ctx.downcall = downcall;
                out._OB_invocationContext(ctx);
                return out;
            }
            catch(FailureException ex)
            {
                handleFailureException(downcall, ex);
            }
        }
    }

    //
    // Invoke a request from a portable stub
    //
    public com.ooc.CORBA.InputStream
    invoke(org.omg.CORBA.Object self, com.ooc.CORBA.OutputStream out)
        throws org.omg.CORBA.portable.ApplicationException,
               org.omg.CORBA.portable.RemarshalException,
               LocationForward, FailureException
    {
        //
        // We should have an InvocationContext associated with the
        // OutputStream
        //
        com.ooc.CORBA.OutputStream o = (com.ooc.CORBA.OutputStream)out;
        InvocationContext ctx = (InvocationContext)o._OB_invocationContext();
        Assert._OB_assert(ctx != null);

        //
        // If the DowncallStub has changed, then remarshal
        //
        if(ctx.downcallStub != this)
            throw new org.omg.CORBA.portable.RemarshalException();

        Downcall down = ctx.downcall;

        //
        // No while loop here - if we need to reinvoke, we must raise
        // RemarshalException
        //
        try
        {
            down.postMarshal();

            boolean response = down.responseExpected();
            if(response)
                down.request();
            else
                down.oneway();

            if(response)
            {
                com.ooc.CORBA.InputStream in = down.preUnmarshal();

                if(down.userException())
                {
                    String id = null;

                    try
                    {
                        //
                        // Extract the exception's repository ID
                        //
                        id = down.unmarshalExceptionId();
                    }
                    catch(org.omg.CORBA.SystemException ex)
                    {
                        down.unmarshalEx(ex);
                    }

                    //
                    // We're using portable stubs, so we'll never
                    // be given the user exception instance. Therefore,
                    // we might as well invoke the interceptors now.
                    //
                    down.setUserException(id);
                    down.postUnmarshal();

                    throw new org.omg.CORBA.portable.
                              ApplicationException(id, in);
                }
                else
                {
                    //
                    // We're using portable stubs, so we'll never
                    // know the unmarshalled results. Therefore,
                    // we might as well invoke the interceptors now.
                    //
                    down.postUnmarshal();

                    return in;
                }
            }
            else
            {
                down.preUnmarshal();
                down.postUnmarshal();
                return null;
            }
        }
        catch(FailureException ex)
        {
            handleFailureException(down, ex);
        }

        //
        // If we reach this point, then we need to reinvoke
        //
        throw new org.omg.CORBA.portable.RemarshalException();
    }
}
