// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OB;

public final class ServerManager
{
    private boolean destroy_; // True if destroy() was called
    private CollocatedServer collocatedServer_; // The collocated server
    // All servers
    private java.util.Vector allServers_ = new java.util.Vector();

    // ----------------------------------------------------------------------
    // ServerManager private and protected member implementations
    // ----------------------------------------------------------------------

    private void
    createServers()
    {
    }

    protected void
    finalize()
	throws Throwable
    {
        Assert._OB_assert(destroy_);
        Assert._OB_assert(allServers_.isEmpty());
        Assert._OB_assert(collocatedServer_ == null);

        super.finalize();
    }

    // ----------------------------------------------------------------------
    // ServerManager public member implementations
    // ----------------------------------------------------------------------

    public
    ServerManager(ORBInstance orbInstance,
                  com.ooc.OCI.Acceptor[] acceptors,
                  com.ooc.OB.OAInterface oaInterface,
                  int concModel)
    {
        destroy_ = false;

	//
	// Create a server for each acceptor, and the collocated server
	//
	for(int i = 0 ; i < acceptors.length ; i++)
	{
	    GIOPServer server = new GIOPServer(orbInstance, acceptors[i],
                                               oaInterface, concModel);
	    allServers_.addElement(server);
	}
	collocatedServer_ = new CollocatedServer(oaInterface, concModel);
        allServers_.addElement(collocatedServer_);
    }

    public synchronized void
    destroy()
    {
        //
        // Don't destroy twice
        //
        if(destroy_)
            return;

        //
        // Set the destroy flag
        //
	destroy_ = true;

	//
	// Destroy all servers
	//
	java.util.Enumeration e = allServers_.elements();
	while(e.hasMoreElements())
	    ((Server)e.nextElement()).destroy();

        allServers_.removeAllElements();
	collocatedServer_ = null;
    }

    public synchronized void
    hold()
    {
	java.util.Enumeration e = allServers_.elements();
	while(e.hasMoreElements())
	    ((Server)e.nextElement()).hold();
    }

    public synchronized void
    activate()
    {
	java.util.Enumeration e = allServers_.elements();
	while(e.hasMoreElements())
	    ((Server)e.nextElement()).activate();
    }

    public synchronized CollocatedServer
    getCollocatedServer()
    {
	return collocatedServer_;
    }
}
