// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test.multi;

import org.omg.CORBA.*;

public class Client extends test.common.TestBase
{
    public static int
    run(ORB orb, String[] args)
	throws org.omg.CORBA.UserException
    {
	//
	// Get "multi" object
	//
	org.omg.CORBA.Object obj =
	    orb.string_to_object("relfile:/Multi.ref");
	if(obj == null)
	{
	    System.err.println("test.multi.Client: cannot read IOR from " +
			       "Multi.ref");
	    return 1;
	}

	Multi multi = MultiHelper.narrow(obj);

	//
	// 5 servers are running. When the 6th is deactivated, I
	// should get an exception.
	//
	int oldpid = 0;
	for(int i = 0 ; i < 5 ; i++)
	{
	    System.out.print("Getting process ID of server #" + (i + 1) +
			     "... ");
	    System.out.flush();

	    while(true)
	    {
		int pid = multi.getpid();
		if(oldpid != pid)
		{
		    oldpid = pid;
		    break;
		}
	    }

	    System.out.println("Done!");

	    System.out.print("Deactivating server... ");
	    System.out.flush();
	    multi.deactivate();
	    System.out.println("Done!");
	}

	System.out.print("Checking whether I now get an exception... ");
	System.out.flush();

	try
	{
	    while(true)
	    {
		int pid = multi.getpid();
		TEST(pid == oldpid);
	    }
	}
	catch(TRANSIENT ex)
	{
	    System.out.println("Done!");
	    System.out.print("Must be COMPLETED_NO... ");
	    System.out.flush();
	    TEST(ex.completed == CompletionStatus.COMPLETED_NO);
	    System.out.println("Done!");
	}

        return 0;
    }

    public static void
    main(String[] args)
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        int status = 0;
        ORB orb = null;

        try
        {
            orb = ORB.init(args, props);
            status = run(orb, args);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
