// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************
 
package com.ooc.CosPropertyService;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POAPackage.*;
import java.io.*;
import java.util.*;

public class Server
{
    private static String progName = "com.ooc.CosPropertyService.Server";

    static void
    usage()
    {
	System.err.println("Usage:");
        System.err.println(progName + " [options]\n" +
                           "\n" +
                           "Options:\n" +
                           "-h, --help             " +
                           "Show this message.\n" +
                           "-v, --version          " +
                           "Show version number.\n" +
                           "-i, --ior              " +
                           "Print IOR on standard output.");
    }

    static int 
    run(ORB orb, String args[])
    {
        //
        // Resolve Root POA
        //
        org.omg.CORBA.Object poaObj = null;
        try
        {
	    poaObj = orb.resolve_initial_references("RootPOA");
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName ex)
        {
	    throw new RuntimeException();
        }
	    POA rootPOA = POAHelper.narrow(poaObj);

        //
        // Get a reference to the POA manager
        //
        POAManager manager = rootPOA.the_POAManager();

        //
        // Create persistent poa
        //
        POA poa = null;
        try
        {
            Policy[] policies = new org.omg.CORBA.Policy[2];
            policies[0] =
                rootPOA.create_lifespan_policy(
                    org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);
            policies[1] =
                rootPOA.create_id_assignment_policy(
                    org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);

            poa = rootPOA.create_POA("PropertyServicePOA", manager, policies);
        }
        catch(org.omg.PortableServer.POAPackage.AdapterAlreadyExists ex)
        {
        }
        catch(org.omg.PortableServer.POAPackage.InvalidPolicy ex)
        {
        }
        com.ooc.OB.Assert._OB_assert(poa != null);

        //
        // Resolve the Boot Manager
        //
        com.ooc.OB.BootManager bootManager = null;
        try
        {
            bootManager = com.ooc.OB.BootManagerHelper.narrow(
                orb.resolve_initial_references("BootManager"));
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName ex)
        {
            throw new RuntimeException();
        }

        //
        // Get options
        //
        boolean ior = false;
	    
        for(int i = 0 ; i < args.length && args[i].startsWith("-") ; i++)
        {
	    if(args[i].equals("--help") || args[i].equals("-h"))
	    {
	        usage();
	        return 0;
	    }
	    else if(args[i].equals("--version") || args[i].equals("-v"))
	    {
	        System.err.println("ORBacus Property Service " +
				   com.ooc.OB.Version.version);
	        return 0;
	    }
	    else if(args[i].equals("--ior") || args[i].equals("-i"))
	    {
	        ior = true;
	    }
	    else
	    {
	        System.err.println(progName + ": Unknown option `" +
	    		           args[i] + "'\n");
	        usage();
	        return 1;
	    }
        }

        //
        // Create factory
        //
        PropertySetDefFactory_impl defFactory =
	    new PropertySetDefFactory_impl(rootPOA, orb);
        byte[] oid = ("PropertyService").getBytes();
        try
        {
	    poa.activate_object_with_id(oid, defFactory);
        }
        catch(WrongPolicy ex)
        {
	    throw new RuntimeException();
        }
        catch(ObjectAlreadyActive ex)
        {
	    throw new RuntimeException();
        }
        catch(ServantAlreadyActive ex)
        {
	    throw new RuntimeException();
        }

        //
        // Register with BootManager
        //
        org.omg.CORBA.Object obj = null;
        try
        {
	    obj = poa.id_to_reference(oid);
	    bootManager.add_binding(oid, obj);
        }
        catch(ObjectNotActive ex)
        {
	    throw new RuntimeException();
        }
        catch(WrongPolicy ex)
        {
	    throw new RuntimeException();
        }
        catch(com.ooc.OB.BootManagerPackage.AlreadyExists ex)
        {
	    throw new RuntimeException();
        }

	//
        // Activate the POAManager before displaying the IOR so that
        // connection attempts are not rejected
        //
        try
        {
	    manager.activate();
        }
        catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
        {
	    throw new RuntimeException();
        }

        //
        // Print IOR on standard output
        //
        if(ior)
        {
	    String s = orb.object_to_string(obj);
	    System.out.println(s);
        }

        //
        // Run implementation
        //
        orb.run();

	return 0;
    }

    public static void
    main(String args[])
    {
        int status = 0;
        ORB orb = null;

        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        try
        {
            args = com.ooc.CORBA.ORB.ParseArgs(args, props, null);

	    //
	    // Check properties first
	    //
	    {
		String propRoot = "ooc.property.";
		Enumeration keys = props.keys();
		while(keys.hasMoreElements())
		{
		    String key = (String)keys.nextElement();
		    if(!key.startsWith(propRoot))
		        continue;

		    String value = props.getProperty(key);
		    com.ooc.OB.Assert._OB_assert(value != null);

		    if(key.equals("ooc.property.endpoint"))
		    {
			//
			// Set the ooc.orb.oa.endpoint property from this
			// value if not already set
			//
			String oaEndpoint =
                            props.getProperty("ooc.orb.oa.endpoint");
			if (oaEndpoint == null)
			    props.put("ooc.orb.oa.endpoint", value);
		    }
		    else
		    {
			System.err.println(progName + ": " + key +
					   "': unknown property");
		    }
		}
            }

	    props.put("ooc.orb.conc_model", "blocking");

            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);
            status = run(orb, argsH.value);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
