// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test;

import org.omg.CORBA.*;
import org.omg.CosPropertyService.*;

//
// Tests for constrained PropertySetDef
//
final class TestConstrained
{
    static void
    test(ORB orb, PropertySetDefFactory factory)
	throws UserException
    {
	PropertySetDef set;
	Any any = orb.create_any();
	boolean except;

        //
        // Set up PropertySetDef constraints, only tk_string is allowed
        //
        TypeCode[] allowedTypes = new TypeCode[1];
        allowedTypes[0] = orb.get_primitive_tc(TCKind.tk_string);
        PropertyDef[] allowedProperties = new PropertyDef[3];
        any.insert_string("property_value");
        allowedProperties[0] =
            new PropertyDef("read_only", any, PropertyModeType.read_only);
        allowedProperties[1] =
            new PropertyDef("fixed_normal", any,
                            PropertyModeType.fixed_normal);
        allowedProperties[2] =
            new PropertyDef("fixed_readonly", any,
                            PropertyModeType.fixed_readonly);

	set = factory.create_constrained_propertysetdef(allowedTypes,
							allowedProperties);

	{
	    System.out.print("  Testing get_allowed_property_types()... ");

	    PropertyTypesHolder types = new PropertyTypesHolder();
	    set.get_allowed_property_types(types);

	    com.ooc.OB.Assert._OB_assert(types.value.length == 1);

	    com.ooc.OB.Assert._OB_assert(types.value[0].equal(
		orb.get_primitive_tc(TCKind.tk_string)));

	    System.out.println("Done!");
	}

	{
	    System.out.print("  Testing get_allowed_properties()... ");

	    PropertyDefsHolder properties = new PropertyDefsHolder();
	    set.get_allowed_properties(properties);

	    com.ooc.OB.Assert._OB_assert(properties.value.length == 3);

	    System.out.println("Done!");
	}

	{
	    System.out.print("  Testing define_property_with_mode()... ");

	    //
	    // Try to add allowed properties
	    //
	    any.insert_string("property_value");
	    try
	    {
		set.define_property_with_mode("fixed_normal", any,
					      PropertyModeType.fixed_normal);
		set.define_property_with_mode("read_only", any,
					      PropertyModeType.read_only);
		set.define_property_with_mode("fixed_readonly", any,
					      PropertyModeType.fixed_readonly);
	    }
	    catch(ReadOnlyProperty ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }
	    catch(ConflictingProperty ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }
	    catch(UnsupportedProperty ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }
	    catch(UnsupportedTypeCode ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }
	    catch(InvalidPropertyName ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }
	    catch(UnsupportedMode ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }

	    //
	    // Try to change readonly property
	    //
	    any.insert_string("property_value");
	    except = false;
	    try
	    {
		set.define_property("read_only", any);
	    }
	    catch(ReadOnlyProperty ex)
	    {
		// This exception must be thrown
		except = true;
	    }
	    catch(ConflictingProperty ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }
	    catch(UnsupportedProperty ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }
	    catch(UnsupportedTypeCode ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }
	    catch(InvalidPropertyName ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }

            com.ooc.OB.Assert._OB_assert(except);

	    //
	    // Try to add illegal TypeCode
	    //
	    any.insert_long(1);
	    except = false;
	    try
	    {
		set.define_property("fixed_normal", any);
	    }
	    catch(ReadOnlyProperty ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }
	    catch(ConflictingProperty ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }
	    catch(UnsupportedProperty ex)
	    {
		// This exception must be thrown
		except = true;
	    }
	    catch(UnsupportedTypeCode ex)
	    {
		// This exception must be thrown
		except = true;
	    }
	    catch(InvalidPropertyName ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }

	    com.ooc.OB.Assert._OB_assert(except);

	    System.out.println("Done!");
	}

	{
	    System.out.print("  Testing get_property_mode()... ");

	    PropertyModeType fixed_normal = null;
	    PropertyModeType read_only = null;
	    PropertyModeType fixed_readonly = null;

	    try
	    {
		fixed_normal = set.get_property_mode("fixed_normal");
		read_only = set.get_property_mode("read_only");
		fixed_readonly = set.get_property_mode("fixed_readonly");
	    }
	    catch(InvalidPropertyName ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }
	    catch(PropertyNotFound ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }

	    com.ooc.OB.Assert._OB_assert(
		fixed_normal == PropertyModeType.fixed_normal ||
		read_only == PropertyModeType.read_only ||
		fixed_readonly == PropertyModeType.fixed_readonly);

	    System.out.println("Done!");
	}
    }
}

