// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test;

import org.omg.CORBA.*;
import org.omg.CosPropertyService.*;

//
// Tests for unconstrained PropertySetDef
//
final class TestUnconstrained
{
    static void
    test(ORB orb, PropertySetDefFactory factory)
    {
	int i, j;

	PropertySetDef set = factory.create_propertysetdef();

	{
	    System.out.print("  Testing define_property()... ");

	    for(i = 0 ; i < 10 ; i++)
	    {
		Any any = orb.create_any();
		any.insert_long(i);

		try
		{
		    set.define_property("property_" + i, any);
		}
		catch(ReadOnlyProperty ex)
		{
		    com.ooc.OB.Assert._OB_assert(false);
		}
		catch(ConflictingProperty ex)
		{
		    com.ooc.OB.Assert._OB_assert(false);
		}
		catch(UnsupportedProperty ex)
		{
		    com.ooc.OB.Assert._OB_assert(false);
		}
		catch(UnsupportedTypeCode ex)
		{
		    com.ooc.OB.Assert._OB_assert(false);
		}
		catch(InvalidPropertyName ex)
		{
		    com.ooc.OB.Assert._OB_assert(false);
		}
	    }

	    System.out.println("Done!");
	}

	{
	    System.out.print("  Testing define_properties()... ");

	    Property[] nproperties = new Property[10];

	    for(i = 10 ; i < 20 ; i++)
	    {
		Any any = orb.create_any();
		any.insert_long(i);
		nproperties[i - 10] = new Property("property_" + i, any);
	    }

	    try
	    {
		set.define_properties(nproperties);
	    }
	    catch(MultipleExceptions ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }

	    System.out.println("Done!");
	}

	{
	    System.out.print("  Testing get_all_property_names() " +
			     "without iterator... ");

	    PropertyNamesHolder nh = new PropertyNamesHolder();
	    PropertyNamesIteratorHolder ih = new PropertyNamesIteratorHolder();

	    int n = set.get_number_of_properties();
	    set.get_all_property_names(n, nh, ih);

	    String[] names = nh.value;
	    com.ooc.OB.Assert._OB_assert(names.length == n);

	    java.util.Vector v = new java.util.Vector();
	    for(i = 0 ; i < n ; i++)
		v.addElement("property_" + i);

	    //
	    // Check if each property name exists only once
	    //
	    for(i = 0 ; i < n ; i++)
	    {
                for(j = 0 ; j < v.size() ; j++)
                    if(names[i].equals(v.elementAt(j)))
                        break;

                com.ooc.OB.Assert._OB_assert(j < v.size());
                v.removeElementAt(j);
	    }

	    System.out.println("Done!");
	}

	{
	    System.out.print("  Testing get_all_property_names() " +
			     "with iterator... ");

	    PropertyNamesHolder nh = new PropertyNamesHolder();
	    PropertyNamesIteratorHolder ih = new PropertyNamesIteratorHolder();

	    //
	    // Get one name without iterator
	    //
	    set.get_all_property_names(1, nh, ih);
	    com.ooc.OB.Assert._OB_assert(nh.value.length == 1);

	    int n = set.get_number_of_properties();

	    String[] names = new String[n];
	    names[0] = nh.value[0];

	    //
	    // Get remaining names with iterator
	    //
	    StringHolder h = new StringHolder();
	    i = 1;
	    while(ih.value.next_one(h))
		names[i++] = h.value;

	    java.util.Vector v = new java.util.Vector();
	    for(i = 0 ; i < n ; i++)
		v.addElement("property_" + i);

	    //
	    // Check if each property name exists only once
	    //
	    for(i = 0 ; i < n ; i++)
	    {
                for(j = 0 ; j < v.size() ; j++)
                    if(names[i].equals(v.elementAt(j)))
                        break;

                com.ooc.OB.Assert._OB_assert(j < v.size());
                v.removeElementAt(j);
	    }

	    ih.value.destroy();

	    System.out.println("Done!");
	}

	{
	    System.out.print("  Testing get_property_value()... ");

	    int n = set.get_number_of_properties();
	    for(i = 0 ; i < n ; i++)
	    {
		Any any = null;

		try
		{
		    any = set.get_property_value("property_" + i);
		}
		catch(InvalidPropertyName ex)
		{
		    com.ooc.OB.Assert._OB_assert(false);
		}
		catch(PropertyNotFound ex)
		{
		    com.ooc.OB.Assert._OB_assert(false);
		}

		int value = any.extract_long();
		com.ooc.OB.Assert._OB_assert(value == i);
	    }

	    System.out.println("Done!");
	}

	{
	    System.out.print("  Testing get_properties()... ");

	    int n = set.get_number_of_properties();
	    String[] names = new String[n];
	    for(i = 0 ; i < n ; i++)
		names[i] = "property_" + i;

	    PropertiesHolder nproperties = new PropertiesHolder();
	    set.get_properties(names, nproperties);

	    java.util.Vector vs =
		new java.util.Vector(set.get_number_of_properties());
	    java.util.Vector vi =
		new java.util.Vector(set.get_number_of_properties());
	    for(i = 0 ; i < set.get_number_of_properties() ; i++)
	    {
		vs.addElement("property_" + i);
		vi.addElement(new Integer(i));
	    }

	    //
	    // Check if each property name exists only once
	    //
	    int match = 0;
	    for(i = 0 ; i < n ; i++)
	    {
		for(j = 0 ; j < n ; j++)
		{
		    String s = nproperties.value[i].property_name;
		    if(!s.equals((String)(vs.elementAt(j))))
			continue;

		    Any any = nproperties.value[i].property_value;
		    int value = any.extract_long();
		    if(value != (((Integer)(vi.elementAt(j))).intValue()))
			continue;

		    //
		    // Mark current element as invalid, it has passed the test
		    //
		    vs.setElementAt("", j);
		    vi.setElementAt(new Integer(-1), j);

		    match++;
		}
	    }

	    com.ooc.OB.Assert._OB_assert(match == n);

	    System.out.println("Done!");
	}

	{
	    System.out.print("  Testing get_all_properties() " +
			     "without iterator... ");

	    PropertiesHolder ph = new PropertiesHolder();
	    PropertiesIteratorHolder ih = new PropertiesIteratorHolder();
	    int n = set.get_number_of_properties();
	    set.get_all_properties(n, ph, ih);
	    com.ooc.OB.Assert._OB_assert(ph.value.length == n);

	    java.util.Vector vs = new java.util.Vector(n);
	    java.util.Vector vi = new java.util.Vector(n);
	    for(i = 0 ; i < n ; i++)
	    {
		vs.addElement("property_" + i);
		vi.addElement(new Integer(i));
	    }

	    //
	    // Check if each property name exists only once
	    //
	    int match = 0;
	    for(i = 0 ; i < n ; i++)
	    {
		for(j = 0 ; j < n ; j++)
		{
		    String s = ph.value[i].property_name;
		    if(!s.equals((String)(vs.elementAt(j))))
			continue;

		    Any any = ph.value[i].property_value;
		    int value = any.extract_long();
		    if(value != (((Integer)(vi.elementAt(j))).intValue()))
			continue;

		    //
		    // Mark current element as invalid, it has passed the test
		    //
		    vs.setElementAt("", j);
		    vi.setElementAt(new Integer(-1), j);

		    match++;
		}
	    }

	    com.ooc.OB.Assert._OB_assert(match == n);

	    System.out.println("Done!");
	}

	{
	    System.out.print("  Testing get_all_properties() " +
			     "with iterator... ");

	    PropertiesHolder ph = new PropertiesHolder();
	    PropertiesIteratorHolder ih = new PropertiesIteratorHolder();

            //
            // Get one property without iterator
            //
 	    set.get_all_properties(1, ph, ih);
	    com.ooc.OB.Assert._OB_assert(ph.value.length == 1);

	    int n = set.get_number_of_properties();
	    Property[] properties = new Property[n];
	    properties[0] = ph.value[0];

	    //
	    // Get remaining properties with iterator
	    //
            PropertyHolder h = new PropertyHolder();
            i = 1;
            while(ih.value.next_one(h))
                properties[i++] = h.value;
 
	    java.util.Vector vs = new java.util.Vector(n);
	    java.util.Vector vi = new java.util.Vector(n);
	    for(i = 0 ; i < n ; i++)
	    {
		vs.addElement("property_" + i);
		vi.addElement(new Integer(i));
	    }

	    //
	    // Check if each property name exists only once
	    //
	    int match = 0;
	    for(i = 0 ; i < n ; i++)
	    {
		for(j = 0 ; j < n ; j++)
		{
		    String s = properties[i].property_name;
		    if(!s.equals((String)(vs.elementAt(j))))
			continue;

		    Any any = properties[i].property_value;
		    int value = any.extract_long();
		    if(value != (((Integer)(vi.elementAt(j))).intValue()))
			continue;

		    //
		    // Mark current element as invalid, it has passed the test
		    //
		    vs.setElementAt("", j);
		    vi.setElementAt(new Integer(-1), j);

		    match++;
		}
	    }

	    com.ooc.OB.Assert._OB_assert(match == n);

	    ih.value.destroy();

	    System.out.println("Done!");
	}

	{
	    System.out.print("  Testing is_property_defined()... ");

	    for(i = 0 ; i < set.get_number_of_properties() ; i++)
	    {
		String name = "property_" + i;

		try
		{
		    com.ooc.OB.Assert._OB_assert(set.is_property_defined(name));
		}
		catch(InvalidPropertyName ex)
		{
		    com.ooc.OB.Assert._OB_assert(false);
		}
	    }

	    System.out.println("Done!");
	}

	{
	    System.out.print("  Testing delete_property()... ");

	    //
	    // Delete first 5 properties, 15 remain
	    //
	    for(i = 0 ; i < 5 ; i++)
	    {
		String name = "property_" + i;

		try
		{
		    set.delete_property(name);
		}
		catch(InvalidPropertyName ex)
		{
		    com.ooc.OB.Assert._OB_assert(false);
		}
		catch(PropertyNotFound ex)
		{
		    com.ooc.OB.Assert._OB_assert(false);
		}
		catch(FixedProperty ex)
		{
		    com.ooc.OB.Assert._OB_assert(false);
		}
	    }

	    System.out.println("Done!");
	}

	{
	    System.out.print("  Testing delete_properties()... ");

	    //
	    // Delete next 10 properties, 5 remain
	    //
	    String[] names = new String[10];
	    for(i = 5 ; i < 15 ; i++)
		names[i - 5] = "property_" + i;

	    try
	    {
		set.delete_properties(names);
	    }
	    catch(MultipleExceptions ex)
	    {
		com.ooc.OB.Assert._OB_assert(false);
	    }

	    PropertyNamesHolder nh = new PropertyNamesHolder();
	    PropertyNamesIteratorHolder ih = new PropertyNamesIteratorHolder();

	    int n = set.get_number_of_properties();
	    com.ooc.OB.Assert._OB_assert(n == 5);
	    set.get_all_property_names(n, nh, ih);

	    String[] names2 = nh.value;

	    for(i = 0 ; i < n ; i++)
	    {
		if(!names2[i].equals("property_15") &&
		   !names2[i].equals("property_16") &&
		   !names2[i].equals("property_17") &&
		   !names2[i].equals("property_18") &&
		   !names2[i].equals("property_19"))
		{
		    com.ooc.OB.Assert._OB_assert(false);
		}
	    }

	    System.out.println("Done!");
	}

	{
	    System.out.print("  Testing delete_all_properties()... ");

	    //
	    // 10 properties remain and are deleted now
	    //
	    com.ooc.OB.Assert._OB_assert(set.delete_all_properties());

	    com.ooc.OB.Assert._OB_assert(set.get_number_of_properties() == 0);

	    System.out.println("Done!");
	}
    }
}
