// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.UDP;

final class Message 
{
    private com.ooc.OCI.Buffer buf_;
    private int payload_;
    private int lastPayload_;
    private int numberOfPackets_;
    private boolean [] packetsReceived_;
    private boolean completed_ = false;
    private long lastPacketTime_ = 0;

    Message(int messageSize, int payload)
    {
	buf_ = new com.ooc.OCI.Buffer(messageSize);
	payload_ = payload;

	//
	// Fragment the message
	//
	lastPayload_ = messageSize % payload;
	numberOfPackets_ = messageSize / payload;
	if(lastPayload_ > 0)
	    numberOfPackets_++;
	
	//
	// Initialize the packets received array
	//
	packetsReceived_ = new boolean[numberOfPackets_];
	for(int i = 0 ; i < numberOfPackets_ ; i++)
	    packetsReceived_[i] = false;
    }

    void 
    add(com.ooc.OCI.Buffer packet, int packetNum)
    {
	//
        // Update the arrival time of the last packet
        //
        lastPacketTime_ = System.currentTimeMillis() / 1000;

	//
	// Calculate current payload
	//
	int currentPayload;
	if(packetNum < numberOfPackets_ - 1)
	{
	    currentPayload = payload_;
	}
	else
	{
	    if(lastPayload_ > 0)
		currentPayload = lastPayload_;
	    else
                currentPayload = payload_;
	}
	
	//
	// Copy the packet at the right location in the message
	//
	com.ooc.OB.Assert._OB_assert(packet.rest_length() >= currentPayload);
	System.arraycopy(packet.data(), packet.pos(), 
			 buf_.data(), packetNum * payload_, currentPayload);
	
        //
	// Mark the packet as received
	//
	packetsReceived_[packetNum] = true;

	//
	// Check if message is complete
	//
	completed_ = true;
	for(int i = 0 ; i < numberOfPackets_ ; i++)
	    if(!packetsReceived_[i])
	    {
		completed_ = false;
		break;
	    }
    }

    com.ooc.OCI.Buffer 
    getBuffer()
    {
	return buf_;
    }

    boolean 
    completed() 
    { 
	return completed_; 
    }

    int 
    numberOfPackets() 
    { 
	return numberOfPackets_; 
    }

    long 
    lastPacketTime() 
    { 
	return lastPacketTime_; 
    }
}
