// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.UDP;

//
// The PacketCollector class
//
class PacketCollector
{
    //
    // The trace level
    //
    private int trace_;

    //
    // The message timeout
    //
    private int messageTimeout_;

    //
    // The logger
    //
    private com.ooc.OB.Logger logger_;

    //
    // Incomplete messages
    //
    private java.util.Hashtable messages_ = null;

    //
    // Add packet to a message and check if the message is complete
    //
    synchronized com.ooc.OCI.Buffer
    addPacket(com.ooc.CORBA.InputStream in, int packetSize,
              com.ooc.UDPIOP.FixedPacketHeader fHeader)
    {
        //
        // Unmarshal the variable header
        //
        com.ooc.UDPIOP.VariablePacketHeader vHeader = 
            com.ooc.UDPIOP.VariablePacketHeaderHelper.read(in);

        //
        // Create a key composed of message ID
        //
        String key = ""; 
        key += fHeader.message_id;

        //
        // Check whether this is a new message
        //
        Message msg = (Message)messages_.get(key);

        if(msg == null)
        {
            //
            // Calculate the payload
            //
            int payload = packetSize - in._OB_pos();

            //
            // This is a new message
            //
            msg = new Message(fHeader.message_size, payload);
            messages_.put(key, msg);

            if(trace_ > 0)
            {
                String str = "new message";
                str += "\nclient id = ";
                str += fHeader.client_id;
                str += "\nmessage id = ";
                str += fHeader.message_id;
                str += "\nmessage size = ";
                str += fHeader.message_size;
                logger_.trace("UDP", str);
            }
        }

        //
        // Add the packet
        //
        msg.add(in._OB_buffer(), vHeader.packet_num);

        if(trace_ > 0)
        {
            String str = "added packet ";
            str += vHeader.packet_num + 1;
            str += '/';
            str += msg.numberOfPackets();
            str += "\nclient id = ";
            str += fHeader.client_id;
            str += "\nmessage id = ";
            str += fHeader.message_id;
            str += "\nmessage size = ";
            str += fHeader.message_size;
            str += "\ncompleted = ";
            str += msg.completed() ? "yes" : "no";
            logger_.trace("UDP", str);
        }

        //
        // Check whether the message is completed
        //
        if(msg.completed())
        {
            //
            // Message completed, get the buffer, and remove the message
            // from the hashtable
            //
            com.ooc.OCI.Buffer buf = msg.getBuffer();
            com.ooc.OB.Assert._OB_assert(buf.pos() == 0);
            messages_.remove(key);
            return buf;
        }
        else
            return null;
    }

    //
    // Collect garbage
    //
    public synchronized void
    sweep()
    {
        //
        // Get the current time
        //
        long currentTime = System.currentTimeMillis() / 1000;

        java.util.Enumeration e = messages_.keys();
        while(e.hasMoreElements())
        {
            Object key = e.nextElement();
            Message msg = (Message)messages_.get(key);
            com.ooc.OB.Assert._OB_assert(msg != null);

            if(currentTime - msg.lastPacketTime() >= messageTimeout_)
            {
                if(trace_ > 0)
                {
                    String str = "garbage collecting message";
                    str += "\nmessage id = ";
                    str += key;
                    logger_.trace("UDP", str);
                }

                messages_.remove(key);
            }
        }
    }

    PacketCollector(int trace, int messageTimeout, com.ooc.OB.Logger logger)
    {
        trace_ = trace;
        messageTimeout_ = messageTimeout;
        logger_ = logger;
        messages_ = new java.util.Hashtable(15, 0.5F);
    }
}
