// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI;

public class udp implements PluginInit
{
    final private static String version_ = "1.0";

    //
    // Compatibility check. The plug-in should verify that it is
    // compatible with the given OCI version, and raise an exception
    // if not.
    //
    public void
    version(org.omg.CORBA.ORB orb, String ver)
    {
        if(!ver.equals(version_))
            throw new org.omg.CORBA.INITIALIZE("udp: incompatible OCI " +
                                               "version");
    }

    //
    // Initialize the plug-in for an ORB
    //
    public com.ooc.OCI.Plugin
    init(org.omg.CORBA.ORB orb, org.omg.CORBA.StringSeqHolder args)
    {
        com.ooc.CORBA.ORB oborb = (com.ooc.CORBA.ORB)orb;
        java.util.Properties props = oborb.properties();

        //
        // Parse arguments
        //
        args.value = parse_args(args.value, props);

        return new com.ooc.OCI.UDP.Plugin_impl(orb);
    }

    //
    // Parse UDP arguments. The return value is the a new array
    // with the UDP arguments removed.
    //
    public static String[]
    parse_args(String[] args, java.util.Properties props)
    {
        String bind = null;
        String host = null;
        boolean multicast = false;
        boolean numeric = false;
        String port = null;
        boolean haveArgs = false;

        int i = 0;
        while(i < args.length)
        {
            if(args[i].equals("-UDPbind"))
            {
                if(i + 1 >= args.length)
                    throw new org.omg.CORBA.INITIALIZE("udp: argument " +
                                                       "expected for " +
                                                       "-UDPbind");
                bind = args[i + 1];
                haveArgs = true;
                i += 2;
            }
            else if(args[i].equals("-UDPhost"))
            {
                if(i + 1 >= args.length)
                    throw new org.omg.CORBA.INITIALIZE("udp: argument " +
                                                       "expected for " +
                                                       "-UDPhost");
                host = args[i + 1];
                haveArgs = true;
                i += 2;
            }
            else if(args[i].equals("-UDPmulticast"))
            {
                multicast = true;
                haveArgs = true;
                i++;
            }
            else if(args[i].equals("-UDPnumeric"))
            {
                numeric = true;
                haveArgs = true;
                i++;
            }
            else if(args[i].equals("-UDPport"))
            {
                if(i + 1 >= args.length)
                    throw new org.omg.CORBA.INITIALIZE("udp: argument " +
                                                       "expected for " +
                                                       "-UDPport");
                port = args[i + 1];
                haveArgs = true;
                i += 2;
            }
            else if(args[i].startsWith("-UDP"))
            {
                throw new org.omg.CORBA.INITIALIZE("udp: unknown option `" +
                                                   args[i] + "'");
            }
            else
                i++;
        }

        if(haveArgs)
        {
            String propName = "ooc.orb.poamanager.RootPOAManager.endpoint";
            String value = props.getProperty(propName);
            if(value == null)
            {
                propName = "ooc.orb.oa.endpoint";
                value = props.getProperty(propName);
            }

            String str = "udp";
            if(bind != null)
            {
                str += " --bind ";
                str += bind;
            }
            if(host != null)
            {
                str += " --host ";
                str += host;
            }
            if(multicast)
            {
                str += " --multicast";
            }
            if(numeric)
            {
                str += " --numeric";
            }
            if(port != null)
            {
                str += " --port ";
                str += port;
            }

            if(value == null)
                props.put(propName, str);
            else
            {
                //
                // Append to existing property value
                //
                props.put(propName, value + ", " + str);
            }
        }

        //
        // Filter arguments
        //
        com.ooc.OB.OptionFilter filter;
        filter = new com.ooc.OB.OptionFilter("udp.parse_args", "-UDP");
        filter.add("bind", 1);
        filter.add("host", 1);
        filter.add("multicast", 0);
        filter.add("numeric", 0);
        filter.add("port", 1);
        return filter.filter(args);
    }
}
