// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA  
//                                
// All Rights Reserved         
//                     
// **********************************************************************

package com.ooc.Util.GUI;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.*;
import javax.swing.*;  
import javax.swing.event.*;  
import javax.swing.border.*;  
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.HTMLDocument;

/**
 * Singleton help dialog box which can be used to display help files
 * written in HTML. This class was taken from CosNotifyConsole by
 * Benoit Foucher.
 *
 * Applications that use this class should define their 'MAIN' help
 * screen by specifying an AppResources file that defines a
 * 'HelpDialogMainScreen' (typically the matching pair will be
 * 'index.html'). Help files themselves should be stored in a
 * subdirectory 'help/' that is in the CLASSPATH of the application.
 */
public class Help extends JDialog
                        implements HyperlinkListener
{
    protected static JFrame parent_;

    private static JButton okBtn_;
    private static JButton backBtn_;
    private static JButton nextBtn_;
    private static JButton homeBtn_;

    protected static Help window_;
    private static JEditorPane htmlPane_;
    private static Stack backStack_;
    private static Stack nextStack_;
    private static Hashtable cache_;
    private static URL url_;
    private static URL mainURL_;

    protected
    Help(JFrame parent, String tittle)
    {
	super(parent, tittle, true);
        setLocationRelativeTo(parent);
	parent_ = parent;

	backStack_ = new Stack();
	nextStack_ = new Stack();
	cache_ = new Hashtable();
	mainURL_ = this.getHelpURL(AppStandards.getString("HelpDialogMainScreen"));
	url_ = mainURL_;

	try
	{
            htmlPane_ = new JEditorPane(url_);
	    htmlPane_.setEditable(false);
	    htmlPane_.setBackground(new Color(255, 255, 231));
	    htmlPane_.addHyperlinkListener(this);
	}
	catch(IOException ex)
	{
	}

        JScrollPane sp = new JScrollPane(htmlPane_);
	sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        sp.setPreferredSize(new Dimension(590, 450));

        okBtn_ = new JButton("Ok");
        backBtn_ = new JButton("< Back");
	backBtn_.setEnabled(false);
        nextBtn_ = new JButton("Next >");
	nextBtn_.setEnabled(false);
        homeBtn_ = new JButton("Home");

	okBtn_.addActionListener(new OkAction());
	backBtn_.addActionListener(new BackAction());
	nextBtn_.addActionListener(new NextAction());
	homeBtn_.addActionListener(new HomeAction());

	JPanel p1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
	p1.add(homeBtn_);
	p1.add(backBtn_);
	p1.add(nextBtn_);

        JPanel p2 = new JPanel(new BorderLayout());
	p2.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
	p2.add(sp, BorderLayout.CENTER);

        JPanel p3 = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        p3.add(okBtn_);

        Container contentPane = getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.add(p1, BorderLayout.NORTH);
        contentPane.add(p2, BorderLayout.CENTER);
        contentPane.add(p3, BorderLayout.SOUTH);

	
        this.pack();
	this.setLocationRelativeTo(parent);
    }

    public static void
    initialize(JFrame parent, String tittle)
    {
	parent_ = parent;
	window_ = new Help(parent_, tittle);
    }
    
    protected void 
    setURL(URL url) 
    {
        try 
        {
	    htmlPane_.setPage(url);
        } 
	catch(IOException ex) 
	{
	    System.err.println("Read bad URL: " + ex.getMessage());
	}
    }

    //
    // Display dialog window
    //
    public static void
    showWindow(String helpFile)
    {
	if(window_ == null)
	{
	    System.err.println("Help is uninitialized...");
	}

	nextStack_ = new Stack();
	backStack_ = new Stack();
	nextBtn_.setEnabled(false);
	backBtn_.setEnabled(false);
	url_ = getHelpURL(helpFile);
	window_.setURL(url_);
	window_.setVisible(true);
    }

    public static URL
    getHelpURL(String helpFile)
    {
        URL url = parent_.getClass().getResource("help/" + helpFile);

        if(url != null)
        {
            return url;
        }

        return null;
    }

    //
    // from HyperlinkListener
    //

    public void hyperlinkUpdate(HyperlinkEvent e) 
    {
        if(e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) 
	{
	    try 
	    {
		backStack_.push(url_);
		url_ = e.getURL();
		window_.setURL(url_);
	        backBtn_.setEnabled(true);
	        nextBtn_.setEnabled(false);
		nextStack_ = new Stack();
            } 
	    catch(Throwable t) 
	    {
            }
        }
    }

    class OkAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
	    window_.setVisible(false);
        }
    }

    class HomeAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
	    try 
	    {
		URL tmpURL = (URL)backStack_.peek();
		
		if(url_.sameFile(tmpURL))
		{
		    return;
		}

		backStack_.push(url_);
		window_.setURL(mainURL_);
	        backBtn_.setEnabled(true);
	        nextBtn_.setEnabled(false);
		nextStack_ = new Stack();
            } 
	    catch(Throwable t) 
	    {
            }
        }
    }

    class BackAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
            nextStack_.push(url_); 
	    nextBtn_.setEnabled(true);
	    url_ = (URL)backStack_.pop();
	    window_.setURL(url_);

	    if(backStack_.empty())
	    {
		backBtn_.setEnabled(false);
	    }
        }
    }

    class NextAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
            backStack_.push(url_); 
	    backBtn_.setEnabled(true);
	    url_ = (URL)nextStack_.pop();
	    window_.setURL(url_);

	    if(nextStack_.empty())
	    {
		nextBtn_.setEnabled(false);
	    }
        }
    }
}
