// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package nested;

public class Peer2
{
    static int
    run(org.omg.CORBA.ORB orb, String[] args)
        throws org.omg.CORBA.UserException
    {
	//
	// Resolve Root POA
	//
	org.omg.PortableServer.POA rootPOA =
	    org.omg.PortableServer.POAHelper.narrow(
		orb.resolve_initial_references("RootPOA"));

	org.omg.PortableServer.POAManager manager = rootPOA.the_POAManager();

	//
	// Create implementation object
	//
	Peer_impl peerImpl = new Peer_impl(rootPOA);
	Peer p2 = peerImpl._this(orb);
	
	org.omg.CORBA.Object obj = orb.string_to_object("relfile:/Peer.ref");
	if(obj == null)
	{
	    System.err.println("nested.Peer2: cannot read IOR from Peer.ref");
	    return 1;
	}

	Peer p1 = PeerHelper.narrow(obj);

	//
	// Initialize the servers so that incoming requests are
	// handled from this point on
	//
        manager.activate();

	while(true)
	{
	    //
	    // Start recursive decrement
	    //
	    try
	    {
		System.out.print("Enter a number (0 to exit): ");
		java.io.DataInputStream dataIn =
		    new java.io.DataInputStream(System.in);
		java.io.BufferedReader in =
		    new java.io.BufferedReader(
			new java.io.InputStreamReader(dataIn));
		String number = in.readLine();
		Integer integer = new Integer(number);

		if(integer.intValue() <= 0)
		    break;

		//
		// Run demo
		//
		long t = System.currentTimeMillis();
		p1.decrement(
		    new org.omg.CORBA.IntHolder(integer.intValue()), p2);
		t = System.currentTimeMillis() - t;
		double td = (double)t / 1000.0;
		System.out.println("Time needed: " + td);
	    }
	    catch(java.io.IOException ex)
	    {
		System.err.println("nested.Peer2: can't read from `" +
				   ex.getMessage() + "'");
		return 1;
	    }
	    catch(java.lang.NumberFormatException ex)
	    {
		// Do nothing. Loop back until valid input.
	    }
	}

	return 0;
    }
    
    public static void
    main(String args[])
    {
	java.util.Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");
	props.put("ooc.orb.id", "Watson-Nested-Peer2");

	int status = 0;
	org.omg.CORBA.ORB orb = null;

        try
        {
	    //
	    // Set the concurrency model to threaded/thread_per_client
	    // if a valid concurrency model wasn't selected
	    //
            args = com.ooc.CORBA.ORB.ParseArgs(args, props, null);
	    String oaModel = props.getProperty("ooc.orb.oa.conc_model");
	    String orbModel = props.getProperty("ooc.orb.conc_model");
	    boolean valid;
	    if(oaModel != null && orbModel != null)
	    {
		valid = orbModel.equals("threaded") &&
		    (oaModel.equals("thread_per_request") ||
		     oaModel.equals("thread_pool"));
	    }
	    else
		valid = false;

	    if(!valid)
	    {
		props.put("ooc.orb.conc_model", "threaded");
		props.put("ooc.orb.oa.conc_model", "thread_per_request");
	    }

            orb = org.omg.CORBA.ORB.init(args, props);
	    status = run(orb, args);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
	    status = 1;
        }

  	if(orb != null)
	{
	    try
	    {
		orb.destroy();
	    }
	    catch(Exception ex)
	    {
                ex.printStackTrace();
                status = 1;
	    }
	}

        System.exit(status);
    }
}
