// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.watson;

//
// AnyWriter is a handy class for displaying Anys, DynAnys and TypeCodes.
//
// Hierarchical structures are represented by indenting; the width of
// the indent can be specified using setSpacing().  The default indent
// is two spaces.
//
// The up() and down() methods control the current indent level.
//
public class AnyWriter
{
    private int level_ = 0;
    private String spacing_ = "   ";
    private java.io.PrintWriter out_;
    private org.omg.DynamicAny.DynAnyFactory factory_;
    private static java.util.Hashtable typeNames_;

    static
    {
        //
        // Put the names of the TCKind values in a hashtable
        //
        typeNames_ = new java.util.Hashtable();
        typeNames_.put(org.omg.CORBA.TCKind.tk_null, "null");
        typeNames_.put(org.omg.CORBA.TCKind.tk_void, "void");
        typeNames_.put(org.omg.CORBA.TCKind.tk_short, "short");
        typeNames_.put(org.omg.CORBA.TCKind.tk_ushort, "unsigned short");
        typeNames_.put(org.omg.CORBA.TCKind.tk_long, "long");
        typeNames_.put(org.omg.CORBA.TCKind.tk_ulong, "unsigned long");
        typeNames_.put(org.omg.CORBA.TCKind.tk_float, "float");
        typeNames_.put(org.omg.CORBA.TCKind.tk_double, "double");
        typeNames_.put(org.omg.CORBA.TCKind.tk_boolean, "boolean");
        typeNames_.put(org.omg.CORBA.TCKind.tk_char, "char");
        typeNames_.put(org.omg.CORBA.TCKind.tk_octet, "octet");
        typeNames_.put(org.omg.CORBA.TCKind.tk_any, "any");
        typeNames_.put(org.omg.CORBA.TCKind.tk_TypeCode, "TypeCode");
        typeNames_.put(org.omg.CORBA.TCKind.tk_Principal, "Principal");
        typeNames_.put(org.omg.CORBA.TCKind.tk_objref, "objref");
        typeNames_.put(org.omg.CORBA.TCKind.tk_struct, "struct");
        typeNames_.put(org.omg.CORBA.TCKind.tk_union, "union");
        typeNames_.put(org.omg.CORBA.TCKind.tk_enum, "enum");
        typeNames_.put(org.omg.CORBA.TCKind.tk_string, "string");
        typeNames_.put(org.omg.CORBA.TCKind.tk_sequence, "sequence");
        typeNames_.put(org.omg.CORBA.TCKind.tk_array, "array");
        typeNames_.put(org.omg.CORBA.TCKind.tk_alias, "alias");
        typeNames_.put(org.omg.CORBA.TCKind.tk_except, "exception");
        typeNames_.put(org.omg.CORBA.TCKind.tk_longlong, "long long");
        typeNames_.put(org.omg.CORBA.TCKind.tk_ulonglong,
                       "unsigned long long");
        typeNames_.put(org.omg.CORBA.TCKind.tk_longdouble, "long double");
        typeNames_.put(org.omg.CORBA.TCKind.tk_wchar, "wchar");
        typeNames_.put(org.omg.CORBA.TCKind.tk_wstring, "wstring");
        typeNames_.put(org.omg.CORBA.TCKind.tk_fixed, "fixed");
        typeNames_.put(org.omg.CORBA.TCKind.tk_value, "valuetype");
        typeNames_.put(org.omg.CORBA.TCKind.tk_value_box, "valuebox");
        typeNames_.put(org.omg.CORBA.TCKind.tk_native, "native");
        typeNames_.put(org.omg.CORBA.TCKind.tk_abstract_interface,
                       "abstract interface");
        typeNames_.put(org.omg.CORBA_2_4.TCKind.tk_local_interface,
                       "local interface");
    }

    //
    // Given a TypeCode, return the TypeCode without any aliasing
    //
    private static org.omg.CORBA.TypeCode
    getOriginalType(org.omg.CORBA.TypeCode tc)
    {
        org.omg.CORBA.TypeCode result = tc;

        try
        {
            while(result.kind() == org.omg.CORBA.TCKind.tk_alias)
                result = result.content_type();
        }
        catch(org.omg.CORBA.TypeCodePackage.BadKind ex)
        {
            // ignore
        }

        return result;
    }

    //
    // Default constructor uses System.out as the output stream
    //
    public
    AnyWriter(org.omg.DynamicAny.DynAnyFactory factory)
    {
        this(new java.io.PrintWriter(System.out, true), factory);
    }

    //
    // User supplies the Writer object
    //
    public
    AnyWriter(java.io.PrintWriter out,
	      org.omg.DynamicAny.DynAnyFactory factory)
    {
        out_ = out;
        factory_ = factory;
    }

    public void
    setWriter(java.io.PrintWriter out)
    {
        out_ = out;
    }

    public java.io.PrintWriter
    getWriter()
    {
        return out_;
    }

    public void
    flush()
    {
        out_.flush();
    }

    public void
    setSpacing(String spacing)
    {
        spacing_ = spacing;
    }

    public String
    getSpacing()
    {
        return spacing_;
    }

    public void
    up()
    {
        level_--;
    }

    public void
    down()
    {
        level_++;
    }

    //
    // "Tabs" over to the appropriate starting point for the current
    // indent level
    //
    public java.io.PrintWriter
    skip()
    {
        for(int i = 0 ; i < level_ ; i++)
            out_.print(spacing_);
        return out_;
    }

    //
    // Prints the full TypeCode
    //
    public void
    print(org.omg.CORBA.TypeCode tc)
    {
        org.omg.CORBA.TCKind kind = tc.kind();

        try
        {
            switch(kind.value())
            {
                case org.omg.CORBA.TCKind._tk_objref:
                    skip().print("interface " + tc.name() + ";");
                    break;

                case org.omg.CORBA.TCKind._tk_struct:
                case org.omg.CORBA.TCKind._tk_except:
                {
                    skip().println("" + typeNames_.get(kind) + " " +
                        tc.name() + " {");
                    down();
                    for(int i = 0 ; i < tc.member_count() ; i++)
                    {
                        skip();
                        org.omg.CORBA.TypeCode memberType = tc.member_type(i);
                        printName(memberType);
                        out_.println(" " + tc.member_name(i) + ";");
                    }
                    up();
                    skip().print("};");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_union:
                {
                    skip().print("union " + tc.name() + " switch(");
                    org.omg.CORBA.TypeCode discrType = tc.discriminator_type();
                    printName(discrType);
                    out_.println(") {");
                    down();
                    for(int i = 0 ; i < tc.member_count() ; i++)
                    {
                        org.omg.CORBA.Any label = tc.member_label(i);
                        org.omg.CORBA.TypeCode type = label.type();

                        if(type.kind() == org.omg.CORBA.TCKind.tk_octet)
                            skip().print("default");
                        else
                        {
                            skip().print("case ");
                            org.omg.DynamicAny.DynAny dynAny = null;
                            try
                            {
                                dynAny = factory_.create_dyn_any(label);
                            }
                            catch(org.omg.DynamicAny.DynAnyFactoryPackage.
                                  InconsistentTypeCode e)
                            {
                                throw new RuntimeException();
                            }
			    printDynAny(dynAny, false);
                        }

                        out_.print(": ");
                        org.omg.CORBA.TypeCode memberType = tc.member_type(i);
                        printName(memberType);
                        out_.println(" " + tc.member_name(i) + ";");
                    }
                    up();
                    skip().print("};");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_enum:
                {
                    skip().print("enum " + tc.name()+ " {");
                    for(int i = 0 ; i < tc.member_count() ; i++)
                    {
                        if(i != 0 )
                            out_.print(", ");
                        out_.print(tc.member_name(i));
                    }
                    out_.print("};");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_string:
                {
                    skip().print("string");
                    int length = tc.length();
                    if(length != 0)
                        out_.print("<" + length + ">");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_wstring:
                {
                    skip().print("wstring");
                    int length = tc.length();
                    if(length != 0)
                        out_.print("<" + length + ">");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_fixed:
                {
                    skip().print("fixed<" + tc.fixed_digits() + "," +
                                 tc.fixed_scale() + ">");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_sequence:
                {
                    skip().print("sequence<");
                    org.omg.CORBA.TypeCode contentType = tc.content_type();
                    printName(contentType);
                    int length = tc.length();
                    if(length != 0)
                        out_.print(", " + length);
                    out_.print(">");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_array:
                {
                    skip();
                    org.omg.CORBA.TypeCode contentType = tc.content_type();
                    printName(contentType);
                    int length = tc.length();
                    out_.print("[" + length + "]");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_alias:
                {
                    skip().print("typedef ");
                    org.omg.CORBA.TypeCode contentType = tc.content_type();
                    printName(contentType);
                    out_.print(" " + tc.name() + ";");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_value:
                {
                    skip().println("valuetype " + tc.name() + " {");
                    down();

                    java.util.Vector vec = new java.util.Vector();
                    vec.addElement(tc);
                    org.omg.CORBA.TypeCode base = tc.concrete_base_type();
                    while(base != null)
                    {
                        vec.insertElementAt(base, 0);
                        base = base.concrete_base_type();
                    }

                    java.util.Enumeration e = vec.elements();
                    while(e.hasMoreElements())
                    {
                        org.omg.CORBA.TypeCode p =
                            (org.omg.CORBA.TypeCode)e.nextElement();
                        for(int i = 0 ; i < p.member_count() ; i++)
                        {
                            skip();
                            org.omg.CORBA.TypeCode memberType =
                                p.member_type(i);
                            printName(memberType);
                            out_.println(" " + p.member_name(i) + ";");
                        }
                    }

                    up();
                    skip().print("};");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_value_box:
                {
                    skip().print("valuetype " + tc.name() + " ");
                    org.omg.CORBA.TypeCode contentType = tc.content_type();
                    printName(contentType);
                    out_.print(";");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_abstract_interface:
                    skip().print("abstract interface " + tc.name() + ";");
                    break;

                case org.omg.CORBA.TCKind._tk_native:
                    skip().print("native " + tc.name() + ";");
                    break;

                case org.omg.CORBA_2_4.TCKind._tk_local_interface:
                    skip().print("local interface " + tc.name() + ";");
                    break;

                default:
                    skip().print("" + typeNames_.get(kind));
                    break;
            }

            out_.println();
        }
        catch(org.omg.CORBA.TypeCodePackage.BadKind e)
        {
        }
        catch(org.omg.CORBA.TypeCodePackage.Bounds e)
        {
        }
    }

    //
    // Prints only the name of a TypeCode
    //
    public void
    printName(org.omg.CORBA.TypeCode tc)
    {
        org.omg.CORBA.TCKind kind = tc.kind();

        try
        {
            switch(kind.value())
            {
                case org.omg.CORBA.TCKind._tk_objref:
                case org.omg.CORBA.TCKind._tk_enum:
                case org.omg.CORBA.TCKind._tk_union:
                case org.omg.CORBA.TCKind._tk_struct:
                case org.omg.CORBA.TCKind._tk_except:
                case org.omg.CORBA.TCKind._tk_alias:
                case org.omg.CORBA.TCKind._tk_value:
                case org.omg.CORBA.TCKind._tk_value_box:
                case org.omg.CORBA.TCKind._tk_abstract_interface:
                case org.omg.CORBA.TCKind._tk_native:
                case org.omg.CORBA_2_4.TCKind._tk_local_interface:
                    out_.print(tc.name());
                    break;

                case org.omg.CORBA.TCKind._tk_fixed:
                {
                    out_.print("fixed<" + tc.fixed_digits() + "," +
                               tc.fixed_scale() + ">");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_string:
                {
                    out_.print("string");
                    int length = tc.length();
                    if(length != 0)
                        out_.print("<" + length + ">");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_wstring:
                {
                    out_.print("wstring");
                    int length = tc.length();
                    if(length != 0)
                        out_.print("<" + length + ">");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_sequence:
                {
                    out_.print("sequence<");
                    org.omg.CORBA.TypeCode contentType = tc.content_type();
                    printName(contentType);
                    int length = tc.length();
                    if(length != 0)
                        out_.print(", " + length);
                    out_.print(">");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_array:
                {
                    org.omg.CORBA.TypeCode contentType = tc.content_type();
                    printName(contentType);
                    int length = tc.length();
                    out_.print("[" + length + "]");
                    break;
                }

                default:
                    out_.print("" + typeNames_.get(kind));
                    break;
            }
        }
        catch(org.omg.CORBA.TypeCodePackage.BadKind e)
        {
        }
    }

    //
    // Indents as appropriate and prints the value of the DynAny,
    // including TypeCode information
    //
    public void
    print(org.omg.DynamicAny.DynAny dynAny)
    {
        skip();
        printDynAny(dynAny);
        out_.println();
    }

    //
    // Prints the value of the DynAny, including TypeCode information
    //
    public void
    printDynAny(org.omg.DynamicAny.DynAny dynAny)
    {
        printDynAny(dynAny, true);
    }

    //
    // Prints the value of the DynAny; showType determines whether
    // TypeCode information is printed
    //
    public void
    printDynAny(org.omg.DynamicAny.DynAny dynAny, boolean showType)
    {
        if(showType)
        {
            org.omg.CORBA.TypeCode tc = dynAny.type();
            printName(tc);
            out_.print("(");
        }

        dynAny.rewind();

        org.omg.CORBA.TypeCode type = dynAny.type();
        org.omg.CORBA.TypeCode tc = getOriginalType(type);
        org.omg.CORBA.TCKind kind = tc.kind();

        try
        {
            switch(kind.value())
            {
                case org.omg.CORBA.TCKind._tk_null:
                    out_.print("<null>");
                    break;

                case org.omg.CORBA.TCKind._tk_void:
                    out_.print("<void>");
                    break;

                case org.omg.CORBA.TCKind._tk_short:
                    out_.print(dynAny.get_short());
                    break;

                case org.omg.CORBA.TCKind._tk_ushort:
                {
                    int i = dynAny.get_ushort();
                    if(i < 0)
                        i += ((int)Short.MAX_VALUE -
                              (int)Short.MIN_VALUE) + 1;
                    out_.print(i);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_long:
                    out_.print(dynAny.get_long());
                    break;

                case org.omg.CORBA.TCKind._tk_ulong:
                {
                    long l = dynAny.get_ulong();
                    if(l < 0)
                        l += ((long)Integer.MAX_VALUE -
                              (long)Integer.MIN_VALUE) + 1;
                    out_.print(l);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_longlong:
                    out_.print(dynAny.get_longlong());
                    break;

                case org.omg.CORBA.TCKind._tk_ulonglong:
                {
                    long l = dynAny.get_ulonglong();
                    if(l < 0)
                        l += (Long.MAX_VALUE - Long.MIN_VALUE) + 1;
                    out_.print(l);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_float:
                    out_.print(dynAny.get_float());
                    break;

                case org.omg.CORBA.TCKind._tk_double:
                    out_.print(dynAny.get_double());
                    break;

                case org.omg.CORBA.TCKind._tk_boolean:
                    out_.print(dynAny.get_boolean() ? "TRUE" : "FALSE");
                    break;

                case org.omg.CORBA.TCKind._tk_char:
                    out_.print(dynAny.get_char());
                    break;

                case org.omg.CORBA.TCKind._tk_wchar:
                    out_.print(dynAny.get_wchar());
                    break;

                case org.omg.CORBA.TCKind._tk_octet:
                {
                    short s = dynAny.get_octet();
                    if(s < 0)
                        s += ((short)Byte.MAX_VALUE -
                              (short)Byte.MIN_VALUE) + 1;
                    out_.print(s);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_string:
                    out_.print(dynAny.get_string());
                    break;

                case org.omg.CORBA.TCKind._tk_wstring:
                    out_.print(dynAny.get_wstring());
                    break;

                case org.omg.CORBA.TCKind._tk_objref:
                {
                    org.omg.CORBA.Object obj = dynAny.get_reference();
                    if(obj == null)
                        out_.print("nil");
                    else
                        out_.print("<object>");
                    break;
                }

                case org.omg.CORBA.TCKind._tk_any:
                {
                    org.omg.CORBA.Any any = dynAny.get_any();
                    printAny(any, showType);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_TypeCode:
                {
                    org.omg.CORBA.TypeCode typeCode = dynAny.get_typecode();
                    printName(typeCode);
                    break;
                }

                case org.omg.CORBA.TCKind._tk_fixed:
                {
                    org.omg.DynamicAny.DynFixed dynFixed =
                        org.omg.DynamicAny.DynFixedHelper.narrow(dynAny);
                    out_.print(dynFixed.get_value());
                    break;
                }

                case org.omg.CORBA.TCKind._tk_except:
                case org.omg.CORBA.TCKind._tk_struct:
                {
                    org.omg.DynamicAny.DynStruct dynStruct =
                        org.omg.DynamicAny.DynStructHelper.narrow(dynAny);
                    out_.println();
                    down();

                    int count = tc.member_count();
                    for(int i = 0 ; i < count ; i++)
                    {
                        String name = dynStruct.current_member_name();
                        skip().print(name + " = ");
                        org.omg.DynamicAny.DynAny component =
                            dynStruct.current_component();
                        printDynAny(component, showType);
                        out_.println();
                        dynStruct.next();
                    }

                    up();
                    skip();
                    break;
                }

                case org.omg.CORBA.TCKind._tk_value:
                {
                    org.omg.DynamicAny.DynValue dynValue =
                        org.omg.DynamicAny.DynValueHelper.narrow(dynAny);
                    out_.println();
                    down();

                    int count = dynValue.component_count();
                    for(int i = 0 ; i < count ; i++)
                    {
                        String name = dynValue.current_member_name();
                        skip().print(name + " = ");
                        org.omg.DynamicAny.DynAny component =
                            dynValue.current_component();
                        printDynAny(component, showType);
                        out_.println();
                        dynValue.next();
                    }

                    up();
                    skip();
                    break;
                }

                case org.omg.CORBA.TCKind._tk_value_box:
                {
                    org.omg.DynamicAny.DynValue dynValue =
                        org.omg.DynamicAny.DynValueHelper.narrow(dynAny);
                    out_.println();
                    down();

                    skip();
                    org.omg.DynamicAny.DynAny component =
                        dynValue.current_component();
                    printDynAny(component, showType);
                    out_.println();

                    up();
                    skip();
                    break;
                }

                case org.omg.CORBA.TCKind._tk_union:
                {
                    org.omg.DynamicAny.DynUnion dynUnion =
                        org.omg.DynamicAny.DynUnionHelper.narrow(dynAny);

                    out_.println();
                    down();

                    org.omg.DynamicAny.DynAny component;

                    component = dynUnion.current_component();
                    skip().print("discriminator = ");
                    printDynAny(component, showType);
                    out_.println();

                    dynUnion.next();

                    component = dynUnion.current_component();
                    String member = dynUnion.member_name();
                    skip().print(member + " = ");

                    if(component != null)
                        printDynAny(component, showType);
                    else
                        out_.println("not initialized");

                    out_.println();

                    up();
                    skip();
                    break;
                }

                case org.omg.CORBA.TCKind._tk_enum:
                {
                    org.omg.DynamicAny.DynEnum dynEnum =
                        org.omg.DynamicAny.DynEnumHelper.narrow(dynAny);
                    out_.print(dynEnum.get_as_string());
                    break;
                }

                case org.omg.CORBA.TCKind._tk_sequence:
                {
                    org.omg.DynamicAny.DynSequence dynSequence =
                        org.omg.DynamicAny.DynSequenceHelper.narrow(dynAny);
                    for(int i = 0 ; i < dynSequence.get_length() ; i++)
                    {
                        if(i != 0)
                            out_.print(", ");
                        org.omg.DynamicAny.DynAny component =
                            dynSequence.current_component();
                        printDynAny(component, showType);
                        dynSequence.next();
                    }
                    break;
                }

                case org.omg.CORBA.TCKind._tk_array:
                {
                    org.omg.DynamicAny.DynArray dynArray =
                        org.omg.DynamicAny.DynArrayHelper.narrow(dynAny);
                    for(int i = 0 ; i < tc.length() ; i++)
                    {
                        if(i != 0)
                            out_.print(", ");
                        org.omg.DynamicAny.DynAny component =
                            dynArray.current_component();
                        printDynAny(component, showType);
                        dynArray.next();
                    }
                    break;
                }

                default:
                    throw new RuntimeException("unsupported type kind");
            }
        }
        catch(org.omg.DynamicAny.DynAnyPackage.InvalidValue e)
        {
        }
        catch(org.omg.DynamicAny.DynAnyPackage.TypeMismatch e)
        {
        }
        catch(org.omg.CORBA.TypeCodePackage.BadKind e)
        {
        }

        dynAny.rewind();

        if(showType)
            out_.print(")");
    }

    //
    // Indents and prints the value of the Any, including TypeCode
    // information
    //
    public void
    print(org.omg.CORBA.Any any)
    {
        printAny(any, true);
    }

    //
    // Prints the value of the Any; showType determines whether
    // TypeCode information is printed
    //
    public void
    printAny(org.omg.CORBA.Any any, boolean showType)
    {
        org.omg.DynamicAny.DynAny dynAny = null;
        try
        {
            dynAny = factory_.create_dyn_any(any);
        }
        catch(org.omg.DynamicAny.DynAnyFactoryPackage.InconsistentTypeCode ex)
        {
            out_.print("<unsupported type>");
            return;
        }
        printDynAny(dynAny, showType);
    }

}
