// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.LoadBalancing.Util;

import com.ooc.LoadBalancing.*;

//
// ORB Initializer for installing the LoadInterceptor and
// LoadAlertInterceptor interceptors and registering the LoadAlert,
// LoadCalculator, and LoadUpdater local objects.
//
final public class UtilInitializer
    extends org.omg.CORBA.LocalObject
    implements org.omg.PortableInterceptor.ORBInitializer
{
    //
    // The logger
    //
    private Logger logger_;

    //
    // The LoadInterceptor
    //
    private LoadInterceptor loadInterceptor_;

    //
    // The LoadAlertInterceptor
    //
    private LoadAlertInterceptor loadAlertInterceptor_;

    //
    // The LoadUpdater frequency
    //
    private int updateFreq_;
    
    //
    // The LoadAlert expire time
    //
    private int alertExpire_;

    //
    // Create the LoadAlert?
    //
    private boolean createAlert_;

    //
    // Create the LoadUpdater?
    //
    private boolean createUpdater_;

    //
    // Create the LoadCalculator?
    //
    private boolean createCalculator_;

    //
    // Are the objects initialized?
    //
    private boolean initialized_;

    // -----------------------------------------------------------------
    // UtilInitializer constructor
    // -----------------------------------------------------------------

    public UtilInitializer()
    {
	updateFreq_ = 1000; // 1 second is the default
	alertExpire_ = 1000; // 1 second is the default
	createAlert_ = true;
	createUpdater_ = true;
	createCalculator_ = true;
	initialized_ = false;
    }

    // -----------------------------------------------------------------
    // IDL to Java Mapping
    // -----------------------------------------------------------------

    public void
    pre_init(org.omg.PortableInterceptor.ORBInitInfo info)
    {
    }

    synchronized public void
    post_init(org.omg.PortableInterceptor.ORBInitInfo info)
    {
	if(!initialized_)
	{
	    com.ooc.OBPortableInterceptor.ORBInitInfo obInfo =
		com.ooc.OBPortableInterceptor.ORBInitInfoHelper.narrow(info);
	    com.ooc.CORBA.ORB orb = (com.ooc.CORBA.ORB) obInfo.orb();
	    com.ooc.OB.Logger messageLogger = orb.logger();
	    java.util.Properties props = orb.properties();

	    //
	    // Create the balancer utility logger.
	    //
	    logger_ = new Logger(messageLogger, props);

	    java.util.Enumeration keys = props.propertyNames();
	    while(keys.hasMoreElements())
	    {
		String key = (String)keys.nextElement();
		if(!key.startsWith("ooc.balancer.util."))
		    continue;

		String value = props.getProperty(key);
		com.ooc.OB.Assert._OB_assert(value != null);

		if(key.equals("ooc.balancer.util.create_alert"))
		{
		    createAlert_ =
			value.equals("true") || value.equals("TRUE");
		}
		else if(key.equals("ooc.balancer.util.create_updater"))
		{
		    createUpdater_ =
			value.equals("true") || value.equals("TRUE");
		}
		else if(key.equals("ooc.balancer.util.create_calculator"))
		{
		    createCalculator_ =
			value.equals("true") || value.equals("TRUE");
		}
		else if(key.equals("ooc.balancer.util.update_freq"))
		{
		    updateFreq_ = Integer.parseInt(value);
		}
		else if(key.equals("ooc.balancer.util.alert_expire"))
		{
		    alertExpire_ = Integer.parseInt(value);
		}
		else if(key.equals("ooc.balancer.util.trace_level"))
		{
		    //
		    // Used by the Logger.
		    //
		}
		else
		    logger_.warning("`" + key + "': unknown property");
	    }

	    if(createAlert_)
	    {
		//
		// Create the LoadAlertInterceptor.
		//
		loadAlertInterceptor_ = new LoadAlertInterceptor(orb, logger_,
								 alertExpire_);

		//
		// Create the LoadAlert object.
		//
		LoadAlert loadAlert = new LoadAlert_impl(loadAlertInterceptor_);

		//
		// Register the LoadAlert initial reference.
		//
		try
		{
		    info.register_initial_reference("LoadAlert", loadAlert);
		}
		catch(org.omg.PortableInterceptor.
			ORBInitInfoPackage.InvalidName ex)
		{
		    com.ooc.OB.Assert._OB_assert(false); // Should not occur
		}
	    }

	    LoadCalculator_impl calc = null;
	    if(createCalculator_)
	    {
		//
		// Create the LoadCalculator local object.
		//
		calc = new LoadCalculator_impl();

		//
		// Create the LoadInterceptor.
		//
		loadInterceptor_ = new LoadInterceptor(calc);

		//
		// Register the LoadCalculator initial reference.
		//
		try
		{
		    info.register_initial_reference("LoadCalculator", calc);
		}
		catch(org.omg.PortableInterceptor.
			ORBInitInfoPackage.InvalidName ex)
		{
		    com.ooc.OB.Assert._OB_assert(false); // Should not occur
		}
	    }

	    if(createUpdater_)
	    {
		//
		// Get the GroupFactory object.
		//
		GroupFactory factory = null;
		try
		{
		    factory = GroupFactoryHelper.narrow(
			orb.resolve_initial_references("Balancer"));
		}
		catch(org.omg.CORBA.ORBPackage.InvalidName ex)
		{
		}
		catch(org.omg.CORBA.SystemException ex)
		{
		}
		if(factory == null)
		{
		    String msg = "unable to locate the Balancer";
		    throw new org.omg.CORBA.INITIALIZE(msg);
		}

		//
		// Create the LoadUpdater.
		//
		LoadUpdater updater =
		    new LoadUpdater_impl(logger_, factory, updateFreq_);

		//
		// Register the LoadUpdater initial reference.
		//
		try
		{
		    info.register_initial_reference("LoadUpdater", updater);
		}
		catch(org.omg.PortableInterceptor.
			ORBInitInfoPackage.InvalidName ex)
		{
		    com.ooc.OB.Assert._OB_assert(false); // Should not occur
		}

		//
		// If the LoadCalculator was also created, then pass it to the
		// LoadUpdater.
		//
		if(createCalculator_)
		    updater.set_load_calculator(calc);
	    }

	    initialized_ = true;
	}

	//
	// Register the LoadAlertInterceptor.
	//
	if(createAlert_)
	{
	    try
	    {
		info.add_server_request_interceptor(loadAlertInterceptor_);
	    }
	    catch(org.omg.PortableInterceptor.
		    ORBInitInfoPackage.DuplicateName ex)
	    {
		com.ooc.OB.Assert._OB_assert(false); // Should not occur
	    }
	}

	//
	// Register the LoadInterceptor.
	//
	if(createCalculator_)
	{
	    try
	    {
		info.add_server_request_interceptor(loadInterceptor_);
	    }
	    catch(org.omg.PortableInterceptor.
		    ORBInitInfoPackage.DuplicateName ex)
	    {
		com.ooc.OB.Assert._OB_assert(false); // Should not occur
	    }
	}
    }
}

//
// Server Request Interceptor which computes load based on the number of
// requests
//
class LoadInterceptor
    extends org.omg.CORBA.LocalObject
    implements org.omg.PortableInterceptor.ServerRequestInterceptor
{
    //
    // The LoadCalculator
    //
    private LoadCalculator_impl calculator_;

    // -----------------------------------------------------------------
    // LoadInterceptor constructor
    // -----------------------------------------------------------------

    public LoadInterceptor(LoadCalculator_impl calc)
    {
	calculator_ = calc;
    }

    // -----------------------------------------------------------------
    // IDL to Java Mapping
    // -----------------------------------------------------------------

    public String
    name()
    {
	return "LoadInterceptor";
    }

    public void
    destroy()
    {
    }

    public void
    receive_request_service_contexts(
    	org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	calculator_.newRequest();
    }

    public synchronized void
    receive_request(org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
    }
	
    public void
    send_reply(org.omg.PortableInterceptor.ServerRequestInfo ri)
    {
	calculator_.requestCompleted();
    }
	
    public void
    send_other(org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	calculator_.requestCompleted();
    }

    public void
    send_exception(org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	calculator_.requestCompleted();
    }
}
