// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.BiDir;

public class CallbackAcceptor_impl extends org.omg.CORBA.LocalObject
    implements com.ooc.OCI.Acceptor
{
    //
    // The TransportManager
    //
    private TransportManager transportManager_;

    //
    // Acceptor information
    //
    private AcceptorInfo_impl info_;

    //
    // Transports waiting to be accepted
    //
    private java.util.Vector transports_ = new java.util.Vector();
    private java.lang.Object transportsMonitor_ = new java.lang.Object();

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    id()
    {
        return transportManager_.id();
    }

    public int
    tag()
    {
        return transportManager_.tag();
    }

    public int
    handle()
    {
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    public void
    close()
    {
        transportManager_.setCallbackAcceptor(null);
    }

    public void
    shutdown()
    {
        //
        // Nothing to do
        //
    }

    public void
    listen()
    {
        //
        // This operation does nothing in the Java implementation
        //
    }

    public com.ooc.OCI.Transport
    accept(boolean block)
    {
        com.ooc.OCI.Transport t = null;

        synchronized(transportsMonitor_)
        {
            //
            // Are there transports waiting to be accepted?
            //
            if(transports_.size() > 0)
            {
                t = (com.ooc.OCI.Transport)transports_.firstElement();
                transports_.removeElementAt(0);
            }
            else if(block)
            {
                while(transports_.isEmpty())
                {
                    try
                    {
                        transportsMonitor_.wait();
                    }
                    catch(InterruptedException ex)
                    {
                    }
                }

                t = (com.ooc.OCI.Transport)transports_.firstElement();
                transports_.removeElementAt(0);
            }
        }

        //
        // Call callbacks
        //
        if(t != null)
        {
            com.ooc.OCI.TransportInfo info = t.get_info();
            try
            {
                info_._OB_callAcceptCB(info);
            }
            catch(org.omg.CORBA.SystemException ex)
            {
                t.close();
                throw ex;
            }
        }

        //
        // Return new transport
        //
        return t;
    }

    public com.ooc.OCI.Transport
    connect_self()
    {
        SelfTransport_impl t = new SelfTransport_impl();

        synchronized(transportsMonitor_)
        {
            transports_.addElement(t);
            transportsMonitor_.notifyAll();
        }

        return t;
    }

    public void
    add_profiles(com.ooc.OCI.ProfileInfo profile_info,
                 org.omg.IOP.IORHolder ref)
    {
        //
        // Filter components
        //
        // TODO: We should really delegate this to something else
        //
        java.util.Vector components = new java.util.Vector();
        if(profile_info.major == 1 && profile_info.minor == 0)
        {
            //
            // No components
            //
        }
        else if(profile_info.major == 1 && profile_info.minor == 1)
        {
            //
            // No code set component
            //
            for(int i = 0 ; i < profile_info.components.length ; i++)
                if(profile_info.components[i].tag !=
                   org.omg.IOP.TAG_CODE_SETS.value)
                {
                    components.addElement(profile_info.components[i]);
                }
        }
        else
        {
            for(int i = 0 ; i < profile_info.components.length ; i++)
                components.addElement(profile_info.components[i]);
        }

        //
        // Create BiDirIOP profile
        //
        com.ooc.BiDirIOP.ProfileBody body =
            new com.ooc.BiDirIOP.ProfileBody();
        body.bidiriop_version = new com.ooc.BiDirIOP.Version((byte)1, (byte)0);
        body.giop_version = new com.ooc.BiDirIOP.Version(profile_info.major,
                                                         profile_info.minor);
        body.peer = transportManager_.peer();
        body.object_key = profile_info.key;
        body.components = new org.omg.IOP.TaggedComponent[components.size()];
        components.copyInto(body.components);
        body.profiles = new org.omg.IOP.TaggedProfile[0];

        //
        // Append the BiDirIOP profile to the IOR
        //
        int len = ref.value.profiles.length + 1;
        org.omg.IOP.TaggedProfile[] profiles =
            new org.omg.IOP.TaggedProfile[len];
        System.arraycopy(ref.value.profiles, 0, profiles, 0,
                         ref.value.profiles.length);
        ref.value.profiles = profiles;
        ref.value.profiles[len - 1] = new org.omg.IOP.TaggedProfile();
        ref.value.profiles[len - 1].tag = tag();
        com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer();
        com.ooc.CORBA.OutputStream out =
            new com.ooc.CORBA.OutputStream(buf);
        out._OB_writeEndian();
        com.ooc.BiDirIOP.ProfileBodyHelper.write(out, body);
        ref.value.profiles[len - 1].profile_data = new byte[buf.length()];
        System.arraycopy(buf.data(), 0,
                         ref.value.profiles[len - 1].profile_data,
                         0, buf.length());
    }

    public com.ooc.OCI.ProfileInfo[]
    get_local_profiles(org.omg.IOP.IOR ref)
    {
        java.util.Vector seq = new java.util.Vector();
        for(int i = 0 ; i < ref.profiles.length ; i++)
        {
            if(ref.profiles[i].tag == tag())
            {
                //
                // Get ProfileBody
                //
                com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer(
                    ref.profiles[i].profile_data,
                    ref.profiles[i].profile_data.length);
                com.ooc.CORBA.InputStream in =
                    new com.ooc.CORBA.InputStream(buf, 0, false);
                in._OB_readEndian();
                com.ooc.BiDirIOP.ProfileBody body =
                    com.ooc.BiDirIOP.ProfileBodyHelper.read(in);

                if(body.peer.equals(transportManager_.peer()) &&
                   body.profiles.length == 0)
                {
                    com.ooc.OCI.ProfileInfo info =
                        new com.ooc.OCI.ProfileInfo();
                    info.major = body.giop_version.major;
                    info.minor = body.giop_version.minor;
                    info.key = body.object_key;
                    info.components = body.components;
                    seq.addElement(info);
                }
            }
        }

        com.ooc.OCI.ProfileInfo[] result =
            new com.ooc.OCI.ProfileInfo[seq.size()];
        seq.copyInto(result);
        return result;
    }

    public com.ooc.OCI.AcceptorInfo
    get_info()
    {
        return info_;
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    CallbackAcceptor_impl(TransportManager transportManager)
    {
        transportManager_ = transportManager;
        info_ = new AcceptorInfo_impl(transportManager.id(),
                                      transportManager.tag(),
                                      transportManager.peer(), null);
        transportManager_.setCallbackAcceptor(this);
    }

    void
    _OB_connect(com.ooc.OCI.Transport t)
    {
        synchronized(transportsMonitor_)
        {
            transports_.addElement(t);
            transportsMonitor_.notifyAll();
        }
    }
}
