// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.BiDir;

public class ConFactory_impl extends org.omg.CORBA.LocalObject
    implements com.ooc.OCI.ConFactory
{
    //
    // The ConFactory delegate
    //
    private com.ooc.OCI.ConFactory delegate_;

    //
    // The TransportManager
    //
    private TransportManager transportManager_;

    //
    // ConFactory information
    //
    private ConFactoryInfo_impl info_;

    // ------------------------------------------------------------------
    // Private and protected member implementations
    // ------------------------------------------------------------------

    private boolean
    compareBodies(com.ooc.BiDirIOP.ProfileBody body1,
                  com.ooc.BiDirIOP.ProfileBody body2)
    {
        //
        // Compare versions
        //
        if(body1.giop_version.major != body2.giop_version.major ||
           body1.giop_version.minor != body2.giop_version.minor)
            return false;

        //
        // Compare peer IDs
        //
        if(!body1.peer.equals(body2.peer))
            return false;

        //
        // Compare object keys
        //
        if(body1.object_key.length != body2.object_key.length)
            return false;

        for(int i = 0 ; i < body1.object_key.length ; i++)
            if(body1.object_key[i] != body2.object_key[i])
                return false;

        //
        // Compare nested profiles
        //
        if(body1.profiles.length != body2.profiles.length)
            return false;
        if(body1.profiles.length > 0)
        {
            org.omg.IOP.IOR ior1 = new org.omg.IOP.IOR();
            ior1.type_id = "";
            ior1.profiles = body1.profiles;
            org.omg.IOP.IOR ior2 = new org.omg.IOP.IOR();
            ior2.type_id = "";
            ior2.profiles = body2.profiles;
            if(!delegate_.equivalent(ior1, ior2))
                return false;
        }

        //
        // OK, found a match
        //
        return true;
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    id()
    {
        return transportManager_.id();
    }

    public int
    tag()
    {
        return transportManager_.tag();
    }

    public String
    describe_profile(org.omg.IOP.TaggedProfile profile)
    {
        //
        // Get the profile body
        //
        byte[] data = profile.profile_data;
        com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer(data, data.length);
        com.ooc.CORBA.InputStream in = new com.ooc.CORBA.InputStream(buf);
        in._OB_readEndian();
        com.ooc.BiDirIOP.ProfileBody body =
            com.ooc.BiDirIOP.ProfileBodyHelper.read(in);

        StringBuffer result = new StringBuffer();

        //
        // Show general info
        //
        result.append("bidiriop_version: " + (int)body.bidiriop_version.major +
                      '.' + (int)body.bidiriop_version.minor + '\n');
        result.append("giop_version: " + (int)body.giop_version.major +
                      '.' + (int)body.giop_version.minor + '\n');
        result.append("peer: " + body.peer + '\n');
        result.append("object_key: (" + body.object_key.length + ")\n");
        result.append(com.ooc.OB.IORUtil.dump_octets(body.object_key, 0,
                                                     body.object_key.length));

        //
        // Components
        //
        for(int i = 0 ; i < body.components.length ; i++)
        {
            result.append(
                com.ooc.OB.IORUtil.describe_component(body.components[i]));
        }

        //
        // Nested profiles
        //
        for(int i = 0 ; i < body.profiles.length ; i++)
        {
            result.append("Encapsulated profile #" + (i + 1) + ":\n");
            result.append(delegate_.describe_profile(body.profiles[i]));
        }

        return result.toString();
    }

    public com.ooc.OCI.Connector[]
    create_connectors(org.omg.IOP.IOR ref, org.omg.CORBA.Policy[] policies)
    {
        //
        // Check whether policies are satisfied
        //
        for(int i = 0 ; i < policies.length ; i++)
        {
            if(policies[i].policy_type() ==
               com.ooc.OB.PROTOCOL_POLICY_ID.value)
            {
                com.ooc.OB.ProtocolPolicy p =
                    com.ooc.OB.ProtocolPolicyHelper.narrow(policies[i]);
                if(!p.contains(transportManager_.id()))
                    return new com.ooc.OCI.Connector[0];
            }
        }

        //
        // Create connectors from profiles
        //
        com.ooc.OCI.ConnectCB[] cbs = info_._OB_getConnectCBSeq();
        java.util.Vector seq = new java.util.Vector();
        for(int i = 0 ; i < ref.profiles.length ; i++)
        {
            if(ref.profiles[i].tag == tag())
            {
                //
                // Get ProfileBody
                //
                byte[] data = ref.profiles[i].profile_data;
                com.ooc.OCI.Buffer buf =
                    new com.ooc.OCI.Buffer(data, data.length);
                com.ooc.CORBA.InputStream in =
                    new com.ooc.CORBA.InputStream(buf);
                in._OB_readEndian();
                com.ooc.BiDirIOP.ProfileBody profile =
                    com.ooc.BiDirIOP.ProfileBodyHelper.read(in);

                //
                // Create a BiDir connector for each connector returned
                // by our delegate
                //
                if(profile.profiles.length > 0)
                {
                    org.omg.IOP.IOR dummy = new org.omg.IOP.IOR();
                    dummy.type_id = ref.type_id;
                    dummy.profiles = profile.profiles;
                    com.ooc.OCI.Connector[] delegates =
                        delegate_.create_connectors(dummy, policies);
                    for(int c = 0 ; c < delegates.length ; c++)
                    {
                        Connector_impl impl =
                            new Connector_impl(transportManager_, profile,
                                               cbs, delegates[c]);
                        seq.addElement(impl);
                    }
                }
                else
                {
                    //
                    // If the BiDir profile contains no encapsulated
                    // profiles, then we must use an existing transport,
                    // so we create a connector with no delegate
                    //
                    Connector_impl impl =
                        new Connector_impl(transportManager_, profile, cbs,
                                           null);
                    seq.addElement(impl);
                }
            }
        }

        com.ooc.OCI.Connector[] result = new com.ooc.OCI.Connector[seq.size()];
        seq.copyInto(result);
        return result;
    }

    public boolean
    equivalent(org.omg.IOP.IOR ior1, org.omg.IOP.IOR ior2)
    {
        int p1, p2, b1, b2;
        int cnt1 = 0, cnt2 = 0;
        com.ooc.BiDirIOP.ProfileBody[] bodies1;
        com.ooc.BiDirIOP.ProfileBody[] bodies2;

        //
        // Calculate number of BiDirIOP profiles in ior1
        //
        for(p1 = 0 ; p1 < ior1.profiles.length ; p1++)
            if(ior1.profiles[p1].tag == tag())
                cnt1++;

        //
        // Calculate number of BiDirIOP profiles in ior2
        //
        for(p2 = 0 ; p2 < ior2.profiles.length ; p2++)
            if(ior2.profiles[p2].tag == tag())
                cnt2++;

        //
        // Return false now if the number of BiDirIOP profile bodies
        // do not match
        //
        if(cnt1 != cnt2)
            return false;

        //
        // Create an array with all BiDirIOP profile bodies of ior1
        //
        bodies1 = new com.ooc.BiDirIOP.ProfileBody[cnt1];
        for(p1 = 0, b1 = 0 ; p1 < ior1.profiles.length ; p1++)
            if(ior1.profiles[p1].tag == tag())
            {
                byte[] data = ior1.profiles[p1].profile_data;
                com.ooc.OCI.Buffer buf =
                    new com.ooc.OCI.Buffer(data, data.length);
                com.ooc.CORBA.InputStream in =
                    new com.ooc.CORBA.InputStream(buf, 0, false);
                in._OB_readEndian();
                bodies1[b1++] = com.ooc.BiDirIOP.ProfileBodyHelper.read(in);
            }

        com.ooc.OB.Assert._OB_assert(b1 == cnt1);

        //
        // Create an array with all BiDirIOP profile bodies of ior2
        //
        bodies2 = new com.ooc.BiDirIOP.ProfileBody[cnt2];
        for(p2 = 0, b2 = 0 ; p2 < ior2.profiles.length ; p2++)
            if(ior2.profiles[p2].tag == tag())
            {
                byte[] data = ior2.profiles[p2].profile_data;
                com.ooc.OCI.Buffer buf =
                    new com.ooc.OCI.Buffer(data, data.length);
                com.ooc.CORBA.InputStream in =
                    new com.ooc.CORBA.InputStream(buf, 0, false);
                in._OB_readEndian();
                bodies2[b2++] = com.ooc.BiDirIOP.ProfileBodyHelper.read(in);
            }

        com.ooc.OB.Assert._OB_assert(b2 == cnt2);

        //
        // Check for profile body matches
        //
        for(b1 = 0 ; b1 < cnt1 ; b1++)
        {
            boolean match = false;
            for(b2 = 0 ; b2 < cnt2 ; b2++)
            {
                if(bodies2[b2] == null)
                    continue;

                //
                // Compare profile bodies
                //
                if(!compareBodies(bodies1[b1], bodies2[b2]))
                    continue;

                //
                // OK, found a match
                //
                bodies2[b2] = null;
                match = true;
            }

            if(match)
                bodies1[b1] = null;
        }

        //
        // Check whether there are any unmatched BiDirIOP profile bodies
        //
        for(b1 = 0 ; b1 < cnt1 ; b1++)
            if(bodies1[b1] != null)
                return false;

        for(b2 = 0 ; b2 < cnt2 ; b2++)
            if(bodies2[b2] != null)
                return false;

        return true;
    }

    public int
    hash(org.omg.IOP.IOR ref, int maximum)
    {
        return delegate_.hash(ref, maximum);
    }

    public com.ooc.OCI.ConFactoryInfo
    get_info()
    {
        return info_;
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    public
    ConFactory_impl(com.ooc.OCI.ConFactory delegate,
                    TransportManager tm)
    {
        delegate_ = delegate;
        transportManager_ = tm;
        info_ = new ConFactoryInfo_impl(tm.id(), tm.tag(), tm.peer(),
                                        delegate.get_info());
    }

    public TransportManager
    _OB_transportManager()
    {
        return transportManager_;
    }
}
