// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.BiDir;

public class Connector_impl extends org.omg.CORBA.LocalObject
    implements com.ooc.OCI.Connector
{
    //
    // The TransportManager
    //
    private TransportManager transportManager_;

    //
    // The profile body
    //
    private com.ooc.BiDirIOP.ProfileBody profile_;

    //
    // The Connector delegate
    //
    private com.ooc.OCI.Connector delegate_;

    //
    // Connector information
    //
    private ConnectorInfo_impl info_;

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    id()
    {
        return transportManager_.id();
    }

    public int
    tag()
    {
        return transportManager_.tag();
    }

    public com.ooc.OCI.Transport
    connect()
    {
        //
        // Delegate to TransportManager
        //
        com.ooc.OCI.Transport tr =
            transportManager_.connect(profile_.peer, this, delegate_);

        if(tr == null)
        {
            throw new org.omg.CORBA.TRANSIENT(
                com.ooc.OB.MinorCodes.describeTransient(
                    com.ooc.OB.MinorCodes.MinorConnectFailed),
                com.ooc.OB.MinorCodes.MinorConnectFailed,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        //
        // Call callbacks
        //
        com.ooc.OCI.TransportInfo info = tr.get_info();
        try
        {
            info_._OB_callConnectCB(info);
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            tr.close();
            throw ex;
        }

        return tr;
    }

    public com.ooc.OCI.Transport
    connect_timeout(int timeout)
    {
        //
        // Delegate to TransportManager
        //
        com.ooc.OCI.Transport tr =
            transportManager_.connectTimeout(profile_.peer, this, delegate_,
                                             timeout);

        //
        // Call callbacks
        //
        if(tr != null)
        {
            com.ooc.OCI.TransportInfo info = tr.get_info();
            try
            {
                info_._OB_callConnectCB(info);
            }
            catch(org.omg.CORBA.SystemException ex)
            {
                tr.close();
                throw ex;
            }
        }

        return tr;
    }

    public com.ooc.OCI.ProfileInfo[]
    get_usable_profiles(org.omg.IOP.IOR ref,
                        org.omg.CORBA.Policy[] policies)
    {
        //
        // Make sure that the set of policies is met
        //
        for(int i = 0 ; i < policies.length ; i++)
        {
            if(policies[i].policy_type() ==
               com.ooc.OB.PROTOCOL_POLICY_ID.value)
            {
                com.ooc.OB.ProtocolPolicy p =
                    com.ooc.OB.ProtocolPolicyHelper.narrow(policies[i]);
                if(!p.contains(transportManager_.id()))
                    return new com.ooc.OCI.ProfileInfo[0];
            }
        }

        java.util.Vector seq = new java.util.Vector();
        for(int i = 0 ; i < ref.profiles.length ; i++)
        {
            if(ref.profiles[i].tag == tag())
            {
                //
                // Get ProfileBody
                //
                byte[] data = ref.profiles[i].profile_data;
                com.ooc.OCI.Buffer buf =
                    new com.ooc.OCI.Buffer(data, data.length);
                com.ooc.CORBA.InputStream in =
                    new com.ooc.CORBA.InputStream(buf);
                in._OB_readEndian();
                com.ooc.BiDirIOP.ProfileBody body =
                    com.ooc.BiDirIOP.ProfileBodyHelper.read(in);

                if(body.peer.equals(profile_.peer))
                {
                    com.ooc.OCI.ProfileInfo profileInfo =
                        new com.ooc.OCI.ProfileInfo();
                    profileInfo.major = body.giop_version.major;
                    profileInfo.minor = body.giop_version.minor;
                    profileInfo.key = body.object_key;
                    profileInfo.id = ref.profiles[i].tag;
                    profileInfo.index = i;

                    //
                    // Compose set of components, including any returned
                    // by delegate
                    //
                    java.util.Vector comp = new java.util.Vector();
                    for(int j = 0 ; j < body.components.length ; j++)
                        comp.addElement(body.components[j]);

                    if(delegate_ != null)
                    {
                        org.omg.IOP.IOR dummyIOR = new org.omg.IOP.IOR();
                        dummyIOR.type_id = ref.type_id;
                        dummyIOR.profiles = body.profiles;
                        com.ooc.OCI.ProfileInfo[] arr =
                            delegate_.get_usable_profiles(dummyIOR, policies);
                        for(int j = 0 ; j < arr.length ; j++)
                            for(int k = 0 ; k < arr[j].components.length ; k++)
                                comp.addElement(arr[j].components[k]);
                    }

                    profileInfo.components =
                        new org.omg.IOP.TaggedComponent[comp.size()];
                    comp.copyInto(profileInfo.components);
                    seq.addElement(profileInfo);
                }
            }
        }

        com.ooc.OCI.ProfileInfo[] result =
            new com.ooc.OCI.ProfileInfo[seq.size()];
        seq.copyInto(result);
        return result;
    }

    public boolean
    equal(com.ooc.OCI.Connector con)
    {
        Connector_impl impl = null;
        try
        {
            impl = (Connector_impl)con;
        }
        catch(ClassCastException ex)
        {
            return false;
        }

        //
        // Compare peer IDs
        //
        if(!profile_.peer.equals(impl.profile_.peer))
            return false;

        //
        // Compare encapsulated profiles
        //
        if(profile_.profiles.length != impl.profile_.profiles.length)
            return false;

        //
        // Compare delegates
        //
        if((delegate_ == null && impl.delegate_ != null) ||
           (delegate_ != null && impl.delegate_ == null))
            return false;
        if(delegate_ != null && !delegate_.equal(impl.delegate_))
            return false;

        //
        // OK, connectors are the same
        //
        return true;
    }

    public com.ooc.OCI.ConnectorInfo
    get_info()
    {
        return info_;
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    Connector_impl(TransportManager transportManager,
                   com.ooc.BiDirIOP.ProfileBody profile,
                   com.ooc.OCI.ConnectCB[] callbacks,
                   com.ooc.OCI.Connector delegate)
    {
        transportManager_ = transportManager;
        profile_ = profile;
        delegate_ = delegate;
        com.ooc.OCI.ConnectorInfo info = null;
        if(delegate_ != null)
            info = delegate_.get_info();
        info_ = new ConnectorInfo_impl(transportManager.id(),
                                       transportManager.tag(),
                                       transportManager.peer(), profile.peer,
                                       callbacks, info);
    }
}
