// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package pi;

//
// Server Request Interceptor logger -- logs server activity
//
final class SRILogger_impl extends org.omg.CORBA.LocalObject
    implements org.omg.PortableInterceptor.ServerRequestInterceptor
{
    //
    // The RILogger object
    //
    private RILogger riLogger_;

    //
    // Set to riLogger_.out_ for ease of use
    //
    private AnyWriter out_;

    // -----------------------------------------------------------------
    // SRILogger private member implementation
    // -----------------------------------------------------------------

    //
    // Get the logging level
    //
    private short
    level(org.omg.PortableInterceptor.ServerRequestInfo ri)
    {
	short l = 0;

	try
	{
	    //
	    // Get the logger policy
	    //
	    org.omg.CORBA.Policy policy =
		ri.get_server_policy(LOGGER_POLICY_ID.value);
	    LoggerPolicy loggerPolicy = LoggerPolicyHelper.narrow(policy);
	    l = loggerPolicy.level();
	}
	catch(org.omg.CORBA.INV_POLICY ex)
	{
	    // Failed to get policy
	}
	catch(org.omg.CORBA.BAD_PARAM ex)
	{
	    // Failed to narrow policy
	}

	return l;
    }

    //
    // Display PortableInterceptor::ServerRequestInfo
    //
    private void
    displayServerRequestInfo(org.omg.PortableInterceptor.ServerRequestInfo info,
			     short logLevel)
    {
	int i;

	riLogger_.displayRequestInfo(info, logLevel);

	//
	// Display sending exception
	//
	try
	{
	    org.omg.CORBA.Any exception = info.sending_exception();

	    out_.skip();
	    out_.getWriter().println("sending exception = ");
	    out_.down();
	    out_.skip();
	    out_.print(exception);
	    out_.getWriter().println();
	    out_.up();
	}
	catch(org.omg.CORBA.BAD_INV_ORDER ex)
	{
	    // Operation not supported in this context
	}

	if(logLevel > 2)
	{
	    //
	    // Display object id
	    //
	    try
	    {
		byte[] octets = info.object_id();

		out_.skip();
		out_.getWriter().print("object id = ");

		for (i = 0 ; i < octets.length ; i++)
		    out_.getWriter().print(riLogger_.getOctet(octets[i]));

		out_.getWriter().println();
	    }
	    catch(org.omg.CORBA.BAD_INV_ORDER ex)
	    {
		// Operation not supported in this context
	    }

	    //
	    // Display adapter id
	    //
	    try
	    {
		byte[] octets = info.adapter_id();

		out_.skip();
		out_.getWriter().print("adapter id = ");

		for (i = 0 ; i < octets.length ; i++)
		    out_.getWriter().print(riLogger_.getOctet(octets[i]));

		out_.getWriter().println();
	    }
	    catch(org.omg.CORBA.BAD_INV_ORDER ex)
	    {
		// Operation not supported in this context
	    }
	}

	//
	// Display target most derived interface
	//
	try
	{
	    String tmdi = info.target_most_derived_interface();
	    out_.skip();
	    out_.getWriter().println("target most derived interface = " + tmdi);
	}
	catch(org.omg.CORBA.BAD_INV_ORDER ex)
	{
	    // Operation not supported in this context
	}
    }

    // -----------------------------------------------------------------
    // CRILogger constructor
    // -----------------------------------------------------------------

    public SRILogger_impl(org.omg.PortableInterceptor.ORBInitInfo info,
    			  int slotId)
    {
	riLogger_ = new RILogger(info, slotId);
	out_ = riLogger_.out_;
    }

    // -----------------------------------------------------------------
    // IDL to Java Mapping
    // -----------------------------------------------------------------

    //
    // The name of the interceptor.
    //
    public String
    name()
    {
	return "SRILogger";
    }

    //
    // Clean up prior to ORB destruction
    //
    public void
    destroy()
    {
    }

    //
    // At this interception point, Interceptors must get their service
    // context information from the incoming request and transfer it
    // to a slot of PortableInterceptor::Current.
    //
    public void
    receive_request_service_contexts(
    	org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	short l = level(ri);
	if(l > 0)
	{
	    out_.getWriter().println("receive request service contexts:");
	    displayServerRequestInfo(ri, l);
	    out_.getWriter().println();

	    //
	    // Get the CallStack from the service context
	    //
	    String[] callStack = null;
	    try
	    {
		//
		// Get encoded CallStack from service context
		//
		org.omg.IOP.ServiceContext sc =
		    ri.get_request_service_context(
			riLogger_.REQUEST_CONTEXT_ID);

		//
		// Decode CallStack
		//
		org.omg.CORBA.Any any = null;
		try
		{
		    any = riLogger_.cdrCodec_.decode_value(
			sc.context_data, CallStackHelper.type());
		}
		catch(org.omg.IOP.CodecPackage.TypeMismatch ex)
		{
		    //
		    // Ignore failure in this demo
		    //
		    return;
		}
		catch(org.omg.IOP.CodecPackage.FormatMismatch ex)
		{
		    //
		    // Ignore failure in this demo
		    //
		    return;
		}

		String[] stack = null;
		try
		{
		    stack = CallStackHelper.extract(any);
		}
		catch(org.omg.CORBA.BAD_OPERATION ex)
		{
		    //
		    // Ignore failure in this demo
		    //
		    return;
		}
		if(stack == null)
		    throw new RuntimeException();

		//
		// Create new CallStack to hold old CallStack plus
		// one new call
		//
		callStack = new String[stack.length + 1];

		//
		// Copy old CallStack to new CallStack
		//
		System.arraycopy(stack, 0, callStack, 0, stack.length);
	    }
	    catch(org.omg.CORBA.BAD_PARAM ex)
	    {
		//
		// Service context was not previously set -- create new
		// CallStack to contain the new call
		//
		callStack = new String[1];
	    }

	    //
	    // Add new call to the CallStack
	    //
	    callStack[callStack.length - 1] = ri.operation();
		
	    //
	    // Save the new CallStack in slot
	    //
	    org.omg.CORBA.Any any = org.omg.CORBA.ORB.init().create_any();
	    CallStackHelper.insert(any, callStack);
	    try
	    {
		ri.set_slot(riLogger_.mySlotId_, any);
	    }
	    catch(org.omg.PortableInterceptor.InvalidSlot ex)
	    {
		//
		// Ignore failure in this demo
		//
		return;
	    }

	    //
	    // Print the CallStack
	    //
	    out_.skip();
	    out_.getWriter().println("stack =");
	    out_.down();
	    for(int i = 0 ; i < callStack.length ; i++)
	    {
		out_.skip();
		out_.getWriter().println((i + 1) + ".	" + callStack[i]);
	    }
	    out_.up();
	    out_.getWriter().println();
	}
    }

    //
    // This interception point allows an Interceptor to query request
    // information after all the information, including operation
    // parameters, are available.
    //
    public void
    receive_request(org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	short l = level(ri);
	if(l > 0)
	{
	    out_.getWriter().println("receive request:");
	    displayServerRequestInfo(ri, l);
	    out_.getWriter().println();
	}
    }
    
    //
    // This interception point allows an Interceptor to query reply
    // information and modify the reply service context after the
    // target operation has been invoked and before the reply is
    // returned to the client.
    //
    public void
    send_reply(org.omg.PortableInterceptor.ServerRequestInfo ri)
    {
	short l = level(ri);
	if(l > 0)
	{
	    out_.getWriter().println("send reply:");
	    displayServerRequestInfo(ri, l);
	    out_.getWriter().println();
	}
    }
    
    //
    // This interception point allows an Interceptor to query the
    // exception information and modify the reply service context
    // before the exception is raised to the client.
    //
    public void
    send_exception(org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	short l = level(ri);
	if(l > 0)
	{
	    out_.getWriter().println("send exception:");
	    displayServerRequestInfo(ri, l);
	    out_.getWriter().println();
	}
    }

    //
    // This interception point allows an Interceptor to query
    // the information available when a request results in
    // something other thatn a normal reply or an exception.
    //
    public void
    send_other(org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	short l = level(ri);
	if(l > 0)
	{
	    out_.getWriter().println("send other:");
	    displayServerRequestInfo(ri, l);
	    out_.getWriter().println();
	}
    }
}
