// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OBPortableServer;

final public class POAManagerFactory_impl extends org.omg.CORBA.LocalObject
    implements POAManagerFactory
{
    //
    // The ORB Instance
    //
    private com.ooc.OB.ORBInstance orbInstance_;

    //
    // Hashtable mapping name -> POAManager
    //
    private java.util.Hashtable managers_ = new java.util.Hashtable();

    //
    // Running count for generating unique names
    //
    private int count_ = 0;

    //
    // The IMRActiveStateHolder
    //
    private com.ooc.IMR.ActiveState activeState_;

    //
    // The POALocator
    //
    private POALocator poaLocator_;

    //
    // The OAD::ProcessEndpoint
    //
    private com.ooc.OAD.ProcessEndpoint_impl processEndpoint_;

    //
    // This has to be a member of the ORB since we need to keep the
    // connection open for the lifespan of the process.
    //
    // TODO: When we have connection reaping then we'll have to have
    // to set a policy on this object to prevent the connection from
    // being reaped.
    //
    private org.omg.CORBA.Object endpointManager_;

    private String
    getUniqueName()
    {
        long now = System.currentTimeMillis();

        String name = "POAManager-" + now;
        name += count_++;

        com.ooc.OB.Assert._OB_assert(!managers_.containsKey(name));

        return name;
    }

    private void
    validateName(String name)
        throws com.ooc.OBPortableServer.
               POAManagerFactoryPackage.POAManagerAlreadyExists
    {
        //
        // Does the POAManager exist?
        //
        POAManager manager = (POAManager)managers_.get(name);
        if(manager != null)
        {
            //
            // If the POAManager is INACTIVE then remove it from
            // the list, and allow the user to re-add the
            // POAManager
            //
            if(manager.get_state() ==
               org.omg.PortableServer.POAManagerPackage.State.INACTIVE)
                managers_.remove(name);
            else
                throw new com.ooc.OBPortableServer.
                    POAManagerFactoryPackage.POAManagerAlreadyExists();
        }
    }

    // ----------------------------------------------------------------------
    // OBPOAManagerFactory_impl public member implementation
    // ----------------------------------------------------------------------

    public
    POAManagerFactory_impl()
    {
        managers_ = new java.util.Hashtable(7);
	poaLocator_ = new POALocator();
    }

    // ----------------------------------------------------------------------
    // Standard IDL to Java mapping
    // ----------------------------------------------------------------------

    public com.ooc.OBPortableServer.POAManager
    create_poa_manager(String name)
        throws com.ooc.OBPortableServer.
               POAManagerFactoryPackage.POAManagerAlreadyExists,
               com.ooc.OCI.InvalidParam
    {
        synchronized(managers_)
        {
            if(name.length() == 0)
	    {
                name = getUniqueName();
	    }
            else
	    {
                validateName(name);
		++count_;
	    }

            java.util.Properties props = orbInstance_.getProperties();
            com.ooc.OB.Logger logger = orbInstance_.getLogger();

            //
            // Get the endpoint configuration
            //
            String rootStr = null;
            String paramStr = null;
            if(name.equals("RootPOAManager"))
                rootStr = props.getProperty("ooc.orb.oa.endpoint");
            String propName = "ooc.orb.poamanager." + name + ".endpoint";
            paramStr = props.getProperty(propName);
            if(paramStr == null && rootStr == null)
                paramStr = "iiop";
            else if(paramStr == null)
                paramStr = rootStr;

            com.ooc.OCI.AccFactoryRegistry registry =
                orbInstance_.getAccFactoryRegistry();
            com.ooc.OCI.AccFactory[] factories = registry.get_factories();

            java.util.Vector acceptors = new java.util.Vector();
            int pos = 0;
            while(pos != -1)
            {
                java.util.Vector params = new java.util.Vector();
                pos = com.ooc.OB.ParseParams.parse(paramStr, pos, params);
                if(!params.isEmpty())
                {
                    String prot = (String)params.firstElement();
                    params.removeElementAt(0);
                    int i;
                    for(i = 0 ; i < factories.length ; i++)
                    {
                        if(prot.equals(factories[i].id()))
                        {
                            try
                            {
                                String[] paramSeq = new String[params.size()];
                                params.copyInto(paramSeq);

                                com.ooc.OCI.Acceptor acceptor =
                                    factories[i].create_acceptor(paramSeq);
                                acceptors.addElement(acceptor);
                                break;
                            }
                            catch(com.ooc.OCI.InvalidParam ex)
                            {
                                String err = "unable to create acceptor: " +
                                    ex.reason;
                                logger.error(err);
                                throw new org.omg.CORBA.INITIALIZE(err);
                            }
                        }
                    }
                    if(i >= factories.length)
                    {
                        String err = "unknown endpoint protocol `" + prot +
                            "'";
                        logger.error(err);
                        throw new org.omg.CORBA.INITIALIZE(err);
                    }
                }
            }

            if(acceptors.size() == 0)
            {
                String err = "no endpoints defined";
                logger.error(err);
                throw new org.omg.CORBA.INITIALIZE(err);
            }

            //
            // Create the new POAManager_impl and add to the table
            //
            com.ooc.OCI.Acceptor[] arr =
                new com.ooc.OCI.Acceptor[acceptors.size()];
            acceptors.copyInto(arr);
            POAManager manager = new POAManager_impl(orbInstance_,
                                                     poaLocator_, name,
                                                     count_, arr);
            managers_.put(name, manager);

            return manager;
	}
    }

    public com.ooc.OBPortableServer.POAManager
    create_poa_manager_with_config(String name,
                                   com.ooc.OBPortableServer.
                                   POAManagerFactoryPackage.AcceptorConfig[]
                                   config)
        throws com.ooc.OBPortableServer.
               POAManagerFactoryPackage.POAManagerAlreadyExists,
               com.ooc.OCI.NoSuchFactory,
               com.ooc.OCI.InvalidParam
    {
	synchronized(managers_)
	{
            if(name.length() == 0)
	    {
                name = getUniqueName();
	    }
            else
	    {
                validateName(name);
		++count_;
	    }


            //
            // Create acceptors
            //
            com.ooc.OCI.AccFactoryRegistry registry =
                orbInstance_.getAccFactoryRegistry();
            com.ooc.OCI.Acceptor[] acceptors =
                new com.ooc.OCI.Acceptor[config.length];
            for(int i = 0 ; i < config.length ; i++)
            {
                com.ooc.OCI.AccFactory factory =
                    registry.get_factory(config[i].id);
                acceptors[i] = factory.create_acceptor(config[i].params);
            }

            //
            // Create the new POAManager_impl and add to the table
            //
            POAManager manager = new POAManager_impl(orbInstance_,
                                                     poaLocator_, name,
                                                     count_, acceptors);
            managers_.put(name, manager);

            return manager;
	}
    }

    public com.ooc.OBPortableServer.POAManager[]
    get_poa_managers()
    {
	java.util.Enumeration e = managers_.keys();
	java.util.Vector result = new java.util.Vector();

	while(e.hasMoreElements())
	{
	    String name = (String)e.nextElement();
	    POAManager manager = (POAManager)managers_.get(name);
	    if(manager != null)
		result.addElement(manager);
        }

	POAManager[] r = new POAManager[result.size()];
	result.copyInto(r);

	return r;
    }

    public void
    destroy()
    {
	//
	// Remove the references to the orbInstance_
	//
	orbInstance_ = null;

	//
	// Shutdown, if necessary
	//
	if(!managers_.isEmpty())
	    _OB_deactivate();
    }

    //
    // Deactivate all POAManagers
    //
    public void
    _OB_deactivate()
    {
        //
        // Deactivate each of the POAManagers
        //
	java.util.Enumeration e = managers_.keys();
	while(e.hasMoreElements())
	{
	    String name = (String)e.nextElement();
            POAManager manager = (POAManager)managers_.get(name);
	    if(manager != null)
	    {
		try
		{
		    manager.deactivate(true, true);
		}
		catch(org.omg.PortableServer.POAManagerPackage.
		      AdapterInactive ex)
		{
		    // Ignore
		}
	    }
        }
	managers_.clear();

        //
        // Tell the IMR that the ORB is STOPPING
        //
        if(activeState_ != null)
        {
            com.ooc.OB.Logger logger = orbInstance_.getLogger();
	    String serverInstance = orbInstance_.getServerInstance();
            try
            {
                activeState_.set_status(serverInstance,
		                        com.ooc.IMR.ServerStatus.STOPPING);
            }
            catch(org.omg.CORBA.SystemException ex)
            {
                String msg = orbInstance_.getServerId() +
			     ": Cannot contact IMR on shutdown";
                logger.warning(msg);
            }

	    //
	    // Clear the IMR::Server record
	    //
	    activeState_ = null;
        }
    }

    public DirectServant
    _OB_getDirectServant(org.omg.IOP.IOR ior, 
    			 com.ooc.OB.RefCountPolicyList policies)
        throws com.ooc.OB.LocationForward
    {
	//
	// Optimization
	//
	if(managers_.isEmpty())
	    return null;

        java.util.Enumeration e = managers_.keys();
        while(e.hasMoreElements())
        {
            try
            {
                String name = (String)e.nextElement();
                POAManager_impl manager = (POAManager_impl)managers_.get(name);
                if(manager != null)
                {
                    com.ooc.OCI.Acceptor[] acceptors = manager.get_acceptors();
                    for(int i = 0 ; i < acceptors.length ; i++)
                    {
                        com.ooc.OCI.ProfileInfo[] profileInfos =
                            acceptors[i].get_local_profiles(ior);

                        //
                        // If the IOR is local then at least one ProfileInfo
                        // will be returned
                        //
                        if(profileInfos.length > 0)
                        {
                            //
                            // In the case that the servant cannot support a
                            // direct invocation, null will be returned
                            //
                            return manager._OB_getDirectServant(
                                profileInfos[0].key, policies);
                        }
                    }
                }
            }
            catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
            {
                //
                // Ignore -- the POAManager isn't valid anymore
                //
            }
        }

        return null;
    }

    public void
    _OB_setORBInstance(com.ooc.OB.ORBInstance instance)
    {
	orbInstance_ = instance;
    }

    public void
    _OB_initializeIMR(POA_impl root, com.ooc.OB.ORBControl orbControl)
    {
        String serverId = orbInstance_.getServerId();
	String serverInstance = orbInstance_.getServerInstance();

	java.util.Properties properties = orbInstance_.getProperties();
	String noIMR = properties.getProperty("ooc.orb.noIMR");
	if(serverId.length() == 0 || noIMR != null)
	    return;

        //
        // Create the OAD::ProcessMonitor servant
        //
        processEndpoint_ =
	    new com.ooc.OAD.ProcessEndpoint_impl(serverId, serverInstance,
	    					 root, orbControl);

        com.ooc.IMR.Domain imrDomain = null;
        try
        {
            com.ooc.OB.InitialServiceManager initServiceManager =
                orbInstance_.getInitialServiceManager();
            org.omg.CORBA.Object imrObj =
                initServiceManager.resolveInitialReferences("IMR");
            imrDomain = com.ooc.IMR.DomainHelper.narrow(imrObj);
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName ex)
        {
	    // Ignore -- this will be handled later
        }
        catch(org.omg.CORBA.BAD_PARAM ex)
        {
            // narrow() failed
        }

	com.ooc.OB.Logger logger = orbInstance_.getLogger();

        //
        // IMR::IMRDomain not reachable?
        //
        if(imrDomain == null)
	{
            String msg = serverId + ": IMRDomain not reachable";
	    logger.error(msg);
            throw new org.omg.CORBA.INITIALIZE(msg);
	}

	//
	// Check if need to register with the IMR
	//
	String exec = properties.getProperty("ooc.orb.imr.register");
	if(exec != null)
	{
	    //
	    // TODO: What do we do for Java?
	    //
            String msg = serverId + ": Self registration not implemented" +
	                 " for java servers";
            logger.error(msg);
            throw new org.omg.CORBA.INITIALIZE(msg);
	}

        //
        // Tell the IMR that we're starting up
        //
        try
        {
            //
            // This is the ProcessEndpointManager
            //
            com.ooc.OAD.ProcessEndpointManagerHolder endpoint =
                new com.ooc.OAD.ProcessEndpointManagerHolder();

	    org.omg.PortableInterceptor.ObjectReferenceTemplate primary =
	        root.adapter_template();

            //
            // Tell the IMR that we are STARTING.
            //
            activeState_ = imrDomain.startup(serverId, serverInstance,
	    				     primary, endpoint);

	    //
	    // Link with the OAD ProcessEndpoint
	    //
            com.ooc.OAD.ProcessEndpoint ref =
	        processEndpoint_._this(orbInstance_.getORB());
            endpoint.value.establish_link(serverId, serverInstance,
	    				  0xFFFFFFFF, ref);
            endpointManager_ = endpoint.value;


	    //
	    // Create an register the IORInterceptor for the IMR
	    //
	    org.omg.PortableInterceptor.IORInterceptor i =
	        new com.ooc.PortableInterceptor.IMRIORInterceptor_impl(
		    logger, activeState_, serverInstance);
	    com.ooc.OB.PIManager piManager = orbInstance_.getPIManager();

	    try
	    {
	        piManager.addIORInterceptor(i, true);
	    }
	    catch(org.omg.PortableInterceptor.ORBInitInfoPackage.
	          DuplicateName ex)
	    {
	        com.ooc.OB.Assert._OB_assert(false);
	    }
        }
        catch(org.omg.CORBA.BAD_PARAM ex)
        {
            String msg = serverId + ": (IMR) Server already running";
            logger.error(msg);
            throw new org.omg.CORBA.INITIALIZE(msg);
        }
        catch(com.ooc.IMR.NoSuchServer ex)
        {
            String msg = serverId + ": (IMR) Not registered with IMR";
            logger.error(msg);
            throw new org.omg.CORBA.INITIALIZE();
        }
        catch(com.ooc.IMR.NoSuchOAD ex)
        {
            String msg = serverId + ": (IMR) No OAD for host";
            logger.error(msg);
            throw new org.omg.CORBA.INITIALIZE();
        }
        catch(com.ooc.IMR.OADNotRunning ex)
        {
            String msg = serverId + ": (IMR) OAD not running";
            logger.error(msg);
            throw new org.omg.CORBA.INITIALIZE();
        }
        catch(com.ooc.OAD.AlreadyLinked ex)
        {
            String msg = serverId + ": (IMR) Process registered with OAD";
            logger.error(msg);
            throw new org.omg.CORBA.INITIALIZE();
        }
    }
}
