// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.UDP;

final public class AccFactory_impl extends org.omg.CORBA.LocalObject
    implements com.ooc.OCI.AccFactory
{
    //
    // The trace level
    //
    private int trace_;

    //
    // The logger
    //
    private com.ooc.OB.Logger logger_;

    //
    // AccFactory information
    //
    private AccFactoryInfo_impl info_;

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    id()
    {
        return PLUGIN_ID.value;
    }

    public int
    tag()
    {
        return com.ooc.UDPIOP.TAG_UDP_IOP.value;
    }

    public com.ooc.OCI.AccFactoryInfo
    get_info()
    {
        return info_;
    }

    public com.ooc.OCI.Acceptor
    create_acceptor(String[] params)
        throws com.ooc.OCI.InvalidParam
    {
        String bind = null;
        int bufferSize = 0;
        String host = null;
        int messageTimeout = 15;
        boolean multicast = false;
        boolean numeric = false;
        int port = 0;
        int transportTimeout = 60;
        boolean noLoopback = false;
        int ttl = -1;

        int i = 0;
        while(i < params.length)
        {
            if(params[i].equals("--bind"))
            {
                if(i + 1 >= params.length)
                    throw new com.ooc.OCI.InvalidParam("argument expected " +
                                                       "for --bind");
                bind = params[i + 1];
                i += 2;
            }
            else if(params[i].equals("--buffer-size"))
            {
                if(i + 1 >= params.length)
                    throw new com.ooc.OCI.InvalidParam("argument expected for "
                                                       + "--buffer-size");
                try
                {
                    bufferSize = Integer.valueOf(params[i + 1]).intValue();
                    if(bufferSize < 0)
                        bufferSize = 0;
                }
                catch(NumberFormatException ex)
                {
                    throw new com.ooc.OCI.InvalidParam("invalid argument for "
                                                       + "--buffer-size");
                }
                i += 2;
            }
            else if(params[i].equals("--host"))
            {
                if(i + 1 >= params.length)
                    throw new com.ooc.OCI.InvalidParam("argument expected " +
                                                       "for --host");
                host = params[i + 1];
                i += 2;
            }
            else if(params[i].equals("--message-timeout"))
            {
                if(i + 1 >= params.length)
                    throw new com.ooc.OCI.InvalidParam("argument expected for "
                                                       + "--message-timeout");
                try
                {
                    messageTimeout = Integer.valueOf(params[i + 1]).intValue();
                    if(messageTimeout < 1)
                        messageTimeout = 15;
                }
                catch(NumberFormatException ex)
                {
                    throw new com.ooc.OCI.InvalidParam("invalid argument for "
                                                       + "--message-timeout");
                }
                i += 2;
            }
            else if(params[i].equals("--multicast"))
            {
                multicast = true;
                i++;
            }
            else if(params[i].equals("--no-loopback"))
            {
                noLoopback = true;
                i++;
            }
            else if(params[i].equals("--numeric"))
            {
                numeric = true;
                i++;
            }
            else if(params[i].equals("--port"))
            {
                if(i + 1 >= params.length)
                    throw new com.ooc.OCI.InvalidParam("argument expected for "
                                                       + "--port");
                try
                {
                    port = Integer.valueOf(params[i + 1]).intValue();
                }
                catch(NumberFormatException ex)
                {
                    throw new com.ooc.OCI.InvalidParam("invalid argument for "
                                                       + "--port");
                }
                if(port < 1 || port > 65535)
                    throw new com.ooc.OCI.InvalidParam("port must be between "
                                                       + "1 and 65535");
                i += 2;
            }
            else if(params[i].equals("--ttl"))
            {
                if(i + 1 >= params.length)
                    throw new com.ooc.OCI.InvalidParam("argument expected for "
                                                       + "--ttl");
                try
                {
                    ttl = Integer.valueOf(params[i + 1]).intValue();
                }
                catch(NumberFormatException ex)
                {
                    throw new com.ooc.OCI.InvalidParam("invalid argument for "
                                                       + "--ttl");
                }
                if(ttl < 0 || ttl > 255)
                    throw new com.ooc.OCI.InvalidParam("ttl must be between "
                                                       + "0 and 255");
                i += 2;
            }
            else if(params[i].equals("--transport-timeout"))
            {
                if(i + 1 >= params.length)
                    throw new com.ooc.OCI.InvalidParam("argument expected " +
                                                       "for " +
                                                       "--transport-timeout");
                try
                {
                    transportTimeout =
                        Integer.valueOf(params[i + 1]).intValue();
                    if(transportTimeout < 1)
                        transportTimeout = 60;
                }
                catch(NumberFormatException ex)
                {
                    throw new com.ooc.OCI.InvalidParam("invalid argument " +
                                                       "for " +
                                                       "--transport-timeout");
                }
                i += 2;
            }
            else
            {
                throw new com.ooc.OCI.InvalidParam("unknown parameter: " +
                                                   params[i]);
            }
        }

        if(host == null)
	{
            host = com.ooc.OB.Net.getCanonicalHostname(numeric);
	}

        if(bind == null)
	{
            return new Acceptor_impl(host, port, multicast, 
				     noLoopback, ttl, trace_,
                                     bufferSize, messageTimeout,
                                     transportTimeout, logger_);
	}
        else
	{
            return new Acceptor_impl(bind, host, port, multicast, 
				     noLoopback, ttl, trace_,
                                     bufferSize, messageTimeout,
                                     transportTimeout, logger_);
	}
    }

    public void
    change_key(org.omg.IOP.IORHolder ior, byte[] key)
    {
        //
        // Extract the UDP profile information from the provided IOR
        //
        for(int profile = 0 ; profile < ior.value.profiles.length ; profile++)
        {
            if(ior.value.profiles[profile].tag ==
               com.ooc.UDPIOP.TAG_UDP_IOP.value)
            {
                //
                // Extract the profile body
                //
                byte[] data = ior.value.profiles[profile].profile_data;
                com.ooc.OCI.Buffer buf =
                    new com.ooc.OCI.Buffer(data, data.length);
                com.ooc.CORBA.InputStream in =
                    new com.ooc.CORBA.InputStream(buf, 0, false);
                in._OB_readEndian();
                com.ooc.UDPIOP.ProfileBody body =
                    com.ooc.UDPIOP.ProfileBodyHelper.read(in);

                //
                // Fill in the new object-key
                //
                body.object_key = key;

                //
                // Remarshal the new body
                //
                com.ooc.OCI.Buffer buf2 = new com.ooc.OCI.Buffer();
                com.ooc.CORBA.OutputStream out =
                    new com.ooc.CORBA.OutputStream(buf2);
                out._OB_writeEndian();
                com.ooc.UDPIOP.ProfileBodyHelper.write(out, body);

                ior.value.profiles[profile].profile_data =
                    new byte[buf2.length()];
                System.arraycopy(buf2.data(), 0,
                                 ior.value.profiles[profile].profile_data,
                                 0, buf2.length());
            }
        }
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    public
    AccFactory_impl(int trace, com.ooc.OB.Logger logger)
    {
        trace_ = trace;
        logger_ = logger;
        info_ = new AccFactoryInfo_impl();
    }
}
