// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.UDP;

final class ClientTransport_impl extends org.omg.CORBA.LocalObject
    implements com.ooc.OCI.Transport
{
    //
    // The socket
    //
    // NOTE: Must not be private, because ClientTransportInfo_impl
    // accesses this member
    //
    java.net.DatagramSocket fd_;

    //
    // The trace level
    //
    private int trace_;

    //
    // The packet size
    //
    private int packetSize_;

    //
    // The packet delay
    //
    private int packetDelay_;

    //
    // The logger
    //
    private com.ooc.OB.Logger logger_;

    //
    // The transport information object
    //
    private ClientTransportInfo_impl info_;

    //
    // The client ID
    //
    private int clientID_;

    //
    // The next message ID
    //
    private int nextMessageID_;

    // ------------------------------------------------------------------
    // Private and protected member implementations
    // ------------------------------------------------------------------

    private int
    getNextMessageID()
    {
        return nextMessageID_++;
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    id()
    {
        return PLUGIN_ID.value;
    }

    public int
    tag()
    {
        return com.ooc.UDPIOP.TAG_UDP_IOP.value;
    }

    public com.ooc.OCI.SendReceiveMode
    mode()
    {
        return com.ooc.OCI.SendReceiveMode.SendOnly;
    }

    public int
    handle()
    {      
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    public void
    close()
    {
        //
        // Call callbacks
        //
        info_._OB_callCloseCB(info_);

        //
        // Destroy the info object
        //
        info_._OB_destroy();

        //
        // Close the socket
        //
        fd_.close();
        fd_ = null;
    }

    public void
    shutdown()
    {
        //
        // Nothing to do here
        //
    }

    public void
    receive(com.ooc.OCI.Buffer buf, boolean block)
    {
        //
        // Not implemented
        //
        throw new InternalError();
    }

    public boolean
    receive_detect(com.ooc.OCI.Buffer buf, boolean block)
    {
        // Should not be called by the client
        com.ooc.OB.Assert._OB_assert(false); 
        return true;
    }

    public void
    receive_timeout(com.ooc.OCI.Buffer buf, int t)
    {
    }

    public boolean
    receive_timeout_detect(com.ooc.OCI.Buffer buf, int t)
    {
        // Should not be called by the client
        com.ooc.OB.Assert._OB_assert(false); 
        return true;
    }

    public void
    send(com.ooc.OCI.Buffer buf, boolean block)
    {
        //
        // Get the next message ID
        //
        int messageID = getNextMessageID();

        //
        // Create the packet buffer and the associated output stream
        //
        com.ooc.OCI.Buffer packet = 
            new com.ooc.OCI.Buffer(packetSize_);
        com.ooc.CORBA.OutputStream out =
            new com.ooc.CORBA.OutputStream(packet);

        //
        // Create and marshal the fixed header
        //
        com.ooc.UDPIOP.FixedPacketHeader fHeader = 
            new com.ooc.UDPIOP.FixedPacketHeader();

        fHeader.flags = false ? (byte)1 : (byte)0; // false is big endian

        //
        // Allocate memory for reserved and extensions, fields.
        //
        fHeader.reserved = new byte[3];
        fHeader.extensions = new com.ooc.UDPIOP.Extension[0];

        fHeader.client_id = clientID_;
        fHeader.message_id = messageID;
        fHeader.message_size = buf.rest_length();
        com.ooc.UDPIOP.FixedPacketHeaderHelper.write(out, fHeader);

        //
        // Save the fixed header marshalling position
        //
        int fHeaderPos = packet.pos();

        //
        // Calculate the payload
        //
        int payload = packet.length() - fHeaderPos - 4;

        //
        // Fragment the message
        //
        int lastPayload = fHeader.message_size % payload;
        int numberOfPackets = fHeader.message_size / payload;
        if(lastPayload > 0)
            numberOfPackets++;

        for(int packetNum = 0 ; packetNum < numberOfPackets ; packetNum++)
        {
            if(trace_ > 0)
            {
                String str = "sending packet ";
                str += packetNum + 1;
                str += '/';
                str += numberOfPackets;
                str += "\nclient id = ";
                str += clientID_;
                str += "\nmessage id = ";
                str += messageID;
                str += "\nmessage size = ";
                str += fHeader.message_size;
                logger_.trace("UDP", str);
            }

            //
            // Calculate current payload
            //
            int currentPayload;
            if(packetNum < numberOfPackets - 1)
            {
                currentPayload = payload;
            }
            else
            {
                if(lastPayload > 0)
                    currentPayload = lastPayload;
                else
                    currentPayload = payload;
            }

            //
            // Reset the marshalling position
            //
            packet.pos(fHeaderPos);

            //
            // Create and marshal the variable header
            //
            com.ooc.UDPIOP.VariablePacketHeader vHeader =
                new com.ooc.UDPIOP.VariablePacketHeader();
            vHeader.packet_num = packetNum;
            com.ooc.UDPIOP.VariablePacketHeaderHelper.write(out, vHeader);
            com.ooc.OB.Assert._OB_assert(packet.pos() == fHeaderPos + 4);

            //
            // Add the data
            //
            System.arraycopy(buf.data(), buf.pos(), packet.data(), 
                             packet.pos(), currentPayload);
            buf.advance(currentPayload);

            //
            // Create the datagram and set the data
            //
            java.net.DatagramPacket datagramPacket;
            datagramPacket = new java.net.DatagramPacket(packet.data(), 
                                                         packet.length());

            //
            // Send the packet
            //
            try
            {
                fd_.send(datagramPacket);
            }
            catch(java.io.IOException ex)
            {
                throw new org.omg.CORBA.COMM_FAILURE(
                    ex.toString(), com.ooc.OB.MinorCodes.MinorSend,
                    org.omg.CORBA.CompletionStatus.COMPLETED_MAYBE);
            }

            //
            // Delay sending of next packet
            //
            if(packetDelay_ > 0)
            {
                try
                {
                    java.lang.Thread.sleep(packetDelay_);
                }
                catch(java.lang.InterruptedException ex)
                {
                }
            }
        }

        com.ooc.OB.Assert._OB_assert(buf.is_full());
    }

    public boolean
    send_detect(com.ooc.OCI.Buffer buf, boolean block)
    {
        try
        {
            send(buf, block);
        }
        catch(org.omg.CORBA.COMM_FAILURE ex)
        {
            return false;
        }

        return true;
    }

    public void
    send_timeout(com.ooc.OCI.Buffer buf, int t)
    {
        send(buf, true);
    }

    public boolean
    send_timeout_detect(com.ooc.OCI.Buffer buf, int t)
    {
        try
        {
            send(buf, true);
        }
        catch(org.omg.CORBA.COMM_FAILURE ex)
        {
            return false;
        }

        return true;
    }

    public com.ooc.OCI.TransportInfo
    get_info()
    {
        return info_;
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    ClientTransport_impl(com.ooc.OCI.Connector connector,
                         java.net.DatagramSocket fd,
                         int trace, int packetSize, int packetDelay,
                         com.ooc.OB.Logger logger)
    {
        fd_ = fd;
        trace_ = trace;
        packetSize_ = packetSize;
        packetDelay_ = packetDelay;
        logger_ = logger;
        info_ = new ClientTransportInfo_impl(this, connector);
        nextMessageID_ = 0;

        //
        // Create a unique client ID
        //
        clientID_ = (int)(System.currentTimeMillis() / 1000);
    }

    protected void
    finalize()
        throws Throwable
    {
        if(fd_ != null)
            close();

        super.finalize();
    }
}
