// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.watson;

import com.ooc.OB.IORUtil;

//
// Client Request Interceptor logger -- logs client activity
//
final class CRILogger_impl
    extends org.omg.CORBA.LocalObject
    implements org.omg.PortableInterceptor.ClientRequestInterceptor
{
    //
    // The RILogger object
    //
    private RILogger riLogger_;

    //
    // The logging level
    //
    private int logLevel_ ;

    // -----------------------------------------------------------------
    // CRILogger private member implementation
    // -----------------------------------------------------------------

    //
    // Display PortableInterceptor::ClientRequestInfo
    //
    private void
    displayClientRequestInfo(
	org.omg.PortableInterceptor.ClientRequestInfo info)
    {
	AnyWriter out_ = riLogger_.out_;

	riLogger_.displayRequestInfo(info, logLevel_);
	
	if(logLevel_ > 2)
	{
	    //
	    // Display effective profile
	    //
	    org.omg.IOP.TaggedProfile profile = info.effective_profile();
	    
	    out_.skip();
	    out_.getWriter().print("effective profile = ");
	    
	    if (profile.tag == org.omg.IOP.TAG_INTERNET_IOP.value)
	    {
		out_.getWriter().println("TAG_INTERNET_IOP");
	    }
	    else if (profile.tag ==
		     org.omg.IOP.TAG_MULTIPLE_COMPONENTS.value)
	    {
		out_.getWriter().println("TAG_MULTIPLE_COMPONENTS");
	    }
	    else
	    {
		out_.getWriter().println("UNKNOWN_TAG");
	    }
	    out_.getWriter().print(": ");
	    out_.getWriter().print(
		IORUtil.dump_octets(profile.profile_data, 0,
				    profile.profile_data.length));
	    out_.getWriter().flush();
	}
	
	//
	// Display received exception
	//
	try
	{
	    org.omg.CORBA.Any exception = info.received_exception();
	    out_.skip();
	    out_.getWriter().println("received exception = ");
	    out_.down();
	    out_.skip();
	    out_.print(exception);
	    out_.getWriter().println();
	    out_.up();
	}
	catch(org.omg.CORBA.BAD_INV_ORDER ex)
	{
	    // Operation not supported in this context
	}
	
	//
	// Display received exception id
	//
	try
	{
	    String exception_id = info.received_exception_id();
	    out_.skip();
	    out_.getWriter().println("received exception id = " 
				     + exception_id);
	}
	catch(org.omg.CORBA.BAD_INV_ORDER ex)
	{
	    // Operation not supported in this context
	}
    }

    // -----------------------------------------------------------------
    // CRILogger constructor
    // -----------------------------------------------------------------

    public CRILogger_impl(org.omg.PortableInterceptor.ORBInitInfo info,
    			  int slotId,
			  int logLevel,
			  com.ooc.OB.WLogger logger)
    {
	riLogger_ = new RILogger(info, slotId, logger);
	logLevel_ = logLevel;
    }

    // -----------------------------------------------------------------
    // CRILogger destroy method
    // -----------------------------------------------------------------

    public void destroy()
    {
	// Intentionally empty
    }

    // -----------------------------------------------------------------
    // IDL to Java Mapping
    // -----------------------------------------------------------------

    //
    // The name of the interceptor.
    //
    public String
    name()
    {
	return "com.ooc.watson.CRILogger_impl";
    }

    //
    // This interception point allows an Interceptor to query request
    // information and modify the service context before the request
    // is sent.
    //
    public void
    send_request(org.omg.PortableInterceptor.ClientRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	if(logLevel_ > 0)
	{
	    //
	    // Try to get the CallStack from slot
	    //
	    try
	    {
		org.omg.CORBA.Any any = ri.get_slot(riLogger_.mySlotId_);
		org.omg.CORBA.TypeCode tc = any.type();
		if(tc.kind() != org.omg.CORBA.TCKind.tk_null)
		{
		    //
		    // Encode CallStack
		    //
		    byte[] data = riLogger_.cdrCodec_.encode_value(any);

		    //
		    // Add encoded CallStack to service context
		    //
		    org.omg.IOP.ServiceContext sc =
			new org.omg.IOP.ServiceContext();
		    sc.context_id = riLogger_.REQUEST_CONTEXT_ID;
		    sc.context_data = data;
		    ri.add_request_service_context(sc, false);
		}
	    }
	    catch(org.omg.PortableInterceptor.InvalidSlot ex)
	    {
		// Ignore failure
	    }
	    catch(org.omg.IOP.CodecPackage.InvalidTypeForEncoding ex)
	    {
		// Ignore failure
	    }

	    //
	    // Send cumulated output to logger
	    //
	    displayClientRequestInfo(ri);
	    riLogger_.logTraceOutput("send request");
	}
    }

    //
    // This interception point allows an Interceptor to query
    // information during a TimeIndependent Infocation (TII)
    // polling get reply sequence.
    //
    public void
    send_poll(org.omg.PortableInterceptor.ClientRequestInfo ri)
    {
	if(logLevel_ > 0)
	{
	    displayClientRequestInfo(ri);
	    riLogger_.logTraceOutput("send poll");
	}
    }

    //
    // This interception point allows an Interceptor to query
    // the information on a reply after it is returned from the
    // server and before control is returned to the client.
    // 
    public void
    receive_reply(org.omg.PortableInterceptor.ClientRequestInfo ri)
    {
	if(logLevel_ > 0)
	{
	    displayClientRequestInfo(ri);
	    riLogger_.logTraceOutput("receive reply");
	}
    }

    //
    // This interception point allows an Interceptor to query
    // the exception information before the exception is raised
    // to the client.
    //
    public void
    receive_exception(org.omg.PortableInterceptor.ClientRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	if(logLevel_ > 0)
	{
	    displayClientRequestInfo(ri);
	    riLogger_.logTraceOutput("receive exception");
	}
    }

    //
    // This interception point allows an Interceptor to query
    // the information available when a request results in
    // something other than a normal reply or an exception.
    //
    public void
    receive_other(org.omg.PortableInterceptor.ClientRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	if(logLevel_ > 0)
	{
	    displayClientRequestInfo(ri);
	    riLogger_.logTraceOutput("receive other");
	}
    }

}




