// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.watson;

import com.ooc.OB.Assert;

//
// ORB Initializer for the server and client request interceptors
//
final public class RIORBInitializer_impl
    extends org.omg.CORBA.LocalObject
    implements org.omg.PortableInterceptor.ORBInitializer
{
    //
    // Logging levels
    //
    private int logLevelIn_;
    private int logLevelOut_;

    // 
    // The properties from the ORB configuration
    //
    private java.util.Properties properties_;

    //
    // The ORBs logger
    //
    private com.ooc.OB.WLogger logger_;

    // -----------------------------------------------------------------
    // IDL to Java Mapping
    // -----------------------------------------------------------------

    public void
    pre_init(org.omg.PortableInterceptor.ORBInitInfo info)
    {
	//
	// Get properties set via ORB config file
	//
	com.ooc.OBPortableInterceptor.ORBInitInfo orbInitInfo =
	    com.ooc.OBPortableInterceptor.ORBInitInfoHelper.narrow(info);

	com.ooc.CORBA.ORB oborb = (com.ooc.CORBA.ORB) orbInitInfo.orb();
	Assert._OB_assert(oborb != null);

	// 
	// The ORBs logger
	//
	com.ooc.OB.Logger logger = oborb.logger();
	try
	{
	    logger_ = com.ooc.OB.WLoggerHelper.narrow(logger);
	}
	catch(org.omg.CORBA.BAD_OPERATION ex)
	{
	    // Fall through
	}
	Assert._OB_assert(logger_ != null);

	logger_.trace("Watson", 
		      "module initialization for ORB " +
	              '"' + info.orb_id() + '"' );
	
	// 
	// The properties from the ORB configuration
	//
	properties_ = oborb.properties();
	Assert._OB_assert(properties_ != null);

	//
	// Set internal switches
	//
	String value;

	value = properties_.getProperty("ooc.watson.trace.requests");
	if (value != null)
	{
	    logLevelIn_ = Integer.parseInt(value);
	    logLevelOut_ = logLevelIn_;
	    logger_.trace("Watson", 
			  "ooc.watson.trace.requests     : " + value);
	}
	
	value = properties_.getProperty("ooc.watson.trace.requests.in");
	if (value != null)
	{
	    logLevelIn_ = Integer.parseInt(value);
	    logger_.trace("Watson", 
			  "ooc.watson.trace.requests.in  : " + value);
	}
	
	value = properties_.getProperty("ooc.watson.trace.requests.out");
	if (value != null)
	{
	    logLevelOut_ = Integer.parseInt(value);
	    logger_.trace("Watson", 
			  "ooc.watson.trace.requests.out : " + value);
	}
    }

    //
    // Note: the interceptors are registered in post_init()
    //       because the CRILogger_impl and SRILogger_impl
    //       constructors require
    //       ORBInitInfo::resolve_initial_reference(), which
    //       cannot be called in pre_init().
    //
    public void
    post_init(org.omg.PortableInterceptor.ORBInitInfo info)
    {
	//
	// Allocate state slot for call stack tracing
	//
	int slotId = info.allocate_slot_id();

	//
	// Create and register the interceptors
	//
	org.omg.PortableInterceptor.ClientRequestInterceptor
	    clientInterceptor = 
	        new CRILogger_impl(info, slotId, logLevelOut_, logger_);
	try
	{
	    if (logLevelOut_ > 0)
	    {
		info.add_client_request_interceptor(clientInterceptor);
	    }
	}
	catch(org.omg.PortableInterceptor.ORBInitInfoPackage.DuplicateName ex)
	{
	    logger_.trace("Watson", 
			  "client_request_interceptor "
			  + '"' + clientInterceptor.name() + '"'
			  + " already registered for ORB " 
			  + '"' + info.orb_id() + '"');
	}

	org.omg.PortableInterceptor.ServerRequestInterceptor
	    serverInterceptor = 
	        new SRILogger_impl(info, slotId, logLevelIn_, logger_);
	try
	{
	    if (logLevelIn_ > 0)
	    {
		info.add_server_request_interceptor(serverInterceptor);
	    }
	}
	catch(org.omg.PortableInterceptor.ORBInitInfoPackage.DuplicateName ex)
	{
	    logger_.warning("Watson: server_request_interceptor "
			    + '"' + serverInterceptor.name() + '"'
			    + " already registered for ORB " 
			    + '"' + info.orb_id() + '"');
	}
    }

}


