// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.watson;

import com.ooc.OB.IORUtil;


//
// Server Request Interceptor logger -- logs server activity
//
final class SRILogger_impl
    extends org.omg.CORBA.LocalObject
    implements org.omg.PortableInterceptor.ServerRequestInterceptor
{
    //
    // The RILogger object
    //
    private RILogger riLogger_;

    //
    // The logging level
    //
    private int logLevel_;

    // -----------------------------------------------------------------
    // SRILogger private member implementation
    // -----------------------------------------------------------------

    //
    // Display PortableInterceptor::ServerRequestInfo
    //
    private void
    displayServerRequestInfo(
	org.omg.PortableInterceptor.ServerRequestInfo info)
    {
	AnyWriter out_ = riLogger_.out_;

	riLogger_.displayRequestInfo(info, logLevel_);
	
	//
	// Display sending exception
	//
	try
	{
	    org.omg.CORBA.Any exception = info.sending_exception();
	    out_.skip();
	    out_.getWriter().println("sending exception = ");
	    out_.down();
	    out_.skip();
	    out_.print(exception);
	    out_.getWriter().println();
	    out_.up();
	}
	catch(org.omg.CORBA.BAD_INV_ORDER ex)
	{
	    // Operation not supported in this context
	}
	
	if(logLevel_ > 2)
	{
	    //
	    // Display object id
	    //
	    try
	    {
		out_.skip();
		out_.getWriter().println("object id = ");
		out_.getWriter().print(
		    IORUtil.dump_octets(info.object_id(), 0,
					info.object_id().length));
		out_.getWriter().flush();
	    }
	    catch(org.omg.CORBA.BAD_INV_ORDER ex)
	    {
		// Operation not supported in this context
	    }
	    
	    //
	    // Display adapter id
	    //
	    try
	    {
		out_.skip();
		out_.getWriter().println("adapter id = ");
		out_.getWriter().print(
		    IORUtil.dump_octets(info.adapter_id(), 0,
					info.adapter_id().length));
		out_.getWriter().flush();
	    }
	    catch(org.omg.CORBA.BAD_INV_ORDER ex)
	    {
		// Operation not supported in this context
	    }
	}
	
	//
	// Display target most derived interface
	//
	try
	{
	    String tmdi = info.target_most_derived_interface();
	    out_.skip();
	    out_.getWriter().println("target most derived interface = " +
				     tmdi);
	}
	catch(org.omg.CORBA.BAD_INV_ORDER ex)
	{
	    // Operation not supported in this context
	}
    }

    // -----------------------------------------------------------------
    // SRILogger constructor
    // -----------------------------------------------------------------

    public SRILogger_impl(org.omg.PortableInterceptor.ORBInitInfo info,
    			  int slotId,
			  int logLevel,
			  com.ooc.OB.WLogger logger)
    {
	riLogger_ = new RILogger(info, slotId, logger);
	logLevel_ = logLevel;
    }

    // -----------------------------------------------------------------
    // SRILogger destroy method
    // -----------------------------------------------------------------

    public void destroy()
    {
	// Intentionally empty
    }

    // -----------------------------------------------------------------
    // IDL to Java Mapping
    // -----------------------------------------------------------------

    //
    // The name of the interceptor.
    //
    public String
    name()
    {
	return "com.ooc.watson.CRILogger_impl";
    }

    //
    // At this interception point, Interceptors must get their service
    // context information from the incoming request and transfer it
    // to a slot of PortableInterceptor::Current.
    //
    public void
    receive_request_service_contexts(
    	org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	if(logLevel_ > 0)
	{
	    AnyWriter out_ = riLogger_.out_;

	    displayServerRequestInfo(ri);

	    //
	    // Get the CallStack from the service context
	    //
	    String[] callStack = null;
	    try
	    {
		//
		// Get encoded CallStack from service context
		//
		org.omg.IOP.ServiceContext sc =
		    ri.get_request_service_context(
			riLogger_.REQUEST_CONTEXT_ID);

		//
		// Decode CallStack
		//
		org.omg.CORBA.Any any = null;
		try
		{
		    any = riLogger_.cdrCodec_.decode_value(
			sc.context_data, CallStackHelper.type());
		}
		catch(org.omg.IOP.CodecPackage.TypeMismatch ex)
		{
		    // Ignore failure
		}
		catch(org.omg.IOP.CodecPackage.FormatMismatch ex)
		{
		    // Ignore failure
		}

		String[] stack = null;
		try
		{
		    stack = CallStackHelper.extract(any);
		}
		catch(org.omg.CORBA.BAD_OPERATION ex)
		{
		    // Ignore failure 
		}
		if(stack == null)
		    throw new RuntimeException();

		//
		// Create new CallStack to hold old CallStack plus
		// one new call
		//
		callStack = new String[stack.length + 1];

		//
		// Copy old CallStack to new CallStack
		//
		System.arraycopy(stack, 0, callStack, 0, stack.length);
	    }
	    catch(org.omg.CORBA.BAD_PARAM ex)
	    {
		//
		// Service context was not previously set -- create new
		// CallStack to contain the new call
		//
		callStack = new String[1];
	    }

	    //
	    // Add new call to the CallStack
	    //
	    callStack[callStack.length - 1] = ri.operation();
		
	    //
	    // Save the new CallStack in slot
	    //
	    org.omg.CORBA.Any any = org.omg.CORBA.ORB.init().create_any();
	    CallStackHelper.insert(any, callStack);
	    try
	    {
		ri.set_slot(riLogger_.mySlotId_, any);
	    }
	    catch(org.omg.PortableInterceptor.InvalidSlot ex)
	    {
		// Ignore failure
	    }

	    //
	    // Print the CallStack
	    //
	    out_.skip();
	    out_.getWriter().println("stack =");
	    out_.down();
	    for(int i = 0 ; i < callStack.length ; i++)
	    {
		out_.skip();
		out_.getWriter().println((i + 1) + ".	" + callStack[i]);
	    }
	    out_.up();

	    //
	    // Send cumulated output to logger
	    //
	    riLogger_.logTraceOutput("receive request service contexts");
	}
    }

    //
    // This interception point allows an Interceptor to query request
    // information after all the information, including operation
    // parameters, are available.
    //
    public void
    receive_request(org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	if(logLevel_ > 0)
	{
	    displayServerRequestInfo(ri);
	    riLogger_.logTraceOutput("receive request");
	}
    }
    
    //
    // This interception point allows an Interceptor to query reply
    // information and modify the reply service context after the
    // target operation has been invoked and before the reply is
    // returned to the client.
    //
    public void
    send_reply(org.omg.PortableInterceptor.ServerRequestInfo ri)
    {
	if(logLevel_ > 0)
	{
	    displayServerRequestInfo(ri);
	    riLogger_.logTraceOutput("send reply");
	}
    }
    
    //
    // This interception point allows an Interceptor to query the
    // exception information and modify the reply service context
    // before the exception is raised to the client.
    //
    public void
    send_exception(org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	if(logLevel_ > 0)
	{
	    displayServerRequestInfo(ri);
	    riLogger_.logTraceOutput("send exception");
	}
    }

    //
    // This interception point allows an Interceptor to query
    // the information available when a request results in
    // something other thatn a normal reply or an exception.
    //
    public void
    send_other(org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	if(logLevel_ > 0)
	{
	    displayServerRequestInfo(ri);
	    riLogger_.logTraceOutput("send other");
	}
    }
}

















